/*****************************************************************

  audiobuf.h

  Copyright (c) 2001 by Burkhard Plaum - plaum@ipf.uni-stuttgart.de

  http://gmerlin.sourceforge.net

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111, USA.

*****************************************************************/

// Audio FIFO buffer

#ifndef _audiobuf_h_
#define _audiobuf_h_

// #include "buffer.h"

// #define GV_AUDIO_BUFFER_INPUT_LOCKED  (1<<0)
// #define GV_AUDIO_BUFFER_OUTPUT_LOCKED (1<<1)
// #define GV_AUDIO_BUFFER_FULL          (1<<2)
// #define GV_AUDIO_BUFFER_EMPTY         (1<<3)

// Conversion modes

#define GV_BUFFER_AUDIO_CONVERT_ENDIAN                   (1<<0)
#define GV_BUFFER_AUDIO_CONVERT_SIGN                     (1<<1)
#define GV_BUFFER_AUDIO_CONVERT_WIDTH_8_TO_16            (1<<2)
#define GV_BUFFER_AUDIO_CONVERT_WIDTH_16_TO_8            (1<<3)
#define GV_BUFFER_AUDIO_CONVERT_DOWNMIX_5_TO_4           (1<<4)
#define GV_BUFFER_AUDIO_CONVERT_DOWNMIX_5_TO_2           (1<<5)
#define GV_BUFFER_AUDIO_CONVERT_DOWNMIX_4_TO_2           (1<<6)
#define GV_BUFFER_AUDIO_CONVERT_INTERLEAVE_2_TO_ALL      (1<<7) 
#define GV_BUFFER_AUDIO_CONVERT_INTERLEAVE_2_TO_NONE     (1<<8) 
#define GV_BUFFER_AUDIO_CONVERT_INTERLEAVE_ALL_TO_2      (1<<9)
#define GV_BUFFER_AUDIO_CONVERT_INTERLEAVE_ALL_TO_NONE  (1<<10)
#define GV_BUFFER_AUDIO_CONVERT_INTERLEAVE_NONE_TO_ALL  (1<<11)
#define GV_BUFFER_AUDIO_CONVERT_INTERLEAVE_NONE_TO_2    (1<<12)

#define GV_BUFFER_AUDIO_CONVERT_WIDTH_MASK \
(GV_BUFFER_AUDIO_CONVERT_WIDTH_8_TO_16|\
GV_BUFFER_AUDIO_CONVERT_WIDTH_16_TO_8)

#define GV_BUFFER_AUDIO_CONVERT_DOWNMIX_MASK \
(GV_BUFFER_AUDIO_CONVERT_DOWNMIX_5_TO_4|\
GV_BUFFER_AUDIO_CONVERT_DOWNMIX_5_TO_2|\
GV_BUFFER_AUDIO_CONVERT_DOWNMIX_4_TO_2)

#define GV_BUFFER_AUDIO_CONVERT_INTERLEAVE_MASK \
(GV_BUFFER_AUDIO_CONVERT_INTERLEAVE_2_TO_ALL|\
GV_BUFFER_AUDIO_CONVERT_INTERLEAVE_2_TO_NONE|\
GV_BUFFER_AUDIO_CONVERT_INTERLEAVE_ALL_TO_2|\
GV_BUFFER_AUDIO_CONVERT_INTERLEAVE_ALL_TO_NONE|\
GV_BUFFER_AUDIO_CONVERT_INTERLEAVE_NONE_TO_ALL|\
GV_BUFFER_AUDIO_CONVERT_INTERLEAVE_NONE_TO_2)


class GVBufferAudio : public GVBuffer<GVAudioFrame,GVAudioFormat>
  {
  void alloc_frame(GVAudioFrame * frame, const GVAudioFormat * format);
  void free_frame(GVAudioFrame * frame, const GVAudioFormat * format);

  void convert(GVAudioFrame * src, GVAudioFrame * dst);
  
  int output_can_alloc();
  int input_can_alloc();
  void set_formats();

  int need_scratch();

  void clear_frame(GVAudioFrame * frame, const GVAudioFormat * format);
  
public:

  GVBufferAudio();
  virtual ~GVBufferAudio();
  
  
  };

#endif
