/*****************************************************************

  audioformat.cc

  Copyright (c) 2001 by Burkhard Plaum - plaum@ipf.uni-stuttgart.de

  http://gmerlin.sourceforge.net

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111, USA.

*****************************************************************/

#include "config.h"

#include <glib.h>
#include <iostream>
#include <iomanip.h>
#include "gvtime.h"
#include "audioformat.h"

GVAudioFormat::GVAudioFormat()
  {
  interleave_mode = GVInterleaveNone; // No interleaving, all channels are separated
  endian_format = GVNativeEndian;
  sample_type = GVAudio_16_signed;
  }

void GVAudioFormat::init()
  {
  switch(sample_type)
    {
    case GVAudio_8_signed:
    case GVAudio_8_unsigned:
      bytes_per_sample = 1;
      break;
    case GVAudio_16_signed:
    case GVAudio_16_unsigned:
      bytes_per_sample = 2;
      break;
    }
  frame_time.set_from_frames((unsigned int)samples_per_frame,
                             (unsigned int)samplerate);

  buffer_size = samples_per_frame * num_channels * bytes_per_sample;
  }

void GVAudioFormat::print(ostream & output) const
  {
  //  cerr << "Audio Format:\n"; 

  output << "Samplerate: " << ((float)samplerate)/1000.0 << " kHz\n";

  output << "Channels: " << num_channels << "\n";
  
  output << "Sample Type: ";
  switch(sample_type)
    {
    case GVAudio_8_signed:
      output << "8 Bit signed\n";
      break;
    case GVAudio_16_signed:
      output << "16 Bit signed\n";
      break;
    case GVAudio_8_unsigned:
      output << "8 Bit unsigned\n";
      break;
    case GVAudio_16_unsigned:
      output << "16 Bit unsigned\n";
      break;
    }
  if(bytes_per_sample > 1)
    {
    output << "Byte Order: ";
    switch(endian_format)
      {
      case GVLittleEndian:
        output << " Little Endian\n";
        break;
      case GVBigEndian:
        output << " Big Endian\n";
        break;
      }
    }
  else
    output << "\n";
  
  output << "Interleave mode: ";

  switch(interleave_mode)
    {
    case GVInterleaveNone:
      output << "None\n";
      break;
    case GVInterleave2:
      output << "2\n";
      break;
    case GVInterleaveAll:
      output << "All\n";
      break;
    }

  //  output << "(" << bytes_per_sample << " bytes per sample)\n";
  
  output << "Total Samples: " << num_samples << "\n";

  output << "Audio Time: ";

  if(!num_samples)
    {
    output << "Unknown\n";
    }
  else
    {
    int hours;
    int minutes;

    double time;
  
    time = (double)(num_samples)/(double)(samplerate);

    int time_i = (int)(time);

    time_i /= 60;
    
    minutes = time_i % 60;
    time_i /= 60;
    
    hours = time_i;
    
    time -= ((minutes*60) + (hours * 3600));
    if(hours)
      {
      if(hours >= 10)
        output << setw(2) << hours;
      else
        output << setw(1) << hours;
      output << ":";
      }
    
    if(!(hours) && (minutes < 10))
      output << setw(1) << minutes;
    else
      output << setw(2) << minutes;
    
    output << ":" << time << "\n";
    }
  output << "Samples per frame: " << samples_per_frame << "\n";
  output << "Buffer Size: " << buffer_size;
  }
