/*****************************************************************

  audioformat.h

  Copyright (c) 2001 by Burkhard Plaum - plaum@ipf.uni-stuttgart.de

  http://gmerlin.sourceforge.net

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111, USA.

*****************************************************************/

// Audio Format description

#ifndef __AUDIOFORMAT_H_
#define __AUDIOFORMAT_H_


union GVAudioSamples
  {
  gint8  * s_8;   //  8 bit signed
  gint16 * s_16;  // 16 bit signed

  guint8  * u_8;  //  8 bit unsigned
  guint16 * u_16; // 16 bit unsigned
  };

union GVAudioChannels
  {
  gint8  ** s_8;   //  8 bit signed
  gint16 ** s_16;  // 16 bit signed

  guint8  ** u_8;  //  8 bit unsigned
  guint16 ** u_16; // 16 bit unsigned
  };

struct GVAudioFrame
  {
  GVTime time;               // Timestamp
  GVAudioSamples samples;    // Pointer to the audio samples
  GVAudioChannels channels;  // Pointer to the audio channels
  unsigned int valid_samples;
  GVAudioFrame * next;
  };

enum GVAudioChannel
{
  GVAudioChannelFrontLeft  = 0,
  GVAudioChannelFrontRight = 1,
  GVAudioChannelRearLeft   = 2,
  GVAudioChannelRearRight  = 3,
  GVAudioChannelCenter     = 4
};

// #define GV_AUDIO_INTERLEAVE_2   (1<<0) // 2 Channels are interleaved
// #define GV_AUDIO_INTERLEAVE_ALL (1<<2) // All Channels are interleaved
// #define GV_AUDIO_BIG_ENDIAN     (1<<3) // Big Endian format

enum GVInterleaveMode
  {
    GVInterleaveNone, // No interleaving, all channels are separated
    GVInterleave2,    // 2 interleaved channels follow the next 2
    GVInterleaveAll   // All channels are interleaved
  };

enum GVEndianFormat
  {
    GVLittleEndian = 0,  // Little endian
    GVBigEndian    = 1   // Big Endian
  };

#ifndef AFMT_U16_NE
#if(G_BYTE_ORDER == G_LITTLE_ENDIAN)
#define GVNativeEndian GVLittleEndian
#else
#define GVNativeEndian GVBigEndian
#endif
#endif


enum GVAudioSampleType
  {
    GVAudio_8_signed,
    GVAudio_16_signed,
    GVAudio_8_unsigned,
    GVAudio_16_unsigned
  };

// class ostream;

struct GVAudioFormat
  {
  GVAudioFormat();

  GVEndianFormat endian_format;
  GVInterleaveMode interleave_mode;

  GVAudioSampleType sample_type;
  int bytes_per_sample;
  //   int flags;
  unsigned int samplerate;         // Samplerate
  unsigned int num_channels;       // Number of channels
  unsigned int num_samples;        // Total number of samples
  unsigned int samples_per_frame;  // Number of samples per frame

  int buffer_size;
  
  GVTime frame_time;

  void print(ostream & output) const;
  
  GVAudioFormat& operator=(const GVAudioFormat & format)
    {
    sample_type = format.sample_type; 
    bytes_per_sample = format.bytes_per_sample;

    interleave_mode = format.interleave_mode;
    endian_format = format.endian_format;

    samplerate = format.samplerate;        
    num_channels = format.num_channels;      
    num_samples = format.num_samples;       
    samples_per_frame = format.samples_per_frame; 
    init();
    return *this;
    }

//   int operator!=(const GVAudioFormat & format) const
//     {
//     return ((sample_type != format.sample_type) ||
//             (interleave_mode != format.interleave_mode) ||
//             (samplerate != format.samplerate) ||
//             (num_channels != format.num_channels) ||      
//             (samples_per_frame != format.samples_per_frame));
//      }

  
  void init();
  
  };

#endif //  __AUDIOFORMAT_H_
