/*****************************************************************

  audioout.cc

  Copyright (c) 2001 by Burkhard Plaum - plaum@ipf.uni-stuttgart.de

  http://gmerlin.sourceforge.net

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111, USA.

*****************************************************************/

// Routines common for all audio output plugins

#ifdef __GNUG__
#pragma implementation "audioout.h"
#endif

#include "config.h"

#include <gtk/gtk.h>
#include <pthread.h>
#include <iostream>

using std::ostream;
using std::cerr;

#include "gvtime.h"

#include "videoformat.h"
#include "audioformat.h"
#include "buffer.h"
#include "videobuf.h"
#include "audiobuf.h"

#include "playlist.h"
#include "baseplugin.h"
#include "plugin.h"
#include "input.h"
#include "output.h"
#include "audioout.h"
#include "videoout.h"

#include "configwindow.h"
#include "engine.h"

GVOutputAudio::GVOutputAudio(const GVPluginAttributes * _attributes) :
  GVOutput(_attributes)
  {
  flags = GV_PLUGIN_TRACK_HAS_AUDIO;
  do_audio = 1;
  do_video = 0;
  }

void GVOutputAudio::playback_func()
  {
  //  cerr << attributes->name << ": playback_func() started\n";

  audio_samples_written = 0;
  pthread_mutex_lock(&internal_time_mutex);
  internal_time.seconds = 0;
  internal_time.nanoseconds = 0;
  pthread_mutex_unlock(&internal_time_mutex);

  //  GVEngineState state;
    
  GVAudioFrame * audio_frame;

  //  struct timespec long_time;
  GVTime short_time;
  GVTime sleep_time;

  //  long_time = audio_format.frame_time;
  //  long_time /= 2;  
  
  short_time = audio_format.frame_time;
  short_time /= 5;  

  //  pthread_mutex_lock(&destroy_mutex);

  gv_engine->plugin_attach();
  gv_engine->wait_for_start();

  int valid_samples;
  
  while(1)
    {
    //    cerr << "Checking Engine State...";
    if(check_engine_state())
      break;
    //    cerr << "Done\n";
    if(audio_buffer->empty())
      {
      if(gv_engine->get_state() == GVEngineCompleting)
        {
        cerr << attributes->name << ": Engine completing\n";
        break;
        }
      else
        {
        //        cerr << "\nAudio Buffer Empty\n";
        short_time.sleep();
        continue;
        }
      }
    //    cerr << "Locking output buffer...";
    audio_frame = audio_buffer->lock_read();
    //    cerr << "done\n";
    put_frame(audio_frame);
    
    audio_samples_written += audio_frame->valid_samples;
    valid_samples = audio_frame->valid_samples;
    pthread_mutex_lock(&internal_time_mutex);
    internal_time = audio_frame->time;
    pthread_mutex_unlock(&internal_time_mutex);
    
    //    cerr << "Unlocking output buffer...";
    audio_buffer->unlock_read();
    //    cerr << "done\n";
    
//     if(((unsigned int)audio_samples_written >= audio_format.num_samples))
// //        (audio_frame->valid_samples < audio_format.samples_per_frame))
//       break;
    
    if(realtime)
      {
      sleep_time.set_from_frames(valid_samples,
                                 audio_format.samplerate);
      sleep_time /= 2;
//       cerr << "Sleeping "
//            << sleep_time.seconds << " seconds, "
//            << sleep_time.nanoseconds << " nanoseconds...";
      sleep_time.sleep();
      //      cerr << "done\n";
      }
    }
  
//   cerr << "Samples written: " << audio_samples_written << "\n";
//   cerr << "Total Samples:   " << audio_format.num_samples << "\n";
  
  cleanup();
  gv_engine->plugin_detach();

  //  pthread_mutex_unlock(&destroy_mutex);
  //  cerr << attributes->name << ": playback_func() finished\n";
  }


GVOutputAudio::~GVOutputAudio()
  {
  
  }
