/*****************************************************************

  baseplugin.h

  Copyright (c) 2001 by Burkhard Plaum - plaum@ipf.uni-stuttgart.de

  http://gmerlin.sourceforge.net

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111, USA.

*****************************************************************/

// Base class for all plugins

#ifdef __GNUG__
#pragma interface
#endif


// Attribute flags

#define GV_PLUGIN_HAS_VIDEO        (1<<0)
#define GV_PLUGIN_HAS_AUDIO        (1<<1)
#define GV_PLUGIN_NEEDS_REALTIME   (1<<2) // needs realtime
#define GV_PLUGIN_IS_FILE          (1<<3) // reads from a file
#define GV_PLUGIN_IS_DEVICE        (1<<4) // reads from a device
#define GV_PLUGIN_NO_OUTPUT        (1<<5) // has it's own output
#define GV_PLUGIN_OWN_TIME         (1<<6) // manages the whole time
#define GV_PLUGIN_CAN_SYNC         (1<<7) // has a working set_time function
#define GV_PLUGIN_CAN_ALLOC        (1<<8) // can (must) alloc it's own frames
#define GV_PLUGIN_CAN_CHANGE       (1<<9) // Can change the track without reset
#define GV_PLUGIN_CAN_SEEK         (1<<10) // Can seek
#define GV_PLUGIN_HAS_DEVICESTRING (1<<11) // Plugin can get a device specific string
#define GV_PLUGIN_IS_RECORDER      (1<<12) // Plugin does recording
#define GV_PLUGIN_HANDLES_URLS     (1<<13) // Plugin can handle URLS

// Attributes of a plugin

struct GVPluginAttributes
  {
  char * name;        // Unique identifier
  char * real_name;   // Name for menus etc
  char * description; // Description
  char * suffixes;    // File suffixes handled by the plugin
  int flags;          // OR'ed combination of the above flags
  };


class GVBasePlugin
  {
protected:

  GVBasePlugin(const GVPluginAttributes * _attributes);
    
public:
  
  // Pointer to the dynamic module handle
  gpointer module;

  // Attributes
  const GVPluginAttributes * attributes;
  };

// extern "C" {
//   extern GVBasePlugin * get_plugin();
//   extern void destroy_plugin(GVBasePlugin * plugin);
// }
