/*****************************************************************

  button.cc

  Copyright (c) 2001 by Burkhard Plaum - plaum@ipf.uni-stuttgart.de

  http://gmerlin.sourceforge.net

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111, USA.

*****************************************************************/

#include "config.h"

#include <iostream>
#include <gtk/gtk.h>

#include "gvtime.h"
#include "videoformat.h"
#include "videoconvert.h"

#include "baseplugin.h"
#include "imagereader.h"

#include "gvloader.h"

#include "button.h"
#include "xmanager.h"

#include "skin.h"

using std::cerr;

static gboolean __expose_callback(GtkWidget * w, GdkEventExpose * event,
                                  gpointer data)
  {
  ((GVButton*)data)->expose_callback();
  return TRUE;
  }

static void __set_style_callback(GtkWidget * w, GtkStyle * old_style,
                                     gpointer data)
  {
  ((GVButton*)data)->set_style_callback();
  //  return true;
  }

static gboolean __callback(GtkWidget * w, GdkEventAny * event,
                                  gpointer data)
  {
  ((GVButton*)data)->callback(event);
  return TRUE;
  }

static void __realize_callback(GtkWidget * w, gpointer data)
  {
  ((GVButton*)data)->realize_callback();
  }

static void __menu_position_func(GtkMenu   *menu,
                          gint      *x,
                          gint      *y,
                          gpointer   user_data)
  {
  GdkEventButton * event = (GdkEventButton*)user_data;
  *x = (gint)(event->x_root);
  *y = (gint)(event->y_root);
  }

void GVButton::callback(GdkEventAny * event)
  {
  switch(event->type)
    {
    case GDK_BUTTON_PRESS:
      state = Pressed;
      expose_callback();
      if(menu)
        gtk_menu_popup(GTK_MENU(menu), NULL, NULL, __menu_position_func,
                       (gpointer)event, 3, ((GdkEventButton*)(event))->time);
      break;
    case GDK_BUTTON_RELEASE:
      if(state == Normal)
        break;
      else
        {
        state = Highlight;
        expose_callback();
        if(press_callback)
          press_callback(this, press_callback_data);
        break;
        }
    case GDK_ENTER_NOTIFY:
      state = Highlight;
      expose_callback();
      break;
    case GDK_LEAVE_NOTIFY:
      state = Normal;
      expose_callback();
      break;
    }
  }

GVButton::GVButton(GtkWidget * _parent)
  {
  drawingarea = gtk_drawing_area_new();

  
  gtk_widget_set_events(drawingarea,
                        GDK_EXPOSURE_MASK|
                        GDK_BUTTON_PRESS_MASK|
                        GDK_BUTTON_RELEASE_MASK|
                        GDK_ENTER_NOTIFY_MASK|
                        GDK_LEAVE_NOTIFY_MASK);

  gtk_signal_connect(GTK_OBJECT(drawingarea), "realize",
                     GTK_SIGNAL_FUNC(__realize_callback), (gpointer)this);
  
  gtk_signal_connect(GTK_OBJECT(drawingarea), "expose_event",
                     GTK_SIGNAL_FUNC(__expose_callback), (gpointer)this);

  gtk_signal_connect(GTK_OBJECT(drawingarea), "style_set",
                     GTK_SIGNAL_FUNC(__set_style_callback),
                     (gpointer)this);
  
  gtk_signal_connect(GTK_OBJECT(drawingarea), "button_press_event",
                     GTK_SIGNAL_FUNC(__callback), (gpointer)this);
  gtk_signal_connect(GTK_OBJECT(drawingarea), "button_release_event",
                     GTK_SIGNAL_FUNC(__callback), (gpointer)this);

  gtk_signal_connect(GTK_OBJECT(drawingarea), "enter_notify_event",
                     GTK_SIGNAL_FUNC(__callback), (gpointer)this);
  gtk_signal_connect(GTK_OBJECT(drawingarea), "leave_notify_event",
                     GTK_SIGNAL_FUNC(__callback), (gpointer)this);
  
  parent = _parent; // Parent (Must be GtkLayout)

  gtk_widget_show(drawingarea);
  gtk_layout_put(GTK_LAYOUT(parent), drawingarea, 0, 0);
  
  width = 0;
  height = 0;
  press_callback = (void (*)(GVButton * button, gpointer callback_data))0;

  press_callback_data = (gpointer)0;

  menu = (GtkWidget*)0;
  skin = (GVButtonSkin*)0;
  state = Normal;
  
  is_realized = 0;
  }

void GVButton::realize_callback()
  {
  is_realized = 1;
  if((skin) && (skin->normal_pixmap))
    set_skin((GVButtonSkin*)0);
  }

void GVButton::set_skin(const GVButtonSkin * _skin)
  {
  if(_skin)
    skin = _skin;

  if(!(skin->normal_pixmap))
    {
    cerr << "GVButton: tried to set NULL skin\n";
    return;
    }
  
  if(!is_realized)
    return;

  //  cerr << "GVButton::set_skin()\n";
    
  int x, y, depth;
  
  gdk_window_get_geometry(skin->normal_pixmap, &x, &y,
                          &width, &height, &depth);

  // Set the size of the drawing area

  gtk_drawing_area_size(GTK_DRAWING_AREA(drawingarea), width, height);

  gtk_layout_move(GTK_LAYOUT(parent), 
                  drawingarea, 
                  skin->x, 
                  skin->y);

  gdk_window_shape_combine_mask(drawingarea->window, skin->mask, 0, 0);
  expose_callback();
  }

void GVButton::expose_callback()
  {
  GdkPixmap * pixmap = (GdkPixmap*)0;
  
  switch(state)
    {
    case Normal:
      pixmap = skin->normal_pixmap;
      break;
    case Highlight:
      pixmap = skin->highlight_pixmap;
      break;
    case Pressed:
      pixmap = skin->pressed_pixmap;
      break;
    }
  gdk_draw_pixmap(drawingarea->window, gv_x_manager->get_gc(), pixmap,
                  0, // xsrc
                  0, // ysrc
                  0, // xdest
                  0, // ydest
                  width,
                  height);
  }

void GVButton::set_style_callback()
  {
  if(drawingarea->window)
    {
    gdk_window_shape_combine_mask(drawingarea->window, skin->mask, 0, 0);
    expose_callback();
    }
  }
