/*****************************************************************

  configsection.cc

  Copyright (c) 2001 by Burkhard Plaum - plaum@ipf.uni-stuttgart.de

  http://gmerlin.sourceforge.net

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111, USA.

*****************************************************************/

// Methods for one section in the configuration database

#include "config.h"

#include <glib.h>
#include <string.h>
#include "gvconfig.h"

//   GList * items;
//   char * name;
  
GVConfigSection::GVConfigSection(const char * _name)
  {
  used = 0;
  items = (GList *)0;
  name = g_strdup(_name);  
  }

GVConfigSection::~GVConfigSection()
  {
  g_free(name);

  GVConfigItem * item;

  int num = g_list_length(items);
  for(int i = 0; i < num; i++)
    {
    item = (GVConfigItem*)(g_list_nth_data(items, i));
    delete item;
    }
  g_list_free(items);
  }

static gint __find_func(gconstpointer a, gconstpointer  b)
  {
  return strcmp(((GVConfigItem*)a)->name, (char*)b);
  }

GVConfigItem * GVConfigSection::find_item(const char * _item_name)
  {
  GList * list = g_list_find_custom(items,
                                    (gpointer)_item_name,
                                    __find_func);
  if(list)
    return (GVConfigItem*)(list->data);
  return (GVConfigItem*)0;
  }

int GVConfigSection::get_int(const char * _name, int & ret)
  {
  GVConfigItem * item = find_item(_name);
  if(!item)
    return 0;
  used = 1;
  return item->get_int(ret);
  }

int GVConfigSection::get_float(const char * _name, float & ret)
  {
  GVConfigItem * item = find_item(_name);
  if(!item)
    return 0;
  used = 1;
  return item->get_float(ret);
  }

int GVConfigSection::get_string(const char * _name, const char ** ret)
  {
  GVConfigItem * item  = find_item(_name);
  if(!item)
    return 0;
  used = 1;
  return item->get_string(ret);
  }

int GVConfigSection::get_stringlist(const char * _name, GList ** ret)
  {
  GVConfigItem * item  = find_item(_name);
  if(!item)
    return 0;
  used = 1;
  return item->get_stringlist(ret);
  }

int GVConfigSection::get_bool(const char * _name, int & ret)
  {
  GVConfigItem * item  = find_item(_name);
  if(!item)
    return 0;
  used = 1;
  return item->get_bool(ret);
  }

int GVConfigSection::get_color(const char * _name, guchar & red,
                               guchar & green, guchar & blue)
  {
  GVConfigItem * item  = find_item(_name);
  if(!item)
    return 0;
  used = 1;
  return item->get_color(red, green, blue);
  }

void GVConfigSection::put_int(const char * _name, int value)
  {
  GVConfigItem * item = find_item(_name);
  used = 1;
  if(!item)
    {
    item = new GVConfigItem(_name, value, GVConfigItemInt);
    items = g_list_append(items, (gpointer)item);
    }
  else
    item->set_int(value);
  item->used = 1;
  }

void GVConfigSection::put_float(const char * _name, float value)
  {
  GVConfigItem * item = find_item(_name);
  used = 1;
  if(!item)
    {
    item = new GVConfigItem(_name, value); 
    items = g_list_append(items, (gpointer)item);
    }
  else
    item->set_float(value);
  item->used = 1;
  }

void GVConfigSection::put_string(const char * _name, const char * value)
  {
  GVConfigItem * item = find_item(_name);
  used = 1;
  if(!item)
    {
    item = new GVConfigItem(_name, value); 
    items = g_list_append(items, (gpointer)item);
    }
  else
    item->set_string(value);
  item->used = 1;
  }

void GVConfigSection::put_stringlist(const char * _name, GList * value)
  {
  GVConfigItem * item = find_item(_name);
  used = 1;
  if(!item)
    {
    item = new GVConfigItem(_name, value); 
    items = g_list_append(items, (gpointer)item);
    }
  else
    item->set_stringlist(value);
  item->used = 1;
  }

void GVConfigSection::put_bool(const char * _name, int value)
  {
  GVConfigItem * item = find_item(_name);
  used = 1;
  if(!item)
    {
    item = new GVConfigItem(_name, value, GVConfigItemBool); 
    items = g_list_append(items, (gpointer)item);
    }
  else
    item->set_bool(value);
  item->used = 1;
  }

void GVConfigSection::put_color(const char * _name, guchar red,
                                guchar green, guchar blue)
  {
  GVConfigItem * item = find_item(_name);
  used = 1;
  if(!item)
    {
    item = new GVConfigItem(_name, red, green, blue); 
    items = g_list_append(items, (gpointer)item);
    }
  else
    item->set_color(red, green, blue);
  item->used = 1;
  }
