/*****************************************************************

  configwindow.cc

  Copyright (c) 2001 by Burkhard Plaum - plaum@ipf.uni-stuttgart.de

  http://gmerlin.sourceforge.net

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111, USA.

*****************************************************************/

// Methods for configuration windows

#include "config.h"

#include <gtk/gtk.h>

#include "configwindow.h"

static void __button_callback(GtkWidget * w, gpointer data)
  {
  ((GVConfigWindow*)data)->button_callback(w);
  }

static gboolean __delete_callback(GtkWidget * w, GdkEventAny * event,
                                  gpointer data)
  {
  ((GVConfigWindow*)data)->button_callback(w);
  return TRUE;
  }

void GVConfigWindow::button_callback(GtkWidget * w)
  {
  if(w == apply_button)
    apply_values();

  else if(w == close_button)
    gtk_main_quit();

  else if(w == ok_button)
    {
    apply_values();
    gtk_main_quit();
    }
  else if(w == window)
    gtk_main_quit();
  }

void GVConfigWindow::run()
  {
  get_values();
  gtk_widget_show(window);
  gtk_main();
  gtk_widget_hide(window);
  }

GVConfigWindow::~GVConfigWindow()
  {
  gtk_widget_destroy(window);
  }


GVConfigWindow::GVConfigWindow(const char * title)
  {
  // Create the objects

  window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
  gtk_window_set_title(GTK_WINDOW(window), title);

  gtk_window_set_position(GTK_WINDOW(window),
                          GTK_WIN_POS_MOUSE);
  
  apply_button = gtk_button_new_with_label("Apply");
  close_button = gtk_button_new_with_label("Close");
  ok_button = gtk_button_new_with_label("Ok");

  // Set attributes

  gtk_window_set_modal(GTK_WINDOW(window), TRUE);
  gtk_window_set_position(GTK_WINDOW(window), GTK_WIN_POS_CENTER);  
  // Set Callbacks

  gtk_signal_connect(GTK_OBJECT(ok_button), "clicked",
                     GTK_SIGNAL_FUNC(__button_callback), (gpointer)this);
  gtk_signal_connect(GTK_OBJECT(close_button), "clicked",
                     GTK_SIGNAL_FUNC(__button_callback), (gpointer)this);
  gtk_signal_connect(GTK_OBJECT(apply_button), "clicked",
                     GTK_SIGNAL_FUNC(__button_callback), (gpointer)this);

  gtk_signal_connect(GTK_OBJECT(window), "delete_event",
                     GTK_SIGNAL_FUNC(__delete_callback), (gpointer)this);
  
  // Show the objects

  gtk_widget_show(apply_button);
  gtk_widget_show(close_button);
  gtk_widget_show(ok_button);
    
  // pack the objects
  
  mainbox = gtk_vbox_new(0, 5);
  //  gtk_box_pack_start_defaults(GTK_BOX(mainbox), notebook);
  GtkWidget * buttonbox = gtk_hbutton_box_new();

  gtk_container_set_border_width(GTK_CONTAINER(buttonbox), 10);
  
  gtk_container_add(GTK_CONTAINER(buttonbox), ok_button);
  gtk_container_add(GTK_CONTAINER(buttonbox), apply_button);
  gtk_container_add(GTK_CONTAINER(buttonbox), close_button);
  gtk_widget_show(buttonbox);
  gtk_box_pack_end_defaults(GTK_BOX(mainbox), buttonbox);

  gtk_widget_show(mainbox);
  gtk_container_add(GTK_CONTAINER(window), mainbox);  
  }

GVMainConfigWindow::GVMainConfigWindow() :
  GVConfigWindow("Gmerlin Configuration")
  {
  notebook = gtk_notebook_new();
  
  // GtkWidget * mainbox = gtk_vbox_new(0, 5);
  gtk_box_pack_start_defaults(GTK_BOX(mainbox), notebook);

  display = (GVConfigWidget *)0;
  playlist_widget = (GVConfigWidget *)0;
#if 0
  input = (GVConfigWidget *)0;
  output_video = (GVConfigWidget *)0;
  output_audio = (GVConfigWidget *)0;
#endif
  global_config = (GVConfigWidget *)0;
  }


void GVMainConfigWindow::run(GVConfigWidget * widget)
  {
  gint page_num = gtk_notebook_page_num(GTK_NOTEBOOK(notebook),
                                        widget->widget);
  gtk_notebook_set_page(GTK_NOTEBOOK(notebook), page_num);
  GVConfigWindow::run();
  }

void GVMainConfigWindow::init()
  {
  GtkWidget * label;
#if 0
  if(input)
    {
    label = gtk_label_new("Input");
    gtk_widget_show(label);
    gtk_notebook_append_page(GTK_NOTEBOOK(notebook),
                             input->widget, label);
    }
  if(output_video)
    {
    label = gtk_label_new("Video Output");
    gtk_widget_show(label);
    gtk_notebook_append_page(GTK_NOTEBOOK(notebook),
                             output_video->widget, label);
    }
  if(output_audio)
    {
    label = gtk_label_new("Audio Output");
    gtk_widget_show(label);
    gtk_notebook_append_page(GTK_NOTEBOOK(notebook),
                             output_audio->widget, label);
    }
#endif
  if(global_config)
    {
    label = gtk_label_new("General");
    gtk_widget_show(label);
    gtk_notebook_append_page(GTK_NOTEBOOK(notebook),
                             global_config->widget, label);
    }
  if(network_config)
    {
    label = gtk_label_new("Network");
    gtk_widget_show(label);
    gtk_notebook_append_page(GTK_NOTEBOOK(notebook),
                             network_config->widget, label);
    }
  if(display)
    {
    label = gtk_label_new("Display");
    gtk_widget_show(label);
    gtk_notebook_append_page(GTK_NOTEBOOK(notebook),
                             display->widget, label);
    }
  if(playlist_widget)
    {
    label = gtk_label_new("Playlist");
    gtk_widget_show(label);
    gtk_notebook_append_page(GTK_NOTEBOOK(notebook),
                             playlist_widget->widget, label);
    }
  
  gtk_widget_show(notebook);
  }

void GVMainConfigWindow::get_values()
  {
  if(display)
    display->get_values();
  if(playlist_widget)
    playlist_widget->get_values();
#if 0
  if(input)
    input->get_values();
  if(output_video)
    output_video->get_values();
  if(output_audio)
    output_audio->get_values();
#endif
  if(global_config)
    global_config->get_values();
  if(network_config)
    network_config->get_values();
  }

void GVMainConfigWindow::apply_values()
  {
  if(display)
    display->apply_values();
  if(playlist_widget)
    playlist_widget->apply_values();
#if 0
  if(input)
    input->apply_values();
  if(output_video)
    output_video->apply_values();
  if(output_audio)
    output_audio->apply_values();
#endif
  if(network_config)
    network_config->apply_values();
  if(global_config)
    global_config->apply_values();

  }
