/*****************************************************************

  audioout.h

  Copyright (c) 2001 by Burkhard Plaum - plaum@ipf.uni-stuttgart.de

  http://gmerlin.sourceforge.net

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111, USA.

*****************************************************************/

// Base class for all configurations windows

#ifndef __GV_CONFIGWINDOW_H_
#define __GV_CONFIGWINDOW_H_

class GVConfigWindow;

// One configuration section

class GVConfigWidget
  {
public:
  
  GtkWidget * widget;

  virtual void apply_values() = 0;
  virtual void get_values() = 0;

  };

// Widget for selecting colors

class GVConfigWidgetColor
  {
  GtkWidget * drawingarea;
  GdkColor color;

  int changed;
  int is_realized;
  GdkColormap * colormap;  

  guchar red, green, blue;

  void update_color();

  GtkWidget * button;
  GtkWidget * label;
  
public:
  static GtkWidget * colorselect;

  void attach(GtkWidget * table, int row, int column);

  // Callbacks
  
  void choose();
  void realize();
  void button_callback();
  
  GVConfigWidgetColor(const gchar * _label);
  ~GVConfigWidgetColor();

  void set_color(guchar _red, guchar _green, guchar _blue);
  void get_color(guchar & _red, guchar & _green, guchar & _blue);

  void set_color(GdkColor & _color);
  int get_color(GdkColor & _color);
  
  void set_color();
  void get_color();
  void set_sensitive(int sensitive);
  
  };

// Widget for selecting fonts

class GVConfigWidgetFont
  {
  static GtkWidget * fontselect;

  GtkWidget * label;
  GtkWidget * entry;
  GtkWidget * button;
  
public:

  void attach(GtkWidget * table, int row, int column);
  
  void set_font(const char * font);
  char * get_font();

  void choose();

  void button_callback(GtkWidget * w);
    
  GVConfigWidgetFont(const char * _label);
  ~GVConfigWidgetFont();
  void set_sensitive(int sensitive);
  };

// Widget for selecting files

// class GVConfigWidgetDevicelist;

class GVConfigWidgetFile
  {
  static GtkWidget * fileselect;
  
  int dirselect; // Directory selector

  GtkWidget * label;
  GtkWidget * entry;
  GtkWidget * button;
  
public:

  GVConfigWidgetFile(const char * _label, int _dirselect);
  ~GVConfigWidgetFile();
  void filesel_button_callback(GtkWidget * w);
  void set_filename(const char * filename);
  const char * get_filename();

  void attach(GtkWidget * table, int row, int column);

  void choose();
  void set_sensitive(int sensitive);
  
  //  friend class GVConfigWidgetDevicelist;
  };

class GVInput;

class GVConfigWidgetDevicelist : public GVConfigWidget
  {
  GVConfigWidgetFile file_widget;
  GtkWidget * devices_list;

  GtkWidget * add_button;
  GtkWidget * delete_button;
  GtkWidget * update_button;

  GVInput * input;
  GList ** devices;
public:

  void button_callback(GtkWidget * w);
  void select_row_callback(int row);

  GVConfigWidgetDevicelist(GVInput * _input);
  void apply_values();
  void get_values();
  virtual ~GVConfigWidgetDevicelist();
  };

class GVConfigWidgetSpinbutton
  {
  GtkWidget * spinbutton;
  GtkWidget * label;
  GtkObject * adjustment;
  
public:

  GVConfigWidgetSpinbutton(const char * _label, float lower, float upper, int num_digits);

  float get_value();
  void set_value(float _val);

  void attach(GtkWidget * table, int row, int column);
  void set_sensitive(int sensitive);
  };

class GVConfigWidgetRecordtime : public GVConfigWidget
  {
  GtkWidget * limit_time;
  GtkWidget * hours;
  GtkWidget * minutes;
  GtkWidget * seconds;

  GtkWidget * hours_label;
  GtkWidget * minutes_label;
  GtkWidget * seconds_label;

  
  GVInput * input;
public:

  GVConfigWidgetRecordtime(GVInput * _input);
  
  void button_callback(GtkWidget * w);
  void get_values();
  void apply_values();
  };


struct GVConfigMenuItemDouble
  {
  double d;
  char * label;
  };

struct GVConfigMenuItemInt
  {
  int i;
  char * label;
  };

struct GVConfigMenuItemString
  {
  char * s;
  char * label;
  };

class GVConfigWidgetOptionmenu
  {
  GtkWidget ** menu_items;
  GtkWidget * menu;

  int no_menu_callback;
  
  int num_items;

  int selected;

  void select_item();

  GtkWidget * optionmenu;
  GtkWidget * label;
  
public:

  void attach(GtkWidget * table, int row, int column);
    
  int get_selected() { return selected; }

  int num_digits; // Number of digits (for double items)

  GVConfigWidgetOptionmenu(char * _label);
    
  void create_menu(int *items, int numitems);
  void create_menu(GVConfigMenuItemInt *items, int numitems);

  void create_menu(double *items, int numitems);
  void create_menu(GVConfigMenuItemDouble *items, int numitems);
    
  void create_menu(char **items, int numitems);
  void create_menu(GVConfigMenuItemString * items, int numitems);
  
  int get_int();               // Get an integer number
  double get_double();         // Get a double number
  const char * get_string();   // Get a string

  void set(int val);
  void set(double val);
  void set(const char * val);
    
  void menu_callback(GtkWidget * w);
  void set_sensitive(int sensitive);
  };

class GVConfigWindow
  {
  GtkWidget * apply_button;
  GtkWidget * close_button;
  GtkWidget * ok_button;

protected:
  GtkWidget * mainbox;
  GtkWidget * window;
  
public:

  GVConfigWindow(const char * title);
    
  void run();

  virtual void apply_values() = 0;
  virtual void get_values() = 0;

  void button_callback(GtkWidget * w);
  virtual ~GVConfigWindow();
  };

// Several widgets for configuration dialogs follow

class GVConfigWidgetString
  {
  int max_length; // Maximum string length, -1 for infinity
  GtkWidget * entry;
  GtkWidget * label;

public:
  
  void set_string(const char * _string);
  const char * get_string();

  GVConfigWidgetString(const char * _label, int _max_length = -1);
  void attach(GtkWidget * table, int row, int column);
  void set_sensitive(int sensitive);
  
  };

class GVConfigWidgetStringmenu
  {
  GtkWidget * combo;
  GtkWidget * label;

  GList * strings;
  
public:
  
  GVConfigWidgetStringmenu(const char * label, char ** strings, int num);
  ~GVConfigWidgetStringmenu();
  void attach(GtkWidget * table, int row, int column);

  void set_string(const char * _string);
  const char * get_string();
  void set_sensitive(int sensitive);

  };

class GVConfigWidgetSlider
  {
  GtkWidget * scale;
  GtkWidget * label;
  GtkObject * adjustment;
  
public:

  GVConfigWidgetSlider(const char * label,
                       float min_value, float max_value, int num_digits);

  int get_value_int();
  float get_value_float();
  void set_value(float value);
  void attach(GtkWidget * table, int row, int column);

  void set_sensitive(int sensitive);
  };

class GVMainConfigWindow : public GVConfigWindow
  {
  GtkWidget * notebook;
  
public:

  GVMainConfigWindow();
  
  GVConfigWidget * display;
  GVConfigWidget * playlist_widget;
#if 0
  GVConfigWidget * input;
  GVConfigWidget * output_video;
  GVConfigWidget * output_audio;
#endif
  GVConfigWidget * global_config;
  GVConfigWidget * network_config;
  
  void apply_values();
  void get_values();
  void init();

  // Run with a special tab shown
  
  void run(GVConfigWidget * widget);
  };


#endif
