/*****************************************************************

  configcolor.cc

  Copyright (c) 2001 by Burkhard Plaum - plaum@ipf.uni-stuttgart.de

  http://gmerlin.sourceforge.net

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111, USA.

*****************************************************************/

// Methods for the color congiguration widget

#include "config.h"

#include <gtk/gtk.h>
#include "configwindow.h"
#include "gtkutil.h"

GtkWidget * GVConfigWidgetColor::colorselect = (GtkWidget*)0;
static GVConfigWidgetColor * current_color_widget = (GVConfigWidgetColor *)0;

static void __ok_callback(GtkWidget * w, gpointer data)
  {
  current_color_widget->get_color();
  gtk_widget_hide(GVConfigWidgetColor::colorselect);
  gtk_main_quit();
  }

static void __cancel_callback(GtkWidget * w, gpointer data)
  {
  gtk_widget_hide(GVConfigWidgetColor::colorselect);
  gtk_main_quit();
  }

static void __realize_callback(GtkWidget * w, gpointer data)
  {
  ((GVConfigWidgetColor*)data)->realize();
  }

static void __button_callback(GtkWidget * w, gpointer data)
  {
  ((GVConfigWidgetColor*)data)->choose();
  }


void GVConfigWidgetColor::choose()
  {
  current_color_widget = this;
  set_color();
  gtk_widget_show(colorselect);
  gtk_main();
  changed = 1;
  }

GVConfigWidgetColor::GVConfigWidgetColor(const gchar * _label)
  {
  if(!colorselect)
    {
    colorselect = gtk_color_selection_dialog_new("Select a color");
    gtk_window_set_modal(GTK_WINDOW(colorselect), TRUE);

    gtk_widget_hide(GTK_COLOR_SELECTION_DIALOG(colorselect)->help_button);
        
    gtk_signal_connect(GTK_OBJECT(GTK_COLOR_SELECTION_DIALOG(colorselect)->ok_button),
                       "clicked", GTK_SIGNAL_FUNC(__ok_callback), (gpointer)0);
    gtk_signal_connect(GTK_OBJECT(GTK_COLOR_SELECTION_DIALOG(colorselect)->cancel_button),
                       "clicked", GTK_SIGNAL_FUNC(__cancel_callback), (gpointer)0);
    }
    
  is_realized = 0;

  button = gtk_button_new();
  drawingarea = gtk_drawing_area_new();
  gtk_widget_show(drawingarea);
  gtk_container_add(GTK_CONTAINER(button), drawingarea);

  gtk_widget_show(button);
  
  colormap = (GdkColormap*)0;
  gtk_widget_set_events(drawingarea, GDK_EXPOSURE_MASK);

  label = gtk_label_new(_label);
  gtk_widget_show(label);
  
  //  gtk_signal_connect(GTK_OBJECT(drawingarea),
  //                     "expose_event", GTK_SIGNAL_FUNC(__expose_callback),
  //                     (gpointer)this);
  
  gtk_signal_connect(GTK_OBJECT(drawingarea),
                     "realize", GTK_SIGNAL_FUNC(__realize_callback),
                     (gpointer)this);
  
  gtk_signal_connect(GTK_OBJECT(button),
                     "clicked", GTK_SIGNAL_FUNC(__button_callback),
                     (gpointer)this);
  }

GVConfigWidgetColor::~GVConfigWidgetColor()
  {
//   gtk_widget_destroy(drawingarea);
//   gtk_widget_destroy(button);
//   gtk_widget_destroy(label);
  }


void GVConfigWidgetColor::set_color(guchar _red, guchar _green, guchar _blue)
  {
  red = _red;
  green = _green;
  blue = _blue;
  update_color();
  }

void GVConfigWidgetColor::get_color(guchar & _red, guchar & _green, guchar & _blue)
  {
  _red = red;
  _green = green;
  _blue = blue;
  }

void GVConfigWidgetColor::set_color(GdkColor & _color)
  {
  red = _color.red >> 8;
  green = _color.green >> 8;
  blue = _color.blue >> 8;
  update_color();
  changed = 0;
  }

// returns 1 if the color really changed

int GVConfigWidgetColor::get_color(GdkColor & _color)
  {
  if(changed)
    {
    _color.red = red *(65535/255);
    _color.green = green * (65535/255);
    _color.blue =  blue  * (65535/255);
    _color.pixel = (gulong)(red*65536+green*256+blue);
    return 1;
    }
  return 0;
  }


void GVConfigWidgetColor::get_color()
  {
  gdouble colors[3];
  
  gtk_color_selection_get_color(GTK_COLOR_SELECTION(GTK_COLOR_SELECTION_DIALOG(colorselect)->colorsel), colors);

  red = (guchar)(colors[0]*255.0);
  green = (guchar)(colors[1]*255.0);
  blue = (guchar)(colors[2]*255.0);
  update_color();
  }

void GVConfigWidgetColor::set_color()
  {
  gdouble colors[3];

  colors[0] = (gdouble)(red) / 255.0;
  colors[1] = (gdouble)(green) / 255.0;
  colors[2] = (gdouble)(blue) / 255.0;
  
  gtk_color_selection_set_color(GTK_COLOR_SELECTION(GTK_COLOR_SELECTION_DIALOG(colorselect)->colorsel), colors);
  }

void GVConfigWidgetColor::update_color()
  {
  if(!is_realized)
    return;

  GdkColor gdkcolor;
  
  gdkcolor.red =   red *(65535/255);
  gdkcolor.green = green * (65535/255);
  gdkcolor.blue =  blue  * (65535/255);
  gdkcolor.pixel = (gulong)(red*65536+green*256+blue);

  gdk_color_alloc(colormap, &gdkcolor);    
  gdk_window_set_background(drawingarea->window, &gdkcolor);
  gdk_window_clear(drawingarea->window);
  }

void GVConfigWidgetColor::realize()
  {
  colormap = gdk_window_get_colormap(drawingarea->window);
  is_realized = 1;
  update_color();
  }

void GVConfigWidgetColor::attach(GtkWidget * table, int row, int column)
  {
  gvgtk_table_attach_noresize(table, label, column, column+1, row, row+1);
  gtk_table_attach_defaults(GTK_TABLE(table), button,
                            column+1, column+2, row, row+1);
  }

void GVConfigWidgetColor::set_sensitive(int sensitive)
  {
  gtk_widget_set_sensitive(button, sensitive);
  gtk_widget_set_sensitive(label, sensitive);
  }
