/*****************************************************************

  cw_devicelist.cc

  Copyright (c) 2001 by Burkhard Plaum - plaum@ipf.uni-stuttgart.de

  http://gmerlin.sourceforge.net

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111, USA.

*****************************************************************/

// Device list for input devices

#include "config.h"

#include <gtk/gtk.h>
#include <pthread.h>
#include <iostream>
// #include <string.h>

#include "gvtime.h"

#include "videoformat.h"
#include "audioformat.h"
#include "buffer.h"

#include "videobuf.h"
#include "audiobuf.h"

#include "playlist.h"
#include "baseplugin.h"
#include "plugin.h"
#include "input.h"

#include "configwindow.h"
#include "gtkutil.h"

static void __button_callback(GtkWidget * w, gpointer data)
  {
  ((GVConfigWidgetDevicelist*)data)->button_callback(w);
  }

static void __select_row_callback(GtkWidget * w, gint row, gint column,
                                  GdkEvent * event, gpointer data)
  {
  ((GVConfigWidgetDevicelist*)data)->select_row_callback(row);
  }


void GVConfigWidgetDevicelist::button_callback(GtkWidget * w)
  {
  if(w == add_button)
    {
    input->add_device(file_widget.get_filename());
    get_values();
    }
  
  else if(w == update_button)
    {

    }
  else if(w == delete_button)
    {
    
    }
  }

void GVConfigWidgetDevicelist::select_row_callback(int row)
  {
  char * c_tmp;
  gtk_clist_get_text(GTK_CLIST(devices_list), row, 0, &c_tmp);
  file_widget.set_filename(c_tmp);
  }

GVConfigWidgetDevicelist::GVConfigWidgetDevicelist(GVInput * _input) :
  file_widget("Device", 0)
  {
  input = _input;
  devices = &(input->devices);

  // Create the objects

  if(input->attributes->flags & GV_PLUGIN_HAS_DEVICESTRING)
    devices_list = gtk_clist_new(2);
  else
    devices_list = gtk_clist_new(1);

  add_button = gtk_button_new_with_label("Add");
  delete_button = gtk_button_new_with_label("Delete");
  update_button = gtk_button_new_with_label("Update");

  // Set attributes

  gtk_clist_set_column_title(GTK_CLIST(devices_list), 0, "Devices");
  if(input->attributes->flags & GV_PLUGIN_HAS_DEVICESTRING)
    gtk_clist_set_column_title(GTK_CLIST(devices_list), 1, "Models");

  gtk_clist_column_titles_show(GTK_CLIST(devices_list));
  gtk_clist_column_titles_passive(GTK_CLIST(devices_list));
  
  // Set callbacks

  gtk_signal_connect(GTK_OBJECT(add_button), "clicked",
                     GTK_SIGNAL_FUNC(__button_callback), (gpointer)this);
  gtk_signal_connect(GTK_OBJECT(update_button), "clicked",
                     GTK_SIGNAL_FUNC(__button_callback), (gpointer)this);
  gtk_signal_connect(GTK_OBJECT(delete_button), "clicked",
                     GTK_SIGNAL_FUNC(__button_callback), (gpointer)this);
  gtk_signal_connect(GTK_OBJECT(devices_list), "select_row",
                     GTK_SIGNAL_FUNC(__select_row_callback), (gpointer)this);
  
  // Show the objects

  gtk_widget_show(add_button);
  gtk_widget_show(delete_button);
  gtk_widget_show(update_button);
  gtk_widget_show(devices_list);
  
  // Pack the objects
  
  GtkWidget * table = gvgtk_table_new(4, 3, 0);

  file_widget.attach(table, 0, 0);
    
  gtk_table_attach_defaults(GTK_TABLE(table), devices_list, 0, 2, 1, 4);

  gvgtk_table_attach_noresize(table, add_button,    2, 3, 1, 2);
  gvgtk_table_attach_noresize(table, update_button, 2, 3, 2, 3);
  gvgtk_table_attach_noresize(table, delete_button, 2, 3, 3, 4);

  gtk_widget_show(table);
  widget = table;
  }

void GVConfigWidgetDevicelist::apply_values()
  {
  input->set_device(file_widget.get_filename());
  }

void GVConfigWidgetDevicelist::get_values()
  {
  gtk_clist_freeze(GTK_CLIST(devices_list));
  gtk_clist_clear(GTK_CLIST(devices_list));

  int num_devices = g_list_length(*devices)/2;

  char * list_text[2];
  
  for(int i = 0; i < num_devices; i++)
    {
    list_text[0] = (char*)g_list_nth_data(*devices, 2*i);
    if(input->attributes->flags & GV_PLUGIN_HAS_DEVICESTRING)
      list_text[1] = (char*)g_list_nth_data(*devices, 2*i+1);
    gtk_clist_append(GTK_CLIST(devices_list), list_text);
    }
  gtk_clist_select_row(GTK_CLIST(devices_list), input->device_index / 2, 0);
  gtk_clist_thaw(GTK_CLIST(devices_list));
  
  }

GVConfigWidgetDevicelist::~GVConfigWidgetDevicelist()
  {

  }
