/*****************************************************************

  cw_file.cc

  Copyright (c) 2001 by Burkhard Plaum - plaum@ipf.uni-stuttgart.de

  http://gmerlin.sourceforge.net

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111, USA.

*****************************************************************/

// File selection widget

#include "config.h"

#include <gtk/gtk.h>

#include "configwindow.h"
#include "gtkutil.h"

GtkWidget * GVConfigWidgetFile::fileselect = (GtkWidget*)0;

GVConfigWidgetFile * current_file_widget = (GVConfigWidgetFile*)0;

static void __file_button_callback(GtkWidget * w, gpointer data)
  {
  ((GVConfigWidgetFile*)data)->choose();
  }

static void __filesel_button_callback(GtkWidget * w, gpointer data)
  {
  current_file_widget->filesel_button_callback(w);
  }

GVConfigWidgetFile::GVConfigWidgetFile(const char * _label, int _dirselect)
  {
  dirselect = _dirselect;
  label = gtk_label_new(_label);
  gtk_widget_show(label);
  entry = gtk_entry_new();
  gtk_widget_show(entry);

  button = gtk_button_new_with_label("Browse...");
  gtk_signal_connect(GTK_OBJECT(button), "clicked",
                     GTK_SIGNAL_FUNC(__file_button_callback),
                     (gpointer)this);
  gtk_widget_show(button);
  }

void GVConfigWidgetFile::set_filename(const char * filename)
  {
  gtk_entry_set_text(GTK_ENTRY(entry), filename);
  }

const char * GVConfigWidgetFile::get_filename()
  {
  return gtk_entry_get_text(GTK_ENTRY(entry));
  }

void GVConfigWidgetFile::filesel_button_callback(GtkWidget * w)
  {
  if(w == GTK_FILE_SELECTION(fileselect)->ok_button)
    {
    gtk_entry_set_text(GTK_ENTRY(entry),
                       gtk_file_selection_get_filename(GTK_FILE_SELECTION(fileselect)));
    }
  gtk_widget_hide(fileselect);
  gtk_main_quit();
  }

void GVConfigWidgetFile::choose()
  {
  if(!fileselect)
    {
    fileselect = gtk_file_selection_new("Select a file or directory");
    gtk_window_set_modal(GTK_WINDOW(fileselect), TRUE);

    gtk_signal_connect(GTK_OBJECT(GTK_FILE_SELECTION(fileselect)->ok_button),
                       "clicked", GTK_SIGNAL_FUNC(__filesel_button_callback),
                       (gpointer)0);
    gtk_signal_connect(GTK_OBJECT(GTK_FILE_SELECTION(fileselect)->cancel_button),
                       "clicked", GTK_SIGNAL_FUNC(__filesel_button_callback),
                       (gpointer)0);
    }
  if(dirselect)
    gtk_widget_set_sensitive(GTK_FILE_SELECTION(fileselect)->file_list, 0);
  else
    gtk_widget_set_sensitive(GTK_FILE_SELECTION(fileselect)->file_list, 1);
  
  current_file_widget = this;

  gtk_file_selection_set_filename(GTK_FILE_SELECTION(fileselect),
                                  gtk_entry_get_text(GTK_ENTRY(entry)));

  gtk_widget_show(fileselect);
  gtk_main();
  }

GVConfigWidgetFile::~GVConfigWidgetFile()
  {
  }

void GVConfigWidgetFile::attach(GtkWidget * table, int row, int column)
  {
  gvgtk_table_attach_noresize(table, label, column, column+1, row, row+1);
  gtk_table_attach_defaults(GTK_TABLE(table), entry,
                            column+1, column+2, row, row+1);
  gvgtk_table_attach_noresize(table, button, column+2, column+3, row, row+1);
  }

void GVConfigWidgetFile::set_sensitive(int sensitive)
  {
  gtk_widget_set_sensitive(button, sensitive);
  gtk_widget_set_sensitive(entry, sensitive);
  gtk_widget_set_sensitive(label, sensitive);
  }
