/*****************************************************************

  cw_font.cc

  Copyright (c) 2001 by Burkhard Plaum - plaum@ipf.uni-stuttgart.de

  http://gmerlin.sourceforge.net

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111, USA.

*****************************************************************/

// Configure a font

#include "config.h"

#include <gtk/gtk.h>

#include "configwindow.h"
#include "gtkutil.h"

GtkWidget * GVConfigWidgetFont::fontselect = (GtkWidget*)0;

GVConfigWidgetFont * current_font_widget = (GVConfigWidgetFont*)0;

static void __button_callback(GtkWidget * w, gpointer data)
  {
  ((GVConfigWidgetFont*)data)->button_callback(w);
  }

static void __fontsel_button_callback(GtkWidget * w, gpointer data)
  {
  current_font_widget->button_callback(w);
  }

void GVConfigWidgetFont::button_callback(GtkWidget * w)
  {
  if(w == button)
    choose();
  else if(w == (GTK_FONT_SELECTION_DIALOG(fontselect)->ok_button))
    {
    char * tmp_fontname
      = gtk_font_selection_dialog_get_font_name(GTK_FONT_SELECTION_DIALOG(fontselect));
    if(tmp_fontname)
      {
      gtk_entry_set_text(GTK_ENTRY(entry), tmp_fontname);
      g_free(tmp_fontname);
      }
    gtk_widget_hide(fontselect);
    gtk_main_quit();
    }
  else if(w == (GTK_FONT_SELECTION_DIALOG(fontselect)->cancel_button))
    {
    gtk_widget_hide(fontselect);
    gtk_main_quit();
    }
  }

void GVConfigWidgetFont::attach(GtkWidget * table, int row, int column)
  {
  gvgtk_table_attach_noresize(table, label, column, column+1, row, row+1);
  gtk_table_attach_defaults(GTK_TABLE(table), entry,
                            column+1, column+2, row, row+1);
  gvgtk_table_attach_noresize(table, button, column+2, column+3, row, row+1);
  }

void GVConfigWidgetFont::set_font(const char * font)
  {
  gtk_entry_set_text(GTK_ENTRY(entry), font);
  }

char * GVConfigWidgetFont::get_font()
  {
  return gtk_entry_get_text(GTK_ENTRY(entry));
  }

void GVConfigWidgetFont::choose()
  {
  if(!fontselect)
    {
    fontselect = gtk_font_selection_dialog_new("Select a Font");
    gtk_window_set_modal(GTK_WINDOW(fontselect), TRUE);
    
    gtk_signal_connect(GTK_OBJECT(GTK_FONT_SELECTION_DIALOG(fontselect)->ok_button),
                       "clicked",
                       GTK_SIGNAL_FUNC(__fontsel_button_callback),
                       (gpointer)0);
    gtk_signal_connect(GTK_OBJECT(GTK_FONT_SELECTION_DIALOG(fontselect)->cancel_button),
                       "clicked", GTK_SIGNAL_FUNC(__fontsel_button_callback),
                       (gpointer)0);
    }
  current_font_widget = this;
  gtk_font_selection_dialog_set_font_name(GTK_FONT_SELECTION_DIALOG(fontselect),
    gtk_entry_get_text(GTK_ENTRY(entry)));

  gtk_widget_show(fontselect);
  gtk_main();
  }

GVConfigWidgetFont::GVConfigWidgetFont(const char * _label)
  {
  // Create the objects

  label = gtk_label_new(_label);
  entry = gtk_entry_new();
  button = gtk_button_new_with_label("Choose...");

  // Set attributes

  // Set Callbacks

  gtk_signal_connect(GTK_OBJECT(button), "clicked",
                     GTK_SIGNAL_FUNC(__button_callback),
                     (gpointer)(this));
  
  // Show the objects

  gtk_widget_show(label);
  gtk_widget_show(entry);
  gtk_widget_show(button);
      
  }

GVConfigWidgetFont::~GVConfigWidgetFont()
  {

  }

void GVConfigWidgetFont::set_sensitive(int sensitive)
  {
  gtk_widget_set_sensitive(button, sensitive);
  gtk_widget_set_sensitive(label, sensitive);
  gtk_widget_set_sensitive(entry, sensitive);
  }

