/*****************************************************************

  cw_optionmenu.cc

  Copyright (c) 2001 by Burkhard Plaum - plaum@ipf.uni-stuttgart.de

  http://gmerlin.sourceforge.net

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111, USA.

*****************************************************************/

// Optionmenu for config windows

#include "config.h"

#include <gtk/gtk.h>
#include <sstream>
using std::stringstream;

//#include <iostream>
#include <string.h>

#include "configwindow.h"
#include "gtkutil.h"

using std::ends;

// GVConfigWidgetOptionmenu

GVConfigWidgetOptionmenu::GVConfigWidgetOptionmenu(char * _label)
  {
  label = gtk_label_new(_label);
  gtk_widget_show(label);

  menu = gtk_menu_new();
  optionmenu = gtk_option_menu_new();

  gtk_option_menu_set_menu(GTK_OPTION_MENU(optionmenu), menu);

  gtk_widget_show(menu);
  gtk_widget_show(optionmenu);

  selected = 0;
  no_menu_callback = 0;
  }

static void __option_menu_callback(GtkWidget * w, gpointer data)
  {
  ((GVConfigWidgetOptionmenu*)data)->menu_callback(w);
  }

void GVConfigWidgetOptionmenu::create_menu(int *items, int _num_items)
  {
  num_items = _num_items;
  menu_items = new GtkWidget*[num_items];

  GtkWidget * item;
  stringstream line;
  for(int i = 0; i < num_items; i++)
    {
    //    line.freeze(0);
    line.seekp(0);
    line << items[i] << ends;
    item = gtk_menu_item_new_with_label(line.str().c_str());

    gtk_object_set_user_data(GTK_OBJECT(item), (gpointer)(&items[i]));
    gtk_signal_connect(GTK_OBJECT(item), "activate",
                       GTK_SIGNAL_FUNC(__option_menu_callback),
                       (gpointer)this);
    gtk_widget_show(item);
    gtk_menu_append(GTK_MENU(menu), item);
    menu_items[i] = item;
    }
  }

void GVConfigWidgetOptionmenu::create_menu(double *items, int _num_items)
  {
  num_items = _num_items;
  menu_items = new GtkWidget*[num_items];

  GtkWidget * item;
  stringstream line;
  for(int i = 0; i < num_items; i++)
    {
    //    line.freeze(0);
    line.seekp(0);
    line << items[i] << ends;
    item = gtk_menu_item_new_with_label(line.str().c_str());

    gtk_object_set_user_data(GTK_OBJECT(item), (gpointer)(&items[i]));
    gtk_signal_connect(GTK_OBJECT(item), "activate",
                       GTK_SIGNAL_FUNC(__option_menu_callback),
                       (gpointer)this);
    gtk_widget_show(item);
    gtk_menu_append(GTK_MENU(menu), item);
    menu_items[i] = item;
    }
  }

void GVConfigWidgetOptionmenu::create_menu(char **items, int _num_items)
  {
  num_items = _num_items;
  menu_items = new GtkWidget*[num_items];

  GtkWidget * item;
  for(int i = 0; i < num_items; i++)
    {
    item = gtk_menu_item_new_with_label(items[i]);
    
    gtk_object_set_user_data(GTK_OBJECT(item), (gpointer)(items[i]));
    gtk_signal_connect(GTK_OBJECT(item), "activate",
                       GTK_SIGNAL_FUNC(__option_menu_callback),
                       (gpointer)this);
    gtk_widget_show(item);
    gtk_menu_append(GTK_MENU(menu), item);
    menu_items[i] = item;
    }
  }

void GVConfigWidgetOptionmenu::create_menu(GVConfigMenuItemInt *items,
                                           int _num_items)
  {
  num_items = _num_items;
  menu_items = new GtkWidget*[num_items];

  GtkWidget * item;
  stringstream line;
  for(int i = 0; i < num_items; i++)
    {
    if(!(items[i].label) || items[i].label[0] == '\0')
      {
      //      line.freeze(0);
      line.seekp(0);
      line << items[i].i << ends;
      item = gtk_menu_item_new_with_label(line.str().c_str());
      }
    else
      item = gtk_menu_item_new_with_label(items[i].label);
    
    gtk_object_set_user_data(GTK_OBJECT(item), (gpointer)(&(items[i].i)));
    gtk_signal_connect(GTK_OBJECT(item), "activate",
                       GTK_SIGNAL_FUNC(__option_menu_callback),
                       (gpointer)this);
    gtk_widget_show(item);
    gtk_menu_append(GTK_MENU(menu), item);
    menu_items[i] = item;
    }

  }

void GVConfigWidgetOptionmenu::create_menu(GVConfigMenuItemDouble *items,
                                           int _num_items)
  {
  num_items = _num_items;
  menu_items = new GtkWidget*[num_items];

  GtkWidget * item;
  stringstream line;
  for(int i = 0; i < num_items; i++)
    {
    if(!(items[i].label) || items[i].label[0] == '\0')
      {
      //      line.freeze(0);
      line.seekp(0);
      line << items[i].d << ends;
      item = gtk_menu_item_new_with_label(line.str().c_str());
      }
    else
      item = gtk_menu_item_new_with_label(items[i].label);
    
    gtk_object_set_user_data(GTK_OBJECT(item), (gpointer)(&(items[i].d)));
    gtk_signal_connect(GTK_OBJECT(item), "activate",
                       GTK_SIGNAL_FUNC(__option_menu_callback),
                       (gpointer)this);
    gtk_widget_show(item);
    gtk_menu_append(GTK_MENU(menu), item);
    menu_items[i] = item;
    }
  }
    
void GVConfigWidgetOptionmenu::create_menu(GVConfigMenuItemString * items,
                                           int _num_items)
  {
  num_items = _num_items;
  menu_items = new GtkWidget*[num_items];

  GtkWidget * item;
  for(int i = 0; i < num_items; i++)
    {
    if(!(items[i].label) || items[i].label[0] == '\0')
      item = gtk_menu_item_new_with_label(items[i].s);
    else
      item = gtk_menu_item_new_with_label(items[i].label);
    gtk_object_set_user_data(GTK_OBJECT(item), (gpointer)(items[i].s));
    gtk_signal_connect(GTK_OBJECT(item), "activate",
                       GTK_SIGNAL_FUNC(__option_menu_callback),
                       (gpointer)this);
    gtk_widget_show(item);
    gtk_menu_append(GTK_MENU(menu), item);
    menu_items[i] = item;
    }

  }

int GVConfigWidgetOptionmenu::get_int()       // Get an integer number
  {
  gpointer data = gtk_object_get_user_data(GTK_OBJECT(menu_items[selected]));
  return *((int*)(data));
  }

double GVConfigWidgetOptionmenu::get_double() // Get a double number
  {
  gpointer data = gtk_object_get_user_data(GTK_OBJECT(menu_items[selected]));
  return *((double*)(data));
  }

const char * GVConfigWidgetOptionmenu::get_string()   // Get a string
  {
  gpointer data = gtk_object_get_user_data(GTK_OBJECT(menu_items[selected]));

  return (char*)(data);
  }
  
void GVConfigWidgetOptionmenu::set(int val)
  {
  gpointer data;
  for(int i = 0; i < num_items; i++)
    {
    data = gtk_object_get_user_data(GTK_OBJECT(menu_items[i]));
    if(*((int*)(data)) == val)
      {
      selected = i;
      break;
      }
    }
  select_item();
  }

void GVConfigWidgetOptionmenu::set(double val)
  {
  gpointer data;
  for(int i = 0; i < num_items; i++)
    {
    data = gtk_object_get_user_data(GTK_OBJECT(menu_items[i]));
    if(*((double*)(data)) == val)
      {
      selected = i;
      break;
      }
    }
  select_item();
  }

void GVConfigWidgetOptionmenu::set(const char * val)
  {
  gpointer data;
  for(int i = 0; i < num_items; i++)
    {
    data = gtk_object_get_user_data(GTK_OBJECT(menu_items[i]));
    if(!strcmp((char*)(data), val))
      {
      selected = i;
      break;
      }
    }
  select_item();
  }

void GVConfigWidgetOptionmenu::select_item()
  {
  no_menu_callback = 1;
  gtk_menu_set_active(GTK_MENU(menu), selected);
  gtk_option_menu_set_history(GTK_OPTION_MENU(optionmenu), selected);
  no_menu_callback = 0;
  }

void GVConfigWidgetOptionmenu::menu_callback(GtkWidget * w)
  {
  // Menu callback
  if(no_menu_callback)
    return;
  
  for(int i = 0; i < num_items; i++)
    {
    if(menu_items[i] == w)
      {
      selected = i;
      break;
      }
    }
  }

void GVConfigWidgetOptionmenu::attach(GtkWidget * table, int row, int column)
  {
  gvgtk_table_attach_noresize(table, label, column, column+1, row, row+1);
  gtk_table_attach_defaults(GTK_TABLE(table), optionmenu,
                            column+1, column+2, row, row+1);
  }

void GVConfigWidgetOptionmenu::set_sensitive(int sensitive)
  {
  gtk_widget_set_sensitive(optionmenu, sensitive);
  gtk_widget_set_sensitive(label, sensitive);
  }
