/*****************************************************************

  cw_recordtime.cc

  Copyright (c) 2001 by Burkhard Plaum - plaum@ipf.uni-stuttgart.de

  http://gmerlin.sourceforge.net

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111, USA.

*****************************************************************/

// Let the user select the recording time

#include "config.h"

#include <gtk/gtk.h>
#include <pthread.h>
#include <iostream>
// #include <string.h>

#include "gvtime.h"

#include "videoformat.h"
#include "audioformat.h"
#include "buffer.h"

#include "videobuf.h"
#include "audiobuf.h"

#include "playlist.h"
#include "baseplugin.h"
#include "plugin.h"
#include "input.h"

#include "configwindow.h"
#include "gtkutil.h"

static void __button_callback(GtkWidget * w, gpointer data)
  {
  ((GVConfigWidgetRecordtime*)data)->button_callback(w);
  }

void GVConfigWidgetRecordtime::button_callback(GtkWidget * w)
  {
  if(w == limit_time)
    {
    int sensitive =
      !!gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(limit_time));
    gtk_widget_set_sensitive(hours,   sensitive);
    gtk_widget_set_sensitive(minutes, sensitive);
    gtk_widget_set_sensitive(seconds, sensitive);
    gtk_widget_set_sensitive(hours_label,   sensitive);
    gtk_widget_set_sensitive(minutes_label, sensitive);
    gtk_widget_set_sensitive(seconds_label, sensitive);
    }
  }

GVConfigWidgetRecordtime::GVConfigWidgetRecordtime(GVInput * _input)
  {
  input = _input;

  GtkObject * adjustment = gtk_adjustment_new(0, 0, 59, 1.0, 0.0, 0.0);
  seconds = gtk_spin_button_new(GTK_ADJUSTMENT(adjustment), 0.1, 0);

  adjustment = gtk_adjustment_new(0, 0, 59, 1.0, 0.0, 0.0);
  minutes    = gtk_spin_button_new(GTK_ADJUSTMENT(adjustment), 0.1, 0);

  adjustment = gtk_adjustment_new(0, 0, 99, 1.0, 0.0, 0.0);
  hours      = gtk_spin_button_new(GTK_ADJUSTMENT(adjustment), 0.1, 0);

  limit_time = gtk_check_button_new_with_label("Limit time");

  hours_label = gtk_label_new("Hours");
  minutes_label = gtk_label_new("Minutes");
  seconds_label = gtk_label_new("Seconds");

  // Set attributes

  // Set callbacks

  gtk_signal_connect(GTK_OBJECT(limit_time), "toggled",
                     GTK_SIGNAL_FUNC(__button_callback), (gpointer)this);
  
  
  // Show the objects

  gtk_widget_show(hours);
  gtk_widget_show(minutes);
  gtk_widget_show(seconds);

  gtk_widget_show(hours_label);
  gtk_widget_show(minutes_label);
  gtk_widget_show(seconds_label);

  gtk_widget_show(limit_time);
  
  // Pack the objects

  widget = gvgtk_table_new(3, 3, 1);

  gtk_table_attach_defaults(GTK_TABLE(widget), limit_time,
                            0, 3, 0, 1);

  gtk_table_attach_defaults(GTK_TABLE(widget), hours_label,   0, 1, 1, 2);
  gtk_table_attach_defaults(GTK_TABLE(widget), minutes_label, 1, 2, 1, 2);
  gtk_table_attach_defaults(GTK_TABLE(widget), seconds_label, 2, 3, 1, 2);

  gtk_table_attach_defaults(GTK_TABLE(widget), hours,   0, 1, 2, 3);
  gtk_table_attach_defaults(GTK_TABLE(widget), minutes, 1, 2, 2, 3);
  gtk_table_attach_defaults(GTK_TABLE(widget), seconds, 2, 3, 2, 3);

  gtk_widget_show(widget);
  }
  
void GVConfigWidgetRecordtime::get_values()
  {
  if(input->record_time.undefined())
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(limit_time), 0);
  else
    {
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(limit_time), 1);
    gint32 tmp = input->record_time.seconds;
    gtk_spin_button_set_value(GTK_SPIN_BUTTON(seconds), tmp % 60);
    tmp /= 60;
    gtk_spin_button_set_value(GTK_SPIN_BUTTON(minutes), tmp % 60);
    tmp /= 60;
    gtk_spin_button_set_value(GTK_SPIN_BUTTON(hours), tmp);
    }
  button_callback(limit_time);
  }

void GVConfigWidgetRecordtime::apply_values()
  {
  if(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(limit_time)))
    {
    input->record_time.nanoseconds = 0;
    input->record_time.seconds
      = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(seconds)) +
        gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(minutes)) * 60 +
        gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(hours)) * 3600;
    }
  else
    input->record_time.set_undefined();
  }
