/*****************************************************************

  cw_slider.cc

  Copyright (c) 2001 by Burkhard Plaum - plaum@ipf.uni-stuttgart.de

  http://gmerlin.sourceforge.net

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111, USA.

*****************************************************************/

// Methods for several configuration widgets

#include "config.h"

#include <gtk/gtk.h>

#include "configwindow.h"
#include "gtkutil.h"

GVConfigWidgetSlider::GVConfigWidgetSlider(const char * _label,
                                           float min_value,
                                           float max_value,
                                           int num_digits)
  {
  adjustment = gtk_adjustment_new(min_value, min_value,
                                  max_value, (max_value - min_value)/100.0,
                                  (max_value - min_value)/10.0,
                                  0.0);

  scale = gtk_hscale_new(GTK_ADJUSTMENT(adjustment));
  gtk_scale_set_digits(GTK_SCALE(scale), num_digits);

  label = gtk_label_new(_label);

  gtk_widget_show(scale);
  gtk_widget_show(label);
  
  }

int GVConfigWidgetSlider::get_value_int()
  {
  return (int)(GTK_ADJUSTMENT(adjustment)->value+0.5);
  }

float GVConfigWidgetSlider::get_value_float()
  {
  return GTK_ADJUSTMENT(adjustment)->value;
  }

void GVConfigWidgetSlider::set_value(float value)
  {
  gtk_adjustment_set_value(GTK_ADJUSTMENT(adjustment), value);
  }
  
void GVConfigWidgetSlider::attach(GtkWidget * table, int row, int column)
  {
  gvgtk_table_attach_noresize(table, label, column, column+1, row, row+1);
  gtk_table_attach_defaults(GTK_TABLE(table), scale,
                            column+1, column+2, row, row+1);
  }

void GVConfigWidgetSlider::set_sensitive(int sensitive)
  {
  gtk_widget_set_sensitive(scale, sensitive);
  gtk_widget_set_sensitive(label, sensitive);
  }
