/*****************************************************************

  cw_string.cc

  Copyright (c) 2001 by Burkhard Plaum - plaum@ipf.uni-stuttgart.de

  http://gmerlin.sourceforge.net

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111, USA.

*****************************************************************/

// Configure a string

#include "config.h"
#include <gtk/gtk.h>
#include <string.h>
// #include <iostream>

#include "configwindow.h"
#include "gtkutil.h"

void GVConfigWidgetString::set_string(const char * _string)
  {
  if(!_string)
    {
    gtk_entry_set_text(GTK_ENTRY(entry), "");
    return;
    }
  if(strlen(_string) > (unsigned int)max_length)
    {
    char * tmp_string = g_strdup(_string);
    tmp_string[max_length-1] = '\0';
    gtk_entry_set_text(GTK_ENTRY(entry), tmp_string);
    g_free(tmp_string);
    }
  else
    gtk_entry_set_text(GTK_ENTRY(entry), _string);
  }

const char * GVConfigWidgetString::get_string()
  {
  return gtk_entry_get_text(GTK_ENTRY(entry));
  }

GVConfigWidgetString::GVConfigWidgetString(const char * _label,
                                           int _max_length = -1)
  {
  if(max_length == -1)
    entry = gtk_entry_new();
  else
    entry = gtk_entry_new_with_max_length(_max_length);
  max_length = _max_length;
  label = gtk_label_new(_label);

  gtk_widget_show(label);
  gtk_widget_show(entry);
  }

void GVConfigWidgetString::attach(GtkWidget * table, int row, int column)
  {
  gvgtk_table_attach_noresize(table, label, column, column+1, row, row+1);
  gtk_table_attach_defaults(GTK_TABLE(table), entry,
                            column+1, column+2, row, row+1);
  }

void GVConfigWidgetString::set_sensitive(int sensitive)
  {
  gtk_widget_set_sensitive(entry, sensitive);
  gtk_widget_set_sensitive(label, sensitive);
  }
