/*****************************************************************

  display.cc

  Copyright (c) 2001 by Burkhard Plaum - plaum@ipf.uni-stuttgart.de

  http://gmerlin.sourceforge.net

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111, USA.

*****************************************************************/

// Methods for the track/time display 

#include <gtk/gtk.h>
#include <pthread.h>
#include <string.h>
#include <iostream>
#include "gv.h"

#include "gvgtk.h"
#include "skin.h"
#include "skinbrowser.h"
#include "gvconfig.h"

#include "pixmaps/big_digits.xpm"
#include "pixmaps/mode.xpm"
#include "pixmaps/state.xpm"

using std::cerr;

#define BIG_DIGITS_WIDTH   18
#define BIG_DIGITS_HEIGHT  27
#define BIG_COLON_WIDTH    4

#define MODE_PIXMAP_WIDTH   50
#define MODE_PIXMAP_HEIGHT  27

#define STATE_WIDTH 40

#define MODE_OFFSET_X (STATE_WIDTH+(BIG_DIGITS_WIDTH*6)+(BIG_COLON_WIDTH*2))
#define MODE_OFFSET_Y 0

#define STATE_OFFSET_X 0
#define STATE_OFFSET_Y 0


#define MODE_OFFSET_TOTAL 30
#define MODE_OFFSET_REPEAT 34

#define DRAWINGAREA_WIDTH  (MODE_OFFSET_X+MODE_PIXMAP_WIDTH)

#define MESSAGE_OFFSET_X  3
#define MESSAGE_OFFSET_Y  3

#define MESSAGE_PIXMAP_HEIGHT 23

//#define DRAWINGAREA_HEIGHT (BIG_DIGITS_HEIGHT)

static gboolean __expose_callback(GtkWidget * w, GdkEventExpose *event,
                                  gpointer data)
  {
  ((GVDisplayWidget*)data)->expose();
  return TRUE;
  }

static gboolean __button_callback(GtkWidget * w, GdkEventButton *event,
                                  gpointer data)
  {
  ((GVDisplayWidget*)data)->button_callback(event);
  return TRUE;
  }

static gboolean  __timeout_func(gpointer data)
  {
  ((GVDisplayWidget*)data)->flash_message(0);
  return TRUE;
  }

void GVDisplayWidget::button_callback(GdkEventButton * event)
  {
  // Switch through the display modes
  // Maybe we could do other things with other buttons??

  switch(event->button)
    {
    case 1:
      if(total_track_time.undefined())
        return;
      switch(display_mode)
        {
        case DisplayTrack:
          display_mode = DisplayTrackRemaining;
          break;
        case DisplayTrackRemaining:
          display_mode = DisplayTotal;
          break;
        case DisplayTotal:
          display_mode = DisplayTotalRemaining;
          break;
        case DisplayTotalRemaining:
          display_mode = DisplayTrack;
          break;
        case DisplayAll:
          break;
        }
      if(display_mode != DisplayAll)
        default_display_mode = display_mode;
      break;
    case 2:
      break;
    case 3:
      gv_engine->repeat();
      break;
    }
  update_mode();
  }

void GVDisplayWidget::flash_message(int force)
  {
  if(!(message))
    return;

  //  cerr << "Flash Message\n";
  
  if((force) || (message_pixmap_width > DRAWINGAREA_WIDTH))
    {
    if(!force)
      {
      message_offset+=2;
      if(message_offset >= message_pixmap_width)
        message_offset=0;
      }
    
    // Draw the first part
    
//     gdk_draw_pixmap(drawingarea->window, gc, message,
//                     message_offset, 0,  // Src coords
//                     0, BIG_DIGITS_HEIGHT,
//                     DRAWINGAREA_WIDTH,
//                     MESSAGE_PIXMAP_HEIGHT);

    gdk_draw_pixmap(message_buffer, gc, message,
                    message_offset, 0,  // Src coords
                    0, 0,
                    DRAWINGAREA_WIDTH,
                    MESSAGE_PIXMAP_HEIGHT);
    
    if(message_pixmap_width - message_offset < DRAWINGAREA_WIDTH)
      {
      // Draw the second part

//       gdk_draw_pixmap(drawingarea->window, gc, message,
//                       0, 0,  // Src coords
//                       message_pixmap_width - message_offset,
//                       BIG_DIGITS_HEIGHT,
//                       DRAWINGAREA_WIDTH -
//                       (message_pixmap_width-message_offset),
//                       MESSAGE_PIXMAP_HEIGHT);
      
      gdk_draw_pixmap(message_buffer, gc, message,
                      0, 0,  // Src coords
                      message_pixmap_width - message_offset,
                      0,
                      DRAWINGAREA_WIDTH -
                      (message_pixmap_width-message_offset),
                      MESSAGE_PIXMAP_HEIGHT);
      }
    gdk_draw_pixmap(drawingarea->window, gc, message_buffer,
                    0, 0,  // Src coords
                    0,
                    BIG_DIGITS_HEIGHT,
                    DRAWINGAREA_WIDTH, 
                    MESSAGE_PIXMAP_HEIGHT);
    
    }
  else
    {
    gdk_draw_pixmap(drawingarea->window, gc, message,
                    0, 0,  // Src coords
                    0, BIG_DIGITS_HEIGHT,
                    DRAWINGAREA_WIDTH,
                    MESSAGE_PIXMAP_HEIGHT);
    }
  
  }

void GVDisplayWidget::set_track_message(const char * message_text)
  {
  if(track_message_text)
    g_free(track_message_text);

  track_message_text = g_strdup(message_text);

  error_displayed = 0;
  
  if(is_realized)
    {
    set_message_pixmap(message_text, &foreground_color);
    flash_message(1);
    }
  }

void GVDisplayWidget::set_error_message(const char * message_text,
                                        int timeout)
  {
  if(error_message_text)
    g_free(error_message_text);

  error_message_text = g_strdup(message_text);

  error_displayed = 1;
  
  if(is_realized)
    {
    set_message_pixmap(message_text, &error_color);
    flash_message(1);
    }
  }

void GVDisplayWidget::set_message_pixmap(const char * text,
                                         GdkColor * text_color)
  {
  int text_width = gdk_string_width(message_font, text);
  message_pixmap_width = text_width + MESSAGE_OFFSET_X*2;

  // Check if we Have To Scroll

  char * tmp_text = (char*)0;
  
  if(message_pixmap_width > DRAWINGAREA_WIDTH)
    {
    tmp_text = g_strdup_printf("%s * * * ", text);
    text_width = gdk_string_width(message_font, tmp_text);
    message_pixmap_width = text_width;
    }
  else if(message_pixmap_width < DRAWINGAREA_WIDTH)
    message_pixmap_width = DRAWINGAREA_WIDTH;

  int tmp_height = 0;
  int tmp_width = 0;
  int tmp_x, tmp_y, tmp_depth;
    
  if(message)
    gdk_window_get_geometry(message,
                            &tmp_x, &tmp_y,
                            &tmp_width,
                            &tmp_height,
                            &tmp_depth);

  //    else message_pixmap_width = 0;
  
   
  if((tmp_width != message_pixmap_width) ||
     (tmp_height != MESSAGE_PIXMAP_HEIGHT)) 
    {
    if(message)
      gdk_pixmap_unref(message);

    message = gdk_pixmap_new(drawingarea->window,
                             message_pixmap_width,
                             MESSAGE_PIXMAP_HEIGHT,
                             message_pixmap_depth);

    if(message_buffer)
      gdk_pixmap_unref(message_buffer);

    message_buffer = gdk_pixmap_new(drawingarea->window,
                                    DRAWINGAREA_WIDTH,
                                    MESSAGE_PIXMAP_HEIGHT,
                                    message_pixmap_depth);
    }

  gdk_gc_set_foreground(gc, &background_color);

  gdk_draw_rectangle(message, gc, 1, 0, 0,
                     message_pixmap_width, MESSAGE_PIXMAP_HEIGHT);
  
  gdk_gc_set_foreground(gc, text_color);
  gdk_gc_set_background(gc, &background_color);

  if(tmp_text)
    {
    if(timeout_tag == -1)
      timeout_tag = gtk_timeout_add(50, __timeout_func, (gpointer)this);

    int tmp_y;
//     tmp_y = message_font->ascent + MESSAGE_OFFSET_Y;
    tmp_y = (MESSAGE_PIXMAP_HEIGHT -
             message_font->ascent - message_font->descent) / 2 +
      message_font->ascent; 
    
    
    gdk_draw_string(message, message_font, gc, 0,
                    tmp_y, tmp_text);
    g_free(tmp_text);
    }
  else
    {
    if(timeout_tag != -1)
      {
      gtk_timeout_remove(timeout_tag);
      timeout_tag = -1;
      }
    int tmp_y;
//     tmp_y = message_font->ascent + MESSAGE_OFFSET_Y;
    tmp_y = (MESSAGE_PIXMAP_HEIGHT -
             message_font->ascent - message_font->descent) / 2 +
      message_font->ascent; 
   
    gdk_draw_string(message, message_font, gc,
                    (message_pixmap_width - text_width)/2,
                    tmp_y, text);
    }
    
  //  message_pixmap_width = width;

  message_offset = 0;
  }

static char __message_font_name[] = "9x15";

GVDisplayWidget::GVDisplayWidget()
  {
  //  cerr << "DRAWINGAREA_WIDTH: " << DRAWINGAREA_WIDTH << "\n";
  message = (GdkPixmap*)0;
  message_buffer = (GdkPixmap*)0;
  engine_state = GVEngineStopped;
  error_displayed = 0;
  
  error_message_text = (char *)0;
  track_message_text = (char *)0;
  
  drawingarea = gtk_drawing_area_new();
  
  gtk_widget_set_events(drawingarea, GDK_EXPOSURE_MASK
                        |GDK_BUTTON_PRESS_MASK
                        |GDK_BUTTON_RELEASE_MASK);

  gtk_signal_connect(GTK_OBJECT(drawingarea), "expose_event",
                     (GtkSignalFunc) __expose_callback, (gpointer)this);

  gtk_signal_connect(GTK_OBJECT(drawingarea), "button_press_event",
                     (GtkSignalFunc) __button_callback, (gpointer)this);
  
  gtk_widget_show(drawingarea);
  widget = drawingarea;
  is_realized = 0;

  timeout_tag = -1;
  
  // Initialize the time values

  current_time.reset();
  
  total_track_time.reset();
  
  total_time.reset();

  total_time_before.reset();

  set_hms(total_time);

  message_pixmap_width = 0;
  message_pixmap_depth = 0;

  // alloc_color(0, 192, 192, foreground_color);
  //   alloc_color(0, 0, 0, background_color);
  //  alloc_color(255, 0, 0, error_color);
  
  // Get some config strings

  //  cerr << "Getting Colors\n";
  
  foreground_color.red = 0;            // Cyan
  foreground_color.green = 65535;
  foreground_color.blue = 65535;

  background_color.red   = 0;  // White
  background_color.green = 0;
  background_color.blue  = 0;

  error_color.red   = 65535;  // White
  error_color.green = 0;
  error_color.blue  = 0;

  gdk_color_parse("#00FFFF", &foreground_color);
  gdk_color_parse("#000000", &background_color);
  gdk_color_parse("#FF0000", &error_color);
  
  gvgtk_get_color_config("Display", "Foreground", foreground_color);
  gvgtk_get_color_config("Display", "Background", background_color);
  gvgtk_get_color_config("Display", "Error", error_color);

  const char * tmp = __message_font_name;

  gv_config->get_string("Display", "Font", &tmp);

  get_colors_from_skin = 1;
  gv_config->get_bool("Display", "GetColorsFromString",
                      get_colors_from_skin);
  
  message_font_name = g_strdup(tmp);
  
  config_widget = new ConfigWidget(this);

  int i_tmp = (int)(DisplayTrack);
  gv_config->get_int("Display", "DefaultMode", i_tmp);

  if((DisplayMode)(i_tmp) == DisplayAll)
    default_display_mode = DisplayTrack;
  else
    default_display_mode = (DisplayMode)(i_tmp);
  
  display_mode = default_display_mode;
  }

GVDisplayWidget::~GVDisplayWidget()
  {
  gvgtk_put_color_config("Display", "Foreground", foreground_color);
  gvgtk_put_color_config("Display", "Background", background_color);
  gvgtk_put_color_config("Display", "Error", error_color);
  gv_config->put_bool("Display", "GetColorsFromString",
                      get_colors_from_skin);

  //  if(display_mode == DisplayAll)
    gv_config->put_int("Display", "DefaultMode", (int)(default_display_mode));
//   else
//     gv_config->put_int("Display", "DefaultMode", (int)(display_mode));

  gv_config->put_string("Display", "Font", message_font_name);
  }

void GVDisplayWidget::expose()
  {
  GVTime current_time;
  gv_engine->get_current_track_time(current_time, 0);
  update_time(current_time);
  update_mode();
  flash_message(1);
  update_state();
  }

void GVDisplayWidget::load_bitmap(char ** pixmap_data, GdkPixmap ** ret,
                                  GdkColor * foreground,
                                  GdkColor * background)
  {
  GdkBitmap * bitmap;
  
  *ret = gdk_pixmap_create_from_xpm_d(drawingarea->window, &bitmap,
                                      (GdkColor*)0, pixmap_data);

  int width, height, x, y, depth;

  gdk_window_get_geometry(*ret, &x, &y, &width, &height, &depth);
  
  gdk_gc_set_foreground(gc, background);

  //  gdk_gc_set_background(gc, &background_color);

  gdk_draw_rectangle(*ret, gc, 1, 0, 0, width, height);

  gdk_gc_set_foreground(gc, foreground);
  gdk_gc_set_clip_mask(gc, bitmap);

  gdk_draw_rectangle(*ret, gc, 1, 0, 0, width, height);

  gdk_gc_set_clip_mask(gc, (GdkBitmap*)0);

  gdk_bitmap_unref(bitmap);
  }

void GVDisplayWidget::realize()
  {
  gtk_widget_realize(drawingarea);
  //  message_pixmap_height = message_font->ascent + message_font->descent +
  //    2 * MESSAGE_OFFSET_Y;
  
  gtk_drawing_area_size(GTK_DRAWING_AREA(drawingarea),
                        DRAWINGAREA_WIDTH, BIG_DIGITS_HEIGHT+
                        MESSAGE_PIXMAP_HEIGHT);
  
  //  GdkColor color;
  
  // Set Foreground Color

  gdk_color_alloc(gtk_widget_get_colormap(drawingarea),
                  &foreground_color);
  
  // Set Background Color

  gdk_color_alloc(gtk_widget_get_colormap(drawingarea),
                  &background_color);
  

  // Set Error Color

  gdk_color_alloc(gtk_widget_get_colormap(drawingarea),
                  &error_color);

  gc = gdk_gc_new(drawingarea->window);

  // Create the pixmaps

  load_bitmap(big_digits_xpm, &big_digits, &foreground_color,
              &background_color);
  
  load_bitmap(mode_xpm, &mode_pixmap, &foreground_color,
              &background_color);

  load_bitmap(state_xpm, &state_pixmap, &foreground_color,
              &background_color);
  
  // Set the font


  // Get the depth of the drawingarea

  int width, height, x, y;
  
  gdk_window_get_geometry(drawingarea->window, &x, &y, &width, &height,
                          &message_pixmap_depth);


  message_font = gdk_font_load(message_font_name);
  gdk_gc_set_font(gc, message_font);
  is_realized = 1;

  // Create the message pixmaps if necessary

  if(error_message_text)
    {
    set_message_pixmap(error_message_text, &error_color);
    flash_message(1);
    }
  else if(track_message_text)
    {
    set_message_pixmap(track_message_text, &foreground_color);
    flash_message(1);
    }
  }

void GVDisplayWidget::set_hms(const GVTime & time)
  {
  int time_i = time.seconds;
  seconds = time_i % 60;
  time_i /= 60;
  
  minutes = time_i % 60;
  time_i /= 60;
  
  hours = time_i % 99;
  }

void GVDisplayWidget::update_time(GVTime & current_time)
  {
  GVTime time1;

  switch(display_mode)
    {
    case DisplayTrack:
      set_hms(current_time);
      break;
    case DisplayTrackRemaining:
      time1 = total_track_time;
      time1 -= current_time;
      set_hms(time1);
      break;
    case DisplayTotal:
      time1 = total_time_before;
      time1  += current_time;
      set_hms(time1);
      break;
    case DisplayTotalRemaining:
      time1 = total_time;
      time1 -= total_time_before;
      time1 -= current_time;
      set_hms(time1);
      break;
    case DisplayAll:
      set_hms(total_time);
      break;
    }

  // Now draw the stuff

  int x = STATE_WIDTH;
  int y = 0;

  int printed = 0;

  // First digit

  // Draw the hours

  gdk_gc_set_foreground(gc, &background_color);
  
  if((hours/10))
    {
    gdk_draw_pixmap(drawingarea->window, gc, big_digits,
                    (hours / 10) * BIG_DIGITS_WIDTH, 0,
                    x, y, BIG_DIGITS_WIDTH, BIG_DIGITS_HEIGHT);
    printed = 1;
    }
  else
    gdk_draw_rectangle(drawingarea->window, gc, 1,
                       x, y,
                       BIG_DIGITS_WIDTH, BIG_DIGITS_HEIGHT);
  
  x += BIG_DIGITS_WIDTH;

  // Second digit
  
  if((hours % 10) || (printed))
    {
    gdk_draw_pixmap(drawingarea->window, gc, big_digits,
                    (hours % 10) * BIG_DIGITS_WIDTH, 0,
                    x, y, BIG_DIGITS_WIDTH, BIG_DIGITS_HEIGHT);
    // Colon
    x += BIG_DIGITS_WIDTH;
    gdk_draw_pixmap(drawingarea->window, gc, big_digits,
                    10 * BIG_DIGITS_WIDTH, 0,
                    x, y, BIG_COLON_WIDTH, BIG_DIGITS_HEIGHT);
    x += BIG_COLON_WIDTH;
    printed = 1;
    }
  else
    {
    gdk_draw_rectangle(drawingarea->window, gc, 1,
                       x, y,
                       BIG_DIGITS_WIDTH + BIG_COLON_WIDTH,
                       BIG_DIGITS_HEIGHT);

    x += (BIG_DIGITS_WIDTH + BIG_COLON_WIDTH);
    }
  
  // Draw minutes

  // First digit

  if((minutes / 10) || (printed))
    {
    gdk_draw_pixmap(drawingarea->window, gc, big_digits,
                    (minutes / 10) * BIG_DIGITS_WIDTH, 0,
                    x, y, BIG_DIGITS_WIDTH, BIG_DIGITS_HEIGHT);
    printed = 1;
    }
  else
    {
    gdk_draw_rectangle(drawingarea->window, gc, 1,
                       x, y,
                       BIG_DIGITS_WIDTH, BIG_DIGITS_HEIGHT);
    }

  x += BIG_DIGITS_WIDTH;
  
  // Second digit
  gdk_draw_pixmap(drawingarea->window, gc, big_digits,
                  (minutes % 10) * BIG_DIGITS_WIDTH, 0,
                  x, y, BIG_DIGITS_WIDTH, BIG_DIGITS_HEIGHT);
  x += BIG_DIGITS_WIDTH;

  // Colon
  gdk_draw_pixmap(drawingarea->window, gc, big_digits,
                  10 * BIG_DIGITS_WIDTH, 0,
                  x, y, BIG_COLON_WIDTH, BIG_DIGITS_HEIGHT);
  x += BIG_COLON_WIDTH;

  // Draw seconds
  
  // First digit
  gdk_draw_pixmap(drawingarea->window, gc, big_digits,
                  (seconds / 10) * BIG_DIGITS_WIDTH, 0,
                  x, y, BIG_DIGITS_WIDTH, BIG_DIGITS_HEIGHT);
  x += BIG_DIGITS_WIDTH;

  // Second digit
  gdk_draw_pixmap(drawingarea->window, gc, big_digits,
                  (seconds % 10) * BIG_DIGITS_WIDTH, 0,
                  x, y, BIG_DIGITS_WIDTH, BIG_DIGITS_HEIGHT);
  x += BIG_DIGITS_WIDTH;
    
  //  gdk_draw_pixmap(drawingarea->window, gc, big_digits,
  //                  0, 0, 0, 0, 6*BIG_DIGITS_WIDTH + 2*BIG_COLON_WIDTH,
  //                  BIG_DIGITS_HEIGHT);
  
  
  }

void GVDisplayWidget::update_mode()
  {
  int x = MODE_OFFSET_X;
  int y = MODE_OFFSET_Y;

  //  cerr << "X: " << x << "\n";
  
  gdk_gc_set_foreground(gc, &background_color);

  if((display_mode == DisplayTrack) ||
     (display_mode == DisplayTrackRemaining) ||
     (display_mode == DisplayAll))
      gdk_draw_rectangle(drawingarea->window, gc, 1,x, y,
                         MODE_OFFSET_TOTAL,MODE_PIXMAP_HEIGHT/2); 
  else
    gdk_draw_pixmap(drawingarea->window, gc, mode_pixmap,
                    0, 0, x, y, MODE_OFFSET_TOTAL,MODE_PIXMAP_HEIGHT/2);

  x += MODE_OFFSET_TOTAL;
  
  if((display_mode == DisplayTrackRemaining) ||
     (display_mode == DisplayTotalRemaining))
    gdk_draw_pixmap(drawingarea->window, gc, mode_pixmap,
                    MODE_OFFSET_TOTAL, 0,x, y,  
                    MODE_PIXMAP_WIDTH - MODE_OFFSET_TOTAL,
                    MODE_PIXMAP_HEIGHT/2);
  else
    gdk_draw_rectangle(drawingarea->window, gc, 1, x, y,
                       MODE_PIXMAP_WIDTH - MODE_OFFSET_TOTAL,
                       MODE_PIXMAP_HEIGHT/2); 

  x -= MODE_OFFSET_TOTAL;
  y += MODE_PIXMAP_HEIGHT/2;

  int height_to_draw = (MODE_PIXMAP_HEIGHT/2) + (MODE_PIXMAP_HEIGHT % 2);
    
  GVRepeatMode repeat_mode = gv_engine->get_repeat_mode();

  switch(repeat_mode)
    {
    case GVRepeatNone:
      gdk_draw_rectangle(drawingarea->window, gc, 1, x, y,
                         MODE_PIXMAP_WIDTH, height_to_draw); 
      break;
    case GVRepeat1:
      gdk_draw_pixmap(drawingarea->window, gc, mode_pixmap,
                      0, MODE_PIXMAP_HEIGHT/2, x, y,  
                      MODE_PIXMAP_WIDTH,
                      height_to_draw);
      break;
    case GVRepeatAll:
      gdk_draw_pixmap(drawingarea->window, gc, mode_pixmap,
                      0, MODE_PIXMAP_HEIGHT/2, x, y,  
                      MODE_OFFSET_REPEAT,
                      height_to_draw);
      gdk_draw_rectangle(drawingarea->window, gc, 1, x+MODE_OFFSET_REPEAT, y,
                         MODE_PIXMAP_WIDTH-MODE_OFFSET_REPEAT,
                         height_to_draw); 
      
    }
  gv_engine->get_current_track_time(current_time, 0);
  update_time(current_time);
  }

void GVDisplayWidget::update_playlist_times()
  {
  gv_playlist->get_total_time(total_time);

  // Return the total time in the playlist before the current track
  
  gv_playlist->get_total_time_before(total_time_before);

  const GVPlaylistTrack * current_track
    = gv_playlist->get_current_track();

  if(current_track)
    total_track_time = current_track->time;
  else
    total_track_time.reset();
  }

GVDisplayWidget::ConfigWidget::~ConfigWidget()
  {
  
  }

static void __config_toggle_callback(GtkWidget * w, gpointer data)
  {
  ((GVDisplayWidget::ConfigWidget*)data)->toggle_callback(w);
  }

void GVDisplayWidget::ConfigWidget::toggle_callback(GtkWidget * w)
  {
  if(w == get_colors_from_skin)
    {
    int sensitive = 1;
    if(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(w)))
      sensitive = 0;
    background_color.set_sensitive(sensitive);
    foreground_color.set_sensitive(sensitive);
    error_color.set_sensitive(sensitive);
    }
  }

GVDisplayWidget::ConfigWidget::ConfigWidget(GVDisplayWidget * _display_widget) :
  background_color("Background Color"),
  foreground_color("Normal Foreground"),
  error_color("Error Foreground"),
  font("Font")
  {
  display_widget = _display_widget;
  GtkWidget * table = gvgtk_table_new(5, 3, 0);

  get_colors_from_skin =
    gtk_check_button_new_with_label("Get colors from skin");

  gtk_signal_connect(GTK_OBJECT(get_colors_from_skin), "toggled",
                     GTK_SIGNAL_FUNC(__config_toggle_callback),
                     (gpointer)this);

  gtk_widget_show(get_colors_from_skin);
  gtk_table_attach_defaults(GTK_TABLE(table),
                            get_colors_from_skin, 0, 3, 0, 1);
  
  background_color.attach(table, 1, 0);
  foreground_color.attach(table, 2, 0);
  error_color.attach(table, 3, 0);
  font.attach(table, 4, 0);
  
  gtk_widget_show(table);

  widget = table;
  }

void GVDisplayWidget::apply_skin(GVSkin & skin)
  {
  if(!get_colors_from_skin)
    return;
  
  skin.get_color(skin.display_color_background, &background_color);
  skin.get_color(skin.display_color_foreground, &foreground_color);
  skin.get_color(skin.display_color_error, &error_color);

  gdk_color_alloc(gtk_widget_get_colormap(drawingarea),
                  &foreground_color);
  gdk_color_alloc(gtk_widget_get_colormap(drawingarea),
                  &background_color);
  gdk_color_alloc(gtk_widget_get_colormap(drawingarea),
                  &error_color);
  update_colors();
  }

void GVDisplayWidget::update_colors()
  {
  if(!is_realized)
    return;

  if(error_displayed)
    {
    set_message_pixmap(error_message_text, &error_color);
    flash_message(1);
    }
  else if(track_message_text)
    {
    set_message_pixmap(track_message_text, &foreground_color);
    flash_message(1);
    }
  
  if(big_digits)
    gdk_pixmap_unref(big_digits);
  
  load_bitmap(big_digits_xpm, &big_digits, &foreground_color,
              &background_color);
  
  if(mode_pixmap)
    gdk_pixmap_unref(mode_pixmap);

  load_bitmap(mode_xpm, &mode_pixmap, &foreground_color,
              &background_color);

  if(state_pixmap)
    gdk_pixmap_unref(state_pixmap);

  load_bitmap(state_xpm, &state_pixmap, &foreground_color,
              &background_color);
  
  update_mode();
  update_state();
  }

void GVDisplayWidget::update_state(GVEngineState state)
  {
  engine_state = state;
  update_state();
  }

void GVDisplayWidget::update_state()
  {
  int x_offset = 0;

  switch(engine_state)
    {
    case GVEngineStopped:    
      x_offset = 2 * STATE_WIDTH;
      break;
    case GVEnginePaused:    
      x_offset = 0;
      break;
    case GVEnginePlaying:
      x_offset = 1 * STATE_WIDTH;
      break;
    case GVEngineCompleting:
    case GVEngineSeeking:
    case GVEngineChanging:
      x_offset = 4 * STATE_WIDTH;
      break;
    case GVEngineError:
      x_offset = 3 * STATE_WIDTH;
      break;
    }

  gdk_draw_pixmap(drawingarea->window, gc, state_pixmap,
                  x_offset, 0,  // Src coords
                  STATE_OFFSET_X, STATE_OFFSET_Y, 
                  STATE_WIDTH, BIG_DIGITS_HEIGHT);
  }


void GVDisplayWidget::set_font(const char * fontname)
  {
  // Don't do this if it's the same font

  if((message_font_name) && !strcmp(fontname, message_font_name))
    return;

  if(message_font_name)
    g_free(message_font_name);
  message_font_name = g_strdup(fontname);

  if(message_font)
    gdk_font_unref(message_font);
    
  message_font = gdk_font_load(message_font_name);
//   message_pixmap_height = message_font->ascent + message_font->descent +
//     2 * MESSAGE_OFFSET_Y;

  gtk_drawing_area_size(GTK_DRAWING_AREA(drawingarea),
                        DRAWINGAREA_WIDTH, BIG_DIGITS_HEIGHT+
                        MESSAGE_PIXMAP_HEIGHT);
  }

void GVDisplayWidget::set_default_display_mode()
  {
  if(total_track_time.undefined())
    display_mode = DisplayTrack;
  else
    display_mode = default_display_mode;
  update_mode();
  }

void GVDisplayWidget::set_display_mode(GVDisplayWidget::DisplayMode new_mode)
  {
  if(new_mode == DisplayAll)
    {
    if(display_mode != DisplayAll)
      default_display_mode = display_mode;
    }
  else
    default_display_mode = new_mode;
  display_mode = new_mode;
  update_mode();
  }

void GVDisplayWidget::ConfigWidget::apply_values()
  {
  if(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(get_colors_from_skin)))
    {
    display_widget->get_colors_from_skin = 1;
    display_widget->apply_skin(*(gv_skin_browser->current_skin));
    }
  else
    {
    display_widget->get_colors_from_skin = 0;
    if(background_color.get_color(display_widget->background_color))
      gdk_color_alloc(gtk_widget_get_colormap(display_widget->drawingarea),
                      &(display_widget->background_color));
        
    if(foreground_color.get_color(display_widget->foreground_color))
      gdk_color_alloc(gtk_widget_get_colormap(display_widget->drawingarea),
                      &(display_widget->foreground_color));
    
    if(error_color.get_color(display_widget->error_color))
      gdk_color_alloc(gtk_widget_get_colormap(display_widget->drawingarea),
                      &(display_widget->error_color));
    display_widget->update_colors();
    }
  
  display_widget->set_font(font.get_font());
  }

void GVDisplayWidget::ConfigWidget::get_values()
  {
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(get_colors_from_skin),
                               display_widget->get_colors_from_skin);
  toggle_callback(get_colors_from_skin);
  background_color.set_color(display_widget->background_color);
  foreground_color.set_color(display_widget->foreground_color);
  error_color.set_color(display_widget->error_color);
  font.set_font(display_widget->message_font_name);
  }
