/*****************************************************************

  display.h

  Copyright (c) 2001 by Burkhard Plaum - plaum@ipf.uni-stuttgart.de

  http://gmerlin.sourceforge.net

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111, USA.

*****************************************************************/

// Track and time  Display widget

class GVSkin;

class GVDisplayWidget
  {

public:
  enum DisplayMode
  {
    DisplayTrack          = 0,
    DisplayTrackRemaining = 1,
    DisplayTotal          = 2,
    DisplayTotalRemaining = 3,
    DisplayAll            = 4
  };

private:

  int get_colors_from_skin;
  
  GVEngineState engine_state;
  
  GtkWidget * drawingarea;

  GdkPixmap * big_digits;
  GdkPixmap * mode_pixmap;
  GdkPixmap * state_pixmap;
  
  GdkPixmap * message;

  // The buffer is needed to prevent
  // the track display from flickering
  
  GdkPixmap * message_buffer;
  
  char * error_message_text;
  char * track_message_text;

  int error_displayed;
  
  GdkFont * message_font;
  char * message_font_name;
  
  //  GdkPixmap * small_digits;

  GdkColor foreground_color;
  GdkColor error_color;
  GdkColor background_color;

  GdkGC * gc;

  gint hours;
  gint minutes;
  gint seconds;

  GVTime current_time;
  GVTime total_track_time;
  GVTime total_time;

  GVTime total_time_before;

  int message_pixmap_depth;
  int message_pixmap_width;
  //  int message_pixmap_height;
  
  int message_offset;
  
  //  float remaining_total_time;

  // Set hours, minutes and seconds according to time
  
  void set_hms(const GVTime & time);

  DisplayMode display_mode;
  DisplayMode default_display_mode;

  int is_realized;

  void set_message_pixmap(const char * text, GdkColor * text_color);

  gint timeout_tag;

  
  void update_colors();
  void set_font(const char * fontname);
  
public:

  // Configuration stuff

  class ConfigWidget : public GVConfigWidget
    {
    GVConfigWidgetColor background_color;
    GVConfigWidgetColor foreground_color;
    GVConfigWidgetColor error_color;
    GVConfigWidgetFont font;
        
    GVDisplayWidget * display_widget;
    GtkWidget * get_colors_from_skin;
    
  public:
    void toggle_callback(GtkWidget * w);
    ConfigWidget(GVDisplayWidget * _display_widget);
    void apply_values();
    void get_values();
    virtual ~ConfigWidget();
    };

  friend class ConfigWidget;
  
  void set_track_message(const char * message);
  void set_error_message(const char * message, int timeout = 2000);
    
  void flash_message(int force);

  void apply_skin(GVSkin & skin);
  
  void update_playlist_times();
  
  void set_default_display_mode();
  void set_display_mode(DisplayMode new_mode);
  
  GtkWidget * widget;
  
  // Callbacks
  
  void realize();
  void expose();
  void button_callback(GdkEventButton * event);
  
//   void set_time(float time);
//   void set_track_time(float time);
//   void set_total_time(float time);
//   void set_track_number(int num);

  void play();
  void stop();

  void load_bitmap(char ** pixmap_data, GdkPixmap ** ret,
                   GdkColor * foreground,
                   GdkColor * background);

    
  // Update the display
  
  void update_time(GVTime & current_time);
  void update_mode();

  void update_state(GVEngineState state);
  void update_state();
    
  GVDisplayWidget();
  ~GVDisplayWidget();

  GVConfigWidget * config_widget;
    
  };
