/*****************************************************************

  engineloop.cc

  Copyright (c) 2001 by Burkhard Plaum - plaum@ipf.uni-stuttgart.de

  http://gmerlin.sourceforge.net

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111, USA.

*****************************************************************/

// Main loop of the engine

#include <glib.h>
#include <pthread.h>
#include <ctype.h>
#include <sys/types.h>
#include <dirent.h>
#include <string.h>

#include <sys/stat.h>
#include <unistd.h>

#include <iostream>

#include "config.h"
#include "gvtime.h"
#include "gvconfig.h"
#include "videoformat.h"
#include "audioformat.h"
#include "buffer.h"
#include "audiobuf.h"
#include "videobuf.h"

#include "playlist.h"

#include "baseplugin.h"
#include "plugin.h"
#include "input.h"

#include "output.h"
#include "audioout.h"
#include "videoout.h"

#include <gtk/gtk.h>
#include "configwindow.h"
#include "engine.h"

static void * __run_func(void * data)
  {
  //  cerr << "Playback_func: data" << data << "\n";
  ((GVEngine*)data)->run_func();
  return (void*)0;
  }
                              
void GVEngine::run() // Play
  {
  pthread_attr_t attr;
  pthread_attr_init(&attr);
  pthread_attr_setdetachstate(&attr, PTHREAD_CREATE_DETACHED);
  //  cerr << attributes->name << ": Creating playback thread...";
  pthread_create(&run_thread, &attr, &__run_func, (void*)this);
  //  cerr << "done\n";
  pthread_attr_destroy (&attr);
  }

// Run loop of the Engine

void GVEngine::run_func()
  {
  GVTime sleep_time;
  sleep_time.seconds = 0;
  sleep_time.nanoseconds = 100000000; // 100 Milliseconds
  GVEngineState tmp_state;
  int num_commands;
  Command * command;

  GVEngineState old_state = GVEngineStopped;
  
  while(1)
    {
    // Check if the state changed
    // Only the states "Error" and "Completing" can be triggered
    // by plugins

    tmp_state = get_state(old_state);
    
    switch(tmp_state)
      {
      case GVEngineError:
        break;
      case GVEngineCompleting:
        {
        wait_for_plugins();
        audio_buffer->free();
        video_buffer->free();
        if(!(get_config_flags() & GV_ENGINE_AUTO_ADVANCE))
          {
          set_state(GVEngineStopped);
          continue;
          }
        switch(repeat_mode)
          {
          case GVRepeatNone:
            {
            int num_tracks = gv_playlist->get_num_tracks();
            // Stop after last track of the playlist
            if(gv_playlist->get_current_track() ==
               gv_playlist->get_track(num_tracks-1))
              {
              set_state(GVEngineStopped);
              continue;
              }
            else
              {
              pthread_mutex_lock(&cmd_mutex);
              next_cmd();
              play_cmd(GVEnginePlaying);
              pthread_mutex_unlock(&cmd_mutex);
              }
            break;
            }
          case GVRepeat1:
            pthread_mutex_lock(&cmd_mutex);
            play_cmd(GVEnginePlaying);
            pthread_mutex_unlock(&cmd_mutex);
            break;
          case GVRepeatAll:
            pthread_mutex_lock(&cmd_mutex);
            next_cmd();
            play_cmd(GVEnginePlaying);
            pthread_mutex_unlock(&cmd_mutex);
            break;
          }
        }
      default:
        break;
      }
      
    // Process all commands

    num_commands = get_num_commands();
    while(num_commands)
      {
      command = pop_command();
      pthread_mutex_lock(&cmd_mutex);
      switch(command->type)
        {
        case CommandNext:
          //          cerr << "CommandNext\n";
          next_cmd();
          break;
        case CommandPrevious:
          //          cerr << "CommandPrevious\n";
          previous_cmd();
          break;
        case CommandPlay:
          //          cerr << "CommandPlay\n";
          if(tmp_state == GVEnginePlaying)
            break;
          play_cmd(GVEnginePlaying);
          break;
        case CommandStop:
          //          cerr << "CommandStop\n";
          if(tmp_state == GVEngineStopped)
            break;
          stop_cmd();
          break;
        case CommandPause:
          //          cerr << "CommandPause\n";
          pause_cmd();
          break;
        case CommandSeek:
          //          cerr << "CommandSearch\n";
          seek_cmd(command->arg_f);
          break;
        case CommandPlaytrack:
          //          cerr << "CommandPlaytrack (" << command->arg_i << ")\n";
          play_track_cmd(command->arg_i);
          break;
        case CommandRefreshTrack:
          refresh_track_cmd(command->arg_i);
          break;
        case CommandPlayfile:
          //          cerr << "CommandPlayfile\n";
          play_file_cmd(command->arg_c);
          break;
        case CommandAddfile:
          //          cerr << "CommandAddfile\n";
          add_file_cmd(command->arg_c);
          break;
        case CommandAddfileWithPlugin:
          //          cerr << "CommandAddfile\n";
          char * pos = strchr(command->arg_c, ':');
          *pos = '\0';
          pos++;
          add_file_with_plugin_cmd(pos, command->arg_c);
          break;

        }
      pthread_mutex_unlock(&cmd_mutex);
      delete command;
      num_commands--;
      }
    // Update the time

    if(tmp_state == GVEnginePlaying)
      {
      pthread_mutex_lock(&time_mutex);
      set_current_track_time();
      pthread_mutex_unlock(&time_mutex);
      }
    sleep_time.sleep();
    }
  
  
  }
