/*****************************************************************

  fileselect.cc

  Copyright (c) 2001 by Burkhard Plaum - plaum@ipf.uni-stuttgart.de

  http://gmerlin.sourceforge.net

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111, USA.

*****************************************************************/

// Methods for selecting media files

#include <gtk/gtk.h>
#include <pthread.h>
#include <iostream>
#include "gv.h"


#include "fileselect.h"
#include "button.h"
#include "gvwindow.h"
#include "playlistwidget.h"
#include "gvconfig.h"

#include "gvloader.h"


#include <string.h>

static void __callback(GtkWidget * w, gpointer data)
  {
  ((GVFileSelect*)data)->callback(w);
  }

 
static gboolean __delete_callback(GtkWidget * w, GdkEventAny * event,
                                  gpointer data)
  {
  ((GVFileSelect*)data)->callback(w);
  return TRUE;
  }

static void __menu_callback(GtkWidget * w, gpointer data)
  {
  ((GVFileSelect*)data)->menu_callback(w);
  }

static int __int_compare_func(gconstpointer a, gconstpointer b)
  {
  if (GPOINTER_TO_INT(a) < GPOINTER_TO_INT(b))
    return -1;
  if (GPOINTER_TO_INT(a) > GPOINTER_TO_INT(b))
    return 1;
  else
    return 0;
  }

void GVFileSelect::menu_callback(GtkWidget * w)
  {
  if(w == plugin_auto)
    plugin_info = (const GVPluginInfo*)0;
  plugin_info
    = (const GVPluginInfo*)(gtk_object_get_user_data(GTK_OBJECT(w)));
  }

void GVFileSelect::callback(GtkWidget * w)
  {
  if(w == (GTK_FILE_SELECTION(fileselect)->ok_button))
    {
    gchar * selected_file
      = gtk_file_selection_get_filename(GTK_FILE_SELECTION(fileselect));

    // Make a path form the filename

    gchar * path = g_strdup(selected_file);

    char * pos = strrchr(path, G_DIR_SEPARATOR);
    pos++;
    *pos = '\0';

    char * file_to_add;

    char * filename;

    GList * selected_files = (GList*)0;

    GList * selection = GTK_CLIST(GTK_FILE_SELECTION(fileselect)->file_list)->selection;
    GList * node;
    
    int num = g_list_length(selection);
    
    for(int i = 0; i < num; i++)
      {
      selected_files = g_list_append(selected_files, selection->data);
      selection = g_list_next(selection);
      }
    selected_files = g_list_sort(selected_files, __int_compare_func);

    node = selected_files;
    for(int i = 0; i < num; i++)
      {
      // if(gtk_clist_get_text(GTK_CLIST(GTK_FILE_SELECTION(fileselect)->file_list)

      gtk_clist_get_text(GTK_CLIST(GTK_FILE_SELECTION(fileselect)->file_list),
                         GPOINTER_TO_INT(node->data), 0, &filename);      
      gtk_clist_unselect_row(GTK_CLIST(GTK_FILE_SELECTION(fileselect)->file_list), GPOINTER_TO_INT(node->data), 0);
      node = g_list_next(node);
      file_to_add = g_strconcat(path, filename, (char*)0); 

      if(!plugin_info)
        gv_engine->add_file(file_to_add);
      else
        gv_engine->add_file_with_plugin(file_to_add, plugin_info->name);

      g_free(file_to_add);
      }
    g_free(path);
    g_list_free(selected_files);
    //    playlist_widget.update();
    }

  else if((w == GTK_FILE_SELECTION(fileselect)->cancel_button) ||
          (w == fileselect))
    hide();
  }

GVFileSelect::GVFileSelect()
  {
  plugin_info = (const GVPluginInfo*)0;
    
  // Create the objects

  fileselect = gtk_file_selection_new("Select File(s)");

  const char * path = (char*)0;
  gv_config->get_string("General", "OpenPath", &path);

  if(path)
    gtk_file_selection_set_filename(GTK_FILE_SELECTION(fileselect), path);

  GtkWidget * menu = gtk_menu_new();
  option_menu = gtk_option_menu_new();

  GtkWidget * item;
  
  // Create the option menu
  
  plugin_auto = gtk_menu_item_new_with_label("By Extension");
  gtk_signal_connect(GTK_OBJECT(plugin_auto), "activate",
                     GTK_SIGNAL_FUNC(__menu_callback), (gpointer)this);
  gtk_widget_show(plugin_auto);
  gtk_menu_append(GTK_MENU(menu), plugin_auto);

  int num = gv_input_file_loader->get_num_plugins();

  const GVPluginInfo * tmp_plugin_info;
  
  for(int i = 0; i < num; i++)
    {
    tmp_plugin_info = gv_input_file_loader->get_plugin_info(i);
    item = gtk_menu_item_new_with_label(tmp_plugin_info->real_name);
    gtk_object_set_user_data(GTK_OBJECT(item), (gpointer)tmp_plugin_info);
    gtk_signal_connect(GTK_OBJECT(item), "activate",
                       GTK_SIGNAL_FUNC(__menu_callback),
                       (gpointer)this);
    gtk_widget_show(item);
    gtk_menu_append(GTK_MENU(menu), item);
    }
  gtk_widget_show(menu);
  gtk_option_menu_set_menu(GTK_OPTION_MENU(option_menu), menu);
  
  // Set Attributes

  gtk_file_selection_show_fileop_buttons(GTK_FILE_SELECTION(fileselect));

  GtkWidget * button_label;
  
  button_label = GTK_BIN((GTK_FILE_SELECTION(fileselect)->ok_button))->child;
  gtk_label_set_text(GTK_LABEL(button_label), "Add");
  
  button_label =
    GTK_BIN((GTK_FILE_SELECTION(fileselect)->cancel_button))->child;
  gtk_label_set_text(GTK_LABEL(button_label), "Close");
  
  gtk_clist_set_selection_mode(GTK_CLIST(GTK_FILE_SELECTION(fileselect)->file_list),
                               GTK_SELECTION_EXTENDED);
  
  // Set Callbacks

  gtk_signal_connect(GTK_OBJECT(GTK_FILE_SELECTION(fileselect)->ok_button),
                     "clicked", GTK_SIGNAL_FUNC(__callback), (gpointer)this);

  gtk_signal_connect(GTK_OBJECT(GTK_WINDOW(fileselect)),
                     "delete_event",
                     GTK_SIGNAL_FUNC(__delete_callback), (gpointer)this);
  
  gtk_signal_connect(GTK_OBJECT(GTK_FILE_SELECTION(fileselect)->cancel_button),
                     "clicked", GTK_SIGNAL_FUNC(__callback), (gpointer)this);
  
  // Show the objects

  gtk_widget_show(option_menu);
    
  // Place the objects

  GtkWidget * hbox = gtk_hbox_new(0, 5);
  GtkWidget * label = gtk_label_new("Plugin");
  gtk_widget_show(label);

  gtk_box_pack_start_defaults(GTK_BOX(hbox), label);
  gtk_box_pack_start_defaults(GTK_BOX(hbox), option_menu);

  gtk_widget_show(hbox);
  
  gtk_container_add(GTK_CONTAINER(GTK_FILE_SELECTION(fileselect)->action_area),
                    hbox);

  //  gtk_widget_show(GTK_FILE_SELECTION(fileselect)->action_area);
  }

void GVFileSelect::show()
  {
  gtk_widget_show(fileselect);
  }

void GVFileSelect::hide()
  {
  gtk_widget_hide(fileselect);
  }

GVFileSelect::~GVFileSelect()
  {
  char * pos;
  char * path = gtk_file_selection_get_filename(GTK_FILE_SELECTION(fileselect));
  if((pos = strrchr(path, G_DIR_SEPARATOR)))
    {
    pos++;
    *pos = '\0';
    }
  gv_config->put_string("General", "OpenPath", path);
  }
