/*****************************************************************

  globalconfig.cc

  Copyright (c) 2001 by Burkhard Plaum - plaum@ipf.uni-stuttgart.de

  http://gmerlin.sourceforge.net

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111, USA.

*****************************************************************/

// Methods for the global configuration items

#ifdef __GNUG__
#pragma implementation "globalconfig.h"
#endif

#include <gtk/gtk.h>
#include <string.h>
#include <iostream>
#include "gv.h"
#include "gvconfig.h"
#include "configwindow.h"
#include "globalconfig.h"
#include "gvgtk.h"
#include "gvmmx.h"

#define SECTION "General"

GVGlobalConfig * gv_global_config;

void GVGlobalConfig::get_config()
  {
  const char * tmp = g_get_home_dir();

  char * home_directory = new char[strlen(tmp)+2];
  strcpy(home_directory, tmp);
  strcat(home_directory, "/");
  tmp = home_directory;
  
  gv_config->get_string(SECTION, "OutputDirectory", &tmp);
  output_directory = g_strdup(tmp);

  delete home_directory;
  
  tmp = "%a - %t";
  gv_config->get_string(SECTION, "TracknameMask", &tmp);
  trackname_mask = g_strdup(tmp);
  
  singlepic_framerate = 25.0;
  
  gv_config->get_float(SECTION, "SinglepicFramerate", singlepic_framerate);

  //  delete_incomplete = 0;
  //  gv_config->get_bool(SECTION, "DeleteIncomplete", delete_incomplete);

  tmp = "";
  gv_config->get_string(SECTION, "HttpProxyServer", &tmp);
  http_proxy_server = g_strdup(tmp);

  http_proxy_port = 8000;
  gv_config->get_int(SECTION, "HttpProxyPort", http_proxy_port);

  http_use_proxy = 0;
  gv_config->get_bool(SECTION, "HttpProxy", http_use_proxy);

  http_use_proxy_env = 0;
  gv_config->get_bool(SECTION, "HttpProxyEnv", http_use_proxy_env);

  use_track_info = 1;
  gv_config->get_bool(SECTION, "UseTrackInfo", use_track_info);

  // Default background color: black
  
  alpha_background[0] = 0x00;
  alpha_background[1] = 0x00;
  alpha_background[2] = 0x00;

  gv_config->get_color(SECTION, "AlphaBackground",
                       alpha_background[0],
                       alpha_background[1],
                       alpha_background[2]);
  }

void GVGlobalConfig::put_config()
  {
  gv_config->put_string(SECTION, "TracknameMask", trackname_mask);
  gv_config->put_string(SECTION, "OutputDirectory", output_directory);
  gv_config->put_float(SECTION, "SinglepicFramerate", singlepic_framerate);
  //  gv_config->put_bool(SECTION, "DeleteIncomplete", delete_incomplete);

  gv_config->put_string(SECTION, "HttpProxyServer", http_proxy_server);

  gv_config->put_int(SECTION, "HttpProxyPort", http_proxy_port);
  gv_config->put_bool(SECTION, "HttpProxy", http_use_proxy);
  gv_config->put_bool(SECTION, "HttpProxyEnv", http_use_proxy_env);
  gv_config->put_bool(SECTION, "UseTrackInfo", use_track_info);

  gv_config->put_color(SECTION, "AlphaBackground",
                       alpha_background[0],
                       alpha_background[1],
                       alpha_background[2]);
  }

GVGlobalConfig::GVGlobalConfig() :
  config_widget(this),
  network_config_widget(this)
  {
  get_config();
  }

GVGlobalConfig::~GVGlobalConfig()
  {
  put_config();
  }

GVGlobalConfig::ConfigWidget::~ConfigWidget()
  {
  //  gtk_widget_destroy(auto_advance);
  }

GVGlobalConfig::ConfigWidget::ConfigWidget(GVGlobalConfig * _global_config) :
  output_directory("Output directory for file writers", 1),
  singlepic_framerate("Framerate for single picture plugins", 1.0, 100.0, 2),
  alpha_background("Background for transparent video")
  {
  global_config = _global_config;

  auto_advance = gtk_check_button_new_with_label("Auto advance");

  use_mmx = gtk_check_button_new_with_label("Use MMX");

  start_play = gtk_check_button_new_with_label("Start play after start");

  delete_incomplete
    = gtk_check_button_new_with_label("Delete incomplete output files");

  remove_error_tracks
    = gtk_check_button_new_with_label("Remove error tracks");
  
  gtk_widget_show(auto_advance);
  gtk_widget_show(start_play);
  gtk_widget_show(delete_incomplete);
  gtk_widget_show(remove_error_tracks);
  gtk_widget_show(use_mmx);

  GtkWidget * table = gvgtk_table_new(6, 3, 0);

  int ypos = 0;
  
  output_directory.attach(table, ypos, 0);

  ypos++;
  
  
  singlepic_framerate.attach(table, ypos, 0);

  ypos++;

  gtk_table_attach_defaults(GTK_TABLE(table),
                            auto_advance, 0, 1, ypos, ypos+1);
  gtk_table_attach_defaults(GTK_TABLE(table),
                            start_play, 1, 2, ypos, ypos+1);
  ypos++;
  
  gtk_table_attach_defaults(GTK_TABLE(table), delete_incomplete, 0, 1,
                            ypos, ypos+1);
  gtk_table_attach_defaults(GTK_TABLE(table), remove_error_tracks, 1, 2,
                            ypos, ypos+1);

  ypos++;
  
  gtk_table_attach_defaults(GTK_TABLE(table), use_mmx, 0, 1, ypos, ypos+1);

  ypos++;
  
  alpha_background.attach(table, ypos, 0);
  
  gtk_widget_show(table);
  widget =  table;
  }


void GVGlobalConfig::ConfigWidget::apply_values()
  {
  g_free(global_config->output_directory);
  global_config->output_directory = g_strdup(output_directory.get_filename());

  
  global_config->singlepic_framerate = singlepic_framerate.get_value();
  
  int config_flags = 0;
  if(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(auto_advance)))
    config_flags |= GV_ENGINE_AUTO_ADVANCE;

  if(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(delete_incomplete)))
    config_flags |= GV_ENGINE_DELETE_INCOMPLETE;

  if(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(start_play)))
    config_flags |= GV_ENGINE_START_PLAY;

  if(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(remove_error_tracks)))
    config_flags |= GV_ENGINE_REMOVE_ERROR_TRACKS;
  gv_engine->set_config_flags(config_flags);

  gv_mmx->set(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(use_mmx)));

  alpha_background.get_color(global_config->alpha_background[0],
                             global_config->alpha_background[1],
                             global_config->alpha_background[2]);
  }

void GVGlobalConfig::ConfigWidget::get_values()
  {
  //  cerr << "GVGlobalConfig::ConfigWidget::get_values\n";
  output_directory.set_filename(global_config->output_directory);
 
  singlepic_framerate.set_value(global_config->singlepic_framerate);
  
  
  int config_flags = gv_engine->get_config_flags();

  
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(auto_advance),
                               !!(config_flags & GV_ENGINE_AUTO_ADVANCE));
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(start_play),
                               !!(config_flags & GV_ENGINE_START_PLAY));
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(delete_incomplete),
                               !!(config_flags & GV_ENGINE_DELETE_INCOMPLETE));
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(remove_error_tracks),
                               !!(config_flags & GV_ENGINE_REMOVE_ERROR_TRACKS));

  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(use_mmx), gv_mmx->get());

  alpha_background.set_color(global_config->alpha_background[0],
                             global_config->alpha_background[1],
                             global_config->alpha_background[2]);
  }

GVGlobalConfig::NetworkConfigWidget::NetworkConfigWidget(GVGlobalConfig * _global_config) :
  http_proxy_server("Proxy Server"),  // Adress of the proxy server
  http_proxy_port("Proxy Port", 1.0, 10000.0, 0)       // http Proxy port
  {
  global_config = _global_config;

  // Use proxy at all
  
  use_http_proxy = gtk_check_button_new_with_label("Use Proxy");
  use_http_proxy_env = gtk_check_button_new_with_label("Use \"http_proxy\" variable");

  gtk_widget_show(use_http_proxy);
  //  gtk_widget_show(use_http_proxy_env);

  GtkWidget * table = gvgtk_table_new(3, 2, 0);

  gtk_table_attach_defaults(GTK_TABLE(table), use_http_proxy,     0, 1, 0, 1);
  gtk_table_attach_defaults(GTK_TABLE(table), use_http_proxy_env, 1, 2, 0, 1);

  http_proxy_server.attach(table, 1, 0);
  http_proxy_port.attach(table, 2, 0);

  gtk_widget_show(table);
  widget = table;  
  }

GVGlobalConfig::NetworkConfigWidget::~NetworkConfigWidget()
  {
  
  }

void GVGlobalConfig::NetworkConfigWidget::get_values()
  {
  http_proxy_server.set_string(global_config->http_proxy_server);

  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(use_http_proxy),
                               global_config->http_use_proxy);

  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(use_http_proxy_env),
                               global_config->http_use_proxy_env);

  http_proxy_port.set_value(global_config->http_proxy_port);
  }

void GVGlobalConfig::NetworkConfigWidget::apply_values()
  {
  g_free(global_config->http_proxy_server);
  global_config->http_proxy_server
    = g_strdup(http_proxy_server.get_string());

  global_config->http_proxy_port = (int)(http_proxy_port.get_value()+0.5);

  global_config->http_use_proxy =
    gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(use_http_proxy));

  global_config->http_use_proxy_env =
    gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(use_http_proxy_env));
  
  }
