/*****************************************************************

  audioout.h

  Copyright (c) 2001 by Burkhard Plaum - plaum@ipf.uni-stuttgart.de

  http://gmerlin.sourceforge.net

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111, USA.

*****************************************************************/

// Program wide configuration database

enum GVConfigItemType
  {
    GVConfigItemBool,
    GVConfigItemInt,
    GVConfigItemFloat,
    GVConfigItemString,
    GVConfigItemColor,
    GVConfigItemStringlist // Needed for multiple devices
  };

struct GVConfigColor
  {
  guchar red;
  guchar green;
  guchar blue;
  };

union GVConfigValue
  {
  int c_int;
  float c_float;
  char * c_string;
  GVConfigColor c_color;
  GList * c_stringlist;
  };

class GVConfigItem
  {
  int used;
  GVConfigValue value;
  GVConfigItemType type;

  void free_data();

  
public:

  char * name;
  
  GVConfigItem(const char * _name, const char * string);
  GVConfigItem(const char * _name, float value);
  GVConfigItem(const char * _name, guchar red, guchar green, guchar blue);

  // Need type here to distuingish between bool and int

  GVConfigItem(const char * _name, int value, GVConfigItemType _type);

  GVConfigItem(const char * _name, GList * _stringlist);
  
  ~GVConfigItem();
  
  int get_int(int & ret);
  int get_float(float & ret);
  int get_string(const char ** ret);
  int get_bool(int & ret);
  int get_color(guchar & red, guchar & green, guchar & blue);
  int get_stringlist(GList ** ret);
    
  void set_int(int val);
  void set_float(float val);
  void set_string(const char * val);
  void set_bool(int val);
  void set_color(guchar red,guchar green, guchar blue);
  void set_stringlist(GList * list);

  friend class GVConfigSection;
  friend class GVConfig;
  };

class GVConfigSection
  {
  int used;
  GVConfigItem * find_item(const char * _item_name);

  GList * items;
  
public:  
  char * name;
  
  GVConfigSection(const char * _name);
  int get_int(const char * _name, int & ret);
  int get_float(const char * _name, float & ret);
  int get_string(const char * _name, const char ** ret);
  int get_bool(const char * _name, int & ret);
  int get_color(const char * _name, GVConfigColor & ret);
  int get_color(const char * _name,
                guchar & red, guchar & green, guchar & blue);

  int get_stringlist(const char * _name, GList ** ret);
    
  void put_int(const char * _name, int value);
  void put_float(const char * _name, float value);
  void put_string(const char * _name, const char * value);
  void put_bool(const char * _name, int value);
  void put_color(const char * _name, guchar red, guchar green, guchar blue);
  void put_stringlist(const char * _name, GList * value);
  
  ~GVConfigSection();
  
  friend class GVConfig;
  };

class GVConfig
  {
  GVConfigSection * find_section(const char * _name);
  
  GList * sections;
  
  int read_int(char * pos, int & ret);
  int read_float(char * pos, float & ret);
  
public:
  
  void put_int(const char * section, const char * name, int value);
  void put_bool(const char * section, const char * name, int value);
  void put_float(const char * section, const char * name, float value);
  void put_string(const char * section, const char * name, const char * value);
  void put_color(const char * section, const char * name,
                 guchar red, guchar green, guchar blue);

  void put_stringlist(const char * section, const char * name,
                      GList * value);

  // get_* functions: return 1 on success
  
  int get_int(const char * section, const char * name, int & value);
  int get_bool(const char * section, const char * name, int & value);
  int get_float(const char * section, const char * name, float & value);
  int get_string(const char * section, const char * name, const char ** ret);
  int get_color(const char * section, const char * name,
                guchar & red, guchar & green, guchar & blue);
  int get_stringlist(const char * section, const char * name,
                      GList ** ret);
  
  int read(const char * filename);
  void write(const char * filename);
  
  GVConfig();
  ~GVConfig();
  };

extern GVConfig * gv_config;
