/*****************************************************************

  gvgtk.cc

  Copyright (c) 2001 by Burkhard Plaum - plaum@ipf.uni-stuttgart.de

  http://gmerlin.sourceforge.net

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111, USA.

*****************************************************************/

// TODO: Misc gtk utilities

#include <gtk/gtk.h>

#include <string.h> 
#include "gv.h"
#include "gvgtk.h"
#include "gvconfig.h"

void gvgtk_get_color_config(char * section, char * name, GdkColor & color)
  {
  guchar red, green, blue;
  if(gv_config->get_color(section, name, red, green, blue))
    {
    color.red = red * (65535/255);
    color.green = green * (65535/255);
    color.blue = blue * (65535/255); 
    color.pixel = (gulong)(red*65536 + green*256 + blue);
    }
  }

void gvgtk_put_color_config(char * section, char * name, GdkColor & color)
  {
  guchar red, green, blue;

  red   = (color.red   >> 8);
  green = (color.green >> 8);
  blue  = (color.blue  >> 8);

  gv_config->put_color(section, name, red, green, blue);
  }

GtkWidget * gvgtk_table_new(int rows, int columns, gboolean homogeneous)
  {
  GtkWidget* ret = gtk_table_new(rows, columns, homogeneous);
  gtk_container_set_border_width(GTK_CONTAINER(ret), 10);
  gtk_table_set_row_spacings(GTK_TABLE(ret), 10);
  gtk_table_set_col_spacings(GTK_TABLE(ret), 10);
  return ret;
  }

void gvgtk_table_attach_noresize(GtkWidget * table, GtkWidget * child,
                                 guint left_attach, guint right_attach,
                                 guint top_attach,  guint bottom_attach)
  {
  gtk_table_attach(GTK_TABLE(table), child, left_attach, right_attach,
                   top_attach, bottom_attach, GTK_FILL, GTK_FILL, 0, 0);
  }

void gvgtk_color_copy(GdkColor & dst, GdkColor & src)
  {
  dst.red = src.red;
  dst.green = src.green;
  dst.blue = src.blue;
  dst.pixel = src.pixel;
  }

GVLabel::GVLabel(const char * new_text)
  {
  text = gtk_text_new((GtkAdjustment*)0,
                      (GtkAdjustment*)0);
  
  gtk_text_set_editable(GTK_TEXT(text), FALSE);

  gtk_text_set_line_wrap(GTK_TEXT(text), 1);
  gtk_text_set_word_wrap(GTK_TEXT(text), 1);


  gtk_widget_show(text);

  widget = gtk_scrolled_window_new((GtkAdjustment*)0,
                                   (GtkAdjustment*)0);

  gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(widget),
                                 GTK_POLICY_NEVER,
                                 GTK_POLICY_AUTOMATIC);

  
  gtk_text_set_adjustments(GTK_TEXT(text),
                           gtk_scrolled_window_get_hadjustment(GTK_SCROLLED_WINDOW(widget)),
                           gtk_scrolled_window_get_vadjustment(GTK_SCROLLED_WINDOW(widget)));

  gtk_container_add(GTK_CONTAINER(widget), text);

  gtk_widget_show(widget);
  if(new_text)
    set_text(new_text);

  }

void GVLabel::set_text(const char * new_text)
  {
  int position = 0;

  gtk_editable_delete_text(GTK_EDITABLE(text),
                           0, -1);

  gtk_editable_insert_text(GTK_EDITABLE(text),
                           new_text,
                           strlen(new_text),
                           &position);

  // Now, we shift everything up

  gtk_adjustment_set_value(gtk_scrolled_window_get_vadjustment(GTK_SCROLLED_WINDOW(widget)),
                           0.0);
                           
  
  }
