/*****************************************************************

  gvloader.h

  Copyright (c) 2001 by Burkhard Plaum - plaum@ipf.uni-stuttgart.de

  http://gmerlin.sourceforge.net

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111, USA.

*****************************************************************/

// Loader Class

// Info about a plugin

class GVOutputAudio;
class GVOutput;
class GVInput;
class GVImageReader;

struct GVPluginInfo
  {
  char * name;        // Name of the plugin
  char * real_name;   // Real name (for GUI)
  char * suffixes;    // File suffixes handled by the plugin
  char * filename;    // Name of the shared object
  char * description; // Description
  int flags;
  
  GVPluginInfo(GVBasePlugin * plugin, const char * _filename);
  
  ~GVPluginInfo();

  int check_suffix(const char * suffix);
  };

// Plugin loader
// There exist 4 global plugin loaders
//
// gv_input_file_loader
// gv_input_loader
// gv_video_output_loader
// gv_audio_output_loader

class GVPluginLoader
  {
  GList * plugins; // List of plugin infos

  int num_plugins; // Number of registered plugins

  static GVBasePlugin * load_plugin(const char * filename);
  
public:

  // Get information for filling menus and stuff

  int get_num_plugins() { return num_plugins; }
  const char * get_plugin_name(int index);
  const char * get_plugin_real_name(int index);
  const GVPluginInfo * get_plugin_info(int index);
  
  // Loader functions
  
  static GVInput *       load_input(GVPluginInfo * info);
  static GVOutput *      load_output(GVPluginInfo * info);
  static GVOutputAudio * load_output_audio(GVPluginInfo * info);
  static GVImageReader * load_image_reader(GVPluginInfo * info);
    
  // Destroy plugin

  static void destroy_plugin(GVBasePlugin * plugin);
    
  // Find plugins
  
  GVPluginInfo * find_plugin_by_name(const char * name);
  GVPluginInfo * find_plugin_by_suffix(const char * filename);
  
  GVPluginLoader();

  // Add a plugin to the list, returns true on success
  
  int add_plugin(const char * module_filename);
  
  };

extern GVPluginLoader * gv_input_file_loader;
extern GVPluginLoader * gv_input_loader;
extern GVPluginLoader * gv_video_output_loader;
extern GVPluginLoader * gv_audio_output_loader;
extern GVPluginLoader * gv_image_reader_loader;

