/*****************************************************************

  gvmmx.cc

  Copyright (c) 2001 by Burkhard Plaum - plaum@ipf.uni-stuttgart.de

  http://gmerlin.sourceforge.net

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111, USA.

*****************************************************************/


// Here we define a global variable (set in main()) which is TRUE if
// we have mmx

#include "config.h"

#include <pthread.h>

extern "C" {
#include "glib.h"
#include "mmx/mm_accel.h"
}

#include "gvmmx.h"
#include "gvconfig.h"
#include <iostream>

using std::cerr;

static pthread_mutex_t mmx_mutex;

GVMMX * gv_mmx = (GVMMX *)0;

int GVMMX::mmx_supported = -1;
int GVMMX::use_mmx = 1;

GVMMX::GVMMX()
  {
  if(mmx_supported == -1)
    mmx_supported = mm_accel();
  pthread_mutex_init(&mmx_mutex,(pthread_mutexattr_t *)0);

  gv_config->get_bool("General", "UseMMX", use_mmx);
    
  if(mmx_supported)
    cerr << "Enabling MMX Instructions\n";
  else
    cerr << "No MMX Available\n";
 
  }

GVMMX::~GVMMX()
  {
  gv_config->put_bool("General", "UseMMX", use_mmx);
  }

void GVMMX::set(int _use_mmx)
  {
  pthread_mutex_lock(&mmx_mutex);
  use_mmx = _use_mmx;
  pthread_mutex_unlock(&mmx_mutex);
  }

int GVMMX::get()
  {
  int ret;
  pthread_mutex_lock(&mmx_mutex);
  ret = use_mmx;
  pthread_mutex_unlock(&mmx_mutex);
  return ret;
  }


int GVMMX::supported()  // Is it supported by this machine?
  {
  return mmx_supported;
  }

int GVMMX::do_mmx()
  {
  int ret;
  pthread_mutex_lock(&mmx_mutex);
  ret = !!((use_mmx) && (mmx_supported));
  pthread_mutex_unlock(&mmx_mutex);
  return ret;
  }

extern "C" {

int gv_do_mmx()
  {
  return gv_mmx->do_mmx();
  }
              
}
