/*****************************************************************

  gvwindow.cc

  Copyright (c) 2001 by Burkhard Plaum - plaum@ipf.uni-stuttgart.de

  http://gmerlin.sourceforge.net

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111, USA.

*****************************************************************/

// Gmerlin managed window

#include <iostream>
using std::ostream;
using std::cerr;

#include "config.h"
#include <gtk/gtk.h>
#include "button.h"
#include "gvwindow.h"
#include "skin.h"

#include "gvtime.h"
#include "videoformat.h"
#include "videoconvert.h"

#include "baseplugin.h"
#include "imagereader.h"

#include "gvloader.h"

#include "xmanager.h"

GdkCursor * GVWindow::left_resize_cursor = (GdkCursor*)0;
GdkCursor * GVWindow::right_resize_cursor = (GdkCursor*)0;
GdkCursor * GVWindow::bottom_resize_cursor = (GdkCursor*)0;
GdkCursor * GVWindow::ll_resize_cursor = (GdkCursor*)0;
GdkCursor * GVWindow::lr_resize_cursor = (GdkCursor*)0;
GdkCursor * GVWindow::move_cursor = (GdkCursor*)0;

static int __button_press_callback(GtkWidget * w, GdkEventButton * event,
                                    gpointer data)
  {
  ((GVWindow*)data)->button_press_callback(w, event);
  return TRUE;
  }

static int __button_release_callback(GtkWidget * w, GdkEventButton * event,
                                      gpointer data)
  {
  ((GVWindow*)data)->button_release_callback(w, event);
  return TRUE;
  }

static int __motion_callback(GtkWidget * w, GdkEventMotion * event,
                              gpointer data)
  {
  ((GVWindow*)data)->motion_callback(w, event);
  return TRUE;
  }

static int __expose_callback(GtkWidget * w, GdkEventExpose * event,
                             gpointer data)
  {
  ((GVWindow*)data)->expose_callback(w, event);
  return TRUE;
  }

static void __close_callback(GVButton * button, gpointer data)
  {
  ((GVWindow*)data)->hide();
  }

void GVWindow::expose_callback(GtkWidget * w, GdkEventExpose * event)
  {
  //  gdk_window_clear(w->window);
  if((w == top_drawingarea) && (skin->title_pixmap))
    {
    gint x, y, width, height, depth;
    gdk_window_get_geometry(top_drawingarea->window, &x, &y, &width, &height,
                            &depth);

    gdk_draw_pixmap(top_drawingarea->window, gc, skin->title_pixmap, 0, 0,
                    (width - title_pixmap_width)/2,
                    (height -  title_pixmap_height)/2,
                     title_pixmap_width, title_pixmap_height);
    }
  }


void GVWindow::button_press_callback(GtkWidget * w, GdkEventButton * event)
  {
  int depth;
  int tmp_x, tmp_y;
  
  if((w == top_drawingarea)||
     (w == ur_drawingarea)||
     (w == ul_drawingarea))
    {
    if((event->button == 1) || (event->button == 3))
      {
      grab_pointer(top_drawingarea->window, event->time, move_cursor);
      flags |= GV_WINDOW_MOVE_ACTIVE;
      }
    else
      return;
    }
  else if(w == ll_drawingarea)
    {
    if((event->button == 1) && (flags & GV_WINDOW_IS_RESIZEABLE))
      {
      grab_pointer(ll_drawingarea->window, event->time);
      flags |= GV_WINDOW_RESIZE_LL_ACTIVE;
      }
    else if(event->button == 3)
      {
      grab_pointer(ll_drawingarea->window, event->time, move_cursor);
      flags |= GV_WINDOW_MOVE_ACTIVE;
      }
    else
      return;
    }
  else if(w == lr_drawingarea)
    {
    if((event->button == 1) && (flags & GV_WINDOW_IS_RESIZEABLE))
      {
      grab_pointer(lr_drawingarea->window, event->time);
      flags |= (GV_WINDOW_RESIZE_LR_ACTIVE);
      }
    else if(event->button == 3)
      {
      grab_pointer(lr_drawingarea->window, event->time, move_cursor);
      flags |= (GV_WINDOW_MOVE_ACTIVE);
      }
    else
      return;
    }
  
  else if(w == left_drawingarea)
    {
    if((event->button == 1) && (flags & GV_WINDOW_IS_RESIZEABLE))
      {
      grab_pointer(left_drawingarea->window, event->time);
      flags |= (GV_WINDOW_RESIZE_LEFT_ACTIVE);
      }
    else if(event->button == 3)
      {
      grab_pointer(left_drawingarea->window, event->time, move_cursor);
      flags |= (GV_WINDOW_MOVE_ACTIVE);
      }
    else
      return;
    }
  else if(w == right_drawingarea)
    {
    if((event->button == 1) && (flags & GV_WINDOW_IS_RESIZEABLE))
      {
      grab_pointer(right_drawingarea->window, event->time);
      flags |= (GV_WINDOW_RESIZE_RIGHT_ACTIVE);
      }
    else if(event->button == 3)
      {
      grab_pointer(right_drawingarea->window, event->time, move_cursor);
      flags |= (GV_WINDOW_MOVE_ACTIVE);
      }
    else
      return;
    }
  else if(w == bottom_drawingarea)
    {
    if((event->button == 1) && (flags & GV_WINDOW_IS_RESIZEABLE))
      {
      grab_pointer(bottom_drawingarea->window, event->time);
      flags |= (GV_WINDOW_RESIZE_BOTTOM_ACTIVE);
      }
    else if(event->button == 3)
      {
      grab_pointer(bottom_drawingarea->window, event->time, move_cursor);
      flags |= (GV_WINDOW_MOVE_ACTIVE);
      }
    else
      return;
    }
  
  gdk_window_get_geometry(window->window, &tmp_x, &tmp_y,
                          &win_width_save, &win_height_save, &depth);

  win_width_save -=  (left_offset + right_offset);
  win_height_save -= (top_offset + bottom_offset);
  
  gdk_window_get_root_origin(window->window,
                             &win_x_save,
                             &win_y_save);

  mouse_x_save = (int)(event->x_root);
  mouse_y_save = (int)(event->y_root);
  }

void GVWindow::set_name(const char * name)
  {
  gtk_window_set_title(GTK_WINDOW(window), name);
  }

void GVWindow::grab_pointer(GdkWindow * window, guint32 time,
                            GdkCursor * cursor)
  {
  if(flags & GV_WINDOW_POINTER_GRABBED)
    return;
  gdk_pointer_grab(window, 0,
                   (GdkEventMask)(GDK_BUTTON1_MOTION_MASK|
                                  GDK_BUTTON3_MOTION_MASK|
                                  GDK_BUTTON_RELEASE_MASK),
                   (GdkWindow*)0, cursor, time);
  //  gdk_flush();
  flags |= GV_WINDOW_POINTER_GRABBED;
  }

void GVWindow::ungrab_pointer(guint32 time)
  {
  if(!(flags & GV_WINDOW_POINTER_GRABBED))
    return;
  gdk_pointer_ungrab(time);
  flags &= ~GV_WINDOW_POINTER_GRABBED;
  }

void GVWindow::button_release_callback(GtkWidget * w, GdkEventButton * event)
  {
  ungrab_pointer(event->time);
  flags &= ~GV_WINDOW_ACTIVE_MASK;
  }

void GVWindow::motion_callback(GtkWidget * w, GdkEventMotion * event)
  {
  int mouse_x = (int)(event->x_root);
  int mouse_y = (int)(event->y_root);
  
  int x_diff =  mouse_x - mouse_x_save;
  int y_diff =  mouse_y - mouse_y_save;
  int new_width;
  int new_height;
  
  switch(flags & GV_WINDOW_ACTIVE_MASK)
    {
    case GV_WINDOW_RESIZE_LL_ACTIVE:
      new_width = win_width_save - x_diff;
      new_height = win_height_save + y_diff;

      if(new_height < min_height)
        new_height = min_height;
      if(new_width < min_width)
        new_width = min_width;
      
      before_resize_hook(win_width_save, win_height_save,
                         new_width, new_height);

      // Calculate the new x_diff and y_diff values
      
      x_diff = new_width - win_width_save;
      y_diff = new_height - win_height_save;
      
      win_x_save -= x_diff;
      //      win_y_save -= y_diff;
      win_width_save = new_width;
      win_height_save = new_height;
      gdk_window_move_resize(window->window,
                             win_x_save,
                             win_y_save,
                             win_width_save + left_offset + right_offset,
                             win_height_save + top_offset + bottom_offset);
      
      break;
    case GV_WINDOW_RESIZE_LR_ACTIVE:
      new_width = win_width_save + x_diff;
      new_height = win_height_save + y_diff;

      if(new_height < min_height)
        new_height = min_height;
      if(new_width < min_width)
        new_width = min_width;

      before_resize_hook(win_width_save, win_height_save,
                         new_width, new_height);

      // Calculate the new x_diff and y_diff values
      
      x_diff = new_width - win_width_save;
      y_diff = new_height - win_height_save;
      
      //      win_x_save -= x_diff;
      //      win_y_save -= y_diff;
      win_width_save = new_width;
      win_height_save = new_height;
      gdk_window_move_resize(window->window,
                             win_x_save,
                             win_y_save,
                             win_width_save + left_offset + right_offset,
                             win_height_save + top_offset + bottom_offset);
      break;
    case GV_WINDOW_RESIZE_BOTTOM_ACTIVE:
      new_height = win_height_save + y_diff;
      new_width = win_width_save;

      if(new_height < min_height)
        new_height = min_height;
      if(new_width < min_width)
        new_width = min_width;
      
      before_resize_hook(win_width_save, win_height_save,
                         new_width, new_height);

      // Calculate the new x_diff and y_diff values
      
      x_diff = new_width - win_width_save;
      y_diff = new_height - win_height_save;
      
      //      win_x_save -= x_diff;
      //      win_y_save -= y_diff;
      win_width_save = new_width;
      win_height_save = new_height;
      gdk_window_move_resize(window->window,
                             win_x_save,
                             win_y_save,
                             win_width_save + left_offset + right_offset,
                             win_height_save + top_offset + bottom_offset);
      break;
    case GV_WINDOW_RESIZE_LEFT_ACTIVE:
      new_width = win_width_save - x_diff;
      new_height = win_height_save;

      if(new_height < min_height)
        new_height = min_height;
      if(new_width < min_width)
        new_width = min_width;

      before_resize_hook(win_width_save, win_height_save,
                         new_width, new_height);

      // Calculate the new x_diff and y_diff values
      
      x_diff = new_width - win_width_save;
      y_diff = new_height - win_height_save;
      
      win_x_save -= x_diff;
      //      win_y_save -= y_diff;
      win_width_save = new_width;
      win_height_save = new_height;
      gdk_window_move_resize(window->window,
                             win_x_save,
                             win_y_save,
                             win_width_save + left_offset + right_offset,
                             win_height_save + top_offset + bottom_offset);


      break;
    case GV_WINDOW_RESIZE_RIGHT_ACTIVE:
      new_width = win_width_save + x_diff;
      new_height = win_height_save;

      if(new_height < min_height)
        new_height = min_height;
      if(new_width < min_width)
        new_width = min_width;

      before_resize_hook(win_width_save, win_height_save,
                         new_width, new_height);

      // Calculate the new x_diff and y_diff values
      
      x_diff = new_width - win_width_save;
      y_diff = new_height - win_height_save;
      
      //      win_x_save -= x_diff;
      //      win_y_save -= y_diff;
      win_width_save = new_width;
      win_height_save = new_height;
      gdk_window_move_resize(window->window,
                             win_x_save,
                             win_y_save,
                             win_width_save + left_offset + right_offset,
                             win_height_save + top_offset + bottom_offset);


      break;
    case GV_WINDOW_MOVE_ACTIVE:
      gdk_window_move(window->window,
                      win_x_save + x_diff, win_y_save + y_diff);
      win_x_save += x_diff;
      win_y_save += y_diff;
      
      break;
    }
  mouse_x_save = mouse_x;
  mouse_y_save = mouse_y;
    
  }

void GVWindow::set_drawingarea_attributes(GtkWidget * drawingarea)
  {
  gtk_widget_set_events(drawingarea,
                        GDK_EXPOSURE_MASK|
                        GDK_BUTTON1_MOTION_MASK|
                        GDK_BUTTON_PRESS_MASK|
                        GDK_BUTTON_RELEASE_MASK);

  gtk_signal_connect(GTK_OBJECT(drawingarea), "expose_event",
                     GTK_SIGNAL_FUNC(__expose_callback), (gpointer)this);
  gtk_signal_connect_after(GTK_OBJECT(drawingarea), "button_press_event",
                           GTK_SIGNAL_FUNC(__button_press_callback),
                           (gpointer)this);
  gtk_signal_connect_after(GTK_OBJECT(drawingarea), "button_release_event",
                           GTK_SIGNAL_FUNC(__button_release_callback),
                           (gpointer)this);
  gtk_signal_connect_after(GTK_OBJECT(drawingarea), "motion_notify_event",
                           GTK_SIGNAL_FUNC(__motion_callback),
                           (gpointer)this);
  }

GVWindow::GVWindow()
  {
  top_offset = 0;
  bottom_offset = 0;
  left_offset = 0;
  right_offset = 0;

  min_width = 0;
  min_height = 0;

  flags = 0;
  // Create the objects

  window = gtk_window_new(GTK_WINDOW_TOPLEVEL);

  default_skin = new GVWindowSkin;
  skin = default_skin;
  
  table = gtk_table_new(3, 3, 0);
  gtk_table_set_col_spacings(GTK_TABLE(table), 0);
  gtk_table_set_row_spacings(GTK_TABLE(table), 0);
  
  left_drawingarea = gtk_drawing_area_new();
  right_drawingarea = gtk_drawing_area_new();
  top_drawingarea = gtk_drawing_area_new();
  bottom_drawingarea = gtk_drawing_area_new();
  
  ll_drawingarea = gtk_drawing_area_new();
  ul_drawingarea = gtk_drawing_area_new();
  lr_drawingarea = gtk_drawing_area_new();
  ur_drawingarea = gtk_drawing_area_new();

  // Create the buttons and layout

  close_layout = gtk_layout_new((GtkAdjustment*)0,(GtkAdjustment*)0);
  menu_layout  = gtk_layout_new((GtkAdjustment*)0,(GtkAdjustment*)0);
    
  // Set attributes & Callbacks

  set_drawingarea_attributes(left_drawingarea);
  set_drawingarea_attributes(right_drawingarea);
  set_drawingarea_attributes(top_drawingarea);
  set_drawingarea_attributes(bottom_drawingarea);
  set_drawingarea_attributes(ll_drawingarea);
  set_drawingarea_attributes(lr_drawingarea);
  set_drawingarea_attributes(ul_drawingarea);
  set_drawingarea_attributes(ur_drawingarea);

  // Show the objects

  gtk_widget_show(left_drawingarea);
  gtk_widget_show(right_drawingarea);
  gtk_widget_show(top_drawingarea);
  gtk_widget_show(bottom_drawingarea);

  gtk_widget_show(ul_drawingarea);
  gtk_widget_show(ur_drawingarea);
  gtk_widget_show(ll_drawingarea);
  gtk_widget_show(lr_drawingarea);

  gtk_widget_show(close_layout);
  gtk_widget_show(menu_layout);
  
  gtk_layout_put(GTK_LAYOUT(close_layout), ur_drawingarea, 0, 0);
  gtk_layout_put(GTK_LAYOUT(menu_layout), ul_drawingarea, 0, 0);

  menu_button = new GVButton(menu_layout);
  close_button = new GVButton(close_layout);

  close_button->press_callback = __close_callback;
  close_button->press_callback_data = (gpointer)this;
  
  gtk_container_set_resize_mode(GTK_CONTAINER(table), GTK_RESIZE_IMMEDIATE);

  // Place the objects

  GtkWidget * hbox_bottom = gtk_hbox_new(FALSE, 0);
  GtkWidget * hbox_top    = gtk_hbox_new(FALSE, 0);

  gtk_box_pack_start(GTK_BOX(hbox_bottom), ll_drawingarea, FALSE, FALSE, 0);
  gtk_box_pack_start_defaults(GTK_BOX(hbox_bottom),
                              bottom_drawingarea);
 
  gtk_box_pack_start(GTK_BOX(hbox_bottom), lr_drawingarea, FALSE, FALSE, 0);
  gtk_widget_show(hbox_bottom);

  gtk_table_attach(GTK_TABLE(table), hbox_bottom, 0, 3, 2, 3,
                   GTK_FILL, GTK_FILL, 0, 0);

  
  gtk_box_pack_start(GTK_BOX(hbox_top), menu_layout, FALSE, FALSE, 0);
  gtk_box_pack_start_defaults(GTK_BOX(hbox_top),
                              top_drawingarea);
 
  gtk_box_pack_start(GTK_BOX(hbox_top), close_layout, FALSE, FALSE, 0);
  gtk_widget_show(hbox_top);

  gtk_table_attach(GTK_TABLE(table), hbox_top, 0, 3, 0, 1,
                   GTK_FILL, GTK_FILL, 0, 0);

  
//   gtk_table_attach(GTK_TABLE(table), ll_drawingarea, 0, 2, 2, 3, 
//                     GtkAttachOptions(GTK_SHRINK), GTK_FILL, 0, 0);
//   gtk_table_attach(GTK_TABLE(table), ul_drawingarea, 0, 2, 0, 1,
//                     GTK_FILL, GTK_FILL, 0, 0);
//   gtk_table_attach(GTK_TABLE(table), lr_drawingarea, 3, 5, 2, 3,
//                     GTK_FILL, GTK_FILL, 0, 0);
//   gtk_table_attach(GTK_TABLE(table), ur_drawingarea, 3, 5, 0, 1,
//                     GTK_FILL, GTK_FILL, 0, 0);

//   gtk_table_attach(GTK_TABLE(table), left_drawingarea, 0, 1, 1, 2,
//                    GTK_FILL, GTK_FILL, 0, 0);
//   gtk_table_attach(GTK_TABLE(table), right_drawingarea, 2, 3, 1, 2,
//                    GTK_FILL, GTK_FILL, 0, 0);

  gtk_table_attach(GTK_TABLE(table), left_drawingarea, 0, 1, 1, 2,
                   GTK_FILL, GTK_FILL, 0, 0);
  gtk_table_attach(GTK_TABLE(table), right_drawingarea, 2, 3, 1, 2,
                   GTK_FILL, GTK_FILL, 0, 0);
  
//   gtk_table_attach(GTK_TABLE(table), top_drawingarea, 2, 3, 0, 1,
//                     GTK_FILL, GTK_FILL, 0, 0);

//   gtk_table_attach(GTK_TABLE(table), bottom_drawingarea, 2, 3, 2, 3,
//                    GtkAttachOptions(GTK_FILL|GTK_EXPAND), GTK_FILL, 0, 0);

  
  //  gtk_table_attach_defaults(GTK_TABLE(table), bottom_drawingarea, 2, 3, 2, 3);
    
  gtk_container_add(GTK_CONTAINER(window), table);

  gtk_widget_realize(window);
  gdk_window_set_decorations(window->window, (GdkWMDecoration)0);
  gdk_window_set_functions(window->window, (GdkWMFunction)0);

  gdk_window_set_icon(window->window, 
                      (GdkWindow*)0,
                      gv_x_manager->gmerlin_icon,
                      gv_x_manager->gmerlin_icon_mask);
  }

void GVWindow::attach(GtkWidget * child)
  {
  gtk_table_attach_defaults(GTK_TABLE(table), child, 1, 2, 1, 2);
  }

void GVWindow::realize()
  {
  gtk_widget_show(table);
  
  gtk_widget_realize(left_drawingarea);
  gtk_widget_realize(right_drawingarea);
  gtk_widget_realize(top_drawingarea);
  gtk_widget_realize(bottom_drawingarea);
  gtk_widget_realize(ll_drawingarea);
  gtk_widget_realize(lr_drawingarea);
  gtk_widget_realize(ul_drawingarea);
  gtk_widget_realize(ur_drawingarea);

  // FIXME: Find some nicer cursors
  
  if(flags & GV_WINDOW_IS_RESIZEABLE)
    {
    // Create the cursors

    if(!left_resize_cursor)
      left_resize_cursor = gdk_cursor_new(GDK_LEFT_SIDE);
    if(!right_resize_cursor)
      right_resize_cursor = gdk_cursor_new(GDK_RIGHT_SIDE);
    if(!bottom_resize_cursor)
      bottom_resize_cursor = gdk_cursor_new(GDK_BOTTOM_SIDE);
    if(!ll_resize_cursor)
      ll_resize_cursor = gdk_cursor_new(GDK_BOTTOM_LEFT_CORNER);
    if(!lr_resize_cursor)
      lr_resize_cursor = gdk_cursor_new(GDK_BOTTOM_RIGHT_CORNER);
    if(!move_cursor)
      move_cursor = gdk_cursor_new(GDK_FLEUR);
    
    gdk_window_set_cursor(left_drawingarea->window, left_resize_cursor);
    gdk_window_set_cursor(right_drawingarea->window, right_resize_cursor);
    gdk_window_set_cursor(bottom_drawingarea->window, bottom_resize_cursor);
    gdk_window_set_cursor(ll_drawingarea->window, ll_resize_cursor);
    gdk_window_set_cursor(lr_drawingarea->window, lr_resize_cursor);
    }
  gc = gdk_gc_new(window->window);
  }


void GVWindow::set_size(int width, int height)
  {
  gdk_window_resize(window->window, width+left_offset+right_offset,
                    height+top_offset+bottom_offset);

  }

void GVWindow::show()
  {
  if(!(flags & GV_WINDOW_IS_VISIBLE))
    {
    gtk_widget_show(window);
    gdk_window_raise(window->window);
    flags |= GV_WINDOW_IS_VISIBLE;
    }
  else
    gdk_window_raise(window->window);
  }

void GVWindow::hide()
  {
  if(flags & GV_WINDOW_IS_VISIBLE)
    {
    gtk_widget_hide(window);
    flags &= ~GV_WINDOW_IS_VISIBLE;
    }
  }

void GVWindow::set_resizeable(int resizeable)
  {
  if(resizeable)
    flags |= GV_WINDOW_IS_RESIZEABLE;
  else
    flags &= ~GV_WINDOW_IS_RESIZEABLE;
    
  }

void GVWindow::before_resize_hook(int old_width, int old_height,
                                  int & new_width, int & new_height)
  {
  
  }

void GVWindow::after_resize_hook(int old_width, int old_height,
                                 int new_width, int new_height)
  {
  
  }

GVWindow::~GVWindow()
  {
  }

void GVWindow::set_pixmap_edge(GtkWidget * drawingarea, GdkPixmap * pixmap,
                               int & width, int & height)
  {
  int x, y, depth;
  gdk_window_get_geometry(pixmap, &x, &y, &width, &height, &depth);

  if(drawingarea)
    {
    gtk_drawing_area_size(GTK_DRAWING_AREA(drawingarea), width, height);
    gdk_window_set_back_pixmap(drawingarea->window, pixmap, 0);
    }
  }

void GVWindow::set_pixmap_corner(GtkWidget * drawingarea, GdkPixmap * pixmap,
                                 GdkBitmap * mask,
                                 int & width, int & height)
  {
  int x, y, depth;
  gdk_window_get_geometry(pixmap, &x, &y, &width, &height, &depth);

  if(drawingarea)
    {
    gtk_drawing_area_size(GTK_DRAWING_AREA(drawingarea), width, height);
    gtk_widget_set_usize(drawingarea, width, height);
    gdk_window_set_back_pixmap(drawingarea->window, pixmap, 0);
//  if(mask)
//   //      gdk_window_shape_combine_mask(drawingarea->window, mask, 0, 0);
//    gtk_widget_shape_combine_mask(drawingarea, mask, 0, 0);
    }
  }


void GVWindow::apply_skin(const GVWindowSkin * _skin)
  {
  int tmp_width, tmp_height, tmp_x, tmp_y, tmp_depth;

  if(_skin)
    skin = _skin;
  
  set_pixmap_edge(top_drawingarea, skin->top_pixmap, tmp_width, tmp_height);
  
  top_offset = tmp_height;
  
  set_pixmap_edge(bottom_drawingarea, skin->bottom_pixmap,
                  tmp_width, tmp_height);

  bottom_offset = tmp_height;
  min_height = top_offset + bottom_offset;

  set_pixmap_edge(left_drawingarea, skin->left_pixmap,
                  tmp_width, tmp_height);
  left_offset = tmp_width;

  set_pixmap_edge(right_drawingarea, skin->right_pixmap,
                  tmp_width, tmp_height);
  right_offset = tmp_width;

  min_width = left_offset + right_offset;

  set_pixmap_corner(ll_drawingarea, skin->ll_pixmap, skin->ll_mask,
                    tmp_width, tmp_height);
  
  set_pixmap_corner(lr_drawingarea, skin->lr_pixmap, skin->lr_mask,
                    tmp_width, tmp_height);
  
  set_pixmap_corner(ul_drawingarea, skin->ul_pixmap, skin->ul_mask,
                    tmp_width, tmp_height);
  gtk_layout_set_size(GTK_LAYOUT(menu_layout), tmp_width, tmp_height);

  gtk_widget_set_usize(menu_layout, tmp_width, tmp_height);


  set_pixmap_corner(ur_drawingarea, skin->ur_pixmap, skin->ur_mask,
                    tmp_width, tmp_height);
  gtk_layout_set_size(GTK_LAYOUT(close_layout), tmp_width, tmp_height);
  gtk_widget_set_usize(close_layout, tmp_width, tmp_height);

  gdk_window_shape_combine_mask(menu_layout->window, skin->ul_mask, 0, 0);
  gdk_window_shape_combine_mask(close_layout->window, skin->ur_mask, 0, 0);

  gdk_window_get_geometry(skin->title_pixmap, &tmp_x, &tmp_y,
                          &title_pixmap_width, &title_pixmap_height,
                          &tmp_depth);
  close_button->set_skin(&(skin->close_button));
  menu_button->set_skin(&(skin->menu_button));

  // Set the shape of the window

  set_shape();  
  }

void GVWindow::set_shape()
  {
  
  }

void GVWindow::set_ul_menu(GtkWidget * menu)
  {
  menu_button->menu = menu;
  };
