/*****************************************************************

  gvwindow.h

  Copyright (c) 2001 by Burkhard Plaum - plaum@ipf.uni-stuttgart.de

  http://gmerlin.sourceforge.net

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111, USA.

*****************************************************************/

// Class for Gmerlin amaged windows

#define GV_WINDOW_IS_RESIZEABLE        (1<<0)
#define GV_WINDOW_IS_VISIBLE           (1<<1)

#define GV_WINDOW_RESIZE_LL_ACTIVE     (1<<2)
#define GV_WINDOW_RESIZE_LR_ACTIVE     (1<<3)
#define GV_WINDOW_RESIZE_LEFT_ACTIVE   (1<<4)
#define GV_WINDOW_RESIZE_RIGHT_ACTIVE  (1<<5)
#define GV_WINDOW_RESIZE_BOTTOM_ACTIVE (1<<6)
#define GV_WINDOW_MOVE_ACTIVE          (1<<7)
#define GV_WINDOW_POINTER_GRABBED      (1<<8)

#define GV_WINDOW_ACTIVE_MASK (GV_WINDOW_RESIZE_LL_ACTIVE|\
                               GV_WINDOW_RESIZE_LR_ACTIVE|\
                               GV_WINDOW_RESIZE_LEFT_ACTIVE|\
                               GV_WINDOW_RESIZE_RIGHT_ACTIVE|\
                               GV_WINDOW_RESIZE_BOTTOM_ACTIVE|\
                               GV_WINDOW_MOVE_ACTIVE)


class GVWindowSkin;

class GVWindow
  {
protected:
  GtkWidget * window;
private:

  const GVWindowSkin * skin;
  GVWindowSkin * default_skin;
  
  //  GdkPixmap * title_pixmap_h;
  
  GtkWidget * table;
  
  GtkWidget * left_drawingarea;
  GtkWidget * right_drawingarea;
  GtkWidget * top_drawingarea;
  GtkWidget * bottom_drawingarea;
  GtkWidget * title_drawingarea;
  
  GtkWidget * ll_drawingarea;
  GtkWidget * ul_drawingarea;
  GtkWidget * lr_drawingarea;
  GtkWidget * ur_drawingarea;

  GtkWidget * menu_layout;
  GtkWidget * close_layout;

  GVButton * menu_button;
  GVButton * close_button;
    
  int title_pixmap_width;
  int title_pixmap_height;
    
  int mouse_x_save, mouse_y_save;
  int win_x_save;
  int win_y_save;
  int win_width_save;
  int win_height_save;

  int top_offset;
  int bottom_offset;
  int left_offset;
  int right_offset;

  int min_width;
  int min_height;
  
  void set_drawingarea_attributes(GtkWidget * drawingarea);
    
protected:

  int flags;
  
  static GdkCursor * left_resize_cursor;
  static GdkCursor * right_resize_cursor;
  static GdkCursor * bottom_resize_cursor;
  static GdkCursor * ll_resize_cursor;
  static GdkCursor * lr_resize_cursor;
  static GdkCursor * move_cursor;

  virtual void before_resize_hook(int old_width, int old_height,
                                  int & new_width, int & new_height);

  virtual void after_resize_hook(int old_width, int old_height,
                                 int new_width, int new_height);

  void grab_pointer(GdkWindow * window, guint32 time,
                    GdkCursor * cursor = (GdkCursor*)0);
  
  void ungrab_pointer(guint32 time);

  GdkGC * gc;
  
public:

  void set_shape();
  
  void set_ul_menu(GtkWidget * menu);
  
  void set_size(int width, int height);

  void set_name(const char * name);
    
  int is_visible() { return flags & GV_WINDOW_IS_VISIBLE; }
  
  void set_resizeable(int resizeable);
  
  
  void attach(GtkWidget * child);
  
  void realize();
  

  virtual void show();
  virtual void hide();
  
  GVWindow();

  void button_press_callback(GtkWidget * w, GdkEventButton * event);
  void button_release_callback(GtkWidget * w, GdkEventButton * event);
  void motion_callback(GtkWidget * w, GdkEventMotion * event);
  void expose_callback(GtkWidget * w, GdkEventExpose * event);
  
  virtual ~GVWindow();

  void apply_skin(const GVWindowSkin * _skin);

  void set_pixmap_edge(GtkWidget * drawingarea, GdkPixmap * pixmap,
                       int & width, int & height);
  
  void set_pixmap_corner(GtkWidget * drawingarea, GdkPixmap * pixmap,
                         GdkBitmap * mask,
                         int & width, int & height);
  
  };
