/*****************************************************************

  imagereader.h

  Copyright (c) 2001 by Burkhard Plaum - plaum@ipf.uni-stuttgart.de

  http://gmerlin.sourceforge.net

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111, USA.

*****************************************************************/

#ifdef __GNUG__
#pragma interface
#endif

// Base class for image reader

class GVImageReader : public GVBasePlugin
  {
protected:
  
  GVVideoFormat video_format;
  GVColorspace best_colorspace;
  
public:

  GVImageReader(const GVPluginAttributes * _attributes);
  
  const GVVideoFormat * get_video_format() { return &video_format; }
  
  // Read the file header, set all information, return 0 on error,
  // 1 else
  // 
  // If the "first" argument is 0, omit stuff which doesn't have to be done
  // if the image is in a series read by a singlepicture plugin.
  
  virtual int init(const char * filename, int first = 1) = 0;
  
  // Return the best colorspace

  GVColorspace get_best_colorspace() { return best_colorspace; }

  // Return true if colorspace is supported

  virtual int colorspace_supported(GVColorspace colorspace) = 0;
  
  // Load image into the video frame structure. The frame must be
  // allocated with the proper size by the caller!!

  virtual void load_image(GVVideoFrame * frame) = 0;

  // Cleanup all allocated resources. If the argument "last" is 0,
  // don't clean resources, which can be reused by series of pictures
  // in singlepicture plugins
  
  virtual void cleanup(int last = 1) = 0;

  virtual ~GVImageReader();
 
  };
