/*****************************************************************

  infowindow.cc

  Copyright (c) 2001 by Burkhard Plaum - plaum@ipf.uni-stuttgart.de

  http://gmerlin.sourceforge.net

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111, USA.

*****************************************************************/

// Gmerlins Info window

#include "config.h"

#include <glib.h>
#include <pthread.h>
#include <sstream>
using std::stringstream;

#include <iostream>
#include "gvtime.h"
#include "gvconfig.h"
#include "videoformat.h"
#include "audioformat.h"
#include "buffer.h"
#include "audiobuf.h"
#include "videobuf.h"

#include "playlist.h"

#include "baseplugin.h"
#include "plugin.h"
#include "input.h"

#include "output.h"
#include "audioout.h"
#include "videoout.h"

#include <gtk/gtk.h>
#include "configwindow.h"
#include "engine.h"

#include "gvloader.h"

#include <gtkutil.h>
#include "infowindow.h"

using std::ends;

GVInfoWindow * gv_info_window  = (GVInfoWindow*)0;


// Methods for the info window

static void __button_callback(GtkWidget * w, gpointer data)
  {
  ((GVInfoWindow*)data)->hide();
  }

static gboolean __delete_callback(GtkWidget * w, GdkEventAny * event,
                                  gpointer data)
  {
  ((GVInfoWindow*)data)->hide();
  return TRUE;
  }

static void __select_row_callback(GtkWidget * w, int row, int column,
                                      GdkEvent * event,
                                      gpointer data)
  {
  ((GVInfoWindow*)data)->select_row_callback(row);
  return;
  }

void GVInfoWindow::select_row_callback(int row)
  {
  stringstream line;
  //  line.freeze(0);
  line.seekp(0);
  line << plugin_infos[row]->real_name << " ("
       << plugin_infos[row]->name << ")\n\n"
       << plugin_infos[row]->description << "\n";
  if(plugin_infos[row]->flags & GV_PLUGIN_IS_FILE)
    line << "\nSuffixes: " << plugin_infos[row]->suffixes;
  line << ends;
  plugins_description->set_text(line.str().c_str());
  }

void GVInfoWindow::show()
  {
  if(visible)
    return;
  gtk_widget_show(window);
  visible = 1;
  }

void GVInfoWindow::hide()
  {
  if(!visible)
    return;
  gtk_widget_hide(window);
  visible = 0;
  }

void GVInfoWindow::update()
  {
  gv_engine->lock_info_strings();
  track_info->set_text(gv_engine->i_track_info);
  input_audio_format->set_text(gv_engine->i_input_audio_format);
  input_video_format->set_text(gv_engine->i_input_video_format);
  output_audio_format->set_text(gv_engine->i_output_audio_format);
  output_video_format->set_text(gv_engine->i_output_video_format);

  input_info->set_text(gv_engine->i_input_info);

  output_audio_info->set_text(gv_engine->i_output_audio_info);
  output_video_info->set_text(gv_engine->i_output_video_info);
  gv_engine->unlock_info_strings();
  }

void GVInfoWindow::update_plugins()
  {
  if(plugin_infos)
    delete plugin_infos;

  int num_input_file_plugins = gv_input_file_loader->get_num_plugins();
  int num_input_plugins = gv_input_loader->get_num_plugins();
  int num_video_output_plugins = gv_video_output_loader->get_num_plugins();
  int num_audio_output_plugins = gv_audio_output_loader->get_num_plugins();
  int num_image_reader_plugins = gv_image_reader_loader->get_num_plugins();
  
  num_plugins = num_input_file_plugins + num_input_plugins +
    num_video_output_plugins + num_audio_output_plugins +
    num_image_reader_plugins;
  
  plugin_infos = new const GVPluginInfo*[num_plugins];

  gtk_clist_clear(GTK_CLIST(plugins_list));
  gtk_clist_freeze(GTK_CLIST(plugins_list));

  int index = 0;

  char * c_tmp[1];
  
  for(int i = 0; i < num_input_file_plugins; i++)
    {
    plugin_infos[index] = gv_input_file_loader->get_plugin_info(i);
    c_tmp[0] = plugin_infos[index]->real_name;
    gtk_clist_append(GTK_CLIST(plugins_list), c_tmp);
    index++;
    }

  for(int i = 0; i < num_input_plugins; i++)
    {
    plugin_infos[index] = gv_input_loader->get_plugin_info(i);
    c_tmp[0] = plugin_infos[index]->real_name;
    gtk_clist_append(GTK_CLIST(plugins_list), c_tmp);
    index++;
    }

  for(int i = 0; i < num_audio_output_plugins; i++)
    {
    plugin_infos[index] = gv_audio_output_loader->get_plugin_info(i);
    c_tmp[0] = plugin_infos[index]->real_name;
    gtk_clist_append(GTK_CLIST(plugins_list), c_tmp);
    index++;
    }

  for(int i = 0; i < num_video_output_plugins; i++)
    {
    plugin_infos[index] = gv_video_output_loader->get_plugin_info(i);
    c_tmp[0] = plugin_infos[index]->real_name;
    gtk_clist_append(GTK_CLIST(plugins_list), c_tmp);
    index++;
    }

  for(int i = 0; i < num_image_reader_plugins; i++)
    {
    plugin_infos[index] = gv_image_reader_loader->get_plugin_info(i);
    c_tmp[0] = plugin_infos[index]->real_name;
    gtk_clist_append(GTK_CLIST(plugins_list), c_tmp);
    index++;
    }
  gtk_clist_thaw(GTK_CLIST(plugins_list));
  }

static char * list_title = "Installed plugins";

GVInfoWindow::GVInfoWindow()
  {
  plugin_infos = (const GVPluginInfo**)0; 
  
  // Create the objects
  
  window = gtk_window_new(GTK_WINDOW_TOPLEVEL);

  gtk_window_set_title(GTK_WINDOW(window), "Gmerlin Info");
  
  notebook = gtk_notebook_new();

  input_audio_format = new GVLabel("No Audio");
  input_video_format = new GVLabel("No Video");;

  output_audio_format = new GVLabel("No Audio");
  output_video_format = new GVLabel("No Video");

  input_info = new GVLabel("No Info available");
  output_audio_info = new GVLabel("No Info available");
  output_video_info = new GVLabel("No Info available");

  track_info = new GVLabel("No Info available");
  
  plugins_list = gtk_clist_new_with_titles(1, &list_title);
  plugins_description = new GVLabel;

  gtk_widget_set_usize(plugins_list, plugins_list->requisition.width, 150);
  
  close_button = gtk_button_new_with_label("Close");

  // Set attributes

  gtk_clist_column_titles_passive(GTK_CLIST(plugins_list));
  
  // Set callbacks

  gtk_signal_connect(GTK_OBJECT(close_button),
                     "clicked", GTK_SIGNAL_FUNC(__button_callback),
                     (gpointer)this);

  gtk_signal_connect(GTK_OBJECT(window), "delete_event",
                     GTK_SIGNAL_FUNC(__delete_callback),
                     (gpointer)this);

  gtk_signal_connect(GTK_OBJECT(plugins_list), "select_row",
                     GTK_SIGNAL_FUNC(__select_row_callback),
                     (gpointer)this);
    
  // Show the objects
  
  gtk_widget_show(plugins_list);
  //  gtk_widget_show(plugins_description);
  
  gtk_widget_show(close_button);
  
  // Pack the objects

  GtkWidget * hbox;
  GtkWidget * frame;
  GtkWidget * tab_label;

  tab_label = gtk_label_new("Track Info");
  gtk_widget_show(tab_label);
  hbox = gtk_hbox_new(1, 5);
  gtk_box_pack_start_defaults(GTK_BOX(hbox), track_info->widget);
  gtk_widget_show(hbox);
  gtk_notebook_append_page(GTK_NOTEBOOK(notebook), hbox, tab_label);
  
  hbox = gtk_hbox_new(1, 5);
  tab_label = gtk_label_new("Audio Format");
  gtk_widget_show(tab_label);
  
  frame = gtk_frame_new("Input");
  gtk_container_add(GTK_CONTAINER(frame), input_audio_format->widget);
  gtk_widget_show(frame);
  gtk_box_pack_start_defaults(GTK_BOX(hbox), frame);

  frame = gtk_frame_new("Output");
  gtk_container_add(GTK_CONTAINER(frame), output_audio_format->widget);
  gtk_widget_show(frame);
  gtk_box_pack_start_defaults(GTK_BOX(hbox), frame);

  gtk_widget_show(hbox);
  gtk_notebook_append_page(GTK_NOTEBOOK(notebook), hbox, tab_label);

  tab_label = gtk_label_new("Video Format");
  gtk_widget_show(tab_label);
  hbox = gtk_hbox_new(1, 5);
  
  frame = gtk_frame_new("Input");
  gtk_container_add(GTK_CONTAINER(frame), input_video_format->widget);
  gtk_widget_show(frame);
  gtk_box_pack_start_defaults(GTK_BOX(hbox), frame);

  frame = gtk_frame_new("Output");
  gtk_container_add(GTK_CONTAINER(frame), output_video_format->widget);
  gtk_widget_show(frame);
  gtk_box_pack_start_defaults(GTK_BOX(hbox), frame);

  gtk_widget_show(hbox);
  gtk_notebook_append_page(GTK_NOTEBOOK(notebook), hbox, tab_label);

  tab_label = gtk_label_new("Input");
  gtk_widget_show(tab_label);
  hbox = gtk_hbox_new(1, 5);
  gtk_box_pack_start_defaults(GTK_BOX(hbox), input_info->widget);
  gtk_widget_show(hbox);
  gtk_notebook_append_page(GTK_NOTEBOOK(notebook), hbox, tab_label);

  tab_label = gtk_label_new("Audio Output");
  gtk_widget_show(tab_label);
  hbox = gtk_hbox_new(1, 5);
  gtk_box_pack_start_defaults(GTK_BOX(hbox), output_audio_info->widget);
  gtk_widget_show(hbox);
  gtk_notebook_append_page(GTK_NOTEBOOK(notebook), hbox, tab_label);

  tab_label = gtk_label_new("Video Output");
  gtk_widget_show(tab_label);
  hbox = gtk_hbox_new(1, 5);
  gtk_box_pack_start_defaults(GTK_BOX(hbox), output_video_info->widget);
  gtk_widget_show(hbox);
  gtk_notebook_append_page(GTK_NOTEBOOK(notebook), hbox, tab_label);

  tab_label = gtk_label_new("Plugins");
  gtk_widget_show(tab_label);
  hbox = gtk_hbox_new(1, 5);

  GtkWidget * scrolledwindow = gtk_scrolled_window_new((GtkAdjustment*)0,
                                                       (GtkAdjustment*)0);
  gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrolledwindow),
                                 GTK_POLICY_NEVER,
                                 GTK_POLICY_ALWAYS);
  gtk_container_add(GTK_CONTAINER(scrolledwindow), plugins_list);
  gtk_widget_show(scrolledwindow);
    
  gtk_box_pack_start_defaults(GTK_BOX(hbox), scrolledwindow);

  frame = gtk_frame_new("Description");
  gtk_container_add(GTK_CONTAINER(frame), plugins_description->widget);
  gtk_widget_show(frame);
  
  gtk_box_pack_start_defaults(GTK_BOX(hbox), frame);
  gtk_widget_show(hbox);
  gtk_notebook_append_page(GTK_NOTEBOOK(notebook), hbox, tab_label);
  
  gtk_widget_show(notebook);
  
  GtkWidget * mainbox = gtk_vbox_new(0, 5);
  
  gtk_box_pack_start_defaults(GTK_BOX(mainbox), notebook);

  gtk_box_pack_start(GTK_BOX(mainbox), close_button, FALSE, FALSE, 0);
  gtk_widget_show(mainbox);
  gtk_container_add(GTK_CONTAINER(window), mainbox);
  visible = 0;
  update_plugins();
  }

