/*****************************************************************

  input.h

  Copyright (c) 2001 by Burkhard Plaum - plaum@ipf.uni-stuttgart.de

  http://gmerlin.sourceforge.net

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111, USA.

*****************************************************************/

// Base class for all input plugins

class GVTrackInfo;

class GVInput : public GVPlugin
  {
protected:

  int num_video_tracks;    // Number of video tracks in the actual file
  int num_audio_tracks;    // Number of audio tracks in the actual file
  
  virtual void fill_audio_buf(GVAudioFrame * frame);
  virtual void fill_video_buf(GVVideoFrame * frame);

  GVTime total_time;

//   GVVideoFormat video_format;
//   GVAudioFormat audio_format;

  int video_frames_written; // Number of the last frame written to the buffer
  guint64 audio_samples_written;
  
  GVInput(const GVPluginAttributes * _attributes);

  // By default GVColorspaceRGB24, change to the colorspace your plugin
  // likes most
  
  GVColorspace best_colorspace;  

  virtual void seek(float percentage);

  // This skippes video frames on input level
  // Has to implemented in derived classes
  
  virtual int skip(int num_frames);
  
  guint frames_to_skip;
  guint skipped_frames;
  
  pthread_mutex_t skip_mutex;
  
  void get_devices_config(const char * default_device);
  void put_devices_config();

  void get_recording_config();
  void put_recording_config();

  // URL Support

  // Check if the current filename is an URL
  
  int is_url();

  void parse_url(char ** host, char ** directory, int & port);
  
public:

  virtual void set_trackname();

  // Record time (undefined for unlimited)
  
  GVTime record_time;
  
  GList * devices;
  const char * device;
  int device_index;    // Index in the device list
  
  virtual char * get_device_string(char * devicefile);

  const char * get_devicename() { return device; }
    
  void add_device(const char * _device);
  void set_device(const char * _device);
  void remove_device(const char * _device);
  
  // Skip a number of video frames
  
  void skip_video_frames(int num_frames);
  
  void preload(); // Load audio and video frames until the buffers are full
    
  const GVAudioFormat * get_audio_format() { return &audio_format; }
  const GVVideoFormat * get_video_format() { return &video_format; }

  virtual int colorspace_supported(GVColorspace colorspace);
  virtual void set_colorspace(GVColorspace colorspace);
 
  GVColorspace get_best_colorspace() { return best_colorspace; }
    
  void get_total_time(GVTime & time)
    {
    time = total_time;
    }
  
  virtual void playback_func(); // Function called in the thread

  int track_has_video() { return !!(flags & GV_PLUGIN_TRACK_HAS_VIDEO); }
  int track_has_audio() { return !!(flags & GV_PLUGIN_TRACK_HAS_AUDIO); }
  int track_is_stream() { return !!(flags & GV_PLUGIN_TRACK_IS_STREAM); }
  int track_is_url()    { return !!(flags & GV_PLUGIN_TRACK_IS_URL); }

  virtual int can_seek() { return !!(attributes->flags & GV_PLUGIN_CAN_SEEK); }
  
  // Fill the playlist
  
  virtual void fill_playlist();
  
  // Cleanup function (called before selecting a new file)

  void seek_cmd(float percentage);
  
  virtual ~GVInput();

  virtual void alloc_frame(GVVideoFrame * frame);
  
  virtual void free_frame(GVVideoFrame * frame);

  virtual void alloc_frame(GVAudioFrame * frame);
  
  virtual void free_frame(GVAudioFrame * frame);

  virtual const GVTrackInfo * get_track_info();
  
  };
