/*****************************************************************

  main.cc

  Copyright (c) 2001 by Burkhard Plaum - plaum@ipf.uni-stuttgart.de

  http://gmerlin.sourceforge.net

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111, USA.

*****************************************************************/

// Main Function

#include <string.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <fcntl.h>
#include <unistd.h>

#include <iostream>
#include <limits.h>
#include <string.h>

#include "gv.h"
#include "gvconfig.h"
#include "globalconfig.h"
#include "gvgtk.h"
#include "remote.h"

#include "xmanager.h"

#include "gvmmx.h"
#include "skin.h"
#include "skinbrowser.h"

// #include <errno.h>

using std::cerr;

void print_help()
  {
  cerr << PACKAGE << " Version " << VERSION << "\n"
       << "Usage: " <<   PACKAGE
       << " [--play] [--input=<input_plugin>]\n" 
       << "              [--audio_output=<audio_output_plugin>]\n"
       << "              [--video-output=<video_output_plugin] <files>\n"
       << " --play                               Start playing after startup\n"
       << " --input=<input_plugin>               Load <input_plugin>\n"
       << " --input=<audio_output_plugin>        Load <audio_output_plugin>\n"
       << " --audio_output=<audio_output_plugin> Load <audio_output_plugin>\n"
       << " --video_output=<video_output_plugin> Load <video_output_plugin>\n";
  }




int check_arg(int & argc, char *** argv, char * option, char ** rest)
  {
  int len = strlen(option);
  for(int i = 1; i < argc; i++)
    {
    //    cerr << "Arg: " << (*argv)[i] << "\n";
    if(!strncmp((*argv)[i], option, len))
      {
      *rest = &((*argv)[i][len]);

      for(int j = i+1; j < argc; j++)
        (*argv)[j-1] = (*argv)[j];
      argc--;
      return 1;
      }
    }
  return 0;
  }

int main(int argc, char ** argv)
  {
  cerr << PACKAGE << "-" << VERSION << "\n";

  // Gtk specific stuff
  
  g_thread_init(NULL);
  gtk_init(&argc, &argv);

  char * gv_home_dir = g_strdup_printf("%s/.%s/", g_get_home_dir(), PACKAGE);

  char * current_directory = new char[PATH_MAX];

  getcwd(current_directory, PATH_MAX-1);

  gv_config = new GVConfig;

  char * gv_config_file;
  
  // Check if the home directory is present
  
  if(chdir(gv_home_dir))
    {
    // Program is run the first time
    mkdir(gv_home_dir, S_IRUSR|S_IWUSR|S_IXUSR);
    gv_config_file = g_strdup_printf("%s/%src", DATA_DIR, PACKAGE);

    // Read the system wide configuration file

    gv_config->read(gv_config_file);

    // Copy it to the new directory

    g_free(gv_config_file);
    gv_config_file = g_strdup_printf("%s%src", gv_home_dir, PACKAGE);
    gv_config->write(gv_config_file);
    }
  else
    {
    chdir(current_directory);
    gv_config_file = g_strdup_printf("%s%src", gv_home_dir, PACKAGE);
    gv_config->read(gv_config_file);
    }
  
  // Get commandline options

  char * input_plugin = (char*)0;
  char * video_output_plugin = (char*)0;
  char * audio_output_plugin = (char*)0;

  int start_play = 0;

  char * rest;

  char *file_to_play;
  
  if(check_arg(argc, &argv, "--help", &rest))
    {
    print_help();
    return 0;
    }
  if(check_arg(argc, &argv, "--input=", &input_plugin))
    {
    if(*input_plugin == '\0')
      {
      cerr << "--input requires a plugin name\n";
      input_plugin = (char*)0;
      }
    }
  if(check_arg(argc, &argv, "--video-output=", &video_output_plugin))
    {
    if(*video_output_plugin == '\0')
      {
      cerr << "--video-output requires a plugin name\n";
      video_output_plugin = (char*)0;
      }
    }
  if(check_arg(argc, &argv, "--audio-output=", &audio_output_plugin))
    {
    if(*audio_output_plugin == '\0')
      {
      cerr << "--audio-output requires a plugin name\n";
      audio_output_plugin = (char*)0;
      }
    }
  if(check_arg(argc, &argv, "--play", &rest))
    {
    cerr << "Start play\n";
    start_play = 1;
    }
  
  // Check if another instance is running

  GVRemote * remote = gv_remote_find();
  if(remote)
    {
    cerr << "Another inscance is running\n";
    cerr << "input_plugin: " << input_plugin << "\n";

    if(input_plugin && (argc == 1))
      gv_remote_set_input(remote, input_plugin);
    else if(argc > 1)
      gv_remote_set_input(remote, "");
    
    if(video_output_plugin)
      gv_remote_set_video_output(remote, video_output_plugin);

    if(audio_output_plugin)
      gv_remote_set_audio_output(remote, audio_output_plugin);

    if(argc > 1)
      {
      // The user wants to play files, we don't need an input
      // plugin here

            
      for(int i = 1; i < argc; i++)
        {
        if(*(argv[i]) != '/')
          file_to_play = g_strdup_printf("%s/%s", current_directory, argv[i]);
        else
          file_to_play = g_strdup(argv[i]);
        if(i == 1)
          gv_remote_play_file(remote, file_to_play); 
        else
          gv_remote_add_file(remote, file_to_play);
        g_free(file_to_play);
        }
      }
    else
      {
      if(start_play)
        gv_remote_play(remote);
      }
    return 0;
    }

  gv_mmx = new GVMMX;

  
  // Configuration stuff

  gv_config->put_string("_", "HomeDir", gv_home_dir);

  // Global configuration values

  gv_global_config = new GVGlobalConfig;

  // Now parse command line arguments and change gv_global_config

  if(audio_output_plugin)
    gv_config->put_string("General", "OutputAudio",
                                 audio_output_plugin);
  if(video_output_plugin)
    gv_config->put_string("General", "OutputVideo",
                                 video_output_plugin);
  if(input_plugin)
    gv_config->put_string("General", "Input",
                                 input_plugin);
  
  
  // Create and initialize the non GUI classes
  
  gv_playlist = new GVPlaylist;
  
  gv_engine = new GVEngine;

  gv_engine->load_plugins(PLUGIN_DIR);

  // Create the X manager (needs plugins and is needed by GUI classes)
  
  gv_x_manager = new GVXManager;
  
  gv_video_window = new GVVideoWindow;

  // Create the main window
  
  gv_main_win = new GVMainWindow;

  gv_info_window = new GVInfoWindow;

  // Create the skin browser

  gv_skin_browser = new GVSkinBrowser;
    
  // Add files listed on the commandline

  int i_tmp = 0;
  gv_config->get_bool("General", "StartPlay", i_tmp);
  if(i_tmp)
    start_play = 1;
  if((argc == 1) && (start_play))
    gv_engine->play();
  else
    {
    for(int i = 1; i < argc; i++)
      {
      if(*(argv[i]) != '/')
        file_to_play = g_strdup_printf("%s/%s", current_directory, argv[i]);
      else
        file_to_play = g_strdup(argv[i]);
      if(i == 1)
        gv_engine->play_file(file_to_play);
      else
        gv_engine->add_file(file_to_play);
      g_free(file_to_play);
      }
    }

  // Show the video window

  i_tmp = 1;
  gv_config->get_bool("VideoWindow", "ShowAtStart", i_tmp); 

  if(i_tmp)
    gv_video_window->show();

  
  // Run the program
  gv_engine->run();
  gv_main_win->run();

  // Call all destructurs
  
  delete gv_playlist;
  delete gv_main_win; 
  delete gv_info_window; 
  delete gv_engine;
  delete gv_video_window;
  delete gv_mmx;
  delete gv_global_config;
  delete gv_skin_browser;
  
  cerr << "Writing " << gv_config_file << "...";
  gv_config->write(gv_config_file);
  cerr << "done\n";
  delete gv_config;
    
  }
