/*****************************************************************

  mainwin.cc

  Copyright (c) 2001 by Burkhard Plaum - plaum@ipf.uni-stuttgart.de

  http://gmerlin.sourceforge.net

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111, USA.

*****************************************************************/

// Routines for gmerlins main window

#include <gtk/gtk.h>
#include <pthread.h>

#include <iostream>
#include <fstream>

using std::ofstream;

#include <gdk/gdkx.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "gv.h"

#include "button.h"
#include "skin.h"
#include "skinbrowser.h"

#include "gvwindow.h"
#include "videowindow.h"
#include "configwindow.h"
#include "fileselect.h"
#include "playlistwidget.h"
#include "display.h"
#include "remote.h"

#include "slider.h"

#include "mainwin.h"
#include "globalconfig.h"
#include "gvconfig.h"

#include "xmanager.h"

#include "gvloader.h"

using std::cerr;

// #include "pixmaps/next.xpm"
// #include "pixmaps/pause.xpm"
// #include "pixmaps/play.xpm"
// #include "pixmaps/previous.xpm"
// #include "pixmaps/stop.xpm"

// Pixmaps for the skin

// #include "pixmaps/window_left.xpm"
// #include "pixmaps/window_right.xpm"
// #include "pixmaps/window_top.xpm"
// #include "pixmaps/window_bottom.xpm"

// #include "pixmaps/window_ll.xpm"
// #include "pixmaps/window_lr.xpm"
// #include "pixmaps/window_ul_dot.xpm"
// #include "pixmaps/window_ur_x.xpm"

// #include "pixmaps/mainwin_title.xpm"

GVMainWindow * gv_main_win;

static void __expose_callback(GtkWidget * w, GdkEventExpose * event,
                              gpointer data)
  {
  ((GVMainWindow*)data)->expose_callback();
  }

static void __gv_button_callback(GVButton * button, gpointer data)
  {
  ((GVMainWindow*)data)->gv_button_callback(button);
  }

static gboolean __delete_callback(GtkWidget * w, GdkEventAny * event,
                              gpointer data)
  {
  ((GVMainWindow*)data)->button_callback(w);
  return TRUE;
  }

static gboolean __property_callback(GtkWidget * w, GdkEventProperty * event,
                                gpointer data)
  {
  ((GVMainWindow*)data)->property_callback(event);
  return TRUE;
  }

static gboolean __window_motion_callback(GtkWidget * w,
                                         GdkEventMotion * event,
                                         gpointer data)
  {
  ((GVMainWindow*)(data))->window_motion_callback(event);
  return TRUE;
  }

static gboolean __window_button_callback(GtkWidget * w,
                                         GdkEventButton * event,
                                         gpointer data)
  {
  ((GVMainWindow*)(data))->window_button_callback(event);
  return TRUE;
  } 

static void __set_style_callback(GtkWidget * w, GtkStyle * old_style,
                                 gpointer data)
  {
  ((GVMainWindow*)data)->set_style_callback();
  //  return true;
  }


static gboolean  __timeout_func(gpointer data)
  {
  return ((GVMainWindow*)data)->timeout_func();
  return TRUE;
  }

// static gboolean __seek_callback(GtkWidget *w, GdkEventButton *event,
//                                 gpointer * data)
//   {
//   ((GVMainWindow*)data)->seek_callback(event);
//   return FALSE;
//   }

static void __slider_change_callback(float percentage, gpointer data)
  {
  ((GVMainWindow*)data)->seek_change_callback(percentage);
  }

static void __slider_seek_callback(float percentage, gpointer data)
  {
  ((GVMainWindow*)data)->seek_callback(percentage);
  }


void GVMainWindow::seek_change_callback(float percentage)
  {
  flags |= GV_MAINWIN_SEEKING_ACTIVE;

  GVTime time;
  double time_d = total_track_time.get_double() * percentage;

  time = time_d;
  display_widget.update_time(time);
  }

void GVMainWindow::seek_callback(float percentage)
  {
  gv_engine->seek(percentage);
  //  flags &= ~GV_MAINWIN_NO_SCALE_UPDATE;
  }

void GVMainWindow::property_callback(GdkEventProperty * event)
  {
  if((!remote) || (event->atom != remote->command_atom))
    return;

  //  cerr << "Property callback\n";

  gtk_signal_handler_block_by_func(GTK_OBJECT(window),
                                   GTK_SIGNAL_FUNC(__property_callback),
                                   (gpointer)this);

  
  if(!gv_remote_get_command(remote))
    {
    gtk_signal_handler_unblock_by_func(GTK_OBJECT(window),
                                     GTK_SIGNAL_FUNC(__property_callback),
                                     (gpointer)this);
    return;
    }
  gtk_signal_handler_unblock_by_func(GTK_OBJECT(window),
                                     GTK_SIGNAL_FUNC(__property_callback),
                                     (gpointer)this);

  //  if(!strcmp(remote->command, GV_REMOTE_ANSWER))
  //    return;
  
  cerr << "Remote Command: " << remote->command << " Arg: " << remote->arg
       << "\n";

  gv_remote_confirm(remote);
  
  if(!strcmp(remote->command, GV_REMOTE_STOP))
    gv_engine->stop();
  else if(!strcmp(remote->command, GV_REMOTE_PLAY))
    gv_engine->play();
  else if(!strcmp(remote->command, GV_REMOTE_NEXT))
    gv_engine->next();
  else if(!strcmp(remote->command, GV_REMOTE_PREVIOUS))
    gv_engine->previous();
  else if(!strcmp(remote->command, GV_REMOTE_PLAYFILE))
    gv_engine->play_file(remote->arg);
  else if(!strcmp(remote->command, GV_REMOTE_ADDFILE))
    gv_engine->add_file(remote->arg);
  else if(!strcmp(remote->command,   GV_REMOTE_SET_INPUT))
    {
    cerr << "Setting Input...";
    gv_engine->set_input(remote->arg);
    cerr << "done\n";
    }
  else if(!strcmp(remote->command,   GV_REMOTE_SET_AUDIO_OUTPUT))
    gv_engine->set_output_audio(remote->arg);
  else if(!strcmp(remote->command,   GV_REMOTE_SET_VIDEO_OUTPUT))
    gv_engine->set_output_video(remote->arg);

  gv_remote_send_done(remote);
  }


void GVMainWindow::window_button_callback(GdkEventButton * event)
  {
  switch(event->type)
    {
    case GDK_BUTTON_PRESS:

      if((event->button == 1) || (event->button == 3))
        {
        gdk_window_get_root_origin(window->window,
                                   &win_x_save,
                                   &win_y_save);
        
        mouse_x_save = (int)(event->x_root);
        mouse_y_save = (int)(event->y_root);
        
        gdk_pointer_grab(drawingarea->window, 0,
                         (GdkEventMask)(GDK_BUTTON1_MOTION_MASK|
                                        GDK_BUTTON3_MOTION_MASK|
                                        GDK_BUTTON_RELEASE_MASK),
                         (GdkWindow*)0, gv_x_manager->move_cursor, event->time);
        move_active = 1;
        }
      else
        return;
      break;
    case GDK_BUTTON_RELEASE:
      if(move_active)
        {
        gdk_pointer_ungrab(event->time);
        move_active = 0;
        }
      break;
    }
  }

void GVMainWindow::window_motion_callback(GdkEventMotion * event)
  {
  if(!move_active)
    return;

  int mouse_x = (int)(event->x_root);
  int mouse_y = (int)(event->y_root);
  
  int x_diff =  mouse_x - mouse_x_save;
  int y_diff =  mouse_y - mouse_y_save;

  gdk_window_move(window->window,
                  win_x_save + x_diff, win_y_save + y_diff);
  win_x_save += x_diff;
  win_y_save += y_diff;

  mouse_x_save = mouse_x;
  mouse_y_save = mouse_y;

  }

gboolean GVMainWindow::timeout_func()
  {

  //  cerr << "Blupp\n";

//   if(gv_engine->audio_buffer.empty())
//     cerr << "Audio Buffer Empty\n";
//   else if(gv_engine->audio_buffer.full())
//     cerr << "Audio Buffer Full\n";
//   else
//     cerr << "Audio Buffer Normal\n";

//   if(gv_engine->get_state_flag(GV_ENGINE_SEEKING))
//     cerr << "Seeking\n";

  GVEngineState state;
  GVEngineState old_state;
  
  int state_changed = gv_engine->get_state_flag(GV_ENGINE_STATE_CHANGED);
  state = gv_engine->get_state(old_state);

  if(gv_playlist->get_state_flags() & GV_PLAYLIST_CURRENT_CHANGED)
    {
    playlist_widget.update_current();
    gv_playlist->clear_state_flag(GV_PLAYLIST_CURRENT_CHANGED);
    }

  if(gv_engine->get_state_flag(GV_ENGINE_INFO_CHANGED))
    {
    gv_info_window->update();
    gv_engine->clear_state_flag(GV_ENGINE_INFO_CHANGED);
    }
    
  if(state_changed)
    {
    display_widget.update_state(state);
    
    switch(state)
      {
      case GVEngineCompleting:
        cerr << "Mainwin: Engine now Completing\n";
        break;
      case GVEngineError:
        {
        display_widget.set_error_message(gv_engine->get_error_message());
        break;
        }
      case GVEnginePlaying:
      case GVEnginePaused:
        {
        display_widget.update_playlist_times();
        if((old_state == GVEngineStopped) ||
           (old_state == GVEngineChanging) ||
           (old_state == GVEngineCompleting)||
           (old_state == GVEngineError))
          {
          gv_engine->get_total_track_time(total_track_time);
          
          //     cerr << "Track changed, new total_time: "
          //          << (total_track_time.seconds)/60 << ":"
          //          << (total_track_time.seconds)%60 << "\n";
          
          const GVPlaylistTrack * current_track =
            gv_playlist->get_current_track();
          if(current_track)
            display_widget.set_track_message(current_track->trackname);
          
          gv_engine->get_total_track_time(total_track_time);
          display_widget.set_default_display_mode();

          if(total_track_time.undefined())
            {
            seek_slider->set_visible(0);
            flags |= GV_MAINWIN_NO_SCALE_UPDATE;
            flags &= ~GV_MAINWIN_NO_TIME_UPDATE;
            }
          else
            {
            seek_slider->set_visible(1);
            if(gv_engine->get_state_flag(GV_ENGINE_CAN_SEEK))
              seek_slider->set_sensitive(1);
            else
              seek_slider->set_sensitive(0);
            flags &= ~GV_MAINWIN_SEEKING_ACTIVE;
            break;
            }
          }
        else if(old_state == GVEngineSeeking)
          flags &= ~GV_MAINWIN_SEEKING_ACTIVE;
        }
        break;
      case GVEngineStopped:
        {
        char * message = g_strdup_printf("%s (Version %s)",
                                         PACKAGE, VERSION);
        display_widget.set_track_message(message);
        g_free(message);
        
        display_widget.set_display_mode(GVDisplayWidget::DisplayAll);
        flags |= GV_MAINWIN_VALUE_UPDATE;
        seek_slider->set_percentage(0.0);
        seek_slider->set_visible(1);
        seek_slider->set_sensitive(0);
        flags &= ~GV_MAINWIN_VALUE_UPDATE;
        }
      }
    gv_engine->clear_state_flag(GV_ENGINE_STATE_CHANGED);
    }
 

  // If the engine is completing, the engine threads stands still,
  // so the current time isn't updated regularly by the engine.
  // Therefore, we need to force the engine to update the time
  // from the plugins, so we can update the slider and the time
  // display properly

  GVTime current_time;
  
  if(state == GVEngineCompleting)
    gv_engine->get_current_track_time(current_time, 1);
  else
    gv_engine->get_current_track_time(current_time, 0);
  
  // Check if the contents of the playlist changed
  
  if(gv_playlist->get_state_flags() & GV_PLAYLIST_CONTENTS_CHANGED)
    {
    playlist_widget.update_album();
    display_widget.update_playlist_times();
    display_widget.update_time(current_time);
    gv_playlist->clear_state_flag(GV_PLAYLIST_CONTENTS_CHANGED);
    }
 
  // Update the time slide

  if((state == GVEnginePlaying)||(state == GVEngineCompleting))
    {
    if(!(flags & GV_MAINWIN_NO_SCALE_UPDATE))
      {
      
      //    cerr << "Current time: " << current_time.seconds << " Seconds "
      //         << current_time.nanoseconds << " Nanoseconds\n";
      //    cerr << "done\n";
      
      float perc = (current_time.get_double()/total_track_time.get_double());
      
      seek_slider->set_percentage(perc);
      
      //    display_widget.flash_message(0);
      }
    if(!(flags & GV_MAINWIN_NO_TIME_UPDATE))
      display_widget.update_time(current_time);
    }

  //   else
  //     if(state != GVEngineStopped)
  //       cerr << "Didn't update slide\n";
  
  // Check if the visible state of the playlist and the video window changed

  if(!!get_toggle_item_state(show_playlist_item) !=
     !!playlist_widget.is_visible())
    set_toggle_item_state(show_playlist_item, !!playlist_widget.is_visible());

  if(!!get_toggle_item_state(show_video_item) !=
     !!gv_video_window->is_visible())
    set_toggle_item_state(show_video_item, !!gv_video_window->is_visible());

  if(!!get_toggle_item_state(show_info_window_item) !=
     !!gv_info_window->is_visible())
    set_toggle_item_state(show_info_window_item, !!gv_info_window->is_visible());
 
  old_state = state;
  idle_tag = gtk_timeout_add(100, __timeout_func, (gpointer)this);
  return FALSE;
  }

void GVMainWindow::button_callback(GtkWidget * w)
  {
  // Closed my Window manager
  if(w == window)
    {
    gdk_threads_leave();
    gv_engine->stop_sync();
    gdk_threads_enter();
    gtk_main_quit();
    }
  }

void GVMainWindow::gv_button_callback(GVButton * w)
  {
  if(w == pause_button)
    gv_engine->pause();
  else if(w == play_button)
    {
    gv_engine->play();
    if(!(flags & GV_MAINWIN_PAUSED))
      {
      display_widget.set_default_display_mode();
      display_widget.update_playlist_times();
      display_widget.update_mode();
      }
    else
      flags &= ~GV_MAINWIN_PAUSED;
    }
  else if(w == next_button)
    {
    gv_engine->next();
    playlist_widget.update_album();
    }
  else if(w == previous_button)
    {
    gv_engine->previous();
    playlist_widget.update_album();
    }
  else if(w == stop_button)
    {
    //    gtk_timeout_remove(idle_tag);
    gv_engine->stop();
    timeout_func();    // Update the new time and stuff
    }
  else if(w == close_button)
    {
    gdk_threads_leave();
    gv_engine->stop_sync();
    gdk_threads_enter();
    gtk_main_quit();
    }
  }

GVMainWindow::GVMainWindow()
  {
  //  set_name("Gmerlin");

  layout = gtk_layout_new((GtkAdjustment*)0, (GtkAdjustment*)0);
  window = gtk_window_new(GTK_WINDOW_TOPLEVEL);

  gtk_widget_set_events(window, GDK_PROPERTY_CHANGE_MASK);
  
  gtk_container_add(GTK_CONTAINER(window), layout);
    
  // Create a file for the window ID

  ofstream output;

  // Create the objects

  move_active = 0;
  flags = 0;
  remote = (GVRemote*)0;

  // Create the background drawingarea

  drawingarea = gtk_drawing_area_new();

  gtk_widget_set_events(drawingarea,
                        GDK_EXPOSURE_MASK|
                        GDK_BUTTON1_MOTION_MASK|
                        GDK_BUTTON_PRESS_MASK|
                        GDK_BUTTON_RELEASE_MASK);

  gtk_signal_connect(GTK_OBJECT(drawingarea), "expose_event",
                     GTK_SIGNAL_FUNC(__expose_callback),
                     (gpointer)this);

  gtk_signal_connect(GTK_OBJECT(drawingarea), "style_set",
                     GTK_SIGNAL_FUNC(__set_style_callback),
                     (gpointer)this);

  gtk_signal_connect(GTK_OBJECT(drawingarea), "button_press_event",
                     GTK_SIGNAL_FUNC(__window_button_callback),
                     (gpointer)this);
  gtk_signal_connect(GTK_OBJECT(drawingarea), "button_release_event",
                     GTK_SIGNAL_FUNC(__window_button_callback),
                     (gpointer)this);
  gtk_signal_connect(GTK_OBJECT(drawingarea), "motion_notify_event",
                     GTK_SIGNAL_FUNC(__window_motion_callback),
                     (gpointer)this);
  
  gtk_widget_show(drawingarea);
  
  gtk_layout_put(GTK_LAYOUT(layout), drawingarea, 0, 0);
  
  play_button = new GVButton(layout);
  pause_button = new GVButton(layout);
  stop_button = new GVButton(layout);
  next_button = new GVButton(layout);
  previous_button = new GVButton(layout);
  menu_button = new GVButton(layout);
  close_button = new GVButton(layout);
  
  //  gtk_window_set_policy(GTK_WINDOW(window), 0, 1, 1);

 // Set Callbacks

  //  gtk_signal_connect(GTK_OBJECT(window), "realize",
  //                     GTK_SIGNAL_FUNC(__realize_callback),
  //                     (gpointer)this);

  play_button->press_callback = __gv_button_callback;
  play_button->press_callback_data = (gpointer)this;

  pause_button->press_callback = __gv_button_callback;
  pause_button->press_callback_data = (gpointer)this;

  next_button->press_callback = __gv_button_callback;
  next_button->press_callback_data = (gpointer)this;

  previous_button->press_callback = __gv_button_callback;
  previous_button->press_callback_data = (gpointer)this;

  stop_button->press_callback = __gv_button_callback;
  stop_button->press_callback_data = (gpointer)this;

  menu_button->press_callback = __gv_button_callback;
  menu_button->press_callback_data = (gpointer)this;

  close_button->press_callback = __gv_button_callback;
  close_button->press_callback_data = (gpointer)this;
  
  gtk_signal_connect(GTK_OBJECT(window), "delete_event",
                     GTK_SIGNAL_FUNC(__delete_callback),
                     (gpointer)this);

  property_handler_id =
    gtk_signal_connect(GTK_OBJECT(window), "property_notify_event",
                       GTK_SIGNAL_FUNC(__property_callback),
                       (gpointer)this);

  
  
  gtk_layout_put(GTK_LAYOUT(layout), display_widget.widget, 0, 0);

  seek_slider = new GVSlider;
  seek_slider->set_sensitive(0);
  
  gtk_layout_put(GTK_LAYOUT(layout), seek_slider->widget, 0, 0);

  gtk_widget_show(layout);
  
  // Show the objects
    
  // Place the objects

  // Do some non-gui stuff

  //  input_window.create_menu();

  // Create the video window

  config_window.global_config = &(gv_global_config->config_widget);
  config_window.network_config = &(gv_global_config->network_config_widget);
  config_window.display = display_widget.config_widget;
  config_window.playlist_widget = &(playlist_widget.config_widget);

#if 0
  config_window.input = &(gv_engine->input_config);
  config_window.output_video = &(gv_engine->output_video_config);
  config_window.output_audio = &(gv_engine->output_audio_config);
#endif
  
  config_window.init();
  get_config();

  gtk_widget_realize(window);
  
  //  realize();

  realize_callback();
  
//   set_top_pixmap(window_top_xpm);
//   set_bottom_pixmap(window_bottom_xpm);
//   set_left_pixmap(window_left_xpm);
//   set_right_pixmap(window_right_xpm);
//   set_ll_pixmap(window_ll_xpm);
//   set_lr_pixmap(window_lr_xpm);
//   set_ul_pixmap(window_ul_dot_xpm);
//   set_ur_pixmap(window_ur_x_xpm);

//   set_title_pixmap(mainwin_title_xpm);

  display_widget.realize();

  seek_slider->realize();
  
  remote = gv_remote_create(window->window);
  create_menu();

  update_plugin_menus();
  
  menu_button->menu = menu; 
  
  //  set_resizeable(1);

  // Set the callbacks for the slider

  seek_slider->change_callback = __slider_change_callback;
  seek_slider->seek_callback = __slider_seek_callback;
  
  seek_slider->change_callback_data = (gpointer)this;
  seek_slider->seek_callback_data = (gpointer)this;

  gdk_window_set_decorations(window->window, (GdkWMDecoration)0);
  gdk_window_set_functions(window->window, (GdkWMFunction)0);

  gdk_window_set_icon(window->window, 
                      (GdkWindow*)0,
                      gv_x_manager->gmerlin_icon,
                      gv_x_manager->gmerlin_icon_mask);
  }

void GVMainWindow::run()
  {
  if(show_playlist)
    playlist_widget.show();

  if(show_info_window)
    gv_info_window->show();
    
  playlist_widget.update_album();

  gtk_widget_show(window);

  update_toggle_items();
  
  display_widget.update_playlist_times();

  idle_tag = gtk_timeout_add(100, __timeout_func, (gpointer)this);
    
  gdk_threads_enter();
  gtk_main();
  gdk_threads_leave();
  }

// Create the pixmap buttons

void GVMainWindow::realize_callback()
  {

  }

GVMainWindow::~GVMainWindow()
  {
  gv_remote_destroy(remote);
  put_config();
  }

#define SECTION "General"

void GVMainWindow::get_config()
  {
  gv_config->get_bool(SECTION, "ShowPlaylist", show_playlist);
  gv_config->get_bool(SECTION, "ShowInfoWindow", show_info_window);
  }

void GVMainWindow::put_config()
  {
  gv_config->put_bool(SECTION, "ShowPlaylist", !!playlist_widget.is_visible());
  gv_config->put_bool(SECTION, "ShowInfoWindow", !!gv_info_window->is_visible());
  }

void GVMainWindow::hide()
  {
  gtk_main_quit();
  }

static void __menu_callback(GtkWidget * w, gpointer data)
  {
  ((GVMainWindow*)data)->menu_callback(w);
  }

void GVMainWindow::menu_callback(GtkWidget * w)
  {
  if(w == quit_item)
    {
    gdk_threads_leave();
    gv_engine->stop_sync();
    gdk_threads_enter();
    gtk_main_quit();
    }
  else if(w == option_preferences_item)
    {
    ((GVConfigWindow*)(&config_window))->run();
    update_toggle_items();
    }
  else if(w == option_skins_item)
    {
    gv_skin_browser->run();
    }
  
  else if(w == option_auto_advance_item)
    gv_engine->set_config_flag(GV_ENGINE_AUTO_ADVANCE,
                               get_toggle_item_state(option_auto_advance_item));
  else if(w == option_sync_software_item)
    gv_engine->set_config_flag(GV_ENGINE_SYNC_SOFTWARE,
                               get_toggle_item_state(option_sync_software_item));
  else if(w == option_play_on_start_item)
    gv_engine->set_config_flag(GV_ENGINE_START_PLAY,
                               get_toggle_item_state(option_play_on_start_item));

  else if(w == show_playlist_item)
    {
    if(get_toggle_item_state(show_playlist_item))
      playlist_widget.show();
    else
      playlist_widget.hide();
    }

  else if(w == show_video_item)
    {
    if(get_toggle_item_state(show_video_item))
      gv_video_window->show();
    else
      gv_video_window->hide();
    }
  else if(w == show_info_window_item)
    {
    if(get_toggle_item_state(show_info_window_item))
      gv_info_window->show();
    else
      gv_info_window->hide();
    }
  else if(w == use_audio_item)
    {
    if(get_toggle_item_state(use_audio_item))
      gv_engine->set_use_audio(1);
    else
      gv_engine->set_use_audio(0);
    }
  }

GtkWidget * GVMainWindow::create_toggle_item(const char * name)
  {
  GtkWidget * ret = gtk_check_menu_item_new_with_label(name);
  gtk_signal_connect(GTK_OBJECT(ret), "toggled",
                     GTK_SIGNAL_FUNC(__menu_callback),
                     (gpointer)this);
  gtk_check_menu_item_set_show_toggle(GTK_CHECK_MENU_ITEM(ret), TRUE);
  gtk_widget_show(ret);
  return ret;
  }

GtkWidget * GVMainWindow::create_item(const char * name)
  {
  GtkWidget * ret = gtk_menu_item_new_with_label(name);
  gtk_signal_connect(GTK_OBJECT(ret), "activate",
                     GTK_SIGNAL_FUNC(__menu_callback),
                     (gpointer)this);
  gtk_widget_show(ret);
  return ret;
  }

GtkWidget * GVMainWindow::create_submenu_item(const char * name,
                                              GtkWidget * submenu)
  {
  GtkWidget * ret = gtk_menu_item_new_with_label(name);
  gtk_menu_item_set_submenu(GTK_MENU_ITEM(ret), submenu);
  gtk_widget_show(ret);
  return ret;
  }

void GVMainWindow::create_menu()
  {
  menu = gtk_menu_new();
    
//   GtkWidget ** input_menu_items;
  
//   GtkWidget * input_item;
//   GtkWidget * input_menu;
  
  option_menu = gtk_menu_new();

  GtkWidget * tearoff_item;

  tearoff_item = gtk_tearoff_menu_item_new();
  gtk_widget_show(tearoff_item);
  gtk_menu_append(GTK_MENU(option_menu), tearoff_item);
  
  option_auto_advance_item = create_toggle_item("Auto Advance");
  gtk_menu_append(GTK_MENU(option_menu), option_auto_advance_item);
  
  option_play_on_start_item = create_toggle_item("Play after start");
  gtk_menu_append(GTK_MENU(option_menu), option_play_on_start_item);

  option_sync_software_item = create_toggle_item("Force software sync");
  gtk_menu_append(GTK_MENU(option_menu), option_sync_software_item);

  option_preferences_item = create_item("Preferences...");
  gtk_menu_append(GTK_MENU(option_menu), option_preferences_item);

  option_skins_item = create_item("Skin Browser...");
  gtk_menu_append(GTK_MENU(option_menu), option_skins_item);
  
  gtk_widget_show(option_menu);

  tearoff_item = gtk_tearoff_menu_item_new();
  gtk_widget_show(tearoff_item);
  gtk_menu_append(GTK_MENU(menu), tearoff_item);
  
  option_item = create_submenu_item("Options", option_menu);

  gtk_menu_append(GTK_MENU(menu), option_item);

  // Create the plugin menus

  media_files_item = gtk_radio_menu_item_new_with_label((GSList*)0,
                                                        "Media Files");
  configure_input_item = gtk_menu_item_new_with_label("Configure...");
  configure_audio_output_item = gtk_menu_item_new_with_label("Configure...");
  configure_video_output_item = gtk_menu_item_new_with_label("Configure...");

  use_audio_item = create_toggle_item("Use this for audio");
  
  GtkWidget * tmp_menu = gtk_menu_new();

  input_items = create_plugin_menu(gv_input_loader,
                                   tmp_menu,
                                   configure_input_item,
                                   num_input_plugins,
                                   media_files_item,
                                   (GtkWidget*)0);

  gtk_widget_show(tmp_menu);
  
  GtkWidget * tmp_item = create_submenu_item("Input Plugin",
                                             tmp_menu);

  gtk_menu_append(GTK_MENU(menu), tmp_item);

  tmp_menu = gtk_menu_new();

  video_output_items = create_plugin_menu(gv_video_output_loader,
                                          tmp_menu,
                                          configure_video_output_item,
                                          num_video_output_plugins,
                                          (GtkWidget*)0,
                                          use_audio_item);
  gtk_widget_show(tmp_menu);
  
  tmp_item = create_submenu_item("Video Output Plugin",
                                 tmp_menu);
  
  gtk_menu_append(GTK_MENU(menu), tmp_item);

  tmp_menu = gtk_menu_new();
  audio_output_items = create_plugin_menu(gv_audio_output_loader,
                                          tmp_menu,
                                          configure_audio_output_item,
                                          num_audio_output_plugins,
                                          (GtkWidget*)0,
                                          (GtkWidget*)0);
  gtk_widget_show(tmp_menu);
  
  tmp_item = create_submenu_item("Audio Output Plugin",
                                 tmp_menu);
  
  gtk_menu_append(GTK_MENU(menu), tmp_item);
  
  // Create the "Show..." items
  
  show_playlist_item = create_toggle_item("Show playlist");
  gtk_menu_append(GTK_MENU(menu), show_playlist_item);
  
  show_video_item = create_toggle_item("Show video window");
  gtk_menu_append(GTK_MENU(menu), show_video_item);

  show_info_window_item = create_toggle_item("Show info window");
  gtk_menu_append(GTK_MENU(menu), show_info_window_item);

  quit_item = create_item("Quit");
  gtk_menu_append(GTK_MENU(menu), quit_item);

  gtk_widget_show(menu);
  
  //  set_ul_menu(menu);

  // TODO: Set up the menu for the menu button
  // TODO: Select Property Change Mask for the window
  }

int GVMainWindow::get_toggle_item_state(GtkWidget * w)
  {
  return (GTK_CHECK_MENU_ITEM(w))->active;
  }

void GVMainWindow::set_toggle_item_state(GtkWidget * w, int value)
  {
  flags |= GV_MAINWIN_NO_MENU_CALLBACK;
  gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(w), !!value);
  flags &= ~GV_MAINWIN_NO_MENU_CALLBACK;
  }

void GVMainWindow::update_toggle_items()
  {
  int flags = gv_engine->get_config_flags();
  
  set_toggle_item_state(option_auto_advance_item,
                        !!(flags & GV_ENGINE_AUTO_ADVANCE));
  set_toggle_item_state(option_play_on_start_item,
                        !!(flags & GV_ENGINE_START_PLAY));
  set_toggle_item_state(option_sync_software_item,
                        !!(flags & GV_ENGINE_SYNC_SOFTWARE));

  set_toggle_item_state(show_video_item, gv_video_window->is_visible());
  set_toggle_item_state(show_info_window_item, gv_info_window->is_visible());
  set_toggle_item_state(show_playlist_item, playlist_widget.is_visible());
  }

void GVMainWindow::apply_skin(GVSkin & skin)
  {
  // Do the buttons
  
  play_button->set_skin(&skin.play_button);
  pause_button->set_skin(&skin.pause_button);
  next_button->set_skin(&skin.next_button);
  previous_button->set_skin(&skin.previous_button);
  stop_button->set_skin(&skin.stop_button);
  menu_button->set_skin(&skin.main_window_menu_button);
  close_button->set_skin(&skin.main_window_close_button);

  // Decorate the seek slider

  seek_slider->set_pixmaps(skin.seek_background,
                           skin.seek_slider,
                           skin.seek_slider_highlight,
                           skin.seek_slider_mask);
  
  gtk_layout_move(GTK_LAYOUT(layout), seek_slider->widget,
                  skin.seek_offset_x, skin.seek_offset_y);

  gtk_widget_show(seek_slider->widget);
  
  // Position the display

  gtk_layout_move(GTK_LAYOUT(layout), display_widget.widget,
                  skin.display_offset_x, skin.display_offset_y);

  // Set up the background

  int tmp_x, tmp_y, tmp_depth;

  background_pixmap = skin.main_window;
  background_mask = skin.main_window_mask;
  
  gdk_window_get_geometry(background_pixmap, &tmp_x, &tmp_y, &width,
                          &height, &tmp_depth);

  gtk_widget_set_usize(window, width, height);
  gdk_window_resize(window->window, width, height);

  gtk_layout_set_size(GTK_LAYOUT(layout), width, height);
  gtk_widget_set_usize(layout, width, height);

  gtk_drawing_area_size(GTK_DRAWING_AREA(drawingarea), width, height);
  
  gdk_window_shape_combine_mask(window->window, background_mask, 0, 0);
  //  gdk_window_set_back_pixmap(layout->window, background_pixmap, 0);
  //  gdk_window_set_child_shapes(window->window);

  playlist_widget.apply_skin(skin);

  display_widget.apply_skin(skin);
  }

void GVMainWindow::expose_callback()
  {
  gdk_draw_pixmap(drawingarea->window,
                  gv_x_manager->get_gc(),
                  background_pixmap,
                  0, 0, 0, 0, width, height);
  }

void GVMainWindow::set_style_callback()
  {
  if(!(window->window))
    return;
  
  gdk_window_shape_combine_mask(window->window, background_mask, 0, 0);
  expose_callback();
  }

static void __plugin_menu_callback(GtkWidget * w, gpointer data)
  {
  ((GVMainWindow*)data)->plugin_menu_callback(w);
  }

void GVMainWindow::plugin_menu_callback(GtkWidget * w)
  {


  if(flags & GV_MAINWIN_NO_MENU_CALLBACK)
    return;
  
  if(w == media_files_item)
    {
    if(get_toggle_item_state(w))
      {
      gv_engine->set_input("");
      gtk_widget_set_sensitive(configure_input_item, 0);
      }
    return;
    }

  else if(w == configure_input_item)
    {
    gv_engine->configure_input();
    return;
    }

  else if(w == configure_audio_output_item)
    {
    gv_engine->configure_output_audio();
    return;
    }
  
  else if(w == configure_video_output_item)
    {
    gv_engine->configure_output_video();
    return;
    }
  
  for(int i = 0; i < num_input_plugins; i++)
    {
    if(w == input_items[i])
      {
      if(get_toggle_item_state(w))
        {
        const GVPluginInfo * info
          = (GVPluginInfo*)(gtk_object_get_user_data(GTK_OBJECT(input_items[i])));
        gdk_threads_leave();
        gv_engine->set_input(info->name);
        gdk_threads_enter();
        gtk_widget_set_sensitive(configure_input_item, 1);
        }
      return;
      }
    }

  for(int i = 0; i < num_audio_output_plugins; i++)
    {
    if(w == audio_output_items[i])
      {
      if(get_toggle_item_state(w))
        {
        const GVPluginInfo * info
          = (GVPluginInfo*)(gtk_object_get_user_data(GTK_OBJECT(audio_output_items[i])));
        gdk_threads_leave();
        gv_engine->set_output_audio(info->name);
        gdk_threads_enter();
        }
      return;
      }
    }

  for(int i = 0; i < num_video_output_plugins; i++)
    {
    if(w == video_output_items[i])
      {
      if(get_toggle_item_state(w))
        {
        const GVPluginInfo * info
          = (GVPluginInfo*)(gtk_object_get_user_data(GTK_OBJECT(video_output_items[i])));
        gdk_threads_leave();
        gv_engine->set_output_video(info->name);
        gdk_threads_enter();
        if(info->flags & GV_PLUGIN_HAS_AUDIO)
          gtk_widget_set_sensitive(use_audio_item, 1);
        else
          gtk_widget_set_sensitive(use_audio_item, 0);
        }
      return;
      }
    }
  }

GtkWidget ** GVMainWindow::create_plugin_menu(GVPluginLoader * plugin_loader,
                                              GtkWidget * _menu,
                                              GtkWidget * _config_item,
                                              int & num_items,
                                              GtkWidget * _media_files_item,
                                              GtkWidget * _use_audio_item)
  {
  num_items = plugin_loader->get_num_plugins();
  
  if(!num_items)
    return (GtkWidget**)0;
  
  GtkWidget ** items = new GtkWidget*[num_items];

  GtkWidget * tearoff_item = gtk_tearoff_menu_item_new();
  gtk_widget_show(tearoff_item);

  gtk_menu_append(GTK_MENU(_menu), tearoff_item);

  gtk_signal_connect(GTK_OBJECT(_config_item), "activate",
                     GTK_SIGNAL_FUNC(__plugin_menu_callback),
                     (gpointer)this);
  gtk_widget_show(_config_item);
  gtk_menu_append(GTK_MENU(_menu), _config_item);

  GSList * group = (GSList*)0;

  if(_media_files_item)
    {
    group = gtk_radio_menu_item_group(GTK_RADIO_MENU_ITEM(_media_files_item));
    gtk_signal_connect(GTK_OBJECT(_media_files_item), "toggled",
                       GTK_SIGNAL_FUNC(__plugin_menu_callback),
                       (gpointer)this);
    gtk_widget_show(_media_files_item);
    gtk_menu_append(GTK_MENU(_menu), _media_files_item);
    }

  for(int i = 0; i < num_items; i++)
    {
    const GVPluginInfo * info = plugin_loader->get_plugin_info(i);
    items[i] = gtk_radio_menu_item_new_with_label(group, info->real_name);
    group = gtk_radio_menu_item_group(GTK_RADIO_MENU_ITEM(items[i]));
    gtk_signal_connect(GTK_OBJECT(items[i]), "toggled",
                       GTK_SIGNAL_FUNC(__plugin_menu_callback),
                       (gpointer)this);
    gtk_object_set_user_data (GTK_OBJECT(items[i]),
                              (gpointer*)info);
    gtk_widget_show(items[i]);
    gtk_menu_append(GTK_MENU(_menu), items[i]);
    }

  if(_use_audio_item)
    gtk_menu_append(GTK_MENU(_menu), _use_audio_item);

  // Initialize the items
  
  set_toggle_item_state(use_audio_item, gv_engine->get_use_audio());
    
  return items;
  }

void GVMainWindow::update_plugin_menus()
  {
  const GVPluginAttributes * attributes;
  const GVPluginInfo * plugin_info;
  
  // Input

  attributes = gv_engine->get_input_attr();
    
  if(!(attributes) || (attributes->flags & GV_PLUGIN_IS_FILE))
    {
    set_toggle_item_state(media_files_item, 1);
    gtk_widget_set_sensitive(configure_input_item, 0);
    }
  else
    {
    gtk_widget_set_sensitive(configure_input_item, 1);
    for(int i = 0; i < num_input_plugins; i++)
      {
      plugin_info
        = (const GVPluginInfo*)gtk_object_get_user_data(GTK_OBJECT(input_items[i]));
      if(!strcmp(plugin_info->name, attributes->name))
        {
        set_toggle_item_state(input_items[i], 1);
        break;
        }
      }
    }

  attributes = gv_engine->get_output_video_attr();
  
  for(int i = 0; i < num_video_output_plugins; i++)
    {
    plugin_info
      = (const GVPluginInfo*)gtk_object_get_user_data(GTK_OBJECT(video_output_items[i]));
    if(!strcmp(plugin_info->name, attributes->name))
      {
      set_toggle_item_state(video_output_items[i], 1);
      if(attributes->flags & GV_PLUGIN_HAS_AUDIO)
        gtk_widget_set_sensitive(use_audio_item, 1);
      else
        gtk_widget_set_sensitive(use_audio_item, 0);
      break;
      }
    }

  attributes = gv_engine->get_output_audio_attr();
  
  for(int i = 0; i < num_audio_output_plugins; i++)
    {
    plugin_info
      = (const GVPluginInfo*)gtk_object_get_user_data(GTK_OBJECT(audio_output_items[i]));
    if(!strcmp(plugin_info->name, attributes->name))
      {
      set_toggle_item_state(audio_output_items[i], 1);
      break;
      }
    }
  
  }
