/*****************************************************************

  mainwin.h

  Copyright (c) 2001 by Burkhard Plaum - plaum@ipf.uni-stuttgart.de

  http://gmerlin.sourceforge.net

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111, USA.

*****************************************************************/

// Main Window

#define GV_MAINWIN_PLAYING              (1<<0)
#define GV_MAINWIN_PAUSED               (1<<1)
#define GV_MAINWIN_VALUE_UPDATE         (1<<2)
#define GV_MAINWIN_NO_SCALE_UPDATE      (1<<3)
#define GV_MAINWIN_NO_PLAYLIST_CALLBACK (1<<4)
#define GV_MAINWIN_NO_REALTIME          (1<<5)
#define GV_MAINWIN_NO_MENU_CALLBACK     (1<<6)
#define GV_MAINWIN_NO_TIME_UPDATE       (1<<7)

#define GV_MAINWIN_SEEKING_ACTIVE (GV_MAINWIN_NO_TIME_UPDATE|\
                                   GV_MAINWIN_NO_SCALE_UPDATE)

class GVSkin;
class GVButton;

class GVMainWindow
  {
  GVButton * play_button;
  GVButton * pause_button;
  GVButton * next_button;
  GVButton * previous_button;
  GVButton * stop_button;
  GVButton * menu_button;
  GVButton * close_button;
  
  GVSlider * seek_slider;

  GtkWidget * window;
  GtkWidget * layout;

  GdkPixmap * background_pixmap;
  GdkBitmap * background_mask;

  GtkWidget * drawingarea;
  
  int width;
  int height;

  // Saved values for moving
  
  int win_x_save;
  int win_y_save;

  int mouse_x_save;
  int mouse_y_save;

  int move_active;
  
public:  
  GVMainConfigWindow config_window;
private:
  GVPlaylistWidget playlist_widget;
  GVDisplayWidget  display_widget;

  int flags;
  int show_playlist;
  int show_info_window;
  
//   int playing;
//   int value_change;
  
  pthread_mutex_t state_mutex;
  int is_playing();

  gint idle_tag;

  GVTime total_track_time;

  GVRemote * remote;
  GtkWidget * button_table;

  int property_handler_id;

  // Menu stuff
  
  GtkWidget * menu;
  
  GtkWidget ** input_items;
  GtkWidget ** audio_output_items;
  GtkWidget ** video_output_items;
  int num_video_output_plugins;
  int num_audio_output_plugins;
  int num_input_plugins;

  GtkWidget * media_files_item;
  GtkWidget * configure_input_item;
  GtkWidget * configure_audio_output_item;
  GtkWidget * configure_video_output_item;
  GtkWidget * use_audio_item;
  
  //   GtkWidget * input_menu;

  GtkWidget * option_item;
  GtkWidget * option_menu;
  
  GtkWidget * option_auto_advance_item;
  GtkWidget * option_play_on_start_item;
  GtkWidget * option_sync_software_item;
  GtkWidget * option_preferences_item;
  GtkWidget * option_skins_item;

  GtkWidget * show_playlist_item;
  GtkWidget * show_video_item;
  GtkWidget * show_info_window_item;
  
  GtkWidget * quit_item;
  
  void create_menu();

  GtkWidget **
  create_plugin_menu(GVPluginLoader * plugin_loader,
                     GtkWidget * _menu,
                     GtkWidget * _config_item,
                     int & num_items,
                     GtkWidget * _media_files_item,
                     GtkWidget * _use_audio_item);
  
  GtkWidget * create_toggle_item(const char * name);
  GtkWidget * create_item(const char * name);
  GtkWidget * create_submenu_item(const char * name, GtkWidget * submenu);
  int get_toggle_item_state(GtkWidget * w);
  void set_toggle_item_state(GtkWidget * w, int value);

  void update_toggle_items();

public:

  void update_plugin_menus();
  
  void apply_skin(GVSkin & skin);
    
  void hide();
  
  gboolean timeout_func();
  
  GVMainWindow();
  virtual ~GVMainWindow();

  void expose_callback();
  void menu_callback(GtkWidget * w);
  void plugin_menu_callback(GtkWidget * w);

  void button_callback(GtkWidget * w);
  void gv_button_callback(GVButton * button);
  
  void property_callback(GdkEventProperty * event);

  void window_button_callback(GdkEventButton * event);
  void window_motion_callback(GdkEventMotion * event);

  void set_style_callback();
  
  void realize_callback();

  void seek_change_callback(float percengate);
  void seek_callback(float percengate);

  void run();

  void get_config();
  void put_config();
  };

extern GVMainWindow * gv_main_win;
