/*****************************************************************

  output.cc

  Copyright (c) 2001 by Burkhard Plaum - plaum@ipf.uni-stuttgart.de

  http://gmerlin.sourceforge.net

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111, USA.

*****************************************************************/

#include "config.h"

#include <gtk/gtk.h>
#include <pthread.h>
#include <ctype.h>
#include <sys/types.h>
#include <dirent.h>
#include <string.h>

#include <sys/stat.h>
#include <unistd.h>

#include <iostream>

using std::ostream;

#include "gvtime.h"
#include "gvconfig.h"
#include "videoformat.h"
#include "audioformat.h"
#include "buffer.h"
#include "audiobuf.h"
#include "videobuf.h"

#include "playlist.h"

#include "baseplugin.h"
#include "plugin.h"
#include "input.h"

#include "output.h"

#include "configwindow.h"
#include "globalconfig.h"
#include "engine.h"

using std::cerr;

void GVOutput::create_trackname()
  {
  if(trackname)
    g_free(trackname);
  trackname = g_strdup(input->get_trackname());
  }

GVOutput::GVOutput(const GVPluginAttributes * _attributes) :
  GVPlugin(_attributes)
  {
  do_audio = 1;
  do_video = 1;
  }

void GVOutput::alloc_frames(GVVideoFrame * frames, int num_frames) { }

void GVOutput::free_frames(GVVideoFrame * frame, int num_frames) { }

void GVOutput::alloc_frames(GVAudioFrame * frames, int num_frames) { }

void GVOutput::free_frames(GVAudioFrame * frame, int num_frames) { }

void GVOutput::put_frame(GVAudioFrame * frame) { }

void GVOutput::put_frame(GVVideoFrame * frame) { }

void GVOutput::playback_func() // Playback Function
  {
  int have_video = !!((flags & GV_PLUGIN_TRACK_HAS_VIDEO) && do_video);
  int have_audio = !!((flags & GV_PLUGIN_TRACK_HAS_AUDIO) && do_audio);
  
  //  cerr << attributes->name << ": playback_func() started\n";

  audio_samples_written = 0;
  video_frames_written = 0;

  internal_video_time.reset();
  internal_audio_time.reset();
  
  pthread_mutex_lock(&internal_time_mutex);
  internal_time.reset();
  pthread_mutex_unlock(&internal_time_mutex);

  //  GVEngineState state;
    
  GVAudioFrame * audio_frame;
  GVVideoFrame * video_frame;
  
  //  struct timespec long_time;

  GVTime frame_time;
  if(have_video && have_audio)
    frame_time= (audio_format.frame_time < video_format.frame_time) ?
      audio_format.frame_time : video_format.frame_time;
  else if(have_video)
    frame_time = video_format.frame_time;
  else if(have_audio)
    frame_time = audio_format.frame_time;
  
  GVTime short_time;
  GVTime long_time;
  
  long_time = frame_time;
  long_time /= 2;  
  
  short_time = frame_time;
  short_time /= 10;  

  //  pthread_mutex_lock(&destroy_mutex);

  gv_engine->plugin_attach();
  gv_engine->wait_for_start();

  video_completed = (flags & GV_PLUGIN_TRACK_HAS_VIDEO) ? 0 : 1;
  audio_completed = (flags & GV_PLUGIN_TRACK_HAS_AUDIO) ? 0 : 1;

  GVTime current_time;
  GVTime diff_time;
  
  int write_video = 0, write_audio = 0;

  if(have_video && !(have_audio))
    {
    write_video = 1;
    write_audio = 0;
    }
  else if(!(have_video) && have_audio)
    {
    write_video = 0;
    write_audio = 1;
    }
  
  while(1)
    {
    if(check_engine_state())
      break;

    // If we write both audio and video (into a file for example)
    // we'll write the one which has a lower time
    
    if(have_video && have_audio)
      {
      if(internal_video_time > internal_audio_time)
        {
        write_audio = 1;
        write_video = 0;
        }
      else
        {
        write_audio = 0;
        write_video = 1;
        }
      }

//     cerr << "Internal audio time: " << internal_audio_time.seconds
//          << " Internal video time: " << internal_video_time.seconds
//          << " Write Audio :" << write_audio 
//          << " Write Video :" << write_video << "\n";
    
    if((audio_completed) && (video_completed))
      break;

    if(write_audio)
      {
      if(audio_buffer->empty())
        {
        if(gv_engine->get_state() == GVEngineCompleting)
          {
          audio_completed = 1;
          break;
          }
        else
          {
          short_time.sleep();
          continue;
          }
        }
      audio_frame = audio_buffer->lock_read();
      put_frame(audio_frame);
      audio_samples_written += audio_frame->valid_samples;
      internal_audio_time = audio_frame->time;
      audio_buffer->unlock_read();
      }
    if(write_video)
      {
      if(video_buffer->empty())
        {
        if(gv_engine->get_state() == GVEngineCompleting)
          {
          video_completed = 1;
          break;
          }
        else
          {
          short_time.sleep();
          continue;
          }
        }
      video_frame = video_buffer->lock_read();

      // Do audio/video sync

      if(realtime)
        {
        gv_engine->get_current_track_time(current_time, 1);
        if(current_time > video_frame->time)
          {
          diff_time = video_frame->time;
          diff_time -= current_time;
          diff_time.sleep();
          }
        }
      put_frame(video_frame);
      internal_video_time = video_frame->time;
      video_frames_written++;
      video_buffer->unlock_read();
      }

    if(!have_audio)
      {
      pthread_mutex_lock(&internal_time_mutex);
      internal_time = internal_video_time;
      pthread_mutex_unlock(&internal_time_mutex);
      }
    else if(!have_video)
      {
      pthread_mutex_lock(&internal_time_mutex);
      internal_time = internal_audio_time;
      pthread_mutex_unlock(&internal_time_mutex);
      }
    else
      {
      pthread_mutex_lock(&internal_time_mutex);
      internal_time = (internal_video_time > internal_audio_time)
        ? internal_video_time : internal_audio_time;
      pthread_mutex_unlock(&internal_time_mutex);
      }
    if(realtime)
      long_time.sleep();
    }
  cleanup();

  if(have_video)
    cerr << " Video Frames Written: " << video_frames_written
         << " Total frames: " << video_format.num_frames << "\n";
  if(have_audio)
    cerr << " Audio Samples written: " << audio_samples_written
         << " Total Samples: " << audio_format.num_samples << "\n";
  gv_engine->plugin_detach();
  }

// Create an output filename, return 1 on Success, return 0 if
// name of output file is the same as the name of the input file

int GVOutput::create_outfilename(const char * suffix)
  {
  if(filename)
    g_free(filename);

  const GVPlaylistTrack * playlist_track = gv_playlist->get_current_track();
    
  filename = g_strdup_printf("%s%s.%s", gv_global_config->output_directory,
                             playlist_track->trackname, suffix);

  if((input->attributes->flags & GV_PLUGIN_IS_FILE) &&
     !strcmp(filename, input->get_filename()))
    {
    g_log(PACKAGE, G_LOG_LEVEL_CRITICAL,
          "Input and output are the same file!");
    return 0;
    }
  return 1;
  }

void GVOutput::resync() // Get the current times from the input plugin
  {
  int have_video = !!((flags & GV_PLUGIN_TRACK_HAS_VIDEO) && do_video);
  int have_audio = !!((flags & GV_PLUGIN_TRACK_HAS_AUDIO) && do_audio);
  
  pthread_mutex_lock(&internal_time_mutex);
  input->get_internal_video_time(internal_video_time);
  input->get_internal_audio_time(internal_audio_time);

  if(!have_audio)
    internal_time = internal_video_time;
  else if(!have_video)
    internal_time = internal_audio_time;
  else
    internal_time = (internal_video_time > internal_audio_time)
      ? internal_video_time : internal_audio_time;

  cerr << attributes->name << " resync, audio time: "
       << internal_audio_time.get_double()
       << ", video time: " << internal_video_time.get_double()
       << ", internal time: " << internal_time.get_double() << "\n";
  
  pthread_mutex_unlock(&internal_time_mutex);
  }

GVOutput::~GVOutput() { }
