/*****************************************************************

  playlistio.cc

  Copyright (c) 2001 by Burkhard Plaum - plaum@ipf.uni-stuttgart.de

  http://gmerlin.sourceforge.net

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111, USA.

*****************************************************************/

// Routines for reading and writing playlists

#include "config.h"

#include <glib.h>
#include <pthread.h>
#include <iostream>
#include <fstream>

using std::ifstream;
using std::ofstream;

#include <limits.h>
#include <string.h>
#include <stdlib.h>
#include <ctype.h>

#include "gvtime.h"
#include "gvconfig.h"
#include "videoformat.h"
#include "audioformat.h"
#include "buffer.h"
#include "audiobuf.h"
#include "videobuf.h"

#include "playlist.h"

#include "baseplugin.h"
#include "plugin.h"
#include "input.h"

#include "output.h"
#include "audioout.h"
#include "videoout.h"

#include <gtk/gtk.h>
#include "configwindow.h"
#include "engine.h"

#include "gvutil.h"

using std::cerr;

// Playlist I/O routines
// Format of verbose playlist is:
// "<filename1>" "<trackname1>" "<plugin1>" <flags1> <seconds1> <nanoseconds1>
// "<filename2>" "<trackname2>" "<plugin2>" <flags2> <seconds2> <nanoseconds2>

#define VERBOSE_PLAYLIST

#define BEGIN_ALBUM_STRING "BeginAlbum"
#define END_ALBUM_STRING   "EndAlbum"

#define STRING_DELIMITER '"'
#define NULL_STRING "NULL"

void GVPlaylist::read(const char * playlistfilename)
  {
  cerr << "Reading playlist file " << playlistfilename << "\n";
  current_track = (GVPlaylistTrack*)0;
  ifstream input;
  input.open(playlistfilename);
  if(!input)
    return;
  clear();

  char * buffer = new char[PATH_MAX+2];

  const char * filename;
  const char * trackname;
  const char * plugin;
  const char * album_name;
  int flags;
  char * pos;
  char * end;
  GVTime time;
  GVPlaylistTrack * new_item;

  while(1)
    {
    input.getline(buffer, PATH_MAX+2);
    //      input.get(delimiter);
    if(input.eof())
      break;

    // Check if we have a new album

    if(!strcmp(buffer, BEGIN_ALBUM_STRING))
      {
      pos = buffer;
      album_name = gv_read_string(pos, &end);
      
      }
    else
      {
// Format of verbose playlist is:
// "<filename1>" "<trackname1>" "<plugin1>" <flags1> <seconds1> <nanoseconds1>

      pos = buffer;
      trackname = gv_read_string(pos, &end);
      pos = end;
      filename = gv_read_string(pos, &end);
      pos = end;
      plugin = gv_read_string(pos, &end);
      pos = end;
      flags = strtol(pos, &end, 10);
      pos = end;
      time.seconds = strtol(pos, &end, 10);
      pos = end;
      time.nanoseconds = strtol(pos, &end, 10);
      
      new_item = new GVPlaylistTrack(filename, trackname, time, flags, plugin);
      current_tracks = g_list_append(current_tracks, (gpointer)new_item);
      num_tracks++;
      if(flags & GV_PLAYLIST_ITEM_IS_CURRENT)
        current_track = new_item;
      }
    }
  input.close();
  if(!current_track)
    {
    current_track = (GVPlaylistTrack*)g_list_nth_data(current_tracks, 0);
    if(current_track)
      current_track->flags |= GV_PLAYLIST_ITEM_IS_CURRENT;
    }
  delete buffer;
  set_state_flag(GV_PLAYLIST_CONTENTS_CHANGED|GV_PLAYLIST_CURRENT_CHANGED);
  }

void GVPlaylist::write(const char * playlistfilename)
  {
  // Never write a private playlist

  if(is_private())
    return;
    
  ofstream output;
  cerr << "Writing playlist file " << playlistfilename << "\n";
  output.open(playlistfilename);
  if(!output)
    return;
  const GVPlaylistTrack * item;
  for(int i = 0; i < num_tracks; i++)
    {
    item = get_track(i);
    if(!(item->flags & GV_PLAYLIST_ITEM_NOSAVE))
      {
      gv_write_string(output, item->trackname);
      output << " ";
      gv_write_string(output, item->filename);
      output << " ";
      gv_write_string(output, item->plugin);
      output << " " << item->flags << " "
             << item->time.seconds << " " << item->time.nanoseconds << "\n";
      }
    }
  output.close();
  }
