/*****************************************************************

  playliststyle.cc

  Copyright (c) 2001 by Burkhard Plaum - plaum@ipf.uni-stuttgart.de

  http://gmerlin.sourceforge.net

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111, USA.

*****************************************************************/

// Routines for the playlist style

#include <gtk/gtk.h>
#include <sstream>
using std::stringstream;

#include <iomanip.h>
#include <string.h>

#include "gv.h"

#include "gvgtk.h"

void GVPlaylistWidget::update_style()
  {
  if(!is_realized)
    return;

  GdkColor test_color;
  gdk_color_parse("#FF8000", &test_color);

  gdk_color_alloc(gtk_widget_get_colormap(list),
                  &test_color);
    
  // A small notice....
  // GTK_STATE_NORMAL,
  // GTK_STATE_ACTIVE,
  // GTK_STATE_PRELIGHT,
  // GTK_STATE_SELECTED,
  // GTK_STATE_INSENSITIVE

  // GdkColor current_color;
  // GdkColor normal_color;
  // GdkColor selected_background_color;
  // GdkColor background_color;

  //  gtk_widget_ensure_style(list);

  GtkRcStyle *rc_style;

  int color_flags[5];

  for(int i = 0; i < 5; i++)
    color_flags[i] = 0;
  
  rc_style = gtk_rc_style_new();
  
  rc_style->bg[GTK_STATE_NORMAL]      = background_color; // Frame around the list
  rc_style->bg[GTK_STATE_SELECTED]    = selected_background_color;
 
  rc_style->bg[GTK_STATE_ACTIVE]      = background_color;
  rc_style->bg[GTK_STATE_PRELIGHT]    = background_color;
  rc_style->bg[GTK_STATE_INSENSITIVE] = background_color;

  for(int i = 0; i < 5; i++)
    color_flags[i] |= GTK_RC_BG;
    
  for(int i = 0; i < 5; i++)
    {
    rc_style->fg[i]  = normal_color;
    color_flags[i] |= GTK_RC_FG;
    }

  // Background of the whole list
  
  rc_style->base[GTK_STATE_NORMAL]      = background_color;
  rc_style->base[GTK_STATE_SELECTED]    = selected_background_color;

  // Lines between the rows

  rc_style->base[GTK_STATE_ACTIVE]      = background_color;
  rc_style->base[GTK_STATE_PRELIGHT]    = selected_background_color;
  rc_style->base[GTK_STATE_INSENSITIVE] = background_color;

  for(int i = 0; i < 5; i++)
    color_flags[i] |= GTK_RC_BASE;
  
  for(int i = 0; i < 5; i++)
    {
    rc_style->text[i] = normal_color;
    color_flags[i] |= GTK_RC_TEXT;
    }

  for(int i = 0; i < 5; i++)
    rc_style->color_flags[i] = (GtkRcFlags)color_flags[i];

//   gdk_window_set_background(list->window, &background_color);
//   gdk_window_set_background(GTK_CLIST(list)->clist_window, &background_color);
  
  gtk_widget_modify_style(list, rc_style);

  // Make the current style

  if(current_style)
    gtk_style_unref(current_style);

  current_style = gtk_style_copy(gtk_widget_get_style(list));

  for(int i = 0; i < 5; i++)
    {
    current_style->fg[i]   = current_color;
    current_style->text[i] = current_color;
    }

  // Draw the new pixmaps

  gdk_gc_set_foreground(gc, &current_color);
  gdk_draw_rectangle(video_pixmap_s, gc, 1, 0, 0,
                     video_pixmap_width, video_pixmap_height);
  gdk_draw_rectangle(audio_pixmap_s, gc, 1, 0, 0,
                     audio_pixmap_width, audio_pixmap_height);

  gdk_gc_set_foreground(gc, &normal_color);
  gdk_draw_rectangle(video_pixmap, gc, 1, 0, 0,
                     video_pixmap_width, video_pixmap_height);
  gdk_draw_rectangle(audio_pixmap, gc, 1, 0, 0,
                     audio_pixmap_width, audio_pixmap_height);

  // 

  //  gtk_style_unref(style);

  gtk_rc_style_unref(rc_style);
  
  //  gtk_widget_ensure_style(list);
  // 
  //  gtk_widget_queue_draw(list);  
  update_album();
  }

void GVPlaylistWidget::create_style()
  {
  //  GtkStyle *default_style;
  //  GtkStyle *sty;
  //  int state;

  //  default_style = gtk_widget_get_default_style();
  //  style = gtk_rc_style_new();
  //  gtk_style_ref(style);

  //  gtk_widget_set_style(list, style);
  
  }
