/*****************************************************************

  playlistwidget.cc

  Copyright (c) 2001 by Burkhard Plaum - plaum@ipf.uni-stuttgart.de

  http://gmerlin.sourceforge.net

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111, USA.

*****************************************************************/

// TODO: Write a short explanation here!!!

#include <gtk/gtk.h>
#include <sstream>
using std::stringstream;

#include <iomanip.h>
#include <string.h>

#include "gv.h"

#include "gvgtk.h"

#include "skin.h"
#include "skinbrowser.h"
#include "urlwindow.h"
#include "fileselect.h"


#include "pixmaps/has_audio.xpm"
#include "pixmaps/has_video.xpm"

#define PIXMAP_WIDTH 12

#define COLUMN_INDEX     0
#define COLUMN_NAME      1
#define COLUMN_HAS_VIDEO 2
#define COLUMN_HAS_AUDIO 3
#define COLUMN_TIME      4

#include "gvconfig.h"
#include "globalconfig.h"

#define SECTION "Playlist"

static gboolean __delete_callback(GtkWidget * w, GdkEventAny * event,
                                  gpointer data)
  {
  ((GVPlaylistWidget*)data)->button_callback(w);
  return TRUE;
  }

static gboolean __list_button_callback(GtkWidget * w,
                                       GdkEventButton * event,
                                       gpointer data)
  {
  //  cerr << "Button Callback\n";
  ((GVPlaylistWidget*)data)->list_button_callback(event);
  return TRUE;
  }

static gboolean __list_motion_callback(GtkWidget * w,
                                       GdkEventMotion * event,
                                       gpointer data)
  {
  return ((GVPlaylistWidget*)data)->list_motion_callback(event);
  //  cerr << "Motion Callback\n";
  //  return TRUE;
  }

static void __button_callback(GtkWidget * w, gpointer data)
  {
  ((GVPlaylistWidget*)data)->button_callback(w);
  }

static void __dnd_callback(GtkWidget * w, GdkDragContext *context,
                           gint x, gint y,
                           GtkSelectionData *selection_data,
                           guint info, guint time, gpointer data)
  {
  ((GVPlaylistWidget*)data)->dnd_callback(w, context, x, y, selection_data,
                                          info, time);
  //  cerr << selection_data->data << "dropped here\n";
  }

void GVPlaylistWidget::before_resize_hook(int old_width, int old_height,
                                          int & new_width, int & new_height)
  {
  if(new_width < 200)
    new_width = 200;
  if(old_width < 100)
    old_width = 100;
  
  int delta_x = new_width - old_width;

  
  
  column_1_width += delta_x;
  
  // Calculate the width of the other columns

  gtk_clist_set_column_width(GTK_CLIST(list), COLUMN_NAME, column_1_width);
  }


void GVPlaylistWidget::dnd_callback(GtkWidget * w,
                                    GdkDragContext *context,
                                    gint x, gint y,
                                    GtkSelectionData *selection_data,
                                    guint info, guint time)
  {
  int num_items = 0;

  for(int i = 0; i < selection_data->length; i++)
    {
    if(selection_data->data[i] == '\n')
      num_items++;
    }

  char * pos1 = (char*)selection_data->data;
  char * pos2;
  char * filename;
  
  for(int i = 0; i < num_items; i++)
    {
    pos2 = strchr(pos1, '\r');
    if(!strncmp(pos1, "file:/", 6))
      {
      pos1 += 5;
      filename = new char[pos2 - pos1 + 1];
      strncpy(filename, pos1, pos2 - pos1);
      filename[pos2 - pos1] = '\0';
      filename = g_strndup(pos1, pos2 - pos1);
      //      cerr << "Adding File: " << filename;
      gv_engine->add_file(filename);

      g_free(filename);
      }
    pos1 = pos2;
    while((*pos1 == '\n') || (*pos1 == '\r')) 
      pos1++;
    }
  }

void GVPlaylistWidget::button_callback(GtkWidget * w)
  {
  if(w == add_item)
    {
    if(!fileselect)
      fileselect = new GVFileSelect;
    fileselect->show();
    }
  if(w == add_url_item)
    {
    if(!url_window)
      url_window = new GVURLWindow;
    url_window->show();
    }

  else if(w == track_remove_item)
    {
    gv_playlist->remove_selected();
    //    update_album();
//     if(!gv_playlist->get_num_tracks())
//       gv_engine->stop();
    }
  else if(w == track_refresh_item)
    {
    if(!gv_playlist->is_private())
      {
      for(int i = 0; i < num_tracks; i++)
        {
        if(playlist_items[i]->flags & GV_PLAYLIST_ITEM_IS_SELECTED)
          gv_engine->refresh_track(i);
        }
      }
    else
      gv_engine->reset_input();
    
    //    gv_playlist->refresh_selected();
    }
  
  else if(w == track_rename_item)
    {
    if(last_clicked_row != -1)
      {
      if(!rename_window)
        rename_window = new RenameWindow(this);
      rename_window->run();
      }
    }
  else if(w == list_sort_item)
    {
    //    fileselect.show();
    gv_playlist->sort();
    update_album();
    }
  else if(w == list_clear_item)
    {
    gv_playlist->clear();
    update_album();
    }
  else if(w == GTK_FILE_SELECTION(playlist_fileselect)->ok_button)
    {
    if(load_playlist)
      {
      gv_playlist->read(gtk_file_selection_get_filename(GTK_FILE_SELECTION(playlist_fileselect)));
      update_album();
      }
    else
      gv_playlist->write(gtk_file_selection_get_filename(GTK_FILE_SELECTION(playlist_fileselect)));
    gtk_widget_hide(playlist_fileselect);
    gtk_main_quit();
    }
  else if(w == GTK_FILE_SELECTION(playlist_fileselect)->cancel_button)
    {
    gtk_main_quit();
    gtk_widget_hide(playlist_fileselect);
    }
  else if(w == list_load_item)
    {
    load_playlist = 1;
    gtk_window_set_title(GTK_WINDOW(playlist_fileselect), "Load Playlist");
    gtk_widget_show(playlist_fileselect);
    gtk_main();
    }
  else if(w == list_save_item)
    {
    load_playlist = 0;
    gtk_window_set_title(GTK_WINDOW(playlist_fileselect), "Save Playlist");
    gtk_widget_show(playlist_fileselect);
    gtk_main();
    }
  else if(w == window)
    hide();
  else if(w == options_item)
    gv_main_win->config_window.run(&config_widget);
  }

static char * playlist_titles[] =
  {
  "000" ,
  "1" ,
  "1" ,
  "1" ,
  "00:00:00" ,
  };

GVPlaylistWidget::GVPlaylistWidget() :
  config_widget(this)
  {
  rename_window = (RenameWindow*)0;
  set_resizeable(1);
  is_realized = 0;
  get_config();
  // Create the objects

  no_update = 0;
  no_resize = 0;
  
  column_1_width = 200;

  fileselect = new GVFileSelect;
  url_window = new GVURLWindow;
  
  //
  // Create the menu
  // 

  GtkWidget * tearoff_item;
  
  main_menu = gtk_menu_new();

  tearoff_item = gtk_tearoff_menu_item_new();
  gtk_widget_show(tearoff_item);
  gtk_menu_append(GTK_MENU(main_menu), tearoff_item);
  
  add_item = gtk_menu_item_new_with_label("Add File(s)");
  gtk_signal_connect(GTK_OBJECT(add_item), "activate",
                     GTK_SIGNAL_FUNC(__button_callback), (gpointer)this);
  gtk_widget_show(add_item);
  gtk_menu_append(GTK_MENU(main_menu), add_item);

  add_url_item = gtk_menu_item_new_with_label("Add URL(s)");
  gtk_signal_connect(GTK_OBJECT(add_url_item), "activate",
                     GTK_SIGNAL_FUNC(__button_callback), (gpointer)this);
  gtk_widget_show(add_url_item);
  gtk_menu_append(GTK_MENU(main_menu), add_url_item);
  
  track_item = gtk_menu_item_new_with_label("Tracks(s)");
  gtk_menu_append(GTK_MENU(main_menu), track_item);

  track_menu = gtk_menu_new();

  tearoff_item = gtk_tearoff_menu_item_new();
  gtk_widget_show(tearoff_item);
  gtk_menu_append(GTK_MENU(track_menu), tearoff_item);
  
  track_remove_item = gtk_menu_item_new_with_label("Remove");
  gtk_signal_connect(GTK_OBJECT(track_remove_item), "activate",
                     GTK_SIGNAL_FUNC(__button_callback), (gpointer)this);
  gtk_widget_show(track_remove_item);
  gtk_menu_append(GTK_MENU(track_menu), track_remove_item);

  track_rename_item = gtk_menu_item_new_with_label("Rename");
  gtk_signal_connect(GTK_OBJECT(track_rename_item), "activate",
                     GTK_SIGNAL_FUNC(__button_callback), (gpointer)this);
  gtk_widget_show(track_rename_item);
  gtk_menu_append(GTK_MENU(track_menu), track_rename_item);
  
  track_refresh_item =
    gtk_menu_item_new_with_label("Refresh");

  gtk_signal_connect(GTK_OBJECT(track_refresh_item), "activate",
                     GTK_SIGNAL_FUNC(__button_callback), (gpointer)this);
  gtk_widget_show(track_refresh_item);
  gtk_menu_append(GTK_MENU(track_menu), track_refresh_item);

  gtk_widget_show(track_menu);
  gtk_menu_item_set_submenu(GTK_MENU_ITEM(track_item), track_menu);
  gtk_widget_show(track_item);
  
  list_item = gtk_menu_item_new_with_label("List");
  gtk_menu_append(GTK_MENU(main_menu), list_item);

  list_menu = gtk_menu_new();

  tearoff_item = gtk_tearoff_menu_item_new();
  gtk_widget_show(tearoff_item);
  gtk_menu_append(GTK_MENU(list_menu), tearoff_item);
  
  list_load_item = gtk_menu_item_new_with_label("Load...");
  gtk_signal_connect(GTK_OBJECT(list_load_item), "activate",
                     GTK_SIGNAL_FUNC(__button_callback), (gpointer)this);
  gtk_widget_show(list_load_item);
  gtk_menu_append(GTK_MENU(list_menu), list_load_item);

  list_save_item = gtk_menu_item_new_with_label("Save...");
  gtk_signal_connect(GTK_OBJECT(list_save_item), "activate",
                     GTK_SIGNAL_FUNC(__button_callback), (gpointer)this);
  gtk_widget_show(list_save_item);
  gtk_menu_append(GTK_MENU(list_menu), list_save_item);

  list_sort_item = gtk_menu_item_new_with_label("Sort");
  gtk_signal_connect(GTK_OBJECT(list_sort_item), "activate",
                     GTK_SIGNAL_FUNC(__button_callback), (gpointer)this);
  gtk_widget_show(list_sort_item);
  gtk_menu_append(GTK_MENU(list_menu), list_sort_item);

  list_clear_item = gtk_menu_item_new_with_label("Clear");

  gtk_signal_connect(GTK_OBJECT(list_clear_item), "activate",
                     GTK_SIGNAL_FUNC(__button_callback), (gpointer)this);
  gtk_widget_show(list_clear_item);
  gtk_menu_append(GTK_MENU(list_menu), list_clear_item);

  gtk_widget_show(list_menu);
  gtk_menu_item_set_submenu(GTK_MENU_ITEM(list_item), list_menu);
  gtk_widget_show(list_item);

  options_item = gtk_menu_item_new_with_label("Options...");
  gtk_signal_connect(GTK_OBJECT(options_item), "activate",
                     GTK_SIGNAL_FUNC(__button_callback), (gpointer)this);
  gtk_widget_show(options_item);
  gtk_menu_append(GTK_MENU(main_menu), options_item);
  
  gtk_widget_show(main_menu);
  
  //
  //

  set_ul_menu(main_menu);
  
  list = gtk_clist_new_with_titles(5, playlist_titles);

  //  gtk_widget_set_name(list, "Playlist");

  //  style = gtk_rc_style_new();
  //  gtk_rc_add_widget_name_style(style, "*Playlist");

  //   remove_button  = gtk_button_new_with_label("Remove");
  //   add_button     = gtk_button_new_with_label("Add");
  //   sort_button    = gtk_button_new_with_label("Sort");
  //   refresh_button = gtk_button_new_with_label("Refresh");
  //   load_button = gtk_button_new_with_label("Load");
  //   save_button = gtk_button_new_with_label("Save");

  playlist_fileselect = gtk_file_selection_new("");
  gtk_window_set_modal(GTK_WINDOW(playlist_fileselect), TRUE);

  //  window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
  
  // Set attributes

  gtk_clist_column_titles_hide(GTK_CLIST(list));

  gtk_clist_set_column_justification(GTK_CLIST(list),
                                     COLUMN_INDEX, GTK_JUSTIFY_RIGHT);
  gtk_clist_set_column_justification(GTK_CLIST(list),
                                     COLUMN_NAME, GTK_JUSTIFY_LEFT);
  gtk_clist_set_column_justification(GTK_CLIST(list),
                                     COLUMN_HAS_VIDEO, GTK_JUSTIFY_CENTER);
  gtk_clist_set_column_justification(GTK_CLIST(list),
                                     COLUMN_HAS_AUDIO, GTK_JUSTIFY_CENTER);
  gtk_clist_set_column_justification(GTK_CLIST(list),
                                     COLUMN_TIME, GTK_JUSTIFY_RIGHT);

  set_name("Playlist");
  
  // FIXME: The next lines have to be changed
  
  gtk_clist_set_column_auto_resize(GTK_CLIST(list), 0, TRUE);
  gtk_clist_set_column_auto_resize(GTK_CLIST(list), 1, FALSE);
  gtk_clist_set_column_auto_resize(GTK_CLIST(list), 2, TRUE);
  gtk_clist_set_column_auto_resize(GTK_CLIST(list), 3, TRUE);
  gtk_clist_set_column_auto_resize(GTK_CLIST(list), 4, TRUE);

  //  gtk_clist_set_column_min_width(GTK_CLIST(list), 1, 200);
  
  gtk_clist_set_column_width(GTK_CLIST(list), COLUMN_HAS_VIDEO, PIXMAP_WIDTH);
  gtk_clist_set_column_width(GTK_CLIST(list), COLUMN_HAS_AUDIO, PIXMAP_WIDTH);
  gtk_clist_set_column_width(GTK_CLIST(list), COLUMN_NAME, 200);
  gtk_clist_set_column_width(GTK_CLIST(list), COLUMN_TIME, 20);
  
//   gtk_clist_set_column_resizeable(GTK_CLIST(list), 0, FALSE);
//   gtk_clist_set_column_resizeable(GTK_CLIST(list), 1, TRUE);
//   gtk_clist_set_column_resizeable(GTK_CLIST(list), 0, FALSE);
//   gtk_clist_set_column_resizeable(GTK_CLIST(list), 0, FALSE);
//   gtk_clist_set_column_resizeable(GTK_CLIST(list), 0, FALSE);
  
  gtk_clist_set_selection_mode(GTK_CLIST(list), GTK_SELECTION_EXTENDED);
  
  gtk_widget_set_events(list, GDK_BUTTON_PRESS_MASK|
                        GDK_BUTTON1_MOTION_MASK|GDK_BUTTON_RELEASE_MASK);
  
  // Set the dnd callback 

  GtkTargetEntry target_entry;
  target_entry.target = "text/plain";
  target_entry.flags = 0; // GTK_TARGET_SAME_WIDGET|GTK_TARGET_SAME_APP;
  target_entry.info = 0;

  gtk_drag_dest_set(list, GTK_DEST_DEFAULT_ALL,
                    &target_entry, 1, GDK_ACTION_COPY);

  gtk_signal_connect(GTK_OBJECT(list), "drag_data_received",
                     GTK_SIGNAL_FUNC(__dnd_callback),
                     (gpointer)this);

  // Set Callbacks

//   gtk_signal_connect(GTK_OBJECT(list), "realize",
//                      GTK_SIGNAL_FUNC(__realize_callback),
//                      (gpointer)this);
  
  
//   gtk_signal_connect(GTK_OBJECT(list), "select_row",
//                      GTK_SIGNAL_FUNC(__select_row_callback),
//                      (gpointer)this);
//   gtk_signal_connect(GTK_OBJECT(list), "unselect_row",
//                      GTK_SIGNAL_FUNC(__unselect_row_callback),
//                      (gpointer)this);
//   gtk_signal_connect(GTK_OBJECT(list), "row_move",
//                      GTK_SIGNAL_FUNC(__row_move_callback),
//                      (gpointer)this);
  
//   gtk_signal_connect(GTK_OBJECT(add_button), "clicked",
//                      GTK_SIGNAL_FUNC(__button_callback),
//                      (gpointer)this);
//   gtk_signal_connect(GTK_OBJECT(remove_button), "clicked",
//                      GTK_SIGNAL_FUNC(__button_callback),
//                      (gpointer)this);
//   gtk_signal_connect(GTK_OBJECT(sort_button), "clicked",
//                      GTK_SIGNAL_FUNC(__button_callback),
//                      (gpointer)this);
//   gtk_signal_connect(GTK_OBJECT(refresh_button), "clicked",
//                      GTK_SIGNAL_FUNC(__button_callback),
//                      (gpointer)this);
//   gtk_signal_connect(GTK_OBJECT(load_button), "clicked",
//                      GTK_SIGNAL_FUNC(__button_callback),
//                      (gpointer)this);
//   gtk_signal_connect(GTK_OBJECT(save_button), "clicked",
//                      GTK_SIGNAL_FUNC(__button_callback),
//                      (gpointer)this);

  gtk_signal_connect(GTK_OBJECT(GTK_FILE_SELECTION(playlist_fileselect)->ok_button),
                     "clicked",
                     GTK_SIGNAL_FUNC(__button_callback),
                     (gpointer)this);
  gtk_signal_connect(GTK_OBJECT(GTK_FILE_SELECTION(playlist_fileselect)->cancel_button),
                     "clicked",
                     GTK_SIGNAL_FUNC(__button_callback),
                     (gpointer)this);

  // We want to disable the selection behaviour of the GtkClist, so we
  // use gtk_signal_connect_after let the signal handlers return TRUE
  
  gtk_signal_connect_after(GTK_OBJECT(list), "button_press_event",
                           (GtkSignalFunc) __list_button_callback,
                           (gpointer)this);
  gtk_signal_connect_after(GTK_OBJECT(list), "button_release_event",
                           (GtkSignalFunc) __list_button_callback,
                           (gpointer)this);
  gtk_signal_connect_after(GTK_OBJECT(list), "motion_notify_event",
                           (GtkSignalFunc) __list_motion_callback,
                           (gpointer)this);


//   gtk_signal_connect(GTK_OBJECT(list), "size_allocate",
//                      (GtkSignalFunc) __list_resize_callback, (gpointer)this);

  gtk_signal_connect(GTK_OBJECT(window), "delete_event",
                     (GtkSignalFunc)__delete_callback, (gpointer)this);
  
  // Show the widgets

//   gtk_widget_show(add_button);
//   gtk_widget_show(remove_button);
//   gtk_widget_show(sort_button);
//   gtk_widget_show(refresh_button);
//   gtk_widget_show(load_button);
//   gtk_widget_show(save_button);
  gtk_widget_show(list);
  
  // Place the widgets

  GtkWidget * mainbox = gtk_vbox_new(FALSE, 5);
  
  //  widget = 

  scrolledwin
    = gtk_scrolled_window_new(gtk_clist_get_hadjustment(GTK_CLIST(list)),
                              gtk_clist_get_vadjustment(GTK_CLIST(list)));

  gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrolledwin),
                                 GTK_POLICY_NEVER, GTK_POLICY_ALWAYS);
  
  gtk_container_add(GTK_CONTAINER(scrolledwin), list);

  gtk_widget_show(scrolledwin);

  //  gtk_container_add(GTK_CONTAINER(event_box), scrolledwin);

  //  gtk_widget_show(event_box);
  
  gtk_box_pack_start_defaults(GTK_BOX(mainbox), scrolledwin);
  
//   GtkWidget * hbox = gtk_hbox_new(TRUE, 0);
//   gtk_box_pack_start_defaults(GTK_BOX(hbox), add_button);
//   gtk_box_pack_start_defaults(GTK_BOX(hbox), remove_button);
//   gtk_box_pack_start_defaults(GTK_BOX(hbox), sort_button);
//   gtk_box_pack_start_defaults(GTK_BOX(hbox), refresh_button);
//   gtk_box_pack_start_defaults(GTK_BOX(hbox), load_button);
//   gtk_box_pack_start_defaults(GTK_BOX(hbox), save_button);

//   gtk_widget_show(hbox);
  
//   gtk_box_pack_start(GTK_BOX(mainbox), hbox, FALSE, FALSE, 0);

  gtk_widget_show(mainbox);
  
  //  gtk_container_add(GTK_CONTAINER(window), mainbox);

  attach(mainbox);
  
  //  gtk_widget_show(window);
  //  gtk_widget_show(widget);
  
  //  widget = mainbox;

  num_tracks = 0;
  dragged_row = -1;
  last_clicked_row = -1;
  playlist_items = (const GVPlaylistTrack**)0;

  create_style();
  
  gtk_widget_realize(list);

  realize();

  realize_callback();
  
  current_style = (GtkStyle *)0;
  
  update_style();

  }

void GVPlaylistWidget::show()
  {
  if(is_visible())
    return;
  gtk_widget_show(window);
  GVWindow::flags |= GV_WINDOW_IS_VISIBLE;
  update_style();
  }


static char * list_text[5] =
{ " ", " ", " ", " ", " "};

void GVPlaylistWidget::update_album()
  {
  if(no_update)
    return;
  
  if(!is_realized)
    return;
  num_tracks = gv_playlist->get_num_tracks();

  num_selected = 0;
  
  stringstream line;

  int track_seconds;
  int track_minutes;
  int track_hours;
  int time;

  //  cerr << num <<  " Tracks\n"; 

  // Remember the first position in the list

  int first_visible_row = 0;
  
  for(int i = 0; i < num_tracks; i++)
    {
    if(gtk_clist_row_is_visible(GTK_CLIST(list), i) != GTK_VISIBILITY_NONE)
      {
      first_visible_row = i;
      break;
      }
    }
  gtk_clist_freeze(GTK_CLIST(list));
  gtk_clist_clear(GTK_CLIST(list));

  if(playlist_items)
    delete playlist_items;
  
  playlist_items = new const GVPlaylistTrack*[num_tracks];
  
  for(int i = 0; i < num_tracks; i++)
    {
    gtk_clist_append(GTK_CLIST(list), list_text);
    
    playlist_items[i] = gv_playlist->get_track(i);

    // Track number

    //    line.freeze(0);
    line.seekp(0);
    line << i+1 << "." << ends;
    gtk_clist_set_text(GTK_CLIST(list), i, COLUMN_INDEX, line.str().c_str());

    // Track name

    gtk_clist_set_text(GTK_CLIST(list), i, COLUMN_NAME,
                       playlist_items[i]->trackname);

    if(playlist_items[i]->time.undefined())
      gtk_clist_set_text(GTK_CLIST(list), i, COLUMN_TIME, "??:??");
    else
      {
      // Time
      time = playlist_items[i]->time.seconds;
      
      track_seconds = time % 60;
      time /= 60;
      
      track_minutes = time % 60;
      time /= 60;
      
      track_hours = time % 99;
      //      line.freeze(0);
      line.seekp(0);
      
      int printed = 0;
      
      if(track_hours)
        {
        if(track_hours >= 10)
          line << setw(2) << setfill('0') << track_hours << ":";
        else
          line << setw(1) << track_hours << ":";
        printed = 1;
        }
      
      if((track_minutes) || (printed))
        {
        if((track_hours >= 10) || (printed))
          line << setw(2) << setfill('0') << track_minutes;
        else
          line << setw(1) << track_minutes;
        }
      
      line << ":" << setw(2) << setfill('0') << track_seconds << ends;
      
      gtk_clist_set_text(GTK_CLIST(list), i, COLUMN_TIME, line.str().c_str());
      }
    
    if(playlist_items[i]->flags & GV_PLAYLIST_ITEM_IS_CURRENT)
      {
      gtk_clist_set_row_style(GTK_CLIST(list), i, current_style);
      
      //      gtk_clist_set_foreground(GTK_CLIST(list), i, &current_color);
      
      // Video Present
      
      if(playlist_items[i]->flags & GV_PLAYLIST_ITEM_HAS_VIDEO)
        gtk_clist_set_pixmap(GTK_CLIST(list),
                             i, COLUMN_HAS_VIDEO, video_pixmap_s,
                             video_mask);
      // Audio Present
      
      if(playlist_items[i]->flags & GV_PLAYLIST_ITEM_HAS_AUDIO)
        gtk_clist_set_pixmap(GTK_CLIST(list),
                             i, COLUMN_HAS_AUDIO, audio_pixmap_s,
                             audio_mask);
      current_row = i;
      }
    else
      {
      // Video Present

      //      gtk_clist_set_foreground(GTK_CLIST(list), i, &normal_color);

      if(playlist_items[i]->flags & GV_PLAYLIST_ITEM_HAS_VIDEO)
        gtk_clist_set_pixmap(GTK_CLIST(list),
                             i, COLUMN_HAS_VIDEO, video_pixmap,
                             video_mask);
      // Audio Present
    
      if(playlist_items[i]->flags & GV_PLAYLIST_ITEM_HAS_AUDIO)
        gtk_clist_set_pixmap(GTK_CLIST(list),
                             i, COLUMN_HAS_AUDIO, audio_pixmap,
                             audio_mask);
      //     gtk_clist_set_row_style(GTK_CLIST(list), i, gtk_widget_get_style(list));
      }

    // Check is the track is selected
    
    if(playlist_items[i]->flags & GV_PLAYLIST_ITEM_IS_SELECTED)
      select_row(i);
    }

  
  gtk_clist_moveto (GTK_CLIST(list),
                    first_visible_row,
                    0, 0.0, 0.0);
  
  gtk_clist_thaw(GTK_CLIST(list));

  //  dragged_row = -1;
  //  last_clicked_row = -1;

  int optimal_width;

  // Set the width of the index and time columns
  
  optimal_width = gtk_clist_optimal_column_width(GTK_CLIST(list), COLUMN_INDEX);
  gtk_clist_set_column_width(GTK_CLIST(list), COLUMN_INDEX, optimal_width);
  //  gtk_clist_set_column_max_width(GTK_CLIST(list), COLUMN_INDEX, optimal_width);

  optimal_width = gtk_clist_optimal_column_width(GTK_CLIST(list), COLUMN_TIME);
  gtk_clist_set_column_min_width(GTK_CLIST(list), COLUMN_TIME, optimal_width);
  gtk_clist_set_column_max_width(GTK_CLIST(list), COLUMN_TIME, optimal_width);
  gtk_clist_set_column_width(GTK_CLIST(list), COLUMN_TIME, optimal_width);


  set_private(gv_playlist->is_private());
  
  }


void GVPlaylistWidget::realize_callback()
  {
  //  gtk_style_attach(style, list->window);

  //  gtk_widget_set_rc_style(list, style);

  //  gtk_widget_set_style(list, style);
  
  GdkColor color;
 
  video_pixmap = gdk_pixmap_create_from_xpm_d(list->window, &video_mask,
                                              &color, has_video_xpm);
  audio_pixmap = gdk_pixmap_create_from_xpm_d(list->window, &audio_mask,
                                              &color, has_audio_xpm);

  int x, y, depth;
  gdk_window_get_geometry(video_pixmap, &x, &y,
                          &video_pixmap_width, &video_pixmap_height,
                      &depth);
  video_pixmap_s = gdk_pixmap_new(list->window,
                                  video_pixmap_width, video_pixmap_height,
                                  depth);

  gdk_window_get_geometry(audio_pixmap, &x, &y, &audio_pixmap_width,
                          &audio_pixmap_height,
                      &depth);
  audio_pixmap_s = gdk_pixmap_new(list->window, audio_pixmap_width,
                                  audio_pixmap_height, depth);
  
  gdk_color_alloc(gtk_widget_get_colormap(list),
                  &current_color);

  gdk_color_alloc(gtk_widget_get_colormap(list),
                  &background_color);

  gdk_color_alloc(gtk_widget_get_colormap(list),
                  &selected_background_color);

  gdk_color_alloc(gtk_widget_get_colormap(list),
                  &normal_color);

  // Paint the selected pixmaps in selected color

  gc = gdk_gc_new(video_pixmap_s);
    
  gdk_window_shape_combine_mask(video_pixmap_s, video_mask, 0, 0);
  gdk_window_shape_combine_mask(audio_pixmap_s, audio_mask, 0, 0);
  
  is_realized = 1;

  // Set the style of the widget

//   GtkRequisition requisition;
//   requisition.width = saved_width;
//   requisition.height = saved_height;

//   gtk_window_set_default_size(GTK_WINDOW(window), saved_width,
//                               saved_height);
  
//   gtk_widget_size_request(window, &requisition);
//   gtk_widget_queue_resize(window);
  }

void GVPlaylistWidget::update_current()
  {
  int num = gv_playlist->get_num_tracks();

  if(!(is_realized) || !(num))
    return;
  const GVPlaylistTrack * track;

  // Reset the old current track
  
  track = gv_playlist->get_track(current_row);
  //  gtk_clist_set_foreground(GTK_CLIST(list), current_row, &normal_color);
  
  gtk_clist_set_row_style(GTK_CLIST(list), current_row,
                          gtk_widget_get_style(list));;

  // Video Present
  if(track)
    {
    if(track->flags & GV_PLAYLIST_ITEM_HAS_VIDEO)
      gtk_clist_set_pixmap(GTK_CLIST(list),
                           current_row, COLUMN_HAS_VIDEO, video_pixmap,
                           video_mask);
    // Audio Present
    
    if(track->flags & GV_PLAYLIST_ITEM_HAS_AUDIO)
      gtk_clist_set_pixmap(GTK_CLIST(list),
                           current_row, COLUMN_HAS_AUDIO, audio_pixmap,
                           audio_mask);
    }
  
  // Set the new current track
  
  track = gv_playlist->get_current_track();
  current_row = gv_playlist->get_current_track_index();


  //  gtk_clist_set_foreground(GTK_CLIST(list), current_row, &current_color);
  gtk_clist_set_row_style(GTK_CLIST(list), current_row, current_style);;

  
  // Video Present
      
  if(track->flags & GV_PLAYLIST_ITEM_HAS_VIDEO)
    gtk_clist_set_pixmap(GTK_CLIST(list),
                         current_row, COLUMN_HAS_VIDEO, video_pixmap_s,
                         video_mask);
  // Audio Present
  
  if(track->flags & GV_PLAYLIST_ITEM_HAS_AUDIO)
    gtk_clist_set_pixmap(GTK_CLIST(list),
                         current_row, COLUMN_HAS_AUDIO, audio_pixmap_s,
                         audio_mask);
  }

void GVPlaylistWidget::select_row(int row)
  {
  //  gtk_clist_set_foreground(GTK_CLIST(list), row, &current_color);

  gtk_clist_select_row(GTK_CLIST(list), row, 0);


  //  gtk_clist_set_background(GTK_CLIST(list), row, &selected_background_color);
  gv_playlist->select_track(row, 1);
  num_selected++;
  }

void GVPlaylistWidget::unselect_row(int row)
  {
  gtk_clist_unselect_row(GTK_CLIST(list), row, 0);
  //  gtk_clist_set_background(GTK_CLIST(list), row, &background_color);
  gv_playlist->select_track(row, 0);
  num_selected--;
  }

static void __menu_position_func(GtkMenu   *menu,
                          gint      *x,
                          gint      *y,
                          gpointer   user_data)
  {
  GdkEventButton * event = (GdkEventButton*)user_data;
  *x = (gint)(event->x_root);
  *y = (gint)(event->y_root);
  }


gboolean GVPlaylistWidget::list_button_callback(GdkEventButton * event)
  {
  int row = 0;
  int column = 0;

  if((event->type == GDK_BUTTON_RELEASE) && (event->button == 1))
    {
    if(dragged_row != -1)
      {
      //      cerr << "Drag finished\n";
      no_update = 0;
      dragged_row = -1;
      }
    return TRUE;
    }

  if(event->type == GDK_2BUTTON_PRESS)
    {
    if(!gtk_clist_get_selection_info(GTK_CLIST(list), (gint)(event->x),
                                     (gint)(event->y), &row, &column))
      return FALSE;
    last_clicked_row = row;
    gv_engine->play_track(row);
    return TRUE;
    }

  if(event->type != GDK_BUTTON_PRESS)
    return FALSE;

  if(event->button == 1)
    {
    if(!gtk_clist_get_selection_info(GTK_CLIST(list), (gint)(event->x),
                                     (gint)(event->y), &row, &column))
      return FALSE;


    if(event->state & GDK_CONTROL_MASK)
      {
      last_clicked_row = row;
      select_row(row);
      return TRUE;
      }

    if(event->state & GDK_SHIFT_MASK)
      {
      if(last_clicked_row == -1)
        {
        last_clicked_row = row;
        select_row(row);
        return TRUE;
        }
      
      if(last_clicked_row != row)
        {
        int i_start = (row < last_clicked_row) ? row : last_clicked_row;
        int i_end = (row > last_clicked_row) ? row : last_clicked_row;
        for(int i = i_start; i <= i_end; i++)
          select_row(i);
        last_clicked_row = row;
        return TRUE;
        }
      }
    
    if(!(event->state))
      {
      dragged_row = row;
      last_clicked_row = row;
      for(int i = 0; i < num_tracks; i++)
        {
        if(i != row)
          unselect_row(i);
        }
      select_row(row);
      no_update = 1;
      }
    }
  else if(event->button == 3)
    gtk_menu_popup(GTK_MENU(main_menu), NULL, NULL, __menu_position_func,
                   (gpointer)event, 3, gdk_time_get());
  return TRUE;
  }

gboolean GVPlaylistWidget::list_motion_callback(GdkEventMotion * event)
  {
  if(dragged_row == -1)
    {
    //    cerr << "No Drag Active\n";
    return FALSE;
    }

  char * track_number;
  int row = 0;
  int column = 0;

  if(!gtk_clist_get_selection_info(GTK_CLIST(list), (gint)(event->x),
                                   (gint)(event->y), &row, &column))
    return TRUE;

  //  cerr << "row: " << row << " dragged_row: " << dragged_row << "\n";

  if(row < 0)
    return TRUE;
  
  if(row != dragged_row)
    {
    // Insert the row in the new position and tell this to the playlist

    gv_playlist->move_track(dragged_row, row);
    
    select_row(dragged_row);
    gtk_clist_row_move(GTK_CLIST(list), dragged_row, row);
    
    //    if(dragged_row == current_row)
    //      current_row = row;
        
    int istart, iend;
    
    if(dragged_row < row)
      {
      istart = dragged_row;
      iend = row;
      }
    else
      {
      istart = row;
      iend = dragged_row;
      }
    for(int i = istart; i <= iend; i++)
      {
      if(i != row)
        unselect_row(i);
      else
        gv_playlist->select_track(i, 1);

      track_number = g_strdup_printf("%d.", i+1);
      gtk_clist_set_text(GTK_CLIST(list), i, COLUMN_INDEX, track_number);
      g_free(track_number);
      }
    dragged_row = row;
    current_row = gv_playlist->get_current_track_index();
    //    cerr << "Dragged Row is now " << dragged_row << "\n";
    }
  return TRUE;
  }

GVPlaylistWidget::~GVPlaylistWidget()
  {
  put_config();
  gdk_gc_destroy(gc);

  if(fileselect)
    delete fileselect;
  if(url_window)
    delete url_window;
  }

void GVPlaylistWidget::get_config()
  {
  gdk_color_parse("#FF0000", &current_color);
  gdk_color_parse("#FFFFFF", &background_color);
  gdk_color_parse("#000000", &background_color);
  gdk_color_parse("#00FFFF", &selected_background_color);
  
  gvgtk_get_color_config("Playlist", "CurrentColor",
                         current_color);
  gvgtk_get_color_config("Playlist", "Background",
                         background_color);
  gvgtk_get_color_config("Playlist", "SelectedBackground",
                         selected_background_color);
  gvgtk_get_color_config("Playlist", "NormalColor",
                         normal_color);

  get_colors_from_skin = 1;
  gv_config->get_bool("Playlist", "GetColorsFromSkin", get_colors_from_skin);
  
  saved_width = 480;
  saved_height = 240;

  gv_config->get_int("Playlist", "Width", saved_width);
  gv_config->get_int("Playlist", "Height", saved_height);
  
  }

void GVPlaylistWidget::put_config()
  {
  gvgtk_put_color_config("Playlist", "CurrentColor",
                         current_color);
  gvgtk_put_color_config("Playlist", "Background",
                         background_color);
  gvgtk_put_color_config("Playlist", "SelectedBackground",
                         selected_background_color);
  gvgtk_put_color_config("Playlist", "NormalColor",
                         normal_color);

  gv_config->put_bool("Playlist", "GetColorsFromSkin", get_colors_from_skin);

  //  gv_config->put_int("Playlist", "Width", saved_width);
  //  gv_config->put_int("Playlist", "Height", saved_height);
  
  int x, y, depth;
  gdk_window_get_geometry(window->window, &x, &y, &saved_width,
                          &saved_height, &depth);

//   gv_config->put_int("Playlist", "Width", saved_width);
//   gv_config->put_int("Playlist", "Height", saved_height);
  }

const char * GVPlaylistWidget::get_track_name()
  {
  return gv_playlist->get_track(last_clicked_row)->trackname;
  }

void GVPlaylistWidget::set_track_name(const char * new_name)
  {
  gv_playlist->set_track_name(last_clicked_row, new_name);
  }

void GVPlaylistWidget::set_private(int is_private)
  {
  int sensitive = (is_private) ? 0 : 1;
  gtk_widget_set_sensitive(add_item, sensitive);
  gtk_widget_set_sensitive(add_url_item, sensitive);
  gtk_widget_set_sensitive(list_load_item, sensitive);
  gtk_widget_set_sensitive(list_save_item, sensitive);
  }

static void __toggle_callback(GtkWidget * w, gpointer data)
  {
  ((GVPlaylistWidget::ConfigWidget*)data)->toggle_callback(w);
  }

GVPlaylistWidget::ConfigWidget::ConfigWidget(GVPlaylistWidget * _playlist_widget) :
  current_color("Current Track Color"),
  normal_color("Normal Track Color"),
  selected_background_color("Selected background color"),
  background_color("Background color"),
  trackname_mask("Template (See documentation)")
  {
  playlist_widget = _playlist_widget;
  
  GtkWidget * table = gvgtk_table_new(7, 2, 0);
  
  use_track_info =
    gtk_check_button_new_with_label("Use track info for playlist");

  gtk_widget_show(use_track_info);

  get_colors_from_skin =
    gtk_check_button_new_with_label("Get colors from skin");

  gtk_signal_connect(GTK_OBJECT(get_colors_from_skin), "toggled",
                     GTK_SIGNAL_FUNC(__toggle_callback),
                     (gpointer)this);
  
  gtk_widget_show(get_colors_from_skin);

  gtk_table_attach_defaults(GTK_TABLE(table),
                            get_colors_from_skin, 0, 2, 0, 1);

  
  normal_color.attach(table, 1, 0);
  current_color.attach(table, 2, 0);
  background_color.attach(table, 3, 0);
  selected_background_color.attach(table, 4, 0);

  gtk_table_attach_defaults(GTK_TABLE(table), use_track_info, 0, 1,
                            5, 6);

  trackname_mask.attach(table, 6, 0);
  
  gtk_widget_show(table);
  widget = table;
  }

GVPlaylistWidget::ConfigWidget::~ConfigWidget() { }

void GVPlaylistWidget::ConfigWidget::get_values()
  {
  background_color.set_color(playlist_widget->background_color);
  selected_background_color.set_color(playlist_widget->selected_background_color);
  normal_color.set_color(playlist_widget->normal_color);
  current_color.set_color(playlist_widget->current_color);

  gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(use_track_info),
                              gv_global_config->use_track_info);
  
  trackname_mask.set_string(gv_global_config->trackname_mask);

  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(get_colors_from_skin),
                               playlist_widget->get_colors_from_skin);
  toggle_callback(get_colors_from_skin);
  }


void GVPlaylistWidget::ConfigWidget::apply_values()
  {
  if(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(get_colors_from_skin)))
    {
    playlist_widget->get_colors_from_skin = 1;
    playlist_widget->apply_skin(*(gv_skin_browser->current_skin));
    }
  else
    {
    playlist_widget->get_colors_from_skin = 0;
    background_color.get_color(playlist_widget->background_color);
    selected_background_color.get_color(playlist_widget->selected_background_color);
    normal_color.get_color(playlist_widget->normal_color);
    current_color.get_color(playlist_widget->current_color);
    playlist_widget->update_style();
    }
  
  gv_global_config->use_track_info =
    gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(use_track_info));
  
  g_free(gv_global_config->trackname_mask);
  gv_global_config->trackname_mask = g_strdup(trackname_mask.get_string());
    
  }

void GVPlaylistWidget::ConfigWidget::toggle_callback(GtkWidget * w)
  {
  if(w == get_colors_from_skin)
    {
    int sensitive = 1;
    if(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(w)))
      sensitive = 0;

    current_color.set_sensitive(sensitive);
    normal_color.set_sensitive(sensitive);
    selected_background_color.set_sensitive(sensitive);
    background_color.set_sensitive(sensitive);
    }

  }


GVPlaylistWidget::RenameWindow::RenameWindow(GVPlaylistWidget *
                                             _playlist_widget) :
  GVConfigWindow("Rename Track"),
  track_name("Track Name")
  {

  table = gvgtk_table_new(1, 1, 0);

  playlist_widget = _playlist_widget;
  track_name.attach(table, 0, 0);
  gtk_widget_show(table);
  gtk_box_pack_start_defaults(GTK_BOX(mainbox), table);   
  gtk_widget_show(table);
  }


void GVPlaylistWidget::RenameWindow::get_values()
  {
  track_name.set_string(playlist_widget->get_track_name());
  }

void GVPlaylistWidget::RenameWindow::apply_values()
  {
  playlist_widget->set_track_name(track_name.get_string());
  }

GVPlaylistWidget::RenameWindow::~RenameWindow()
  {
  
  }

void GVPlaylistWidget::apply_skin(GVSkin & skin)
  {
  GVWindow::apply_skin(&(skin.playlist_window));

  if(!get_colors_from_skin)
    return;
  
  skin.get_color(skin.playlist_color_background, &background_color);
  skin.get_color(skin.playlist_color_normal_foreground, &normal_color);
  skin.get_color(skin.playlist_color_current_foreground, &current_color);
  skin.get_color(skin.playlist_color_selected_background,
                 &selected_background_color);

  gdk_color_alloc(gtk_widget_get_colormap(list), &current_color);
  gdk_color_alloc(gtk_widget_get_colormap(list), &normal_color);
  
  gdk_color_alloc(gtk_widget_get_colormap(list), &selected_background_color);
  gdk_color_alloc(gtk_widget_get_colormap(list), &background_color);
  update_style();
  }
