/*****************************************************************

  playlistwidget.h

  Copyright (c) 2001 by Burkhard Plaum - plaum@ipf.uni-stuttgart.de

  http://gmerlin.sourceforge.net

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111, USA.

*****************************************************************/

// Playlist window

class GVPlaylistTrack;
class GVSkin;
class GVURLWindow;
class GVFileSelect;

class GVPlaylistWidget : public GVWindow
  {
public:
  
  class ConfigWidget : public GVConfigWidget
    {
    GtkWidget * get_colors_from_skin;
    GVConfigWidgetColor current_color;
    GVConfigWidgetColor normal_color;
    GVConfigWidgetColor selected_background_color;
    GVConfigWidgetColor background_color;
    GVConfigWidgetString trackname_mask;
    GtkWidget * use_track_info;

    GVPlaylistWidget * playlist_widget;
   
  public:

    ConfigWidget(GVPlaylistWidget * _playlist_widget);
    void toggle_callback(GtkWidget * w);

    void get_values();
    void apply_values();
    virtual ~ConfigWidget();
    };
  
private:
  
  class RenameWindow : public GVConfigWindow
    {
    GVConfigWidgetString track_name;
    GVPlaylistWidget * playlist_widget;
    GtkWidget * table;
  public:

    RenameWindow(GVPlaylistWidget * _playlist_widget);

    void get_values();
    void apply_values();
    virtual ~RenameWindow();

    };

  friend class ConfigWidget;
  friend class RenameWindow;
    
  RenameWindow * rename_window;
  
  int saved_width;
  int saved_height;

  int num_selected;
  
  GdkGC * gc;

  int audio_pixmap_width;
  int audio_pixmap_height;
  int video_pixmap_width;
  int video_pixmap_height;
  
  GtkWidget * list;           // List

  GtkWidget* scrolledwin;

  //  GtkWidget * window;
  
//   GtkWidget * remove_button;
//   GtkWidget * add_button;
//   GtkWidget * sort_button;
//   GtkWidget * refresh_button;
//   GtkWidget * load_button;
//   GtkWidget * save_button;

  GtkWidget * playlist_fileselect;
  
  int active;                 // Active (playing) item

  int load_playlist;  // Load or save a playlist?
  //  int visible;        // Are we visible?
  
  GdkPixmap * video_pixmap;
  GdkBitmap * video_mask;
  
  GdkPixmap * audio_pixmap;
  GdkBitmap * audio_mask;

  GdkPixmap * video_pixmap_s;
  GdkPixmap * audio_pixmap_s;

  GtkWidget * main_menu;
  GtkWidget * add_item;
  GtkWidget * add_url_item;
  
  GtkWidget * track_item;
  GtkWidget * list_item;

  GtkWidget * list_menu;
  GtkWidget * track_menu;
    
  //  GtkWidget * track_add_item;
  GtkWidget * track_remove_item;
  GtkWidget * track_refresh_item;
  GtkWidget * track_rename_item;
  GtkWidget * list_load_item;
  GtkWidget * list_save_item;
  GtkWidget * list_sort_item;
  GtkWidget * list_clear_item;

  GtkWidget * options_item;
  
  GVFileSelect * fileselect;
  GVURLWindow * url_window;

  GdkColor current_color;
  GdkColor normal_color;

  GdkColor selected_background_color;
  GdkColor background_color;

  int get_colors_from_skin;
  
  int is_realized;
  
  int dragged_row;

  int current_row;

  int no_update; // Don't update the playlist
  int no_resize; // Don't do a rezize callback
  
  int last_clicked_row; // For multiple selection with SHIFT
  
  int num_tracks;

  const GVPlaylistTrack ** playlist_items;

  void select_row(int row);
  void unselect_row(int row);

  GtkStyle * current_style;
  
  void get_config();
  void put_config();

  void update_style();
  void create_style();
  
//   int old_allocation_width;
//   int old_name_width;

  int column_1_width;

  void before_resize_hook(int old_width, int old_height,
                          int & new_width, int & new_height);


  const char * get_track_name();
  void set_track_name(const char * new_name);

  void set_private(int is_private);
        
public:

  void apply_skin(GVSkin & skin);
  
  void show();
  
  ConfigWidget config_widget;

  void realize_callback();
  void button_callback(GtkWidget * w);
  
//   void select_row_callback(GtkWidget * w, gint row, gint column,
//                            GdkEventButton *event);
//   void unselect_row_callback(GtkWidget * w, gint row, gint column,
//                              GdkEventButton *event);
//   void row_move_callback(GtkWidget * w, gint src, gint dst);


  void dnd_callback(GtkWidget * w, GdkDragContext *context,
                    gint x, gint y, GtkSelectionData *selection_data,
                    guint info, guint time);
  
  gboolean list_button_callback(GdkEventButton * event);
  gboolean list_motion_callback(GdkEventMotion * event);
  
  void update_album();
  void update_current();
  
  GVPlaylistWidget();
  ~GVPlaylistWidget();

  };
