/*****************************************************************

  plugin.cc

  Copyright (c) 2001 by Burkhard Plaum - plaum@ipf.uni-stuttgart.de

  http://gmerlin.sourceforge.net

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111, USA.

*****************************************************************/

// Methods which are common to all media plugins

#include "config.h"
#include <gtk/gtk.h>
#include <pthread.h>
#include <iostream>

using std::ostream;

#include <string.h>

#include "gvtime.h"
#include "buffer.h"
#include "videoformat.h"
#include "audioformat.h"

#include "videobuf.h"
#include "audiobuf.h"

#include "playlist.h"
#include "baseplugin.h"
#include "plugin.h"
#include "configwindow.h"
#include "engine.h"

GVPlugin::GVPlugin(const GVPluginAttributes * _attributes) :
  GVBasePlugin(_attributes)
  {
  pthread_mutex_init (&destroy_mutex,(pthread_mutexattr_t *)0);
  pthread_mutex_init (&internal_time_mutex,(pthread_mutexattr_t *)0);
  error = GV_PLUGIN_ERROR_NONE;
  filename = (char*)0;
  trackname = (char*)0;
  }

void GVPlugin::set_filename(const char * _filename)
  {
  if(filename)
    g_free(filename);
  filename = g_strdup(_filename);
  }


int GVPlugin::get_error()
  {
  return error;
  }

static void * __playback_func(void * data)
  {
  ((GVPlugin*)data)->playback_func();
  return (void*)0;
  }
                              
void GVPlugin::play() // Play
  {
  pthread_attr_t attr;
  pthread_attr_init(&attr);
  pthread_attr_setdetachstate(&attr, PTHREAD_CREATE_DETACHED);
  pthread_create(&playback_thread, &attr, &__playback_func, (void*)this);
  pthread_attr_destroy (&attr);
  }

GVPlugin::~GVPlugin()
  {
  //  cerr << "GVPlugin::~GVPlugin\n";
  pthread_mutex_destroy(&destroy_mutex);

  if(filename)
    g_free(filename);
  if(trackname)
    delete trackname;
  }

void GVPlugin::configure()
  {
  
  }

void GVPlugin::init()
  {
  }


void GVPlugin::get_current_track_time(GVTime & old_time)
  {
  pthread_mutex_lock(&internal_time_mutex);
  old_time = internal_time;
  pthread_mutex_unlock(&internal_time_mutex);
  }

void GVPlugin::reset() { }

int GVPlugin::check_engine_state()
  {
  GVEngineState state = gv_engine->get_state();

  if(state != GVEnginePlaying)
    {
    switch(state)
      {
      case GVEngineStopped:
      case GVEngineChanging:
        return 1;
      case GVEngineSeeking:
      case GVEnginePaused:
        reset();
        gv_engine->plugin_detach();
        //        cerr << attributes->name << ": waiting for start..\n";
        gv_engine->wait_for_start();
        gv_engine->plugin_attach();
        state = gv_engine->get_state();
        if(state == GVEngineStopped)
          return 1;
        else
        //        cerr << attributes->name << ": started again\n";
      default:
        break;
      }
    }
  return 0;
  }

void GVPlugin::print_info(ostream & output)
  {
  output << "No Info available";
  }

void GVPlugin::get_internal_video_time(GVTime & ret)
  {
  pthread_mutex_lock(&internal_time_mutex);
  ret = internal_video_time;
  pthread_mutex_unlock(&internal_time_mutex);
  }

void GVPlugin::get_internal_audio_time(GVTime & ret)
  {
  pthread_mutex_lock(&internal_time_mutex);
  ret = internal_audio_time;
  pthread_mutex_unlock(&internal_time_mutex);
  }
