/*****************************************************************

  plugin.h

  Copyright (c) 2001 by Burkhard Plaum - plaum@ipf.uni-stuttgart.de

  http://gmerlin.sourceforge.net

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111, USA.

*****************************************************************/

// Base class for all media plugins

// Error codes

#define GV_PLUGIN_ERROR_NONE                   0 // No error
#define GV_PLUGIN_ERROR_FILE_INVALID           1 // File format unknown
#define GV_PLUGIN_ERROR_DEVICE_OPEN_FAILED     2 
#define GV_PLUGIN_ERROR_DEVICE_SETTINGS_FAILED 3 // Device settings failed
#define GV_PLUGIN_ERROR_NO_MEDIUM              4 // No medium found
#define GV_PLUGIN_ERROR_OPEN_FILE              5 // Error opening the file
#define GV_PLUGIN_ERROR_CONNECT                6 // Error opening a socket

class GVBufferAudio;
class GVBufferVideo;

#define GV_PLUGIN_TRACK_HAS_VIDEO (1<<0)
#define GV_PLUGIN_TRACK_HAS_AUDIO (1<<1)
#define GV_PLUGIN_TRACK_IS_STREAM (1<<2) // Stream means: unknown total length
#define GV_PLUGIN_TRACK_IS_URL    (1<<3) // Track is an URL

class ostream;

class GVPlugin : public GVBasePlugin
  {
protected:
  int flags;
  pthread_t playback_thread;
   
  GVPlugin(const GVPluginAttributes * _attributes);

  //  int paused;

  int error;
  //  char * error_message;
  char * filename;
  char * trackname;        // Name of the track

  GVVideoFormat video_format;
  GVAudioFormat audio_format;

  GVTime internal_audio_time; // Time according to the written samples
  GVTime internal_video_time; // Time according to the written frames
  GVTime internal_time;       // Official track time of this plugin
  
  pthread_mutex_t internal_time_mutex;

  int video_completed;
  int audio_completed;

  // Check the state of the engine, return 1 if the
  // playback thread should exit
  
  int check_engine_state();
  
public:

  void get_internal_video_time(GVTime & ret);
  void get_internal_audio_time(GVTime & ret);
  
  
  virtual void print_info(ostream & output);
  
  virtual int can_sync() { return !!(attributes->flags & GV_PLUGIN_CAN_SYNC); }
  
  int do_audio; // Plugin should handle audio
  int do_video; // Plugin should handle video
  
  virtual void get_current_track_time(GVTime & old_time);
  
  const GVVideoFormat * get_video_format() { return &video_format; }
  const GVAudioFormat * get_audio_format() { return &audio_format; }
  
  GVBufferAudio * audio_buffer;
  GVBufferVideo * video_buffer;
  
  int realtime;   // Set to 1 if you want realtime behaviour
  
  const char * get_filename() { return filename; }
  virtual void set_filename(const char * _filename);

  const char * get_trackname() { return trackname; }
  
  virtual void configure();

  virtual int can_alloc_audio() { return 0; }
  virtual int can_alloc_video() { return 0; }
  
  int get_error();

  // Everyone must unlock this mutex before calling a destructor
  
  pthread_mutex_t destroy_mutex;


  void play();

  virtual void playback_func() = 0;

  virtual ~GVPlugin();

  // These functions MUST be defined
  
  virtual void init() = 0;
  virtual void cleanup() = 0;

  // This function CAN be defined (called before pause and seek)

  virtual void reset();
  
  };

