#!/bin/sh
#

acdir=""
use_copy=""

while [ $# -gt 0 ]; do
	case "$1" in
		--acdir) 
			acdir=$2
                        shift
			;;
		--copy) 
			use_copy="--copy"
                        shift
			;;
		*)
			echo "WARNING: unrecognized option: $1"
			;;
	esac
	shift
done

echo "Generating build information..."

# Touch the timestamps on all the files since CVS messes them up
directory=`dirname $0`
touch $directory/configure.in

# Debian unstable allows to have several different versions of autoconf
# in the one system
use_autoconf=autoconf
use_autoheader=autoheader
# I think links are now OK
test -h configure.ac || ln -sf configure.in configure.ac
# Debian will use 2.50 for configure.ac files automaticaly
#( which autoconf2.50 2>/dev/null | grep -q "^/" ) && use_autoconf=autoconf2.50
#( which autoheader2.50 2>/dev/null | grep -q "^/" ) && use_autoheader=autoheader2.50

DIE=0

($use_autoconf --version) < /dev/null > /dev/null 2>&1 || {
  echo
  echo "**Error**: You must have \`autoconf' installed ."
  DIE=1
}

(automake --version) < /dev/null > /dev/null 2>&1 || {
  echo
  echo "**Error**: You must have \`automake' installed."
  DIE=1
  NO_AUTOMAKE=yes
}

(aclocal --version) < /dev/null > /dev/null 2>&1 || {
  echo
  echo "**Error**: Missing \`aclocal'."
  DIE=1
}

(libtool --version) < /dev/null > /dev/null 2>&1 || {
  echo
  echo "**Error**: You must have \`libtool' installed."
  DIE=1
}


if test -z "$acdir" -a -d /usr/local/share/aclocal ; then
  if !(ls -ld /usr/local/share | grep /usr/share > /dev/null 2>/dev/null ); then
  if test "`aclocal --print-ac-dir`" != "/usr/local/share/aclocal" ; then
    ACLOCAL_FLAGS="-I /usr/local/share/aclocal $ACLOCAL_FLAGS"
    ACLOCAL_MAINDIR=`aclocal --print-ac-dir`
    for FILENAME in `ls $ACLOCAL_MAINDIR`; do
    if test -e /usr/local/share/aclocal/$FILENAME ; then
	echo "FATAL: both $ACLOCAL_MAINDIR/$FILENAME and /usr/local/share/aclocal/$FILENAME exist; you must manually delete one of them"
	echo "or use: autogen.sh --acdir /usr/share/aclocal"
    fi
    done
  fi
  else
    echo /usr/local/share is link to /usr/share
  fi
fi

# Regenerate configuration files
ok=0

rm -f aclocal.m4 configure config.guess config.log config.sub ltmain.sh libtool ltconfig missing mkinstalldirs
rm -f libmmxnow/configure libmmxnow/mmxnow-config libmmxnow/aclocal.m4

echo "Running libtoolize..."
libtoolize --force $use_copy > /dev/null || ok=1
if test -z "$acdir" ; then
	echo "Running aclocal $ACLOCAL_FLAGS..."
	aclocal $ACLOCAL_FLAGS || ok=1
else
	echo "Running aclocal with forced acdir: $acdir"
	aclocal --acdir=$acdir || ok=1
fi
# echo "Running autoupdate..."
# autoupdate || ok=1
echo "Running autoheader..."
$use_autoheader || ok=1
echo "Running autoconf..."
$use_autoconf || ok=1
echo "Running automake..."
automake --add-missing $use_copy --foreign || ok=1

if [ "${ok}" -eq 0 ]; then
	echo "Now you are ready to run ./configure"
    else
	echo "Problems detected. Please investigate."
	echo "Suggested version of used programs (try to use them)"
	echo "   libtool  1.4.1 or better"
	echo "   automake 1.4 or better"
	echo "   autoconf 2.52 or better"
	echo "Your installed version:"
        libtool --version | head -1
        automake --version | head -1
        autoconf --version | head -1
        echo "Report aclocal = "
        aclocal --print-ac-dir
        echo "Please report your problem on avifile-admin@prak.org"
        echo "with this log of build process and with system description."
#        echo "If generated 'configure' script is useless try following:"
#        echo "   rm -f configure ltmain.sh config.guess condig.sub"
# 	 echo "   cvs update"
#        echo "   automake"
#        echo "   configure"
fi
