#ifndef AVIFILE_STREAMINFO_H
#define AVIFILE_STREAMINFO_H

// this file could be a part of infotypes in future  > so be careful


#include "default.h"
//#include <ostream.h>

class FileInfo
{
    enum FileType { UNKNOWN, AVI, ASF };
    FileType m_FileType;

    // here will follow various info about the movie file

    // DO NOT USE - it's just experimental

    // IARL - Indicates where the subject of the file is archived
    avm::string m_ArchivalLocation;
    // IART - Lists the artist of the original subject of the file;
    // for example, "Michaelangelo."
    avm::string m_Artist;
    // ICMS - Lists the name of the person or organization that commissioned
    // the subject of the file; for example "Pope Julian II."
    avm::string m_Commissioned;
    // ICMT - Provides general comments about the file or the subject
    // of the file. If the comment is several sentences long, end each
    // sentence with a period. Do not include new-line characters.
    avm::string m_Comments;
    // ICOP - Records the copyright information for the file; for example,
    // "Copyright Encyclopedia International 1991." If there are multiple
    // copyrights, separate them by semicolon followed by a space.
    avm::string m_Copyright;
    // ICRD - Specifies the date the subject of the file was created. List
    // dates in year-month-day format, padding one-digit months and days with
    // a zero on the left; for example, "1553-05-03" for May 3, 1553.
    avm::string m_CreationDate;
    // ICRP - Describes whether an image has been cropped and, if so, how it
    // was cropped; for example, "lower-right corner."
    avm::string m_Cropped;
    // IDIM - Specifies the size of the original subject of the file; for
    // example, "8.5 in h, 11 in w."
    avm::string m_Dimensions;
    // IDPI - Stores dots per inch setting of the digitizer used to
    // produce the file, such as "300."
    avm::string m_DotsPerInch;
    // IENG - Stores the of the engineer who worked on the file. If there are
    // multiple engineers, separate the names by a semicolon and a blank;
    // for example, "Smith, John; Adams, Joe."
    avm::string m_Engineer;
    // IGNR - Describes the original work, such as "landscape,", "portrait,"
    // "still liefe," etc.
    avm::string m_Genre;
    // IKEY - Provides a list of keywords that refer to the file or subject of the
    // file. Separate multiple keywords with a semicolon and a blank;
    // for example, "Seattle, aerial view; scenery."
    avm::string m_Keywords;
    // ILGT - Describes the changes in the lightness settings on the digitizer
    // required to produce the file. Note that the format of this information
    // depends on the hardware used.
    avm::string m_Lightness;
    // IMED - Decribes the original subject of the file, such as
    // "computer image," "drawing," "lithograph," and so on.
    avm::string m_Medium;
    // INAM - Stores the title of the subject of the file, such as
    // "Seattle from Above."
    avm::string m_Name;
    // IPLT - Specifies the number of colors requested when digitizing
    // an image, such as "256."
    avm::string m_PaletteSetting;
    // IPRD - Specifies the name of title the file was originally intended
    // for, such as "Encyclopedia of Pacific Northwest Geography."
    avm::string m_Product;
    // ISBJ - Decsribes the contents of the file, such as
    // "Aerial view of Seattle."
    avm::string m_Subject;
    // ISFT - Identifies the name of the software packages used to create the
    // file, such as "Microsoft WaveEdit"
    avm::string m_Software;
    // ISHP - Identifies the change in sharpness for the digitizer
    // required to produce the file (the format depends on the hardware used).
    avm::string m_Sharpness;
    // ISRC - Identifies the name of the person or organization who
    // suplied the original subject of the file; for example, "Try Research."
    avm::string m_Source;
    // ISRF - Identifies the original form of the material that was digitized,
    // such as "slide," "paper," "map," and so on. This is not necessarily
    // the same as IMED
    avm::string m_SourceForm;
    // ITCH - Identifies the technician who digitized the subject file;
    // for example, "Smith, John."
    avm::string m_Technician;

    // ISMP
    avm::string m_TimeCode;
    // IDIT
    avm::string m_DigitizationTime;
public:
    FileInfo() : m_FileType(UNKNOWN) {}
    FileType GetFileType() const;
};

class IMediaReadStream;
class AviMediaReadStream;
class ASFReadStream;

class StreamInfo
{
    friend class IMediaReadStream;
    friend class AviMediaReadStream;
    friend class ASFReadStream;
public:
    enum Type { Video, Audio, Other };

    StreamInfo();
    ~StreamInfo();

    Type GetType() const;
    fourcc_t GetFourCC() const;

    double GetBps() const;	// calculated Bps
    double GetFps() const;	// calculated Fps

    double GetLengthTime() const;
    int64_t GetStreamSize() const;
    uint_t GetStreamFrames() const;

    uint_t GetMaxKfFrameSize() const;
    uint_t GetMinKfFrameSize() const;
    uint_t GetAvgKfFrameSize() const;
    uint_t GetKfFrames() const;
    int64_t GetKfFramesSize() const;

    uint_t GetMaxFrameSize() const;
    uint_t GetMinFrameSize() const;
    uint_t GetAvgFrameSize() const;
    uint_t GetFrames() const;
    int64_t GetFramesSize() const;

    // for non video returns -1
    int GetVideoWidth() const;
    int GetVideoHeight() const;

    // for non audio returns -1
    int GetAudioChannels() const;
    int GetAudioSamplesPerSec() const;
    int GetAudioBitsPerSample() const;
    bool IsAudioVbr() const;//??? no sure
    avm::string GetString() const;
//    friend ostream & operator << (ostream&, StreamInfo &);
//    friend ostream & operator << (ostream&, StreamInfo *);
protected:
    Type m_Type;
    fourcc_t m_FourCC;		// fourcc type of stream

    double m_dLengthTime;	// time in seconds for all packets
    uint_t m_uiMaxKfFrameSize;	// biggest keyframe chunk size
    uint_t m_uiMinKfFrameSize;	// smallest keyframe chunk size
    uint_t m_uiKfFrames;	// keyframe chunks
    int64_t m_uiKfFramesSize;	// total keyframe chunk size

    // should make sence probably only for video stream
    uint_t m_uiMaxFrameSize;	// biggest non-keyframe chunk size
    uint_t m_uiMinFrameSize;	// smallest non-keyframe chunk size
    uint_t m_uiFrames;		// non-keyframe chunks
    int64_t m_uiFramesSize;	// total non-keyframe chunk size

    union
    {
        char b[512];
	struct
	{
	    int	m_iChannels;
	    int	m_iSamplesPerSec;
	    int	m_iBitsPerSample;
            bool m_bVbr;
	} aud;
	struct
	{
	    int m_iWidth;
	    int m_iHeight;

	} vid;
    } u;
};

#endif // AVIFILE_STREAMSTATS_H
