#ifndef AVIFILE_DEFAULT_H
#define AVIFILE_DEFAULT_H

#ifndef	WIN32

#ifdef HAVE_CONFIG_H
#include "config.h"	/* to get the HAVE_xxx_H defines */
#endif /* HAVE_CONFIG_H */

#if !defined(HAVE_CONFIG_H) || defined(HAVE_SYS_TYPES_H)
/* do we still need this header ??? #include <sys/types.h> */
/* we should add this header file only in places where it's really needed */
#endif

#include <inttypes.h>	/* for int64_t */

#if WITH_DMALLOC
#include <dmalloc.h>
#endif

//#include <stdio.h>

#ifndef __WINE_WINDEF_H
typedef long HRESULT;
#endif /* __WINE_WINDEF_H */


#else /* WIN32 */

/* '<Unknown'> has C-linkage specified, but returns UDT 'basic_string<char, struct ...' */
 #pragma warning (disable: 4190)

/* 'std::_Tree(std::basic_string<... ': identifier was truncated to '255' characters in the debug information */
 #pragma warning (disable: 4786)

/* 'unsigned char': forcing value to bool 'true' or 'false' */
 #pragma warning (disable: 4800)
typedef __int64  int64_t;
typedef __int32  int32_t;
typedef __int16  int16_t;
typedef __int8   int8_t;

typedef __uint64 uint64_t;
typedef __uint32 uint32_t;
typedef __uint16 uint16_t;
typedef __uint8  uint8_t;
#endif /* WIN32 */

#define E_ERROR -2

typedef uint32_t fourcc_t;
typedef unsigned int uint_t;    // use as generic type -
typedef uint_t framepos_t;
typedef uint_t streamid_t;
typedef uint_t Unsigned; 	// just for compatibility with older software

#ifdef WITHOUT_X
typedef int Display;
#endif

#ifdef __cplusplus

#define AVM_BEGIN_NAMESPACE namespace avm {
#define AVM_END_NAMESPACE   };

// some basic types for our C++ usage
// we are replacing overbloated STL

AVM_BEGIN_NAMESPACE

// simple class for storing char*
// the only reason for this class is about 100% faster compilation
// we do not need overcomplicated  std::string class for our purpose

class string
{
    char* str;
public:
    typedef uint_t size_type;
    static const size_type npos = ~0U;
    string();
    string(char s);
    string(const char* s, uint_t len = 0);
    string(const string& s, uint_t len = 0);
    ~string();
    uint_t size() const;
    char operator[](int i) const { return str[i]; }
    char& operator[](int i) { return str[i]; }
    bool operator==(const char* s) const;
    bool operator==(const string& s) const;
    operator const char*() const { return str; }
    const char* c_str() const { return str; }
    string& operator=(const string& s);
    string& operator=(const char* s);
    string& operator+=(const string& s);
    string& operator+=(const char* s);
    string operator+(const string& s) { return string(str) += s; }
    string operator+(const char* s) { return string(str) += s; }
    //string operator+(const char* s) { return string(str) += s; }
    string substr(uint_t to) const { return string(str, to); };
    string substr(uint_t from, uint_t to) const { return string(str + from, to); };
    size_type find(const string& s, size_type startpos = 0) const;
    void insert(uint_t pos, const string& s);
    void erase(uint_t from, uint_t to = npos);
};

// very simple vector class
template <class Type> class vector
{
    struct vec {
	Type* m_Type;
	uint_t m_uiSize;
	uint_t m_uiAlloc;
	int m_iRefcount;
    };
    vec* m_pV;
public:
    typedef Type* iterator;
    typedef const Type* const_iterator;
    vector<Type>(int prealloc = 0);
    // we will not count references - we have to program with this in mind!
    vector<Type>(const vector<Type>& t) : m_pV(0) { operator=(t); }
    vector<Type>& operator=(const vector<Type>& t);
    ~vector();
    Type& operator[](int i) const { return m_pV->m_Type[i]; }
    //const Type& operator[](int i) const { return m_Type[i]; }
    const Type* begin() const { return m_pV->m_Type; }
    Type* begin() { return m_pV->m_Type; }
    const Type* end() const { return m_pV->m_Type + m_pV->m_uiSize; }
    Type* end() { return m_pV->m_Type + m_pV->m_uiSize; }
    void erase(iterator pos)
    {
	if (m_pV->m_uiSize > 0)
	{
	    while (pos < end() - 1)
	    {
		pos[0] = pos[1];
		pos++;
	    }
	    pop_back();
	}
    }
    void push_back(const Type& m)
    {
	if (m_pV->m_uiSize + 1 >= m_pV->m_uiAlloc)
	{
            m_pV->m_uiAlloc *= 2;
	    //printf("Alloc %d  %d   %p\n", m_pV->m_uiAlloc, sizeof(Type), m_pV);
	    Type* tn = new Type[m_pV->m_uiAlloc];
	    for (uint_t i = 0; i < m_pV->m_uiSize; i++)
		tn[i] = m_pV->m_Type[i];
	    delete[] m_pV->m_Type;
	    m_pV->m_Type = tn;
	}
	m_pV->m_Type[m_pV->m_uiSize++] = m;
    }
    void pop_back()
    {
	m_pV->m_uiSize--;

	if ((m_pV->m_uiAlloc >= 8) && (m_pV->m_uiSize < m_pV->m_uiAlloc / 4))
	{
	    //printf("Popback %d\n", m_uiAlloc);
            m_pV->m_uiAlloc /= 2;
            Type* tn = new Type[m_pV->m_uiAlloc];
	    for (uint_t i = 0; i < m_pV->m_uiSize; i++)
		tn[i] = m_pV->m_Type[i];
            delete[] m_pV->m_Type;
	    m_pV->m_Type = tn;
	}

    }
    Type& back() { return m_pV->m_Type[m_pV->m_uiSize - 1]; }
    const Type& back() const { return m_pV->m_Type[m_pV->m_uiSize - 1]; }
    uint_t size() const { return m_pV->m_uiSize; }
    void resize(uint_t sz)
    {
        m_pV->m_uiAlloc = sz;
	Type* tn = new Type[sz];
	if (m_pV->m_uiSize > sz)
            m_pV->m_uiSize = sz;
	for (uint_t i = 0; i < m_pV->m_uiSize; i++)
	    tn[i] = m_pV->m_Type[i];
	delete[] m_pV->m_Type;
	m_pV->m_Type = tn;
        m_pV->m_uiSize = m_pV->m_uiAlloc;
    }
    void clear()
    {
	if (m_pV->m_uiAlloc > 4)
	{
	    delete[] m_pV->m_Type;
	    m_pV->m_uiAlloc = 4;
	    m_pV->m_Type = new Type[m_pV->m_uiAlloc];
	}
	m_pV->m_uiSize = 0;
    }
    void pop_front()
    {
	for (uint_t i = 1; i < m_pV->m_uiSize; i++)
	    m_pV->m_Type[i - 1] = m_pV->m_Type[i];
	pop_back();
    }
    void remove(const Type& t);
    operator const Type*() const { return m_pV->m_Type; }
};

template <class Type>
vector<Type>::vector<Type>(int prealloc = 0)
{
    m_pV = new vec;
    m_pV->m_uiAlloc = prealloc;
    if (prealloc < 4 )
	m_pV->m_uiAlloc = 4;
    m_pV->m_Type = new Type[m_pV->m_uiAlloc];
    m_pV->m_uiSize = prealloc;
    m_pV->m_iRefcount = 1;
}

template <class Type>
vector<Type>::~vector()
{
    //printf("~vector %d\n", m_pV->m_iRefcount);
    m_pV->m_iRefcount--;
    if (m_pV->m_iRefcount < 1)
    {
	delete[] m_pV->m_Type;
	//m_pV->m_Type = const_cast<Type*>(0xdeadbeef);
	delete m_pV;
        //m_pV = const_cast<vec*>(0xdeadbeef);
    }
}

template <class Type>
vector<Type>& vector<Type>::operator=(const vector<Type>& t)
{
    //printf("const normal1=   %p\n", t.m_pV);
    if (this != &t)
    {
	if (m_pV)
	{
	    m_pV->m_iRefcount--;
	    if (m_pV->m_iRefcount < 1)
	    {
		delete[] m_pV->m_Type;
		//m_pV->m_Type = const_cast<Type*>(0xdeadbeef);
		delete m_pV;
                //m_pV = const_cast<vec*>(0xdeadbeef);
	    }
	}
	m_pV = t.m_pV;
	m_pV->m_iRefcount++;
    }
    return *this;
}

template <class Type>
void vector<Type>::remove(const Type& t)
{
    int d = 0;
    iterator from = begin();
    for (iterator it = from; it != end(); it++)
    {
	if (t == *it)
	{
	    d++;
	    continue;
	}
	if (from != it)
	    *from++ = *it;
    }
    while (--d >= 0)
	pop_back();
}

AVM_END_NAMESPACE

#endif /* __cplusplus */

//#define std avm

#endif /* AVIFILE_DEFAULT_H */
