#ifndef AVIFILE_EXCEPT_H
#define AVIFILE_EXCEPT_H

/********************************************************

	Exception object
	Copyright 2000 Eugene Kuznetsov  (divx@euro.ru)

*********************************************************/

/**
 *
 *   The idea is to use error codes for non-fatal problems
 * ( for example, problem in decompressing one frame is not fatal
 * for video decoder ) and exceptions for problems which leave
 * object in unusable state. GenError is an intermediate case.
 *
 */
class BaseError
{
protected:
    const char* file;
    char* module;
    char* description;
    char* severity;
    int line;
public:
    BaseError();
    BaseError(const BaseError& f);
    BaseError(const char* severity, const char* module, const char* file,
	      int line, const char* desc);
    virtual ~BaseError();
    BaseError& operator=(const BaseError& f);
    void Print();
    void PrintAll();
    const char* GetModule() const;
    const char* GetDesc() const;
};

class FatalError: public BaseError
{
public:
    FatalError(const char* mod, const char* f, int l, const char* desc,...);
    FatalError(const FatalError& f):BaseError(f){}
};

class GenError: public BaseError
{
public:
    GenError(const char* mod, const char* f, int l, const char* desc,...);
    GenError(const GenError& f):BaseError(f){}
};

#define FATAL(X...) FatalError(__MODULE__,__FILE__,__LINE__,X)
#define WARNING(X...) GenError(__ERR_MODULE__,__FILE__,__LINE__,X)
#define MEMORY_FATAL FATAL("Out of memory")

#endif // AVIFILE_EXCEPT_H
