#include "AviMediaReadHandler.h"
#include "AviMediaReadStream.h"
#include "utils.h"
#include "fourcc.h"
#include <stdio.h>
#include <string.h>

#define __MODULE__ "MediaReadHandler"
#define Debug if(0)

// reset profiler status - just for debuging
//extern "C" void kprof_reset_stats(void);

//template vector<AviMediaReadStream>;

AviMediaReadHandler::AviMediaReadHandler(const char* pszFile) throw(FatalError)
    :m_Input(pszFile)
{
    m_Input.seek(0);
    checkDword(FOURCC_RIFF);
    /*int riff_chunk=*/
    m_Input.readDword();
    checkDword(formtypeAVI);

    int movie_chunk = -1;
    bool index_chunk_found = false;
    uint32_t chunk_type;
    uint_t type, nextpos;
    uint_t chunk_size;
    while (!m_Input.eof())
    {
	chunk_type = m_Input.readDword();
	chunk_size = m_Input.readDword();
//	if(chunk_size<0)throw FATAL("Damaged AVI file");
	nextpos = m_Input.pos() + chunk_size + (chunk_size & 1);
	switch(chunk_type)
	{
	case FOURCC_LIST:
	    if (chunk_size < 4)
		throw FATAL("Damaged AVI file");
	    type = m_Input.readDword();
	    switch(type)
	    {
	    case listtypeAVIMOVIE:
		movie_chunk = m_Input.pos();
	        m_Input.seek(nextpos);
//		readMovieChunk();
		break;
	    case listtypeAVIHEADER:
		continue;
	    case listtypeSTREAMHEADER:
		readStreamHeader();
		break;
	    default:
		break;
	    };
	    break;
	case ckidAVIMAINHDR:
	    readMainHeader(chunk_size);
	    break;
	case ckidAVINEWINDEX:
	    index_chunk_found = true;
	    readIndexChunk(chunk_size, (movie_chunk < 0) ? 0 : movie_chunk);
	    break;
	case RIFFINFO_IARL:
	case RIFFINFO_IART:
	case RIFFINFO_ICMS:
	case RIFFINFO_ICMT:
	case RIFFINFO_ICOP:
	case RIFFINFO_ICRD:
	case RIFFINFO_ICRP:
	case RIFFINFO_IDIM:
	case RIFFINFO_IDPI:
	case RIFFINFO_IENG:
	case RIFFINFO_IGNR:
	case RIFFINFO_IKEY:
	case RIFFINFO_ILGT:
	case RIFFINFO_IMED:
	case RIFFINFO_INAM:
	case RIFFINFO_IPLT:
	case RIFFINFO_IPRD:
	case RIFFINFO_ISBJ:
	case RIFFINFO_ISFT:
	case RIFFINFO_ISHP:
	case RIFFINFO_ISRC:
	case RIFFINFO_ISRF:
	case RIFFINFO_ITCH:
	case RIFFINFO_ISMP:
	case RIFFINFO_IDIT:
    	default:
	    break;
	};
	m_Input.seek(nextpos);
    }
/*
    checkInt(FOURCC_LIST);
    int header_chunk=m_Input.readInt();
    checkInt(listtypeAVIHEADER);
    header_chunk-=4;
//    riff_chunk-=(header_chunk+8);

    header_chunk-=readMainHeader();
    if(header_chunk<0)throw FATAL("Damaged AVI file");

    int nextpos=m_Input.pos()+header_chunk;
    readStreams(header_chunk);
    m_Input.seek(nextpos);

    int movie_chunk=readMovieChunk();
    if(m_Input.eof() || (readIndexChunk(movie_chunk)<0))
	reconstructIndexChunk(movie_chunk);
*/
    if (movie_chunk < 0)
	throw FATAL("Movie chunk not found");
    if (!index_chunk_found)
	reconstructIndexChunk(movie_chunk);

    //vector<AviMediaReadStream>::iterator it;
    //    for(it=m_Streams.begin(); it!=m_Streams.end(); it++)
    for (unsigned i = 0; i < m_Streams.size(); i++)
    {
	char b[100];

	if (m_Streams[i].m_Header.fccType == streamtypeAUDIO)
	{
	    strncpy(b, avm_wave_format_name(m_Streams[i].m_Header.fccHandler), sizeof(b));
            b[sizeof(b) - 1] = 0;
	}
	else
	{
	    strncpy(b, (const char*)&m_Streams[i].m_Header.fccHandler, 4);
            b[4] = 0;
	}

	printf("Stream %d: 0x%x (%.4s) : 0x%x (%s)   %d chunks\n",
	       i, m_Streams[i].m_Header.fccType,
	       (const char*)&m_Streams[i].m_Header.fccType,
	       m_Streams[i].m_Header.fccHandler, b,
	       (int) m_Streams[i].m_Index.size());

#if 0
	AviMediaReadStream& stream = m_Streams[j];
	for (int i = 0 ; i < 150; i++)
	    printf("%3d  %Ld  %d  %d %Ld\n", i, stream.m_Index[i].qwChunkOffset,
		   stream.m_Index[i].dwChunkLength, stream.m_Index[i].dwFlags,
		   stream.m_Index[i].qwTimestamp);
#endif

	//hack   interleaved format  -  iavs
	//if (m_Streams[i].m_Header.fccType == mmioFOURCC('i', 'a', 'v', 's'))
	//    m_Streams[i].m_Header.fccType = streamtypeVIDEO;

	m_Input.addStream(m_Streams[i].m_iId, m_Streams[i].m_Index);
    }
    m_Input.async();
}

AviMediaReadHandler::~AviMediaReadHandler()
{
}

HRESULT AviMediaReadHandler::GetHeader(void* pheader, uint_t size) const
{
    if (!pheader || (size < sizeof(MainAVIHeader)))
        return -1;
    memset(pheader, 0, size);
    memcpy(pheader, &m_MainHeader, sizeof(MainAVIHeader));
    return 0;
}

void AviMediaReadHandler::checkDword(uint32_t value)
{
    uint_t v = m_Input.readDword();
    if (v != value)
    {
	//printf("%.4s  %.4s\n", (char*)&v, (char*)&value);
	throw FATAL("Not an AVI file!");
    }
}

int AviMediaReadHandler::readMainHeader(uint_t data_size)
{
//    checkInt(ckidAVIMAINHDR);//avih
//    uint_t data_size=m_Input.readInt();
    data_size += (data_size & 1);
    uint_t read_size = sizeof(MainAVIHeader);

    char b[read_size];
    memset(b, 0, read_size);

    if (data_size > read_size)
	printf("WARNING: unexpected main AVI header size\n");
    else
	read_size = data_size;

    m_Input.read(b, read_size);
    if (read_size < data_size)
        m_Input.seekCur(data_size - read_size);

    m_MainHeader.dwMicroSecPerFrame = avm_read_be32(b);
    m_MainHeader.dwMaxBytesPerSec = avm_read_be32(b + 4);
    m_MainHeader.dwPaddingGranularity = avm_read_be32(b + 8);
    m_MainHeader.dwFlags = avm_read_be32(b + 12);
    m_MainHeader.dwTotalFrames = avm_read_be32(b + 16);
    m_MainHeader.dwInitialFrames = avm_read_be32(b + 20);
    m_MainHeader.dwStreams = avm_read_be32(b + 24);
    m_MainHeader.dwSuggestedBufferSize = avm_read_be32(b + 28);
    m_MainHeader.dwWidth = avm_read_be32(b + 32);
    m_MainHeader.dwHeight = avm_read_be32(b + 36);

    for (int i = 0; i < 4; i++)
	m_MainHeader.dwReserved[i] = avm_read_be32(b + 40 + i * 4);

    printf("AviMainHeader: MicroSecPerFrame=%d MaxBytesPerSec=%d\n"
	   " PaddingGranularity=%d  Flags=[ %s%s%s%s%s ] TotalFrames=%d\n"
           " InitialFrames=%d  Streams=%d  SuggestedBufferSize=%d WxH=%dx%d\n",
	   m_MainHeader.dwMicroSecPerFrame, m_MainHeader.dwMaxBytesPerSec,

	   m_MainHeader.dwPaddingGranularity,
	   (m_MainHeader.dwFlags & AVIF_HASINDEX) ? " HasIndex " : "",
	   (m_MainHeader.dwFlags & AVIF_MUSTUSEINDEX) ? " MustUseIndex " : "",
	   (m_MainHeader.dwFlags & AVIF_ISINTERLEAVED) ? " IsInterleaved " : "",
	   (m_MainHeader.dwFlags & AVIF_COPYRIGHTED) ? " Copyrighted " : "",
	   (m_MainHeader.dwFlags & AVIF_WASCAPTUREFILE) ? " WasCapturedFile " : "",
	   m_MainHeader.dwTotalFrames,

	   m_MainHeader.dwInitialFrames, m_MainHeader.dwStreams,
	   m_MainHeader.dwSuggestedBufferSize,
	   m_MainHeader.dwWidth, m_MainHeader.dwHeight);

    return data_size + 8;
}

int AviMediaReadHandler::readStreamHeader()
{
    checkDword(ckidSTREAMHEADER);//strh
    uint_t data_size = m_Input.readDword();
    data_size += (data_size & 1);
    uint_t read_size = sizeof(AVIStreamHeader);
    char b[read_size];
    memset(b, 0, read_size);
    if (data_size > read_size)
	printf("WARNING: unexpected AVI stream header size\n");
    else
        read_size = data_size;

    m_Input.read(b, read_size);
    if (read_size < data_size)
        m_Input.seekCur(data_size - read_size);

    m_Streams.push_back(AviMediaReadStream(this));
    AviMediaReadStream& s = m_Streams.back();

    s.m_iId = m_Streams.size() - 1;

    s.m_Header.fccType = avm_read_be32(b);
    s.m_Header.fccHandler = avm_read_be32(b + 4);
    s.m_Header.dwFlags = avm_read_be32(b + 8);
    s.m_Header.wPriority = avm_read_be16(b + 12);
    s.m_Header.wLanguage = avm_read_be16(b + 14);
    s.m_Header.dwInitialFrames = avm_read_be32(b + 16);
    s.m_Header.dwScale = avm_read_be32(b + 20);
    s.m_Header.dwRate = avm_read_be32(b + 24);
    s.m_Header.dwStart = avm_read_be32(b + 28);
    s.m_Header.dwLength = avm_read_be32(b + 32);
    s.m_Header.dwSuggestedBufferSize = avm_read_be32(b + 36);
    s.m_Header.dwQuality = avm_read_be32(b + 40);
    s.m_Header.dwSampleSize = avm_read_be32(b + 44);
    s.m_Header.rcFrame.left = avm_read_be16(b + 48);
    s.m_Header.rcFrame.right = avm_read_be32(b + 50);
    s.m_Header.rcFrame.top = avm_read_be32(b + 52);
    s.m_Header.rcFrame.bottom = avm_read_be32(b + 54);

    char sb[10];
    if (s.m_Header.fccType == streamtypeVIDEO)
    {
	memcpy(sb, (const char*)&s.m_Header.fccHandler, 4);
	sb[4] = 0;
    }
    else
        sprintf(sb, "0x%x", s.m_Header.fccHandler);

    printf("AviStreamHeader: FccType=%.4s FccHandler=%s Flags=[ %s%s%s%s%s ]\n"
	   " InitialFrames=%d Scale=%d Rate=%d Start=%d Length=%d\n"
	   " SuggestedBufferSize=%d Quality=%d SampleSize=%d"
	   " Rect l,r,t,b=%d,%d,%d,%d\n",
	   (const char*)&s.m_Header.fccType, sb,
	   (s.m_Header.dwFlags & AVIF_HASINDEX) ? " HasIndex " : "",
	   (s.m_Header.dwFlags & AVIF_MUSTUSEINDEX) ? " MustUseIndex " : "",
	   (s.m_Header.dwFlags & AVIF_ISINTERLEAVED) ? " IsInterleaved " : "",
	   (s.m_Header.dwFlags & AVIF_COPYRIGHTED) ? " Copyrighted " : "",
	   (s.m_Header.dwFlags & AVIF_WASCAPTUREFILE) ? " WasCapturedFile " : "",

	   s.m_Header.dwInitialFrames, s.m_Header.dwScale,
	   s.m_Header.dwRate, s.m_Header.dwStart, s.m_Header.dwLength,

	   s.m_Header.dwSuggestedBufferSize,
	   s.m_Header.dwQuality, s.m_Header.dwSampleSize,
	   s.m_Header.rcFrame.left, s.m_Header.rcFrame.right,
	   s.m_Header.rcFrame.top, s.m_Header.rcFrame.bottom);

    //if (m_Header.dwStart && m_Header.dwStart)
    //    cout << "Calc starttime " << m_Header.dwStart * (m_Header.dwRate / (double)m_Header.dwStart) << endl;

    checkDword(ckidSTREAMFORMAT);//strf

    data_size = m_Input.readDword();
    s.m_uiFormatSize = data_size;
    data_size += (data_size & 1);
    // FIXME - read Format in  ENDIAN indepenent way
    // WAVEFORMATEX / BITMAPINFO
    s.m_pcFormat = new char[data_size];
    m_Input.read(s.m_pcFormat, data_size);

    //printf("format %x  %.4s  %d\n",
    //       s.m_Header.fccHandler, (char*)&s.m_Header.fccHandler, data_size);
    if (s.m_Header.fccType == streamtypeAUDIO)
    {
	if (!s.m_Header.dwSampleSize && s.m_Header.dwLength > 400000)
	{
	    // ((*(short*)s.m_pcFormat == WAVE_FORMAT_MPEGLAYER3)
	    // only if we have a lot of chunks
	    // if the stream would be VBR then ther would be far
	    // less chunks
	    printf("Warning: setting SampleSize=1\n");
	    s.m_Header.dwSampleSize = 1;
	}
    }
    else
	s.m_Header.dwSampleSize = 0;  // I'm not sure if should do this

#if 0
    if (s.m_Header.dwScale > 100)
        s.m_Header.dwScale = 9600;
    printf("out rate: %d    scale: %d\n", s.m_Header.dwRate, s.m_Header.dwScale);
#endif
    s.init();
    return 0;
}

/*
void AviMediaReadHandler::readStreams(int streams_size)
{
    int read_bytes=0;
    while(read_bytes<streams_size)
    {
	if(m_Input.readInt()!=FOURCC_LIST)
	    return;
	int stream_size=m_Input.readInt();
	if(m_Input.readInt()!=listtypeSTREAMHEADER)//strl
	    return;
	read_bytes+=12;
	stream_size-=4;
	int nextpos=m_Input.pos()+stream_size;
	readStreamHeader();
	m_Input.seek(nextpos);
	read_bytes+=stream_size;
    }
}
int AviMediaReadHandler::readMovieChunk()
{
    int moviem_uiChunk;
    while(!m_Input.eof())
    {
	if(m_Input.readInt()!=FOURCC_LIST)
	{
	    int chunk_size=m_Input.readInt();
	    if(chunk_size<0)throw FATAL("Damaged AVI file");
	    if(chunk_size%2) chunk_size++;
	    m_Input.seekCur(chunk_size);
	    continue;
	}
	int chunk_size=m_Input.readInt();
	if(chunk_size<4)throw FATAL("Damaged AVI file");
	if(chunk_size%2)chunk_size++;
	checkInt(listtypeAVIMOVIE);
	moviem_uiChunk=m_Input.pos();
	m_Input.seekCur(chunk_size-4);
	return moviem_uiChunk;
    }
    throw FATAL("Movie chunk not found");
}
*/
int AviMediaReadHandler::readIndexChunk(uint_t index_size, int movie_chunk_offset)
{
//    if(m_Input.readInt()!=ckidAVINEWINDEX)
//	return -1;

    bool zero_based_offsets = false;
//    movie_chunk_offset-=4;

//    int index_size=m_Input.readInt();
//    if(index_size<0)return -1;
    int64_t positions[m_Streams.size()];

    for (unsigned i = 0; i < m_Streams.size(); i++)
	positions[i] = m_Streams[i].m_Header.dwStart;

    AVIINDEXENTRY entry;

    for(; (index_size >= sizeof(AVIINDEXENTRY)) && !m_Input.eof();
	index_size -= sizeof(AVIINDEXENTRY))
    {
	AVIINDEXENTRY2 entry2;

	entry.ckid = m_Input.readDword();
	entry.dwFlags = m_Input.readDword();
	entry.dwChunkOffset = m_Input.readDword();
	entry.dwChunkLength = m_Input.readDword();

	uint_t id = StreamFromFOURCC(entry.ckid);
	if (id >= m_Streams.size())
	    continue;

	AviMediaReadStream& stream = m_Streams[id];

	entry2.dwFlags = entry.dwFlags;
	if (entry.dwChunkOffset < movie_chunk_offset)
	    zero_based_offsets = true;

	entry2.qwChunkOffset = entry.dwChunkOffset;
	if (zero_based_offsets)
	    entry2.qwChunkOffset += movie_chunk_offset - 4;

	entry2.dwChunkLength = entry.dwChunkLength;
	entry2.qwTimestamp = positions[id];
	//printf ("Id  %d    ps: %Ld   len:%d\n", id, entry2.qwTimestamp, entry2.dwChunkLength);
	if (stream.m_Header.dwSampleSize)
	    positions[id] += entry2.dwChunkLength;
	else
	    positions[id]++;  // video stream, one frame / chunk
	stream.m_Index.push_back(entry2);
    }

    //kprof_reset_stats();
    return 0;
}

void AviMediaReadHandler::reconstructIndexChunk(int offset)
{
    m_Input.seek(offset);
    int64_t positions[m_Streams.size()];
    int isdivx[m_Streams.size()];

    printf("Reconstructing index for this file");
    // fixing KeyFrames for DivX movies
    // they contain flag bit so we will check it
    for (unsigned i = 0; i < m_Streams.size(); i++)
    {
	positions[i] = m_Streams[i].m_Header.dwStart;
	isdivx[i] = 0;

	if (m_Streams[i].m_Header.fccType == streamtypeVIDEO)
	    switch (m_Streams[i].m_Header.fccHandler)
	    {
	    case fccDIV3:
	    case fccdiv3:
	    case fccDIV4:
	    case fccdiv4:
	    case fccDIV5:
	    case fccdiv5:
	    case fccDIV6:
	    case fccdiv6:
	    case fccMP43:
	    case fccmp43:
	    case fccMP42:
	    case fccmp42:
	    case fccAP41:
		isdivx[i] = 1;
                break;
    /* any heuristic for these broken DivX files ?? */
	    case fccDIVX:
	    case fccdivx:
	    case fccDIV1:
	    case fccdiv1:
	    case fccMP4S:
	    case fccmp4s:
	    case 0x4:
		isdivx[i] = 2;
	    }
    }

    // set when trying to reconstruct even very broken .avi files
    bool seriouslyBroken = false;
    while (!m_Input.eof())
    {
//	cout << hex << m_Input.pos() << dec << endl;
	uint_t ckid = m_Input.readDword();
	streamid_t id = StreamFromFOURCC(ckid);
	if (id >= m_Streams.size())
	{
	    if (m_Input.eof())
		break;
	    m_Input.seekCur(-2);
            if (!seriouslyBroken)
		printf("Trying to reconstruct very broken avi stream (could take a lot of time)\n");
	    seriouslyBroken = true;
	    continue;
	}
	AviMediaReadStream& stream = m_Streams[id];
	AVIINDEXENTRY2 entry2;
	entry2.dwFlags = AVIIF_KEYFRAME;
	entry2.qwChunkOffset = m_Input.pos() - 4;
	uint32_t size = m_Input.readDword();
        // max size for frame will be 250MB
	if (size > ((seriouslyBroken) ? 100000U : 0xfffffffU))
	// || size == 0)
	// size = 0 is legal
	    continue;
	entry2.dwChunkLength = size;
	entry2.qwTimestamp = positions[id];
	//printf ("Id  %d    ps: %Ld   len:%d\n", id, entry2.qwTimestamp, entry2.dwChunkLength);
	if (size & 1)
	    size++;

	if (isdivx[id] == 1 && size)
	{
	    int b = m_Input.readByte();
	    if (b & 0x40)
		entry2.dwFlags = 0;
            size--;
	}
        else if (isdivx[id] == 2 && size)
	{
            // DivX4 keyframe detection
	    uint32_t a = m_Input.readDword();
	    //printf("Dword 0x%x\n", a);
	    if (a != 0x00010000) // old OpenDivX
	    {
		if (a == 0xb6010000)
		{
		    // looks like new DivX frame
		    uint8_t b = m_Input.readByte() & 0xc0;
		    //printf("Byte 0x%x\n", b);
		    if (b != 0)
			entry2.dwFlags = 0;
		    // 0x00 Key frame,  0x40 P-frame
                    size--;
		}
                else
		    entry2.dwFlags = 0;
	    }
            size -= 4;
	}

	if (stream.m_Header.dwSampleSize) //audio stream, chunks contain variable number of frames
	    positions[id] += entry2.dwChunkLength;
	else //video stream, one frame / chunk
	    positions[id]++;

	//cout << "Size " << entry2.dwChunkLength << " offset " << entry2.qwChunkOffset << endl;
	stream.m_Index.push_back(entry2);
	m_Input.seekCur(size);
    }
}

IMediaReadStream* AviMediaReadHandler::GetStream(fourcc_t fccType, uint_t lParam)
{
    avm::vector<AviMediaReadStream>::iterator it;
    streamid_t match = 0;
    for (it = m_Streams.begin(); it != m_Streams.end(); it++)
	if (it->m_Header.fccType == fccType)
	{
	    if (match == lParam)
	    {
		//printf("GETHDR  %.4s   %.4s\n", (char*) &fccType, (char*)&it->m_Header.fccHandler);
		return &(*it); //safe for gcc3.0
	    }
	    match++;
	}
    return 0;
}

IMediaReadHandler *CreateAviMediaReadHandler(const char *pszFile) throw(FatalError)
{
    return new AviMediaReadHandler(pszFile);
}
