/********************************************************

	AviWriteStream/AviWriteFile class implementation
	Copyright 2000 Eugene Kuznetsov  (divx@euro.ru)

*********************************************************/

#define _LARGEFILE64_SOURCE
#include "AviWrite.h"

#include <fcntl.h>
#include <cstdio>

using namespace std;

#if defined(__FreeBSD__) || defined(__NetBSD__)
#define lseek64 lseek
#define O_LARGEFILE 0
#endif
Registrator Registrator::Reg;

AviWriteFile::AviWriteFile(const char* name, int flags, int mask):m_filename(name), m_fd(0)
{
    try
    {
        m_status=0;
	m_fd=new FileBuffer(name, O_WRONLY | O_CREAT | O_TRUNC | O_LARGEFILE, mask);

	const int junk_size=0x800;
        char* junk=new char[junk_size];
	memset(junk, 0, junk_size);
        m_fd->write(junk, junk_size);
	delete junk;
        m_status=1;

	memset(&m_header, 0, sizeof(m_header));
        m_header.dwFlags=flags;
	Registrator::Reg.push_back(this);
    }
    catch(FatalError&)
    {
	delete m_fd;
	throw;
    }
}

IAviWriteStream* AviWriteFile::AddStream(AviStream::StreamType type,
					 const void* format,
					 uint_t format_size,
					 fourcc_t handler, int frame_rate,
					 uint_t samplesize, int quality,
					 int flags)
{
    int ckid;
    ckid=MAKEAVICKID((type==AviWriteStream::Video)?cktypeDIBcompressed:cktypeWAVEbytes, m_streams.size());

    AviWriteStream* result=new AviWriteStream(this, ckid, type,
					      format, format_size,
					      handler, frame_rate,
					      samplesize,
					      quality, flags);

    m_streams.push_back(result);
    return result;
}

IAviVideoWriteStream* AviWriteFile::AddVideoStream(fourcc_t fourcc,
						   BITMAPINFOHEADER* srchdr,
						   int frame_rate, int flags)
{
    int ckid;
    ckid=MAKEAVICKID(cktypeDIBcompressed, m_streams.size());

    AviVideoWriteStream* result=new AviVideoWriteStream(this, ckid, fourcc,
							srchdr,	frame_rate, flags);
    if(result==0)
    	throw FATAL("Failed to create new video stream");

//    result->Create(fourcc, srchdr);
    m_streams.push_back(result);
    return result;
}
IAviAudioWriteStream* AviWriteFile::AddAudioStream(fourcc_t fourcc,
						   WAVEFORMATEX* fmt,
						   int bitrate, int flags)
{
    int ckid = MAKEAVICKID(cktypeWAVEbytes, m_streams.size());

    AviAudioWriteStream* result = new AviAudioWriteStream(this, ckid, fourcc,
							  fmt, bitrate, flags);
    if (!result)
    	throw FATAL("Failed to create new video stream");

    m_streams.push_back(result);
    return result;
}

//    temp=(X);
//    write(m_fd, &temp, 4);
AviWriteFile::~AviWriteFile()
{
    if (!m_status)
	return;

    try
    {
	uint_t videoendpos = m_fd->lseek(0, SEEK_CUR);
	if (videoendpos & 1)
	{
	    int junk = -1;
	    m_fd->write(&junk, 1);
	    videoendpos++;
	}
	WriteHeaders();

	write_32le(ckidAVINEWINDEX);
	write_32le(m_index.size() * sizeof(AVIINDEXENTRY));
	m_fd->write(&m_index[0], m_index.size()*sizeof(AVIINDEXENTRY));

	videoendpos=m_fd->lseek(0, SEEK_END);
	m_fd->lseek(4, SEEK_SET);
	write_32le(videoendpos-8);
	_destruct();
    }
    catch(...)
    {
	_destruct();
	throw;
    }
    if (!Registrator::Reg._destroying)
	Registrator::Reg.remove(this);
}

void AviWriteFile::_destruct()
{
    if (m_fd)
	delete m_fd;
    m_fd = 0;

    for (unsigned i=0; i<m_streams.size(); i++)
        delete m_streams[i];
    m_streams.clear();
    m_index.clear();
    m_status=0;
}

void AviWriteFile::AddChunk(uint_t offset, uint_t size, uint_t id, int flags)
{
//    m_index=(AVIINDEXENTRY*)realloc(m_index, (m_indsize+1)*sizeof(AVIINDEXENTRY));
    AVIINDEXENTRY entry;
    entry.ckid=id;
    entry.dwFlags=flags;
    entry.dwChunkOffset=offset;
    entry.dwChunkLength=size;
    try
    {
	m_index.push_back(entry);
    }
    catch(...)
    {
	throw FATAL("Chunk table reallocation failure");
    }
    if (m_index.size()%1000==0)
	WriteHeaders();
}

int64_t AviWriteFile::GetFileSize() const
{
    return m_fd->lseek(0, SEEK_CUR);
}

AviWriteStream::AviWriteStream(AviWriteFile* file, int ckid,
			       enum AviStream::StreamType type,
			       const void* format, uint_t format_size,
			       fourcc_t handler, int frame_rate,
			       uint_t samplesize,
			       int quality, int flags)
    :m_file(file), m_ckid(ckid)
{
    if (format == 0)
        throw FATAL("Bad format");
    if (type == Other)
        throw FATAL("Unsupported stream type");

    m_type = type;

    m_fd = file->m_fd;
    m_header.dwLength = 0;

    m_format = new uint8_t[format_size];
    m_forsize = format_size;
    memcpy(m_format, format, format_size);

    memset(&m_header, 0, sizeof(m_header));
    m_header.fccType = ((m_type == Video)?streamtypeVIDEO:streamtypeAUDIO);
    m_header.fccHandler = handler;
    m_header.dwFlags = flags;
    if (m_type == Video)
    {
        m_header.rcFrame.right = *(const int*)(m_format+4);
	m_header.rcFrame.bottom = *(const int*)(m_format+8);

	m_header.dwRate = 1000000;
        m_header.dwScale = frame_rate;
    }
    else
    {
        m_header.dwRate = frame_rate;
        m_header.dwScale = samplesize;
    }
    m_header.dwSampleSize = samplesize;
    m_header.dwQuality = quality;
    m_header.dwFlags = flags;
}
AviWriteStream::AviWriteStream(AviWriteFile* file, int ckid,
			       enum AviStream::StreamType type,
			       fourcc_t handler, int frame_rate, int flags)
    :m_file(file), m_format(0), m_ckid(ckid)
{
    m_fd = file->m_fd;

    memset(&m_header, 0, sizeof(m_header));

    m_header.fccHandler = handler;
    m_header.dwFlags = flags;

    switch(type)
    {
    case Other:
	throw FATAL("Bad type");
    case Video:
	m_type=Video;
        m_header.fccType = streamtypeVIDEO;
	m_header.dwRate = 1000000;
	m_header.dwScale = frame_rate;
	break;
    case Audio:
	m_type=Audio;
        m_header.fccType = streamtypeAUDIO;
	m_header.dwRate = frame_rate;
	m_header.dwScale = 1;
	break;
    }
}

HRESULT AviWriteStream::AddChunk(const void* chunk, uint_t size, int flags)
{
    if((chunk==0) && (size!=0))
    {
	printf("Invalid argument to AviWriteStream::AddChunk()\n");
	return -1;
    }
    uint_t offset=m_fd->lseek(0, SEEK_CUR);
    if (offset>0x7f000000)
	return -1;
    m_fd->write(&m_ckid, 4);
    m_fd->write(&size, 4);
    if (chunk)
    {
	m_fd->write(chunk, size);
	if (size & 1)
	    m_fd->write(chunk, 1);
    }
//    fsync(m_fd);
    m_file->AddChunk(offset-0x7FC, size, m_ckid, flags);
    if (m_header.dwSampleSize==0)
    	m_header.dwLength++;
    else
	m_header.dwLength += size/m_header.dwSampleSize;
    return 0;
}

HRESULT AviWriteFile::Reserve(uint_t size)
{
    return -1;
}

HRESULT AviWriteFile::WriteChunk(fourcc_t fourcc, void* data, uint_t size)
{
    return -1;
}

AviWriteStream::~AviWriteStream()
{
    if (m_format)
	delete m_format;
}

IAviWriteStream::~IAviWriteStream(){}
IAviVideoWriteStream::~IAviVideoWriteStream(){}
IAviWriteFile::~IAviWriteFile(){}

IAviWriteFile* CreateIAviWriteFile(const char* name, int flags, int mask)
{
    return new AviWriteFile(name, flags, mask);
}

void AviWriteFile::WriteHeaders()
{
    //cerr<<"**** Writing headers ****"<<endl;
    if (m_status == 0)
	return;
    m_header.dwFlags |= AVIF_HASINDEX | AVIF_TRUSTCKTYPE;
    m_header.dwPaddingGranularity = 0;
    m_header.dwTotalFrames = 0;
    for (unsigned i=0; i<m_streams.size(); i++)
    {
	if (m_streams[i]->GetType() == AviWriteStream::Video)
	{
	    m_header.dwTotalFrames = m_streams[i]->GetLength();
	    m_header.dwMicroSecPerFrame = m_streams[i]->m_header.dwScale;
	    m_header.dwWidth = m_streams[i]->m_header.rcFrame.right;
	    m_header.dwHeight = labs(m_streams[i]->m_header.rcFrame.bottom);
	    break;
	}
    }
    if (m_header.dwTotalFrames == 0)
	if (m_streams.size())
	{
	    m_header.dwTotalFrames = m_streams[0]->GetLength();
	    m_header.dwWidth = m_header.dwHeight = 0;
	}
    m_header.dwStreams = m_streams.size();
    uint_t endpos = m_fd->lseek(0, SEEK_END);
    m_fd->lseek(0, SEEK_SET);
    write_32le(FOURCC_RIFF);
    write_32le(endpos-8);
    write_32le(formtypeAVI);// Here goes chunk with all headers
    write_32le(FOURCC_LIST);
    int hdr_pos = 0x10;
    write_32le(0);//here header chunk size hdr_size will be
    int hdr_size = 12 + sizeof(MainAVIHeader);
    write_32le(listtypeAVIHEADER);
    write_32le(ckidAVIMAINHDR);
    write_32le(sizeof(MainAVIHeader));

    // Write MainAVIHeader
    write_32le(m_header.dwMicroSecPerFrame);
    write_32le(m_header.dwMaxBytesPerSec);
    write_32le(m_header.dwPaddingGranularity);
    write_32le(m_header.dwFlags);
    write_32le(m_header.dwTotalFrames);
    write_32le(m_header.dwInitialFrames);
    write_32le(m_header.dwStreams);
    write_32le(m_header.dwSuggestedBufferSize);
    write_32le(m_header.dwWidth);
    write_32le(m_header.dwHeight);
    for (unsigned i = 0; i < 4; i++)
	write_32le(m_header.dwReserved[i]);

    for (unsigned j = 0; j < m_streams.size(); j++)
    {
	int s = sizeof(AVIStreamHeader) + m_streams[j]->m_forsize
	    + (m_streams[j]->m_forsize & 1);
	hdr_size += 28 + s;
	write_32le(FOURCC_LIST);
	write_32le(20 + s);
        write_32le(listtypeSTREAMHEADER);
	write_32le(ckidSTREAMHEADER);
        write_32le(sizeof(AVIStreamHeader));

	m_fd->write(&m_streams[j]->m_header, sizeof(AVIStreamHeader));

	write_32le(ckidSTREAMFORMAT);
	write_32le(m_streams[j]->m_forsize);
	m_fd->write(m_streams[j]->m_format, m_streams[j]->m_forsize);

	if (m_streams[j]->m_forsize & 1)
	    m_fd->write(&m_fd, 1);
    }

    if (hdr_size > 0x700)
	throw FATAL("Too large header. Aborting");

    int curpos=m_fd->lseek(0, SEEK_CUR);
    write_32le(ckidAVIPADDING);
    write_32le(0x7F4 - (curpos + 8));

    m_fd->lseek(0x7F4, SEEK_SET);
    write_32le(FOURCC_LIST);
    write_32le(endpos-0x7FC);
    write_32le(listtypeAVIMOVIE);

    m_fd->lseek(hdr_pos, SEEK_SET);
    write_32le(hdr_size);
    m_fd->lseek(0, SEEK_END);
}
