/********************************************************

	Classes for writing AVIs
	Copyright 2000 Eugene Kuznetsov  (divx@euro.ru)

*********************************************************/
#ifndef AVIFILE_AVIWRITE_H
#define AVIFILE_AVIWRITE_H

#include "videoencoder.h"
#include "avifmt.h"
#include "avifile.h"
#include "except.h"
#include "FileBuffer.h"

#include <unistd.h>
#include <stdlib.h>

#define __MODULE__ "AviWriteFile"

class Component
{
public:
    virtual ~Component(){}
};

class Registrator: public avm::vector<Component*>
{
public:
    Registrator():_destroying(false){}
    static Registrator Reg;
    bool _destroying;
    ~Registrator()
    {
	_destroying=true;
	for (avm::vector<Component*>::iterator it=begin(); it!=end(); it++)
	    delete *it;
    }
};

class AviWriteFile;

class AviWriteStream : public IAviWriteStream
{
friend class AviWriteFile;
protected:
    AviWriteFile* m_file;
    FileBuffer* m_fd;
    AVIStreamHeader m_header;
    StreamType m_type;
    uint8_t* m_format;
    uint_t m_forsize;
    int m_ckid;
    AviWriteStream(AviWriteFile* file, int ckid, enum AviStream::StreamType,
		   fourcc_t handler, int frame_rate, int flags=0);
    void SetVideoData(int quality, int width, int height)
    {
    	m_header.dwQuality=quality;
        m_header.rcFrame.right=width;
	m_header.rcFrame.bottom=labs(height);
    }
public:
    AviWriteStream(AviWriteFile* file, int ckid,
		   enum AviStream::StreamType type,
		   const void* format, uint_t format_size,
		   fourcc_t handler, int frame_rate,
		   uint_t samplesize=0, int quality=0, int flags=0);

    ~AviWriteStream();
    //
    // these two should be called before data insertion begins
    //
    //
    // for video streams, specify time in microsecs per frame in frame_rate
    // for audio streams - count of bytes per second
    //
    StreamType GetType() const { return m_type; }
    HRESULT AddChunk(const void* chunk, uint_t size, int flags=0);

    uint_t GetLength() const { return m_header.dwLength; }
};

class AviAudioWriteStream : public IAviAudioWriteStream, public AviWriteStream
{
    IAudioEncoder* m_pIEnc;
    WAVEFORMATEX srcfmt;
    int _status;
    int _bitrate;
public:
    AviAudioWriteStream(AviWriteFile* file, int ckid, fourcc_t fourcc,
			WAVEFORMATEX* fmt, int bitrate, int flags=0);
    virtual ~AviAudioWriteStream();
    virtual HRESULT Start();
    virtual HRESULT AddData(void* data, uint_t size);
    virtual HRESULT Stop();
    virtual HRESULT SetQuality(int quality);
    uint_t GetLength() const;
};

class AviVideoWriteStream : public IAviVideoWriteStream, public AviWriteStream
{
    IVideoEncoder* m_pIEnc;
    int m_vstatus;
    uint8_t* m_local_buffer;
public:
    AviVideoWriteStream(AviWriteFile* file, int ckid, fourcc_t fourcc,
			BITMAPINFOHEADER* srchdr,
			int frame_rate, int flags=0);
    ~AviVideoWriteStream();
    virtual HRESULT Start();
    virtual HRESULT Stop();
    HRESULT AddFrame(CImage* chunk, uint_t* =0, int* =0);
    virtual HRESULT SetQuality(int quality);
    HRESULT SetKeyFrame(int frequency);
    uint_t GetLength() const;
};

//
//maintains global file info
//
class AviSegWriteFile;
class AviWriteFile: public IAviWriteFile, public Component
{
friend class AviWriteStream;
friend class AviSegWriteFile;
private:
    avm::vector<AviWriteStream*> m_streams;
    int m_strcnt;
    MainAVIHeader m_header;
    int m_status;
    avm::vector<AVIINDEXENTRY> m_index;
    avm::string m_filename;
protected:
    FileBuffer* m_fd;
//    int m_fd;
    void _destruct();
public:
    AviWriteFile(const char* name, int flags=0, int mask=00777);
    ~AviWriteFile();
    void WriteHeaders();
//    void Create(const char* name, int flags=0, int mask=00777);
    IAviAudioWriteStream* AddAudioStream(fourcc_t fourcc,
					 WAVEFORMATEX* fmt, int bitrate,
					 int flags=0);
    IAviVideoWriteStream* AddVideoStream(fourcc_t fourcc,
					 BITMAPINFOHEADER* srchdr,
					 int frame_rate, int flags);

    IAviWriteStream* AddStream(enum AviStream::StreamType type,
			       const void* format, uint_t format_size,
			       fourcc_t handler, int frame_rate,
			       uint_t samplesize=0,
			       int quality=0, int flags=0);
    HRESULT Reserve(uint_t size);
    HRESULT WriteChunk(fourcc_t fourcc, void* data, uint_t size);
    int64_t GetFileSize() const;
    const char* GetFileName() const { return m_filename.c_str(); }
protected:
    //
    //called when data is added to stream
    //
    void AddChunk(uint_t offset, uint_t size, uint_t id, int flags=0);

    void write_32le(int x) { m_fd->write_int(x); }
};

#endif // AVIFILE_AVIWRITE_H
