#include "audiodecoder.h"
#include "except.h"
#include "utils.h"

#include <stdio.h>

#define __MODULE__ "IAudioDecoder"

IAudioDecoder::IAudioDecoder(const CodecInfo& info, const WAVEFORMATEX* inw)
    :record(info)
{
    if (inw == 0)
	throw FATAL("Bad argument: NULL");

    in_fmt = *inw;
}

int IAudioDecoder::GetOutputFormat(WAVEFORMATEX* destfmt)
{
    if (!destfmt)
	return -1;

    // couple tricks here
    switch (in_fmt.wBitsPerSample)
    {
    case 0:
    case 2: // IMC
    case 4: // IMA_ADPCM
	in_fmt.wBitsPerSample = 16;
	break;
    }

    *destfmt = in_fmt; // copy just

    if (destfmt->nChannels == 5)
	destfmt->nChannels = 2; // AC3 5 channel

    switch (in_fmt.wFormatTag)
    {
    case 6: // A-Law  8 -> 16b
    case 7: // u-Law  8 -> 16b
        destfmt->wBitsPerSample = 16;
	break;
    case 0x2000:
	if (destfmt->nSamplesPerSec > 48000)
            destfmt->nSamplesPerSec = 48000;
    }

    destfmt->wFormatTag = 1;//PCM
    destfmt->nBlockAlign = destfmt->nChannels * destfmt->wBitsPerSample / 8;
    destfmt->nAvgBytesPerSec = destfmt->nSamplesPerSec * destfmt->nBlockAlign;
    destfmt->cbSize = 0;
    char b[200];
    avm_wave_format(b, sizeof(b), &in_fmt);
    printf("src %s\n", b);
    avm_wave_format(b, sizeof(b), destfmt);
    printf("dst %s\n", b);

    return 0;
}
