/*****************************************************************

  cdaudio_common.cc

  Copyright (c) 2001 by Burkhard Plaum - plaum@ipf.uni-stuttgart.de

  http://gmerlin.sourceforge.net

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111, USA.

*****************************************************************/

#ifdef __GNUG__
#pragma implementation "cdaudio_common.h" 
#endif

#include <iostream>
using std::ostream;
using std::cerr;
using std::ios;

#include <sstream>
using std::stringstream;

#include <iomanip>

using std::setw;
using std::setfill;
using std::ends;

#include <glib.h>
#include <ctype.h>
#include <stdlib.h>
#include <time.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>

#include <sys/ioctl.h>

#include <fstream.h>

#include <linux/cdrom.h>
#include <linux/hdreg.h>
#include <stdio.h>
#include <string.h>
#include <limits.h>

#include <dirent.h>

#include "../gvconfig.h"
#include "../gv.h"

#include "../configwindow.h"
#include "../trackinfo.h"
#include "../gtkutil.h"
#include "../socket.h"

#include "cdaudio_common.h"

static const char * __read_queue_file = "cddb_read_queue";

// create name of the read queue file, returned string must be g_freed

static char * get_read_queue_name()
  {
  const char * tmp;
  gv_config->get_string("_", "HomeDir", &tmp);
  return g_strdup_printf("%s/%s", tmp, __read_queue_file);
  }

// This was taken from grip

static int cddb_sum(int val)
{
   char *bufptr, buf[16];
   int ret = 0;
   
   snprintf(buf, 16, "%u", val);
   for(bufptr = buf; *bufptr != '\0'; bufptr++)
     ret += (*bufptr - '0');
   
   return ret;
}

// Calcutale the disc ID for the current CD

guint32 GVCdaudioCommon::get_cddb_id()
  {
  int tracksum = 0;

  guint32 ret;
  int discid;
  for(int i = 0; i < num_tracks; i++)
    {
    tracksum += cddb_sum(tracks[i].start_frame/75);
    }
  discid = (tracks[num_tracks-1].end_frame/75) - (tracks[0].start_frame/75);
  
  ret = (tracksum % 0xFF) << 24 | discid << 8 | num_tracks;

  return ret;
  }

int GVCdaudioCommon::cddb_get_info()
  {
  int ret;
  guint32 disc_id = get_cddb_id();

  char * filename = new char[PATH_MAX];

  ifstream cddb_file;
  
  // First: Try to search system wide CDDB database
  // Not implemented yet, will do this if I have the database
  
  //   sprintf(filename, "%s/%8x", cddb_system_dir, disc_id);
  //   cddb_file = fopen(filename, "r");

  // Second: Try to open $HOME/.cddb

//   if(!cddb_file)
//     {
//    cddb_file.clear();
    sprintf(filename, "%s/.cddb/%8x", g_get_home_dir(), disc_id);
    cddb_file.open(filename);
//     }

  // Third: Try to open a cddb connection and get the index file
  // If this succeeds, we have the file in $HOME/.cddb/ after
  
    if(!cddb_file)
      {
      cddb_file.clear();

      int * start_frames = new int[num_tracks];

      for(int i = 0; i < num_tracks; i++)
        start_frames[i] = tracks[i].start_frame;
      
      
      cddb_download_info(disc_id, start_frames, num_tracks,
                         tracks[num_tracks-1].end_frame/75);
      delete start_frames;
      cddb_file.open(filename);
      }
  if(cddb_file)
    {
    cddb_file.close();
    cddb_parse_file(filename);
    ret = 1;
    }
  else
    {
    // We cannot get the info, put the discid into the toget file
    // for later reading (TODO)
    cddb_queue(disc_id);
    ret = 0;
    }
  delete filename;
  return ret;
  }

static void __cddb_substitute(char * buffer)
  {
  int len = strlen(buffer);
  int substitute = 0;

  for(int i = 0; i < len; i++)
    {
    substitute = 0;
    if(buffer[i] == '\\')
      {
      switch(buffer[i+1])
        {
        case 't':
          buffer[i] = '\t';
          substitute = 1;
          break;
        case 'n':
          buffer[i] = '\n';
          substitute = 1;
          break;
        case '\\':
          buffer[i] = '\\';
          substitute = 1;
          break;
        default:
          break;
        }
      if(substitute)
        {
        for(int j = i+1; j < len; j++)
          buffer[j] = buffer[j+1];
        len--;
        }

      }
    }
  }

// Convert something like "DTITLE=Gong / Angels Egg" to
// "DTITLE\0Gong / Angels Egg"
// where value points to "Gong / Angels Egg" after

static void __cddb_get_key_value(char * buffer, char ** value)
  {
  if(buffer[0] == '#')
    {
    *value = (char*)0;
    return;
    }

  char * pos = strchr(buffer, '=');
  if(!pos)
    {
    *value = (char*)0;
    return;
    }
  else
    {
    *pos = '\0';
    *value = pos;
    (*value)++;

    // Set value to NULL if the field is empty
    
    if(*value == '\0')
      *value = (char*)0; 
    }
  }

static void __str_add(char ** old_string, char * new_string)
  {
  char * c_tmp;

  __cddb_substitute(new_string);

  if(*old_string)
    {
    c_tmp = g_strconcat(*old_string, new_string, (char*)0);
    g_free(*old_string);
    *old_string = c_tmp;
    }
  else
    *old_string = g_strdup(new_string);
  }

void GVCdaudioCommon::cddb_parse_file(const char * filename)
  {
  //  cerr << "Parsing File...";
  char * read_buffer = new char[257];

  // We allocate some temporary strings here

  char * disc_title = (char*)0;
  char * extended_disc_title = (char*)0;

  char * genre = (char*)0;
  char ** track_titles;
  char ** extended_track_titles;

  // Temporary variables
  
  char * album;
  char * artist;
  
  track_titles = new char *[num_tracks];
  extended_track_titles = new char *[num_tracks];

  for(int i = 0; i < num_tracks; i++)
    {
    track_titles[i] = (char*)0;
    extended_track_titles[i] = (char*)0;
    }
  
  char * value;

  char * c_tmp;
  char * c_tmp1;
  int i_tmp;

  ifstream input;

  input.open(filename);

  //  cerr << "Opened file " << filename << "\n";
  
  char delimiter;
  
  while(1)
    {
    //    cerr << "Reading line...";

    input.get(read_buffer, 256, '\n');
    input.get(delimiter);
    //    cerr << "done, Line: " << read_buffer << "\n";
    if(input.eof())
      break;
    
    // Remove the trailing newline

    c_tmp = strchr(read_buffer, '\n');
    if(c_tmp)
      *c_tmp = '\0';

    // Some ill conditioned operating systems also need '\r'
    // for linebreaks

    c_tmp = strchr(read_buffer, '\r');
    if(c_tmp)
      *c_tmp = '\0';
    
    __cddb_get_key_value(read_buffer, &value);
    if(!value)
      continue;

    if(!strcmp(read_buffer, "DTITLE"))
      __str_add(&disc_title, value);

    if(!strcmp(read_buffer, "EXTD"))
      __str_add(&extended_disc_title, value);
    
    else if(!strcmp(read_buffer, "DGENRE"))
      __str_add(&genre, value);

    else if(!strncmp(read_buffer, "TTITLE",6))
      {
      i_tmp = atoi(&read_buffer[6]);
      if(i_tmp < num_tracks)
        __str_add(&track_titles[i_tmp], value);
      }
    else if(!strncmp(read_buffer, "EXTT", 4))
      {
      i_tmp = atoi(&read_buffer[4]);
      if(i_tmp < num_tracks)
        __str_add(&extended_track_titles[i_tmp], value);
      }
    }

  // Adjust the data for out needs
  
  // Convert the first letter of the genre to uppercase

  if(genre)
    genre[0] = toupper(genre[0]);

  // Create artist and album. Artist will be NULL if we have a sampler
  
  artist = disc_title;
  c_tmp = strpbrk(disc_title, "/");

  // Format: Artist / Album
  
  if(c_tmp)
    {
    // Set the album

    album = c_tmp;
    album++;
    while(isspace(*album))
      album++;

    // Set the trailing zero for artist
    
    c_tmp--;
    while(isspace(*c_tmp))
      c_tmp--;
    c_tmp++;
    *c_tmp = '\0';
    }

  // Format: Artist (Artist and album are the same)
  
  else
    album = disc_title;

  cerr << "Album: " << album << "\n";
  
  // Set artist to zero if we have a sampler
  
  if(!strncasecmp(artist, "Various", 7))
    artist = (char*)0;

  // Now we have all the data, let's fill the track infos

  if(track_infos)
    delete [] track_infos;
  
  track_infos = new GVTrackInfo[num_tracks];
    
  for(int i = 0; i < num_tracks; i++)
    {
    track_infos[i].track_number = i+1; // Set track number
    track_infos[i].set_genre(genre);   // Set genre
    track_infos[i].set_album(album);   // Set album

    // Set artist and title, different for samplers
    
    if(artist)
      {
      if(artist_from_extended)
        track_infos[i].set_artist(extended_track_titles[i]);
      else
        track_infos[i].set_artist(artist);
      track_infos[i].set_title(track_titles[i]);
      }
    else
      {
      c_tmp = track_titles[i];
      c_tmp = strpbrk(track_titles[i], "-/");

      // Create the trailing zero for the artist

      if(!artist_from_extended)
        {
        c_tmp1 = c_tmp;
        c_tmp1--;
        while(isspace(*c_tmp1))
          c_tmp1--;
        c_tmp1++;
        *c_tmp1 = '\0';
        }
      
      // Set start pointer for track title

      c_tmp++;
      while(isspace(*c_tmp))
        c_tmp++;
      if(!artist_from_extended)
        track_infos[i].set_artist(track_titles[i]);
      else
        track_infos[i].set_artist(extended_track_titles[i]);
      track_infos[i].set_title(c_tmp);
      }
    }
  cerr << "Disc Title: " << disc_title << "\n";
//   cerr << "Genre: " << genre << "\n";
//   for(int i = 0; i < num_tracks; i++)
//     cerr << "Track " << i+1 << ": " << track_titles[i] << "\n";

  if(disc_title)
    g_free(disc_title);

  if(extended_disc_title)
    g_free(extended_disc_title);

  if(genre)
    g_free(genre);

  for(int i = 0; i < num_tracks; i++)
    {
    if(track_titles[i])
      g_free(track_titles[i]);
    if(extended_track_titles[i])
      g_free(extended_track_titles[i]);
    }

  delete track_titles;
  delete extended_track_titles;
    
  input.close();
  delete read_buffer;
  }

GVCdaudioCommon::GVCdaudioCommon(const GVPluginAttributes * _attributes) :
  GVInput(_attributes)
  {
  get_config();
  tracks = (GVCdaudioTrack*)0;
  track_infos = (GVTrackInfo*)0;
  cddb_widget = (CDDBWidget *)0;
  device_widget = (GVConfigWidgetDevicelist*)0;
  }

GVCdaudioCommon::~GVCdaudioCommon()
  {
  if(tracks)
    delete [] tracks;
  if(track_infos)
    delete [] track_infos;
  put_config();
  }

int GVCdaudioCommon::get_playlist_track(int track_index)
  {
  const GVPlaylistTrack * item = gv_playlist->get_track(track_index);
  if(item)
    return ((GVCdaudioTrack*)(item->data))->index;
  else
    return -1;
  }

// Get Current CD track from playlist

int GVCdaudioCommon::get_current_playlist_track()
  {
  const GVPlaylistTrack * item = gv_playlist->get_current_track();
  if(item)
    return ((GVCdaudioTrack*)(item->data))->index;
  else
    return -1;
  }

void GVCdaudioCommon::fill_playlist()
  {
  read_toc();

  gv_playlist->make_private(1);
  gv_playlist->clear();
  
  if(!tracks)
    return;
  
  int cddb_present = cddb_get_info();

  if(cddb_present)
    cerr << "Got CDDB Data\n";
  else
    cerr << "Got no CDDB data\n";
  

  if(!tracks)
    return;

  stringstream line;

  //  cerr << "Filling Playlist\n";

  if(cddb_present)
    {
    for(int i = 0; i < num_tracks; i++)
      {
      line.seekp(0);
      track_infos[i].print_string(line);
      gv_playlist->append_track((char*)0, line.str().c_str(), tracks[i].length,
                                GV_PLAYLIST_ITEM_HAS_AUDIO|
                                GV_PLAYLIST_ITEM_NOSAVE,
                                attributes->name,
                                &tracks[i]);
      }
    }
  else
    {
    for(int i = 0; i < num_tracks; i++)
      {
      //      line.freeze(0);
      line.seekp(0);
      line << "CD Track " << setw(2) << setfill('0') << i+1 << ends;
      gv_playlist->append_track((char*)0, line.str().c_str(), tracks[i].length,
                                GV_PLAYLIST_ITEM_HAS_AUDIO|
                                GV_PLAYLIST_ITEM_NOSAVE,
                                attributes->name,
                                &tracks[i]);
      }
    }
  }

int GVCdaudioCommon::check_for_cd()
  {
  struct cdrom_tochdr toc_header;

  error = GV_PLUGIN_ERROR_NONE;
  
  if((cd_fd = open(device, O_RDONLY | O_NONBLOCK)) < 0)
    {
    g_log(PACKAGE, G_LOG_LEVEL_CRITICAL,
          "Couldn't open CD device %s",
          filename);
    error = GV_PLUGIN_ERROR_DEVICE_OPEN_FAILED;
    return 0;
    }

  if(ioctl(cd_fd, CDROMREADTOCHDR, &toc_header) < 0)
    {
    g_log(PACKAGE, G_LOG_LEVEL_CRITICAL,
          "Insert Audio CD");
    error = GV_PLUGIN_ERROR_NO_MEDIUM;
    close(cd_fd);
    return 0;
    }
  return 1;
  }

// Read the table of contents into the tracks array

void GVCdaudioCommon::read_toc()
  {
  num_tracks = 0;
  if(tracks)
    {
    delete tracks;
    tracks = (GVCdaudioTrack*)0;
    }
  
  if(!check_for_cd())
    return;

  
  //  struct cdrom_subchnl subchannel;
  struct cdrom_tochdr toc_header;
  cdrom_tocentry toc_entry;
  
  if(ioctl(cd_fd, CDROMREADTOCHDR, &toc_header) < 0)
    {
    g_log(PACKAGE, G_LOG_LEVEL_CRITICAL,
          "Insert Audio CD");
    error = GV_PLUGIN_ERROR_NO_MEDIUM;
    return;
    }
  
  // Get the number of tracks
  
  num_tracks = toc_header.cdth_trk1 - toc_header.cdth_trk0 + 1;

  //  cerr << num_tracks << "Tracks\n";
  
  tracks = new GVCdaudioTrack[num_tracks];

  toc_entry.cdte_format = CDROM_MSF;

  int track_frames;
  
  for(int i = 0; i <= num_tracks; i++)
    {
    if(i < num_tracks)
      {
      toc_entry.cdte_track = i+1;
      tracks[i].index = i+1;
      }
    else
      toc_entry.cdte_track = CDROM_LEADOUT;

    ioctl(cd_fd, CDROMREADTOCENTRY, &toc_entry);

    track_frames = toc_entry.cdte_addr.msf.frame +
      75 * toc_entry.cdte_addr.msf.second +
      60 * 75 * toc_entry.cdte_addr.msf.minute;
    
    if(i < num_tracks)
      tracks[i].start_frame = track_frames;
    if(i)
      tracks[i-1].end_frame = track_frames-1;
    }

  for(int i = 0; i < num_tracks; i++)
    tracks[i].length.set_from_frames((unsigned int)(tracks[i].end_frame -
                                                  tracks[i].start_frame),
                                     (unsigned int)75);

  close(cd_fd);
  }

static char * _cddb_config_section = "CDDB";

// The following placeholders are defined:
// %a: artist
// %A: album
// %t: track name
// %n: track number
// %X: Extended album info
// %x: Extended track info

void GVCdaudioCommon::get_config()
  {
//   char * cddb_track_template;
//   char * cddb_album_template;

//   char * cddb_server;
//   char * cddb_path;

  const char * tmp;

  tmp = "freedb.freedb.org";
  gv_config->get_string(_cddb_config_section, "Server", &tmp);
  cddb_server = g_strdup(tmp);

  tmp = "~cddb/cddb.cgi";
  gv_config->get_string(_cddb_config_section, "CGIPath", &tmp);
  cddb_path = g_strdup(tmp);

  queue_cddb_read = 1;
  gv_config->get_bool(_cddb_config_section, "QueueRead", queue_cddb_read);

  artist_from_extended = 0;
  gv_config->get_bool(_cddb_config_section, "ArtistFromExtended",
                      artist_from_extended);
  }

void GVCdaudioCommon::put_config()
  {
  gv_config->put_string(_cddb_config_section, "Server", cddb_server);
  gv_config->put_string(_cddb_config_section, "CGIPath", cddb_path);
  gv_config->put_bool(_cddb_config_section, "QueueRead", queue_cddb_read);
  gv_config->put_bool(_cddb_config_section, "ArtistFromExtended",
                      artist_from_extended);
  }

const GVTrackInfo * GVCdaudioCommon::get_track_info()
  {
  if(!track_infos)
    return (const GVTrackInfo*)0;
  else
    return &track_infos[get_current_playlist_track() - 1];
  }

void GVCdaudioCommon::cddb_queue(guint32 disc_id)
  {
  ofstream output;
  ifstream input;
  
  char * filename = get_read_queue_name();

  char * c_tmp_disc_id = g_strdup_printf("%08x", disc_id);

  char * buffer = new char[4096];
  char delimiter;

  int do_add = 1;
  
  // Check if the disc id is already in the queue

  input.open(filename);

  if(input)
    {
    while(!input.eof())
      {
      input.get(buffer, 4096, '\n');
      input.get(delimiter);
      if(!strncmp(buffer, c_tmp_disc_id, 8))
        {
        cerr << "discid " << c_tmp_disc_id << " is already in the queue\n";
        do_add = 0;
        break;
        }
      }
    input.close();
    }
  
  if(do_add)
    {
    output.open(filename, ios::out|ios::app);
    //  output.seekp(ios::end);
    output << c_tmp_disc_id << " " << num_tracks << " ";
    for(int i = 0; i < num_tracks; i++)
      output << tracks[i].start_frame << " ";
    output << tracks[num_tracks-1].end_frame / 75 << "\n";
    output.close();
    }
  g_free(filename);
  g_free(c_tmp_disc_id);
  delete buffer;
  }

void GVCdaudioCommon::cddb_queue_download()
  {
  if(!queue_cddb_read)
    return;

  //  const char * tmp;

  char * filename = get_read_queue_name();

  char * buf;

  char delimiter;
  guint32 i_tmp_disc_id;
  
  ifstream input;
  
  input.open(filename);

  int * start_frames;
  int num;
  int disc_seconds;

  char * pos;
  char * rest;
  
  if(!input)
    {
    g_free(filename);
    return;
    }

  buf = new char[4096];
  start_frames = new int[100];
  
  while(!input.eof())
    {
    input.get(buf, 4096, '\n');
    input.get(delimiter);

    // First: i_tmp_disc_id in hexadecimal

    pos = buf;

    i_tmp_disc_id = strtoul(pos, &rest, 16);

    // We reached the end of the file, break here to prevent gmerlin from
    // doing some real stupid stuff
    
    if(pos == rest)
      break;

    pos = rest;
    num = strtol(pos, &rest, 10);
    pos = rest;

    for(int i = 0; i < num; i++)
      {
      start_frames[i] = strtol(pos, &rest, 10);
      pos = rest;
      }
    disc_seconds = strtol(pos, &rest, 10);
    cddb_download_info(i_tmp_disc_id, start_frames, num, disc_seconds);
    }
  remove(filename);

  delete buf;
  delete start_frames;

  g_free(filename);
  }

static void __button_callback(GtkWidget * w, gpointer data)
  {
  ((GVCdaudioCommon*)data)->cddb_queue_download();
  }

GVCdaudioCommon::CDDBWidget::CDDBWidget(GVCdaudioCommon * _plugin) :
  cddb_server("Server"),
  cddb_path("CGI Path")
  {
  plugin = _plugin;

  queue_cddb_read = gtk_check_button_new_with_label("Queue Lookups");

  artist_from_extended
    = gtk_check_button_new_with_label("Get Artist from extended track info");

  download_now = gtk_button_new_with_label("Download Now");

  gtk_signal_connect(GTK_OBJECT(download_now), "clicked",
                     GTK_SIGNAL_FUNC(__button_callback), (gpointer)plugin);
  
  gtk_widget_show(queue_cddb_read);
  gtk_widget_show(artist_from_extended);
  gtk_widget_show(download_now);

  widget = gvgtk_table_new(4, 2, 0);
  cddb_server.attach(widget, 0, 0);
  cddb_path.attach(widget, 1, 0);

  gvgtk_table_attach_noresize(widget, artist_from_extended, 0, 2, 2, 3);
  
  gvgtk_table_attach_noresize(widget, queue_cddb_read, 0, 1, 3, 4);
  gtk_table_attach_defaults(GTK_TABLE(widget), download_now,
                            1, 2, 3, 4);
  
  gtk_widget_show(widget);
  }

void GVCdaudioCommon::CDDBWidget::get_values()
  {
  cddb_server.set_string(plugin->cddb_server);
  cddb_path.set_string(plugin->cddb_path);
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(queue_cddb_read),
                               plugin->queue_cddb_read);

  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(artist_from_extended),
                               plugin->artist_from_extended);
  
  }

void GVCdaudioCommon::CDDBWidget::apply_values()
  {
  g_free(plugin->cddb_server);
  plugin->cddb_server = g_strdup(cddb_server.get_string());
  
  g_free(plugin->cddb_path);
  plugin->cddb_path = g_strdup(cddb_path.get_string());

  plugin->queue_cddb_read =
    gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(queue_cddb_read));

  plugin->artist_from_extended =
    gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(artist_from_extended));
  
  }

void GVCdaudioCommon::cddb_download_info(guint32 i_tmp_disc_id,
                                         int * start_frames,
                                         int num, int seconds)
  {
  GVSocket cddb_socket;
  cddb_socket.open(cddb_server, 80);
  if(cddb_socket.sock == -1)
    return;

  char * c_tmp_disc_id = g_strdup_printf("%08x", i_tmp_disc_id);

  stringstream line;
  line.seekp(0);
  line << "/" << cddb_path << "?cmd=cddb+query+";

  line << c_tmp_disc_id;

  line << "+" << num << "+";
  
  for(int i = 0; i < num; i++)
    line << start_frames[i] << "+";

  line << seconds;
  
  line << "&hello=private+free.the.cddb+" << PACKAGE << "+" << VERSION <<
    "&proto=3";
  
  line << ends;
 
  //  cerr << "Made request: " << line.str() << "\n";

  cddb_socket.send_http_request(line.str().c_str(), "text/plain");
  cddb_socket.skip_http();
    
  char * pos;

  const char * c_pos;
  while(1)
    {
    //    line.freeze(0);
    line.seekp(0);
    line.seekg(0);
    cddb_socket.read_line(line);
    line << ends;
    
    c_pos = line.str().c_str();
    cerr << "Got: " << c_pos << "\nlength: " << strlen(c_pos) << "\n";
    if(isdigit(*c_pos) && (strlen(c_pos) >= 5))
      break;
    }

  char * genre = (char*)0;
  char * start_genre;
  char * end_genre;

  int error_code = atoi(c_pos);

  int got_info = 1;

  char * c_old_disc_id = (char*)0;
  
  switch(error_code)
    {
    case 200:
      {
      pos = g_strdup(c_pos);
      
      start_genre = pos;
      while(isdigit(*start_genre))
        start_genre++;
      while(isspace(*start_genre))
        start_genre++;
      end_genre = start_genre;
      while(!isspace(*end_genre))
        end_genre++;
      *end_genre = '\0';
      genre = g_strdup(start_genre);
      cerr << "Found CD: " << pos << "\n";
      g_free(pos);
      break;
      }
    case 211:
      // If we get only inexact matches, we take the first one.

      //      line.freeze(0);
      line.seekp(0);
      line.seekg(0);
      cddb_socket.read_line(line);
      line << ends;
        
      pos = g_strdup(line.str().c_str());
      //      cerr << "Got: " << pos << "\nlength: " << strlen(pos) << "\n";

      cerr << "Found inexact match, taking " << pos << "\n";

      start_genre = pos;
      end_genre = pos;
      while(!isspace(*end_genre))
        end_genre++;
      *end_genre = '\0';
      genre = g_strdup(start_genre);
      //      cerr << "Found genre: " << genre << "\n";

      // Read the disc id
      
      end_genre++;
      
      sscanf(end_genre, "%x", &i_tmp_disc_id);
      c_old_disc_id = c_tmp_disc_id;
      c_tmp_disc_id = g_strdup_printf("%08x", i_tmp_disc_id);
      g_free(pos);
      break;
    default:
      got_info = 0;
      cerr << "Cannot handle server code " << error_code << "\n";
    }
  
  cddb_socket.close();
  cddb_socket.open(cddb_server, 80);

  if(got_info)
    {
    //    line.freeze(0);
    line.seekp(0);
    line << "/" << cddb_path << "?cmd=cddb+read+" << genre << "+"
         << c_tmp_disc_id;
    line << "&hello=private+free.the.cddb+" << PACKAGE << "+" << VERSION <<
      "&proto=3";
    
    line << ends;
    
    //    cerr << "Made request: " << line.str() << "\n";
    
    cddb_socket.send_http_request(line.str().c_str(), "text/plain");
    
    cddb_socket.skip_http();
    
    while(1)
      {
      //      line.freeze(0);
      line.seekp(0);
      line.seekg(0);
      cddb_socket.read_line(line);
      line << ends;
      
      c_pos = line.str().c_str();
      //      cerr << "Got: " << pos << "\n";
      if(isdigit(*c_pos) && (strlen(c_pos) >= 5))
        break;
      }
    
    error_code = atoi(c_pos);
    
    switch(error_code)
      {
      case 210:
        {
        // Copy the following lines to the cddb file
        
        ofstream output;
        char * filename;
        char * local_cddb_dir
          = g_strdup_printf("%s/.cddb", g_get_home_dir());
        DIR * dir = opendir(local_cddb_dir);
        if(!dir)
          mkdir(local_cddb_dir, S_IRUSR|S_IWUSR|S_IXUSR);
        else
          closedir(dir);
        
        // Now, download the file
        
        filename = g_strdup_printf("%s/%s", local_cddb_dir, c_tmp_disc_id);
        output.open(filename);
        while(1)
          {
          line.seekp(0);
          line.seekg(0);
          cddb_socket.read_line(line);
          line << ends;
          c_pos = line.str().c_str();
          //          cerr << "Got: " << pos << "\n";
          if((strlen(c_pos) < 5) && (*c_pos != '#'))
            break;
          output << c_pos << "\n";
          }
        output.close();
        
        // Create a symbolic link from the exact match to the inexact match

        if(c_old_disc_id)
          {
          char * old_filename = g_strdup_printf("%s/%s", local_cddb_dir,
                                                c_old_disc_id); 
          symlink(filename, old_filename);
          g_free(old_filename);
          }
        g_free(filename);
        g_free(local_cddb_dir);
        break;
        }
      default:
        cerr << "Cannot handle server code " << error_code << "\n";
      }
    }
  cddb_socket.close();
  g_free(c_tmp_disc_id);
  if(c_old_disc_id)
    g_free(c_old_disc_id);
  }
