/*****************************************************************

  cdaudio_common.h

  Copyright (c) 2001 by Burkhard Plaum - plaum@ipf.uni-stuttgart.de

  http://gmerlin.sourceforge.net

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111, USA.

*****************************************************************/

// Common definitions for the cdparanioa and cd player plugins

#ifdef __GNUG__
#pragma interface
#endif

struct GVCdaudioTrack
  {
  int index;        // Index for cdrom!!
  int start_frame;  // Starting frame
  int end_frame;    // End frame
  GVTime length;
  };

class GVCdaudioCommon : public GVInput
  {
  int queue_cddb_read;
  int artist_from_extended;
  
protected:
  
  class CDDBWidget : public GVConfigWidget
    {
    GVConfigWidgetString cddb_server;
    GVConfigWidgetString cddb_path;

    GtkWidget * queue_cddb_read;
    GtkWidget * download_now;
    GtkWidget * artist_from_extended;
    GVCdaudioCommon * plugin;

  public:
    CDDBWidget(GVCdaudioCommon * _plugin);
    void get_values();
    void apply_values();
    };

  CDDBWidget * cddb_widget;

  GVConfigWidgetDevicelist * device_widget;
  
  friend class ConfigWidget;
  friend class CDDBWidget;
  
private:
  
  
  char * cddb_system_dir;
  char * cddb_server;

  char * cddb_track_template;
  char * cddb_album_template;

  char * cddb_path;
  
  // CDDB Stuff

  GVTrackInfo * track_infos; // Array to fill with cddb
  guint32 get_cddb_id();     // Calcutale the disc ID for the current CD

  // This gets the cddb info of the disc and fills the track infos
  // Returns 1 on Success
  
  int cddb_get_info();

  void cddb_download_info(guint32 disc_id, int * start_frames,
                          int num, int seconds);

  
  void cddb_parse_file(const char * filename);

  
  void get_config();
  void put_config();
    
protected:
  
  int cd_fd;
  GVCdaudioTrack * tracks;
  
  //
  
  int num_tracks;

  void read_toc();  // Read the table of contents into the tracks array

  GVCdaudioCommon(const GVPluginAttributes * _attributes);
  virtual ~GVCdaudioCommon();
 
  // Get the track number from the playlist
  int get_playlist_track(int track);
  int get_current_playlist_track();

  // Check for an Audio CD, return 1 on success
  // If an audio CD is found, the filedescriptor is open after this call
  
  int check_for_cd();
    
public:

  const GVTrackInfo * get_track_info();
  
  void fill_playlist();

  void cddb_queue_download();
  void cddb_queue(guint32 disc_id);
  };
