/*****************************************************************

  i_cdaudio.cc

  Copyright (c) 2001 by Burkhard Plaum - plaum@ipf.uni-stuttgart.de

  http://gmerlin.sourceforge.net

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111, USA.

*****************************************************************/

// CD player for gmerlin
// This is like almost every cdplayer: It plays audio directly
// to the soundcard

#ifdef __GNUG__
#pragma implementation "i_cdaudio.h"
#endif

#include <iostream>
using std::ostream;
using std::cerr;

//#include <iomanip.h>
#include <glib.h>
#include <ctype.h>
#include <stdlib.h>
#include <time.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>

#include <sys/ioctl.h>
#include <stdio.h> // For cdaudio_common

#include <linux/cdrom.h>

#include "../gvconfig.h"

#include <gtk/gtk.h>
#include <gmodule.h>
//#include "../configwindow.h"
#include "../gv.h"

#include "../gtkutil.h"
#include "i_cdaudio.h"

#define NSEC2MSEC 1000000

static GVPluginAttributes _attributes = 
  {
    "i_cdaudio",
    "CD Player",
    "CD Player plugin for linux cdrom drives",
    (char*)0,
    GV_PLUGIN_HAS_AUDIO|
    GV_PLUGIN_NEEDS_REALTIME|
    GV_PLUGIN_IS_DEVICE|
    GV_PLUGIN_NO_OUTPUT|
    GV_PLUGIN_OWN_TIME|
    GV_PLUGIN_CAN_SEEK|
    GV_PLUGIN_CAN_CHANGE
  };

#define SECTION attributes->name

GVInputCdaudio::GVInputCdaudio() :
  GVCdaudioCommon(&_attributes)
  {
  get_devices_config("/dev/cdrom");
  
  //  cerr << "device: " << filename << "\n";
  
  tracks = (GVCdaudioTrack*)0;
  config_window = (ConfigWindow*)0;

  // Audio format is quite easy for CDs
    
  audio_format.samplerate = 44100;
  audio_format.sample_type = GVAudio_16_signed;
  audio_format.endian_format = GVNativeEndian;
  audio_format.num_channels = 2;
  audio_format.samples_per_frame = 588;
  audio_format.interleave_mode = GVInterleaveAll;
  flags |= GV_PLUGIN_TRACK_HAS_AUDIO;
  must_seek = 0;
  }

GVInputCdaudio::~GVInputCdaudio()
  {
  //  cerr << "device: " << filename << "\n";
  put_devices_config();

  if(config_window)
    delete config_window;
  }

void GVInputCdaudio::init()
  {
  check_for_cd();
  start_cd_track = end_cd_track = current_cd_track = -1;
  }

// Get CD track from playlist

void GVInputCdaudio::play(int startpos)
  {
  struct cdrom_msf msf;

  int start_frame;
  int end_frame;

  start_cd_track = get_current_playlist_track();

  start_frame = tracks[start_cd_track-1].start_frame + startpos;
  
  // Start position
  
  msf.cdmsf_frame0 = start_frame % 75;   /* start frame */
  start_frame /= 75;
  msf.cdmsf_sec0 = start_frame % 60;     /* start second */
  start_frame /= 60;
  msf.cdmsf_min0 = start_frame;
  
  // End position

  end_cd_track = -1;

  int playlist_index = gv_playlist->get_current_track_index();
  int max_playlist_index = gv_playlist->get_num_tracks();

  int old_playlist_track = start_cd_track;
  int tmp_playlist_track;
  
  for(int i = playlist_index+1; i < max_playlist_index; i++)
    {
    if((tmp_playlist_track = get_playlist_track(i)) != old_playlist_track+1)
      {
      end_cd_track = get_playlist_track(i-1);
      break;
      }
    old_playlist_track = tmp_playlist_track;
    }
  if(end_cd_track == -1)
    end_cd_track = num_tracks;
    
  end_frame = tracks[end_cd_track-1].end_frame;

//   cerr << "Playing CD from tracks "
//        << start_cd_track << " To " << end_cd_track << "\n";

  msf.cdmsf_frame1 = end_frame % 75;   /* start frame */
  end_frame /= 75;
  msf.cdmsf_sec1 = end_frame % 60;     /* start second */
  end_frame /= 60;
  msf.cdmsf_min1 = end_frame;
  
  ioctl(cd_fd, CDROMPLAYMSF, &msf);

  audio_format.num_samples = (tracks[start_cd_track-1].end_frame -
    tracks[start_cd_track-1].start_frame + 1)*588;
  audio_format.init();
  gv_engine->update_info();
  }

void GVInputCdaudio::playback_func()
  {
  gv_engine->plugin_attach();

  current_cd_track = get_current_playlist_track();

  total_time = tracks[current_cd_track-1].length;
  gv_engine->set_total_track_time(total_time);

  GVTime total_track_time;
  GVTime current_track_time;
  
  int startpos = 0;

  // Set the current total time

  total_track_time = tracks[current_cd_track-1].length;
  gv_engine->set_total_track_time(total_track_time);
  
  GVTime sleep_time;

  sleep_time.seconds = 0;
  sleep_time.nanoseconds = NSEC2MSEC*200;

  struct cdrom_subchnl subchannel;
  
  subchannel.cdsc_format = CDROM_MSF;

  int playlist_cd_track;
  int restart = 0;
  int do_exit;

  // Play the current track

  GVEngineState state = gv_engine->get_state();
  GVEngineState old_state = GVEngineStopped;;

  gv_engine->wait_for_start();
  
  while(1)
    {
    // Break if the engine stopped

    state = gv_engine->get_state();

    restart = 0;
    do_exit = 0;
    
    if(state != old_state)
      {
      switch(state)
        {
        case GVEnginePlaying:
          //          cerr << "GVEnginePlaying\n";
          restart = 1;
          break;
        case GVEngineSeeking:
//           cerr << "GVEngineSeeking\n";
          gv_engine->plugin_detach();
          gv_engine->wait_for_start();
          gv_engine->plugin_attach();
          old_state = state = GVEnginePlaying;
          restart = 1;
          break;
        case GVEnginePaused:
//           cerr << "GVEnginePaused\n";
          gv_engine->plugin_detach();
          ioctl(cd_fd, CDROMPAUSE, (int*)0);
          gv_engine->wait_for_start();
          ioctl(cd_fd, CDROMRESUME, (int*)0);
          gv_engine->plugin_attach();
          old_state = state = GVEnginePlaying;
          break;
        case GVEngineStopped:
//           cerr << "GVEngineStopped\n";
          do_exit = 1;
          break;
        case GVEngineChanging:
//           cerr << "GVEngineChanging: " << ++num_changes << "...";
          gv_engine->plugin_detach();
          //          cerr << "Waiting for start...";
          gv_engine->wait_for_start();
          //          cerr << "Done :-)\n";
          state = GVEnginePlaying;
          playlist_cd_track = current_cd_track =
            get_current_playlist_track();
//           cerr << "Engine changed track, new track: " << current_cd_track
//                << "\n";
          gv_engine->plugin_attach();
          restart = 1;
          break;
        }
      if(do_exit)
        break;
      }
    else
      {
      // Get the current state of the CD
      if(ioctl(cd_fd, CDROMSUBCHNL, &subchannel) == -1)
        {
        // IOctl failed, maybe the CD is missing
        gv_engine->stop();
        break;
        }
      // Get the current playlist track
      playlist_cd_track = get_current_playlist_track();

      // Check if the song is over
      if(subchannel.cdsc_audiostatus == CDROM_AUDIO_COMPLETED)
        {
        gv_playlist->next();
        //      gv_engine->set_state_flag(GV_ENGINE_TRACK_CHANGED);
        playlist_cd_track = current_cd_track = get_current_playlist_track();
        restart = 1;
        }

      // Check if the drive changed the track
      else
        {
        if(subchannel.cdsc_trk != current_cd_track)
          {
          // We must restart if we are out of sync with the playlist

//         cerr << "subchannel.cdsc_trk: " << subchannel.cdsc_trk <<
//           " current_cd_track: " << current_cd_track << "\n";
        
          int current_playlist_index
            = gv_playlist->get_current_track_index();
          
          const GVPlaylistTrack * next_playlist_item
            = gv_playlist->get_track(current_playlist_index+1);
          
          // This was the last track in the playlist
          
          if(!(next_playlist_item) &&
             (gv_engine->get_config_flags() & GV_ENGINE_AUTO_ADVANCE))
            {
            GVRepeatMode repeat_mode = gv_engine->get_repeat_mode();
            switch(repeat_mode)
              {
              case GVRepeatNone:
                break;
              case GVRepeat1:
                restart = 1;
                break;
              case GVRepeatAll:
                gv_playlist->set_current_track(0);
                restart = 1;
                break;
              }
            }
          else
            {
            if(!(gv_engine->get_config_flags() & GV_ENGINE_AUTO_ADVANCE))
              break;
            
            if(((GVCdaudioTrack*)(next_playlist_item->data))->index ==
               current_cd_track+1)
              {
              // The drive went to another track, but we are still in sync
              // with the playlist
              //           cerr << "Current cd track: " << current_cd_track
              //                << " Current playlist track: "
              //                << ((GVCdaudioTrack*)(next_playlist_item->data))->index << "\n";

              if(gv_engine->get_repeat_mode() != GVRepeat1)
                {
                current_cd_track++;
                gv_playlist->next();
                }
              else
                restart = 1;
              
              // Tell this to the main window
              
              total_track_time = tracks[current_cd_track-1].length;
              audio_format.num_samples =
                (tracks[current_cd_track-1].end_frame - 
                 tracks[current_cd_track-1].start_frame+1)*588;
              gv_engine->set_total_track_time(total_track_time);
              gv_engine->set_state(GVEngineChanging);
              gv_engine->set_state(GVEnginePlaying);
              
              gv_engine->update_info();
              }
            else
              {
              // Out of sync
              //        cerr << "Out of sync, restart\n";

              if(gv_engine->get_repeat_mode() != GVRepeat1)
                {
                gv_playlist->next();
                current_cd_track = get_current_playlist_track();
                }
              
              //        cerr << "switching to track " << current_cd_track << "\n";
              restart = 1;
              //           cerr << "Blupp2\n";
              }
            }
          }
        }
      }
    // Restart the CD Player
    
    if(restart)
      {
//       cerr << "Restart\n";
      total_track_time = tracks[current_cd_track-1].length;
      gv_engine->set_total_track_time(total_track_time);
      if(must_seek)
        startpos = (int)(seek_percentage*75.0*
                         total_track_time.get_double()+0.5);
      else
        startpos = 0;

      play(startpos);
      //      gv_engine->clear_state_flag(GV_ENGINE_MUST_SEEK);
      //      gv_engine->set_state_flag(GV_ENGINE_TRACK_CHANGED);
      ioctl(cd_fd, CDROMSUBCHNL, &subchannel);
      must_seek = 0;

      gv_engine->set_state(GVEngineChanging);
      gv_engine->set_state(GVEnginePlaying);
      }
    
    // Update the current time

    current_track_time.seconds =
      subchannel.cdsc_reladdr.msf.minute*60+
      subchannel.cdsc_reladdr.msf.second;

    guint64 nanoseconds = subchannel.cdsc_reladdr.msf.frame;
    nanoseconds *= GV_NANOSEC_PER_SEC;
    nanoseconds /= 75;
    
    current_track_time.nanoseconds = nanoseconds;
      
    gv_engine->set_current_track_time(current_track_time);
    sleep_time.sleep();
    old_state = state;
    }
  cleanup();
  gv_engine->stop();
  gv_engine->plugin_detach();
  }

void GVInputCdaudio::cleanup()
  {
  ioctl(cd_fd, CDROMSTOP, (int*)0);
  close(cd_fd);
  }

void GVInputCdaudio::configure()
  {
  if(!device_widget)
    device_widget = new GVConfigWidgetDevicelist(this);

  if(!cddb_widget)
    cddb_widget = new CDDBWidget(this);
  
  if(!config_window)
    config_window = new ConfigWindow(this);
  config_window->run();
  }

void GVInputCdaudio::seek(float percentage)
  {
  must_seek = 1;
  seek_percentage = percentage;
  }

void GVInputCdaudio::print_info(ostream & output)
  {
  output << "Playing Audio CD in " << device;
  if(start_cd_track != -1)
    {
    if(start_cd_track != end_cd_track)
      {
      output << " Tracks: " << start_cd_track << " - " << end_cd_track << "\n";
      output << "Current Track: " << current_cd_track;
      }
    else
      output << " Track: " << current_cd_track << "\n";
    }
  else
    output << " Not Playing yet\n";
  }

GVInputCdaudio::ConfigWindow::ConfigWindow(GVInputCdaudio * _plugin) :
  GVConfigWindow(_plugin->attributes->real_name)
  {
  plugin = _plugin;

  GtkWidget * notebook = gtk_notebook_new();
  GtkWidget * label;

  GtkWidget * table = gvgtk_table_new(1, 3, 0);

  gtk_widget_show(table);

  label = gtk_label_new("Devices");
  gtk_widget_show(label);
  gtk_notebook_append_page(GTK_NOTEBOOK(notebook),
                           plugin->device_widget->widget,
                           label);
  
  label = gtk_label_new("CDDB");
  gtk_widget_show(label);
  gtk_notebook_append_page(GTK_NOTEBOOK(notebook),
                           plugin->cddb_widget->widget, label);
    
  gtk_widget_show(notebook);
  gtk_box_pack_start_defaults(GTK_BOX(mainbox), notebook);
  }

GVInputCdaudio::ConfigWindow::~ConfigWindow()
  {
  }

void GVInputCdaudio::ConfigWindow::get_values()
  {
  //  devicename.set_filename(plugin->get_filename());
  plugin->cddb_widget->get_values();
  plugin->device_widget->get_values();
  }

void GVInputCdaudio::ConfigWindow::apply_values()
  {
  //  plugin->set_filename(devicename.get_filename());
  plugin->device_widget->apply_values();
  plugin->cddb_widget->apply_values();
  }

#define PLUGIN_NAME GVInputCdaudio
#include "interface.cc"
