/*****************************************************************

  i_paranoia.cc

  Copyright (c) 2001 by Burkhard Plaum - plaum@ipf.uni-stuttgart.de

  http://gmerlin.sourceforge.net

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111, USA.

*****************************************************************/

// CD reader for gmerlin
// Reads digital audio data from Audio CDs

#include <gtk/gtk.h>
#include <iostream>

using std::cerr;
using std::ostream;


#include "../gvconfig.h"
#include "../gv.h"
#include "../gtkutil.h"

#include <stdio.h>
#include <string.h>
#include <stdio.h> // For cdaudio_common

#include "i_cdparanoia.h"

#define NUM_SECTORS 10

static void paranoia_callback(long inpos, int function)
  {
//   cerr << "Paranoia Callback: inpos: " << inpos
//        << " function: " << function << "\n";
  }

static GVPluginAttributes _attributes = 
  {
    "i_cdparanoia",
    "CD Ripper",
    "CD plugin for reading digital audio data\n\
Based on cdparanoia. See\n\
http://www.xiph.org/paranoia/index.html\n\
for more information",
    (char*)0,
    GV_PLUGIN_HAS_AUDIO|
    GV_PLUGIN_IS_DEVICE|
    GV_PLUGIN_CAN_SEEK
  };

void GVInputCdparanoia::init()
  {
  current_track = get_current_playlist_track();
  if(trackname)
    delete trackname;

  const GVPlaylistTrack * item = gv_playlist->get_current_track();
  trackname = g_strdup(item->trackname);
  
  start_sector = tracks[current_track-1].start_frame - tracks[0].start_frame;
  end_sector = tracks[current_track-1].end_frame - tracks[0].start_frame;
  last_sector = 0;
  
  // Set up the paranoia stuff
  
  //  cerr << "Searching for cdrom...";

  drive = cdda_identify(device, 1, NULL);
  
  if(drive)
    cerr << "done\n";
  else
    cerr << "failed\n";

  //  cdda_verbose_set(drive,CDDA_MESSAGE_PRINTIT,CDDA_MESSAGE_PRINTIT);
  cdda_verbose_set(drive,CDDA_MESSAGE_FORGETIT,CDDA_MESSAGE_FORGETIT);

  cdda_speed_set(drive, 4);
    
  cdda_open(drive);
    
  paranoia = paranoia_init(drive);
    
  paranoia_seek(paranoia, start_sector, SEEK_SET);
    
//   cerr << "Playing track " << current_track << " Sector " << start_sector
//        << ".." << end_sector << "\n";

  paranoia_mode = PARANOIA_MODE_FULL^PARANOIA_MODE_NEVERSKIP;
  if(disable_paranoia) paranoia_mode=PARANOIA_MODE_DISABLE;
  if(disable_extra_paranoia)
    {
    paranoia_mode|=PARANOIA_MODE_OVERLAP;
    paranoia_mode&=~PARANOIA_MODE_VERIFY;
    }
  if(disable_scratch_detection)
    paranoia_mode&=~(PARANOIA_MODE_SCRATCH|PARANOIA_MODE_REPAIR);
  if(disable_scratch_repair) 
    paranoia_mode&=~PARANOIA_MODE_REPAIR;
  
  paranoia_modeset(paranoia,paranoia_mode);

  //  cerr << drive->nsectors << " Sectors\n";

  num_sectors = drive->nsectors; // Number of cd sectors to be read at once
  
  // Format values are fixed for audio CDs

  audio_format.num_samples = (end_sector-start_sector+1)*588;
  
  //  audio_format.samples_per_frame = 588*drive->nsectors;
  audio_format.samples_per_frame = 588*num_sectors;
  audio_format.init();
  total_time.set_from_frames((unsigned int)audio_format.num_samples,
                             (unsigned int)44100); 
  }

GVInputCdparanoia::GVInputCdparanoia() :
  GVCdaudioCommon(&_attributes)
  {
  error_strings = strerror_tr; // Keep gcc quiet

  get_config();
  
  flags = GV_PLUGIN_TRACK_HAS_AUDIO;
  
  //  cerr << "device: " << filename << "\n";
  
  //  track_index = (int*)0;

  paranoia = (cdrom_paranoia*)0;
  drive = (cdrom_drive*)0;
  paranoia_mode = PARANOIA_MODE_FULL^PARANOIA_MODE_NEVERSKIP;
  flags = GV_PLUGIN_TRACK_HAS_AUDIO;
  config_window = (ConfigWindow*)0;

  audio_format.samplerate = 44100;

  audio_format.interleave_mode = GVInterleave2;
  audio_format.sample_type = GVAudio_16_signed;
  audio_format.num_channels = 2;

  }

#define SECTION attributes->name

void GVInputCdparanoia::get_config()
  {
  get_devices_config("/dev/cdrom");
    
  disable_paranoia = 0;
  disable_extra_paranoia = 0;
  disable_scratch_detection = 0;
  disable_scratch_repair = 0;

  gv_config->get_bool(SECTION, "DisableParanoia", disable_paranoia);
  gv_config->get_bool(SECTION, "DisableExtraParanoia", disable_extra_paranoia);
  gv_config->get_bool(SECTION, "DisableScratchDetection",
                      disable_scratch_detection);
  gv_config->get_bool(SECTION, "DisableScratchRepair",
                      disable_scratch_repair);
  }

void GVInputCdparanoia::put_config()
  {
  gv_config->put_string(SECTION, "Device", filename);
  gv_config->put_bool(SECTION, "DisableParanoia", disable_paranoia);
  gv_config->put_bool(SECTION, "DisableExtraParanoia", disable_extra_paranoia);
  gv_config->put_bool(SECTION, "DisableScratchDetection",
                      disable_scratch_detection);
  gv_config->put_bool(SECTION, "DisableScratchRepair",
                      disable_scratch_repair);
  put_devices_config();
  }

void GVInputCdparanoia::fill_audio_buf(GVAudioFrame * frame)
  {
  //  cerr << "Filling audio buffer...";

  frame->valid_samples = 0;
  gint16 * dst = frame->samples.s_16;  
  size16 * src;
  
  for(int i = 0; i < num_sectors; i++)
    {
    src = paranoia_read(paranoia, paranoia_callback);
    memcpy(dst, src, audio_format.buffer_size/num_sectors);
    last_sector++;
    frame->valid_samples += 588;
    if(last_sector*588 == audio_format.num_samples)
      {
      audio_completed = 1;
      cerr << "Ripping finished\n";
      break;
      }
    dst += 588*audio_format.num_channels;
    }
  }

void GVInputCdparanoia::cleanup()
  {
  if(paranoia)
    {
    paranoia_free(paranoia);
    paranoia = (cdrom_paranoia*)0;
    }
  if(drive)
    {
    cdda_close(drive);
    drive = (cdrom_drive*)0;
    }
  }

GVInputCdparanoia::~GVInputCdparanoia()
  {
  cleanup();
  if(config_window)
    delete config_window;
  put_config();
  }

void GVInputCdparanoia::configure()
  {
  if(!device_widget)
    device_widget = new GVConfigWidgetDevicelist(this);
  if(!cddb_widget)
    cddb_widget = new CDDBWidget(this);
  if(!config_window)
    config_window = new ConfigWindow(this);
  config_window->run();
  }

GVInputCdparanoia::ConfigWindow::ConfigWindow(GVInputCdparanoia * _plugin) :
  GVConfigWindow(_plugin->attributes->real_name)
  {
  plugin = _plugin;

  GtkWidget * notebook = gtk_notebook_new();
  GtkWidget * label;

  label = gtk_label_new("Devices");
  gtk_widget_show(label);
  gtk_notebook_append_page(GTK_NOTEBOOK(notebook),
                           plugin->device_widget->widget,
                           label);
  
  disable_paranoia = gtk_check_button_new_with_label("Disable Paranoia");
  disable_extra_paranoia =
    gtk_check_button_new_with_label("Disable extra paranoia");
  disable_scratch_detection =
    gtk_check_button_new_with_label("scratch detection");
  disable_scratch_repair =
    gtk_check_button_new_with_label("scratch repair");;

  gtk_widget_show(disable_paranoia);
  gtk_widget_show(disable_extra_paranoia);
  gtk_widget_show(disable_scratch_detection);
  gtk_widget_show(disable_scratch_repair);
    
  GtkWidget * table = gvgtk_table_new(2, 3, 0);
  gtk_table_attach_defaults(GTK_TABLE(table), disable_paranoia, 0, 1, 0, 1);
  gtk_table_attach_defaults(GTK_TABLE(table), disable_extra_paranoia,
                            1, 3, 0, 1);
  label = gtk_label_new("Disable");
  gtk_widget_show(label);

  gtk_table_attach_defaults(GTK_TABLE(table), label, 0, 1, 1, 2);
  gtk_table_attach_defaults(GTK_TABLE(table), disable_scratch_detection,
                            1, 2, 1, 2);
  gtk_table_attach_defaults(GTK_TABLE(table), disable_scratch_repair,
                            2, 3, 1, 2);
  gtk_widget_show(table);
  label = gtk_label_new("Paranoia");
  gtk_widget_show(label);
  gtk_notebook_append_page(GTK_NOTEBOOK(notebook), table, label);
    
  label = gtk_label_new("CDDB");
  gtk_widget_show(label);
  gtk_notebook_append_page(GTK_NOTEBOOK(notebook),
                           plugin->cddb_widget->widget, label);
    
  gtk_widget_show(notebook);
  gtk_box_pack_start_defaults(GTK_BOX(mainbox), notebook);
  }

void GVInputCdparanoia::seek(float percentage)
  {
  start_sector = tracks[current_track-1].start_frame;
  end_sector = tracks[current_track-1].end_frame;

  int offset = (int)((float)(end_sector-start_sector)*percentage + 0.5);
  
  audio_samples_written = offset * 588;
  paranoia_seek(paranoia, start_sector + offset, SEEK_SET);
  }

void GVInputCdparanoia::print_info(ostream & output)
  {
  output << "CD Device: " << device << "\n";
  output << "Ripping CD track " << current_track
         << " (Sectors " << tracks[current_track-1].start_frame
         << " - " << tracks[current_track-1].end_frame << ")\n";
  }

GVInputCdparanoia::ConfigWindow::~ConfigWindow()
  {
  }

void GVInputCdparanoia::ConfigWindow::get_values()
  {
  //  devicename.set_filename(plugin->get_filename());

  plugin->device_widget->get_values();
  
  plugin->cddb_widget->get_values();

  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(disable_paranoia),
                               plugin->disable_paranoia);
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(disable_extra_paranoia),
                               plugin->disable_extra_paranoia);

  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(disable_scratch_detection),
                               plugin->disable_scratch_detection);
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(disable_scratch_repair),
                               plugin->disable_scratch_repair);
  

  }

void GVInputCdparanoia::ConfigWindow::apply_values()
  {
  
  plugin->device_widget->apply_values();

  //  plugin->set_filename(devicename.get_filename());
  plugin->cddb_widget->apply_values();

  plugin->disable_paranoia =
    gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(disable_paranoia));

  plugin->disable_extra_paranoia =
    gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(disable_extra_paranoia));

  plugin->disable_scratch_detection =
    gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(disable_scratch_detection));

  plugin->disable_scratch_repair =
    gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(disable_scratch_repair));


  }

#define PLUGIN_NAME GVInputCdparanoia
#include "interface.cc"
