/*****************************************************************

  i_paranoia.h

  Copyright (c) 2001 by Burkhard Plaum - plaum@ipf.uni-stuttgart.de

  http://gmerlin.sourceforge.net

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111, USA.

*****************************************************************/

// CD reader for gmerlin
// Reads digital audio data from Audio CDs

#define size32 gint32
#define size16 gint16


#include "cdaudio_common.h"
extern "C" {
#include "cdparanoia/interface/cdda_interface.h"
#include "cdparanoia/paranoia/cdda_paranoia.h"
}

// CD input plugin for reading digital audio data from the CD

class GVInputCdparanoia : public GVCdaudioCommon
  {
  int num_sectors; // Number of cd sectors to be read at once
  int current_track;

  char ** error_strings;

  int disable_paranoia;
  int disable_extra_paranoia;
  int disable_scratch_detection;
  int disable_scratch_repair;

  void get_config();
  void put_config();
  
  
protected:
  
  cdrom_drive *drive;
  cdrom_paranoia *paranoia;

  int paranoia_mode;
  
  int start_sector;
  int end_sector;
  unsigned int last_sector;

  class ConfigWindow : public GVConfigWindow
    {
    GVInputCdparanoia * plugin;

    GtkWidget * disable_paranoia;
    GtkWidget * disable_extra_paranoia;
    GtkWidget * disable_scratch_detection;
    GtkWidget * disable_scratch_repair;
    
  public:

    ConfigWindow(GVInputCdparanoia * _plugin);
    virtual ~ConfigWindow();

    void get_values();
    void apply_values();
    };

  ConfigWindow * config_window;
  friend class ConfigWindow;
  
public:

  void seek(float percentage);
  
  void configure();
  
  void init();

  GVInputCdparanoia();
  ~GVInputCdparanoia();

  void fill_audio_buf(GVAudioFrame * frame);
  void cleanup();

  void print_info(ostream & output);
  
  };
