/*****************************************************************

  i_sndfile.cc

  Copyright (c) 2001 by Burkhard Plaum - plaum@ipf.uni-stuttgart.de

  http://gmerlin.sourceforge.net

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111, USA.

*****************************************************************/

// sndfile input plugin

#include <iostream>

using std::cerr;
using std::ostream;

#include "i_sndfile.h"

// IRCAM - ".sf" or ".irc" or ".ircam" extension
// PAF - ".paf" extension
// REX - ".rex"


static GVPluginAttributes _attributes = 
  {
    "i_sndfile",
    "Sndfile Input Plugin",
    "Input plugin for lots of audio file formats\n\
See http://www.zip.com.au/~erikd/libsndfile/\n\
for supported formats",
    "wav au snd aiff aifc aif sf irc ircam paf rex",
    GV_PLUGIN_HAS_AUDIO|GV_PLUGIN_IS_FILE
  };



GVInputSndfile::GVInputSndfile() :
  GVInput(&_attributes)
  {
  flags = GV_PLUGIN_TRACK_HAS_AUDIO;
  file = (SNDFILE*)0;
  }

int GVInputSndfile::can_seek()
  {
  if(!file)
    return 0;
  return !!(file_info.seekable);
  }

void GVInputSndfile::fill_audio_buf(GVAudioFrame * frame)
  {
  unsigned int samples_read = sf_readf_short(file, frame->samples.s_16,
                                             audio_format.samples_per_frame);

  if(samples_read < audio_format.samples_per_frame)
    audio_completed = 1;

  //  cerr << "i_sndfile: Valid Samples: " << samples_read << "\n";
  
  frame->valid_samples = samples_read;
  }

void GVInputSndfile::init()
  {
  file = sf_open_read(filename, &file_info);

  audio_format.samplerate = file_info.samplerate;
  audio_format.num_samples = file_info.samples;
  audio_format.num_channels = file_info.channels;
  audio_format.sample_type = GVAudio_16_signed;
 
  audio_format.samples_per_frame = audio_format.samplerate / 4;
  //  audio_format.flags = 0;
  audio_format.interleave_mode = GVInterleaveAll;
  video_completed = 1;
  audio_completed = 0;

  audio_format.init();
  
  total_time.set_from_frames((unsigned int)audio_format.num_samples,
                             (unsigned int)audio_format.samplerate);
  audio_format.samples_per_frame = 4096;
  }

void GVInputSndfile::seek(float percentage)
  {
  sf_seek(file,(int)((percentage)*audio_format.num_samples+0.5),
          SEEK_SET);
  }


void GVInputSndfile::cleanup()
  {
  if(file)
    sf_close(file);
  file = (SNDFILE*)0;
  }

#define PLUGIN_NAME GVInputSndfile
#include "interface.cc"
