#ifdef __GNUG__
#pragma implementation "if_gmpeg.h"
#endif

#include <unistd.h>
#include <string.h>

#include <iostream>
#include <sstream>

using std::stringstream;
using std::ostream;
using std::cerr;

#include "../gv.h"
#include "../socket.h"
#include "libgmerlin_mpeg/mpeg.h"
#include "libgmerlin_mpeg/in_file.h"
#include "libgmerlin_mpeg/in_http.h"

#include "mpeg_common.h"
#include "if_gmpeg.h"

static GVPluginAttributes _attributes = 
  {
    "i_gmerlin_mpeg",
    "New mpeg input Plugin",
    "Currently for MPEG 1/2/2.5 audio layer I,II,III\n\
Uses: mad (http://mad.sourceforge.net) for\n\
high quality audio decoding and id3lib \n\
(http://id3lib.sourceforge.net) for ID3 support",
    "mp2 mp3 m3u",
    GV_PLUGIN_HAS_AUDIO|
    GV_PLUGIN_HAS_VIDEO|
    GV_PLUGIN_IS_FILE|
    GV_PLUGIN_HANDLES_URLS|
    GV_PLUGIN_CAN_SEEK
  };

#define INPUT_HTTP 1
#define INPUT_FILE 2

GVInputMpegFile::GVInputMpegFile() :
  GVInputMpegCommon(&_attributes)
  {
  input_mode = 0;
  mpeg_http.socket.keep_header = 1;
  }

static int __check_suffix(char * filename, const char * suffix)
  {
  const char * pos = strrchr(filename, '.');
  if(!pos)
    return 0;
  pos++;

  if(g_strcasecmp(pos, suffix))
    return 0;
  else
    return 1;
  }

void GVInputMpegFile::init()
  {
  error = GV_PLUGIN_ERROR_NONE;

  // Check, if we have an ".m3u" file
    
  if(__check_suffix(filename, "m3u"))
    {
    stringstream line;
    //    line.freeze(0);
    ifstream input;
    input.open(filename);
    if(!input)
      return;
    input.get(*line.rdbuf(), '\n');
    set_filename(line.str().c_str());
    input.close();
    }
  
  if(is_url())
    {
    if(!mpeg_http.open(filename))
      {
      error = GV_PLUGIN_ERROR_CONNECT;
      cerr << "Could not connect\n";
      return;
      }
    input_mode = INPUT_HTTP;
    mpeg_decoder.set_input(&mpeg_http);
    }
  else
    {
    if(!mpeg_file.open(filename))
      {
      error = GV_PLUGIN_ERROR_OPEN_FILE;
      return;
      }
    input_mode = INPUT_FILE;
    mpeg_decoder.set_input(&mpeg_file);
    }
  init_common();
  }

void GVInputMpegFile::set_trackname()
  {
  if(trackname)
    delete trackname;

  if(input_mode == INPUT_HTTP)
    {
    //     cerr << "Getting station name...";
    const char * new_trackname;
    if((new_trackname =
        mpeg_http.socket.get_header_key("x-audiocast-name:")))
      trackname = g_strdup(new_trackname);
    else if((new_trackname =
             mpeg_http.socket.get_header_key("icy-name:")))
      trackname = g_strdup(new_trackname);
    else
      trackname = g_strdup(filename);
    //     cerr << "done: Name: " << trackname << "\n";
    }
  else
    {
    const char * pos1;
    const char * pos2;
    
    if(!(pos1 = strrchr(filename, G_DIR_SEPARATOR)))
      pos1 = filename;
    else
      pos1++;
    
    if(!(pos2 = strrchr(filename, '.')))
      pos2 = &(filename[strlen(filename)]);
    
    trackname = new char[pos2 - pos1 + 1];
    strncpy(trackname, pos1, pos2-pos1);
    trackname[pos2-pos1] = '\0';
    }
  }

void GVInputMpegFile::cleanup()
  {
  cleanup_common();
  mpeg_decoder.cleanup();

  if(input_mode == INPUT_FILE)
    {
    mpeg_file.close();
    mpeg_file.cleanup();
    }
  else if(input_mode == INPUT_HTTP)
    {
    mpeg_http.close();
    mpeg_http.cleanup();
    }
  }

#define PLUGIN_NAME GVInputMpegFile
#include "interface.cc"
