/*****************************************************************

  ir_jpg.cc

  Copyright (c) 2001 by Burkhard Plaum - plaum@ipf.uni-stuttgart.de

  http://gmerlin.sourceforge.net

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111, USA.

*****************************************************************/

// Reads JPG files

#ifdef __GNUG__
#pragma implementation "ir_jpg.h"
#endif

#include "../gv.h"
#include "../configwindow.h"
#include "../globalconfig.h"

extern "C" {
#include <stdio.h>
  // #include "../jpeg-mmx/jpeglib.h"
#include <jpeglib.h>
}

#include "ir_jpg.h"

static GVPluginAttributes _attributes = 
  {
    "ir_jpg",
    "JPG loader",
    "Plugin for loading jpg images",
    "jpg",
    GV_PLUGIN_HAS_VIDEO|
    GV_PLUGIN_IS_FILE
  };

GVImageReaderJPG::GVImageReaderJPG() :
  GVImageReader(&_attributes)
  {
  cinfo.err = jpeg_std_error(&jerr);
  jpeg_create_decompress(&cinfo);
  input = (FILE*)0;
  }

GVImageReaderJPG::~GVImageReaderJPG()
  {
  jpeg_destroy_decompress(&cinfo);
  }

int GVImageReaderJPG::init(const char * filename, int first = 1)
  {
  input = fopen(filename, "rb");
  if(!input)
    return 0;
  
  jpeg_stdio_src(&cinfo, input);
  
  if(jpeg_read_header(&cinfo, TRUE) != JPEG_HEADER_OK)
    return 0;

  if(first)
    {
    //    jpeg_abort_decompress(&cinfo);
    //    fclose(input);

    video_format.width  = cinfo.image_width;
    video_format.height = cinfo.image_height;
    
    switch(cinfo.jpeg_color_space)
      {
      case JCS_YCbCr:
        
      // Direct YUV output is only possible if width and
      // Height are multiples of 16

        if(!(video_format.width % 16) &&
           !(video_format.height % 16))
          {
          if((cinfo.comp_info[0].h_samp_factor == 2) &&
             (cinfo.comp_info[0].v_samp_factor == 2) &&
             (cinfo.comp_info[1].h_samp_factor == 1) &&
             (cinfo.comp_info[1].v_samp_factor == 1) &&
             (cinfo.comp_info[2].h_samp_factor == 1) &&
             (cinfo.comp_info[2].v_samp_factor == 1))
            best_colorspace = GVColorspaceYUV_420_P;
          else if((cinfo.comp_info[0].h_samp_factor == 2) &&
                  (cinfo.comp_info[0].v_samp_factor == 1) &&
                  (cinfo.comp_info[1].h_samp_factor == 1) &&
                  (cinfo.comp_info[1].v_samp_factor == 1) &&
                  (cinfo.comp_info[2].h_samp_factor == 1) &&
                  (cinfo.comp_info[2].v_samp_factor == 1))
            best_colorspace = GVColorspaceYUV_422_P;
          else
            {
            cinfo.out_color_space = JCS_RGB;
            best_colorspace = GVColorspaceRGB24;
            }
          }
      else
        {
        cinfo.out_color_space = JCS_RGB;
        best_colorspace = GVColorspaceRGB24;
        }
        break;
      default:
        cinfo.out_color_space = JCS_RGB;
        best_colorspace = GVColorspaceRGB24;
        break;
      }
    }
  video_format.colorspace = best_colorspace;
  video_format.init();
  return 1;
  }

// Return true if colorspace is supported

int GVImageReaderJPG::colorspace_supported(GVColorspace colorspace)
  {
  if(colorspace != best_colorspace)
    return 0;
  return 1;
  }
  
// Load image into the video frame structure. The frame must be
// allocated with the proper size by the caller!!

void GVImageReaderJPG::load_image(GVVideoFrame * frame)
  {
  if((video_format.colorspace == GVColorspaceYUV_420_P) ||
     (video_format.colorspace == GVColorspaceYUV_422_P))
    cinfo.raw_data_out = TRUE;
  
  jpeg_start_decompress(&cinfo);
  
  switch(video_format.colorspace)
    {
    case GVColorspaceRGB24:
      while(cinfo.output_scanline < cinfo.output_height)
        jpeg_read_scanlines(&cinfo,
                            (JSAMPLE**)(&frame->pixels_rows[cinfo.output_scanline]),
                            cinfo.output_height - cinfo.output_scanline);
      
      break;
    case GVColorspaceYUV_420_P:
      while(cinfo.output_scanline < cinfo.output_height)
        {
        yuv_rows[0] = &(frame->y_rows[cinfo.output_scanline]);
        yuv_rows[1] = &(frame->u_rows[cinfo.output_scanline/2]);
        yuv_rows[2] = &(frame->v_rows[cinfo.output_scanline/2]);
        jpeg_read_raw_data(&cinfo, yuv_rows, 16);
        }
      break;
    case GVColorspaceYUV_422_P:
      while(cinfo.output_scanline < cinfo.output_height)
        {
        yuv_rows[0] = &(frame->y_rows[cinfo.output_scanline]);
        yuv_rows[1] = &(frame->u_rows[cinfo.output_scanline]);
        yuv_rows[2] = &(frame->v_rows[cinfo.output_scanline]);
        jpeg_read_raw_data(&cinfo, yuv_rows, 8);
        }
      break;
    }
  }

// Cleanup all allocated resources. If the argument "last" is 0,
// don't clean resources, which can be reused by series of pictures
// in singlepicture plugins
  
void GVImageReaderJPG::cleanup(int last = 1)
  {
  jpeg_finish_decompress(&cinfo);  
  fclose(input);
  }

#define PLUGIN_NAME GVImageReaderJPG
#include "interface.cc"
