/*****************************************************************

  imagereader_png.cc

  Copyright (c) 2001 by Burkhard Plaum - plaum@ipf.uni-stuttgart.de

  http://gmerlin.sourceforge.net

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111, USA.

*****************************************************************/

// Reads PNG files

// #ifdef __GNUG__
// #pragma implementation "ir_png.h"
// #endif

#include <iostream>
using std::cerr;
using std::ostream;

#include "../gv.h"
#include "../configwindow.h"
#include "../globalconfig.h"

#include <png.h>

#include <stdio.h>

#include "ir_png.h"

static GVPluginAttributes _attributes = 
  {
    "ir_png",
    "PNG loader",
    "Plugin for loading png images",
    "png",
    GV_PLUGIN_HAS_VIDEO|
    GV_PLUGIN_IS_FILE
  };

GVImageReaderPNG::GVImageReaderPNG() :
  GVImageReader(&_attributes)
  {
//   cerr << "GVImageReaderPNG::GVImageReaderPNG(): attributes: "
//        << (const void*)attributes << "\n";
  }

GVImageReaderPNG::~GVImageReaderPNG()
  {
  
  }

int GVImageReaderPNG::init(const char * filename, int first = 1)
  {
//   cerr << "GVImageReaderPNG::init\n";
  FILE * file = fopen(filename, "rb");
  if(!file)
    return 0;

  png_byte signature[8];

  fread(signature, 1, 8, file);

  if(png_sig_cmp(signature, 0, 8))
    return 0;

  png_ptr = png_create_read_struct
    (PNG_LIBPNG_VER_STRING, (png_voidp)0,
     NULL, NULL);

  if (!png_ptr)
    return 0;

  setjmp(png_jmpbuf(png_ptr));
  
  info_ptr = png_create_info_struct(png_ptr);
  if (!info_ptr)
    {
    png_destroy_read_struct(&png_ptr, (png_infopp)NULL, (png_infopp)NULL);
    return 0;
    }

  end_info = png_create_info_struct(png_ptr);
  if(!end_info)
    {
    png_destroy_read_struct(&png_ptr, &info_ptr,
                            (png_infopp)NULL);
    return 0;
    }

  // 
  
  png_init_io(png_ptr, file);
  png_set_sig_bytes(png_ptr, 8);

  // Now, read info from the file
  
  png_read_info(png_ptr, info_ptr);

  // Read the image header

  if(first)
    {
    video_format.width  = png_get_image_width(png_ptr,  info_ptr);
    video_format.height = png_get_image_height(png_ptr, info_ptr);
    png_bit_depth = png_get_bit_depth(png_ptr, info_ptr);
    png_color_type = png_get_color_type(png_ptr, info_ptr);

    video_format.framerate = gv_global_config->singlepic_framerate;

    // Get the best colorspace
    
    if(png_color_type & PNG_COLOR_MASK_ALPHA)
      best_colorspace = GVColorspaceRGBA32;
    else
      best_colorspace = GVColorspaceRGB24;
    }

  // Maybe we can also do the following only once
  
  switch(png_color_type)
    {
    case PNG_COLOR_TYPE_GRAY:       //  (bit depths 1, 2, 4, 8, 16)
      if(png_bit_depth < 8)
        png_set_gray_1_2_4_to_8(png_ptr);
      if (png_get_valid(png_ptr, info_ptr, PNG_INFO_tRNS))
        {
        png_set_tRNS_to_alpha(png_ptr);
        best_colorspace = GVColorspaceRGBA32;
        }
      png_set_gray_to_rgb(png_ptr);
      break;
    case PNG_COLOR_TYPE_GRAY_ALPHA: //  (bit depths 8, 16)
      png_set_gray_to_rgb(png_ptr);
      break;
    case PNG_COLOR_TYPE_PALETTE:    //  (bit depths 1, 2, 4, 8)
      png_set_palette_to_rgb(png_ptr);
      if (png_get_valid(png_ptr, info_ptr, PNG_INFO_tRNS))
        {
        png_set_tRNS_to_alpha(png_ptr);
        best_colorspace = GVColorspaceRGBA32;
        }
      break;
    case PNG_COLOR_TYPE_RGB:        //  (bit_depths 8, 16)
      if(png_get_valid(png_ptr, info_ptr, PNG_INFO_tRNS))
        {
        png_set_tRNS_to_alpha(png_ptr);
        best_colorspace = GVColorspaceRGBA32;
        }
      if(png_bit_depth == 16)
        png_set_strip_16(png_ptr);
      break;
    case PNG_COLOR_TYPE_RGB_ALPHA:  //  (bit_depths 8, 16)
      if(png_bit_depth == 16)
        png_set_strip_16(png_ptr);
      break;
    }
  video_format.colorspace = best_colorspace;
  video_format.init();
  return 1;
  }

// Return true if colorspace is supported

int GVImageReaderPNG::colorspace_supported(GVColorspace colorspace)
  {
  if(colorspace == best_colorspace)
    return 1;
  else
    return 0;
  }

// Load image into the video frame structure. The frame must be
// allocated with the proper size by the caller!!

void GVImageReaderPNG::load_image(GVVideoFrame * frame)
  {
  setjmp(png_jmpbuf(png_ptr));
  png_read_image(png_ptr, frame->pixels_rows);
  png_read_end(png_ptr, end_info);
  }

// Cleanup all allocated resources. If the argument "last" is 0,
// don't clean resources, which can be reused by series of pictures
// in singlepicture plugins

void GVImageReaderPNG::cleanup(int last = 1)
  {
  setjmp(png_jmpbuf(png_ptr));

  png_destroy_read_struct(&png_ptr, &info_ptr, &end_info);
  }

#define PLUGIN_NAME GVImageReaderPNG
#include "interface.cc"
