/*****************************************************************

  imagereader_tga.h

  Copyright (c) 2001 by Burkhard Plaum - plaum@ipf.uni-stuttgart.de

  http://gmerlin.sourceforge.net

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111, USA.

*****************************************************************/

#ifdef __GNUG__
#pragma interface
#endif

// This one loads tga files with Alpha
// It can be used for any case, where TGAs should be loaded

class GVImageReaderTGA : public GVImageReader
  {
  // Elementary scanline read functions

  void read_scanline_left_right_uncompressed(guchar * out_pixels);
  void read_scanline_right_left_uncompressed(guchar * out_pixels);

  void read_scanline_left_right_rle(guchar * out_pixels);
  void read_scanline_right_left_rle(guchar * out_pixels);
  
  // TGA data

  int new_tga;
  int image_type;

  int colormap_first_index;
  int colormap_length;
  int colormap_entry_size;

  int pixel_depth;

  int alpha_bits;

  // Array of 4 colors
  // Order is:
  // color[0]: red
  // color[1]: green
  // color[2]: blue
  // color[3]: alpha
  
  struct TgaColor
    {
    guchar colors[4];
    };

  TgaColor * colormap;
  
  // Pixel orientation

  int bottom_to_top;
  int right_to_left;

  // Reads one pixel from input 
  
  void read_pixel_raw(guchar * pixel, int depth);

  // Reads one pixel with colormap lookup
  
  void read_pixel(guchar * pixel);
  
  // Other stuff

  ifstream input;

  guint16 read_ushort();
  gint16 read_short();

  guint32 read_ulong();
  gint32 read_long();

  guint8 read_ubyte();
  gint8  read_byte();
  
public:

  GVImageReaderTGA();
  ~GVImageReaderTGA();

  int init(const char * filename, int first = 1);

  // Return true if colorspace is supported

  int colorspace_supported(GVColorspace colorspace);
  
  // Load image into the video frame structure. The frame must be
  // allocated with the proper size by the caller!!

  void load_image(GVVideoFrame * frame);

  // Cleanup all allocated resources. If the argument "last" is 0,
  // don't clean resources, which can be reused by series of pictures
  // in singlepicture plugins
  
  void cleanup(int last = 1);
  };
