/*****************************************************************

  audio.h

  Copyright (c) 2001 by Burkhard Plaum - plaum@ipf.uni-stuttgart.de

  http://gmerlin.sourceforge.net

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111, USA.

*****************************************************************/

// enum for stereo mode

enum GVMpegStereoMode
  {
    GVMpegStereoModeNone,
    GVMpegStereoModeMono,
    GVMpegStereoModeStereo,
    GVMpegStereoModeJStereo,
    GVMpegStereoModeDualChannel
  };

enum GVMpegAudioMode
  {
    GVMpegAudioMPEG1,   // Mpeg 1   layers I - III
    GVMpegAudioMPEG2,   // Mpeg 2   layers I - III
    GVMpegAudioMPEG2_5, // Mpeg 2.5 layers I - III
    GVMpegAudioAC3      // AC3 audio (Not supported yet)
  };

struct GVMpegAudioInfo
  {
  int vbr;

  GVMpegStereoMode stereo_mode;
  GVMpegAudioMode audio_mode;
  // Interesting for the user  
  int bitrate;
  int layer;
  // Interesting for the software
  int samples_per_frame;
  int samplerate;
  int num_channels;

  //  int vbr; // Variable bitrate

  //  int frame_size; // Size of a frame, invalid for VBR files
  
  guint64 num_samples;
  };

// Table of contents for audio streams

class GVMpegAudioDecoder
  {
protected:
  
  GVMpegFifo * fifo;
  
  //  int frame_size; // Size of one frame, 0 for VBR file

  guint64 decoded_samples; // Number of audio samples which are done yet
  
  GVMpegAudioInfo audio_info;

  GVMpegToc * toc;
  
  virtual void build_toc() = 0;

  GVMpegInput * input;
  
public:
  
  const GVMpegAudioInfo * get_audio_info() const { return &audio_info; }
  
  virtual void init() = 0;
  virtual void cleanup() = 0;
  
  GVMpegAudioDecoder(GVMpegFifo * _fifo);

  // Decode a frame of audio data, return the number of
  // samples or zero
  
  virtual int decode_frame(gint16 ** samples) = 0;

  // Get the size of the next frame
  
  virtual int get_frame_size() = 0;

  // Read one undecoded frame
  
  int read_frame(guchar * ptr);
  
  // Works only for elemental audio streams
  
  virtual void seek(float percentage);

  friend class GVMpegDecoder;
  
  };
