/*****************************************************************

  decoder.h

  Copyright (c) 2001 by Burkhard Plaum - plaum@ipf.uni-stuttgart.de

  http://gmerlin.sourceforge.net

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111, USA.

*****************************************************************/

// Mpeg decoder class
// All demultiplexing seeking and ID3 stuff is done here
// Data sources can be set using the set_input method.

enum GVMpegStreamType
{
  GVMpegElementalAudio,  // Elemental audio stream
  GVMpegElementalAC3,    // Elemental AC3 stream
  GVMpegElementalVideo,  // Elemental video stream
  GVMpegSystem,          // System stream (audio and video)
  GVMpegIfo,             // Ifo file (points to several VOB files)
};

struct GVTrackInfo;
struct GVMpegAudioDecoder;
struct GVMpegVideoDecoder;
struct GVMpegDemuxer;
//struct GVMpegFifo;
struct GVMpegSource;
struct GVMpegInput;

class GVMpegDecoder
  {
  GVMpegStreamType stream_type;
  
  GVMpegInput * input;

  int num_audio_streams;
  int num_video_streams;
    
  int audio_stream_id;
  int video_stream_id;

  int check_audio();  // Check for audio stream
  int check_video();  // Check for video stream
  int check_system(); // Check for system stream
  int check_ac3();    // Check for AC3 file

    
  const GVTrackInfo * track_info;

  // Skip the RIFF header present in some mp3s which were
  // created on some minor operating systems
  
  void skip_riff_header();
  
  // Read all ID3 information from the file and
  // set the offsets
  
//   void read_id3();

//   void string_to_id3v1(const guchar * ptr);

//   void read_id3v2_start();
//   void read_id3v2_end();
  
  GVMpegDemuxer * demuxer;
  
public:
  
  GVMpegStreamType get_stream_type() { return stream_type; }
  
  const GVTrackInfo * get_track_info() { return track_info; }
  
  GVMpegAudioDecoder * audio_decoder;
  GVMpegVideoDecoder * video_decoder;
  
  GVMpegDecoder();
  
  // Seek in the file, percentage is from 0 to 1
  
  void seek(float percentage);
  
  void set_input(GVMpegInput * _input);
  
  int get_video_frame(guchar * y, guchar * u, guchar * v);
  
  int get_audio_samples(gint16 * samples, int num_samples);
  
  // Called after the input is set up
  
  int init();
  void cleanup();
  
  };
