/*****************************************************************

  demuxer.cc

  Copyright (c) 2001 by Burkhard Plaum - plaum@ipf.uni-stuttgart.de

  http://gmerlin.sourceforge.net

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111, USA.

*****************************************************************/

#include <glib.h>

#include <iostream>
using std::cerr;
using std::ostream;

#include "toc.h"
#include "source.h"
#include "fifo.h"
#include "demuxer.h"

void GVMpegDemuxer::seek(float percentage)
  {
  
  }

void GVMpegDemuxer::build_toc()
  {
  
  }

GVMpegDemuxer::GVMpegDemuxer(GVMpegFifo * _fifo) :
  audio_fifo(this),
  video_fifo(this)
  {
  fifo = _fifo;
  }

guint32 GVMpegDemuxer::read_timestamp()
  {
  guint32 timestamp;

  guchar * ptr;

  guint32 tmp;
    
  ptr = fifo->read_data(5);

  timestamp = (ptr[0] >> 1) & 7;  
  timestamp <<= 15;

  tmp = (ptr[1] << 8) | ptr[2];
    
  timestamp |= (tmp >> 1);

  timestamp <<= 15;

  tmp = (ptr[3] << 8) | ptr[4];

  timestamp |= (tmp >> 1);

  return timestamp;
  }

int GVMpegDemuxer::pack_header()
  {
  // The pack start code is 4 bytes long

  cerr << "GVMpegDemuxer::pack_header...\n";
  
  guint32 header = fifo->read_32bits();
  if(header != 0x000001ba)
    {
    return 0;
    }

  // After the 32 bit header, there come 9 bytes, which aren't decoded
  // right now (maybe later)
  
  //  source->read_data(9);

  guchar * ptr = fifo->get_data(1);
  if((*ptr >> 4) == 2)
    {
    cerr << "MPEG1\n";
    read_timestamp();
    fifo->read_data(3);
    }
  else
    {
    cerr << "MPEG2\n";

    fifo->read_data(9);

    // Now, we have 5 reserved bits and then 3 bits for the number of stuffing
    // bytes
    
    ptr = fifo->read_data(1);
    
    int num_stuffing_bytes = ((int)(*ptr)) & 0x74;
    
    if(num_stuffing_bytes)
      {
      cerr << "Skipping " << num_stuffing_bytes << " stuffing bytes\n";
      fifo->read_data(num_stuffing_bytes);
      }
    
    }
  // Check, if there is a system header start code

  header = fifo->get_32bits();
  if(header == 0x000001bb)
    {
    cerr << "Found system header\n";
    }
  cerr << "GVMpegDemuxer::pack_header: Done\n";
  return 1;
  }

#define PROGRAM_STREAM_MAP 0xbc
#define PRIVATE_STREAM_1   0xbd
#define PADDING_STREAM     0xbe
#define PRIVATE_STREAM_2   0xbf

#define AUDIO_MASK         0xe0
#define AUDIO_STREAM       0xc0

#define VIDEO_MASK         0xf0
#define VIDEO_STREAM       0xe0

int GVMpegDemuxer::pes_packet()
  {
  cerr << "GVMpegDemuxer::pes_packet()\n";
  int audio_stream_index;
  int video_stream_index;

  guint32 header = fifo->read_32bits();

  guint stream_id = header & 0xff;

  if(stream_id == PROGRAM_STREAM_MAP)
    {
    cerr << "PROGRAM_STREAM_MAP\n";
    }
  else if((stream_id == PRIVATE_STREAM_1) ||
          (stream_id == PRIVATE_STREAM_2))
    {
    cerr << "PRIVATE_STREAM\n";
    }
  else if(stream_id == PADDING_STREAM)
    {
    cerr << "PADDING_STREAM\n";
    }
  else if((stream_id & AUDIO_MASK) == AUDIO_STREAM)
    {
    audio_stream_index = stream_id & ~AUDIO_MASK;
    cerr << "Audio Stream Nr. " << audio_stream_index << "\n";
    }
  else if((stream_id & VIDEO_MASK) == VIDEO_STREAM)
    {
    video_stream_index = stream_id & ~VIDEO_MASK;
    cerr << "Video Stream Nr. " << video_stream_index << "\n";
    }

  // Read the packet length

  //  guint16 length = 
  
  cerr << "GVMpegDemuxer::pes_packet(): Done\n";

  
  
  
  return 1;
  }

int GVMpegDemuxer::pack()
  {
  if(!pack_header())
    return 0;
  
  while(1)
    {
    guint32 header = fifo->get_32bits();
    if((header & 0xffffff00) == 0x00000100)
      pes_packet();
    else
      {
      cerr << "found no pes header, header: " << (void*)header << "\n";
      break;
      }
    }
  return 1;
  }

#define AUDIO_BUFFER_SIZE 1024
#define VIDEO_BUFFER_SIZE 1024

int GVMpegDemuxer::init()
  {
  if(!pack())
    return 0;
  
  audio_fifo.set_buffer_size(AUDIO_BUFFER_SIZE);
  video_fifo.set_buffer_size(VIDEO_BUFFER_SIZE);

  return 0;
  
  }
