/*****************************************************************

  fifo.cc

  Copyright (c) 2001 by Burkhard Plaum - plaum@ipf.uni-stuttgart.de

  http://gmerlin.sourceforge.net

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111, USA.

*****************************************************************/

#ifndef __GNUG__
#pragma implementation "fifo.h"
#endif

#include <string.h>
#include <glib.h>
#include "source.h"
#include "fifo.h"

#include <iostream.h>

GVMpegFifo::GVMpegFifo(GVMpegSource * _source)
  {
  source = _source;
  
  write_position = 0;
  read_position = 0;
  buffer_size = 0;

  flags = 0;
  
  buffer = (guchar*)0;
  }

GVMpegFifo::~GVMpegFifo()
  {
  if(buffer) delete buffer;
  }

void GVMpegFifo::set_buffer_size(int size)
  {
  guchar * new_buffer = new guchar[size];

  // Copy old contents to the new buffer

  int old_size = write_position - read_position;
  
  if(buffer)
    {
    memcpy(new_buffer, &(buffer[read_position]),
           write_position - read_position);
    delete buffer;
    }
  
  buffer = new_buffer;
  read_position = 0;
  write_position = old_size;

  buffer_size = size;
  }

int GVMpegFifo::add_data(guchar * data, int size)
  {
//   cerr << "Add Data1, read_position: " << read_position
//        << " write_position: " << write_position << "\n"; 
  int bytes_to_write = get_available_write_bytes();
  if(bytes_to_write < size)
    {
    if(flags & GV_MPEG_FIFO_AUTORESIZE)
      set_buffer_size(bytes_to_write + size);
    else
      {
      cerr << "GVMpegFifo::add_data(): Too much data (" << size << ")\n";
      return 0;
      }
    }

  // Move the unread section to the beginning

  if((read_position) && (write_position - read_position))
    memmove(buffer, &(buffer[read_position]), write_position - read_position);
  
  write_position -= read_position;
  read_position = 0;

  memcpy(&(buffer[write_position]), data, size);
  write_position += size;

//   cerr << "Add Data2, read_position: " << read_position
//        << " write_position: " << write_position << "\n"; 
  
  return 1;
  }

void GVMpegFifo::seek(gint64 difference, GVMpegSeekMode mode)
  {
  if(mode == GVMpegSeekCurrent)
    read_position += difference; 
  }

guchar * GVMpegFifo::read_data(int size, int * size_read)
  {
  //  cerr << "GVMpegFifo::read_data...";
  guchar * ret = get_data(size, size_read);
  read_position += size;
  //  cerr << "done\n";
  return ret;
  }


guchar * GVMpegFifo::get_data(int size, int * size_read)
  {
  //  cerr << "Fifo get data\n";
  
  if(size > buffer_size)
    {
    if(!(flags & GV_MPEG_FIFO_AUTORESIZE))
      cerr << "Fifo overflow in get_data(), buffer size is " << buffer_size
           << " requested " << size << " bytes\n";
    else
      {
      set_buffer_size(size + 1024); // We add 1024 bytes
      cerr << "Enlarging buffer to " << size + 1024 << " bytes\n";
      }
    }
  if(write_position - read_position < size)
    {
//     cerr << "Refilling fifo, " << write_position << " "
//          << read_position << " " << size << "\n";
    source->refill_fifo(this);
    }
  
  if(size_read)
    {
    if(write_position - read_position < size)
      *size_read = write_position - read_position;
    else
      *size_read = size;
    }
  guchar * ret = &(buffer[read_position]);
  return ret;
  }
