/*****************************************************************

  fifo.h

  Copyright (c) 2001 by Burkhard Plaum - plaum@ipf.uni-stuttgart.de

  http://gmerlin.sourceforge.net

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111, USA.

*****************************************************************/

#ifndef __GNUG__
#pragma interface
#endif

#define GV_MPEG_FIFO_EOF              (1<<0)
#define GV_MPEG_FIFO_AUTORESIZE       (1<<1)

// Fifo class
// This is used by the input stream as well as
// for communication between demuxer and decoders

// #include <iostream.h>

class GVMpegFifo
  {
  int flags;
  
  // Positions in the buffer

  int read_position;  // Index of the last byte read
  int write_position; // Index of the last byte written
    
  guchar * buffer;    // Buffer for data output

  int buffer_size;    // Buffer size can vary during operation

  GVMpegSource * source;
  
public:
  
  // Get the number of bytes available to read in the buffer

//   gint64 get_available_read_bytes()
//     {
//     return write_position - read_position;
//     }

  // Request bytes from the fifo, return value is the numbeer of bytes,
  // which is really available (sould be num_bytes or less if stream is over)
  
  //  int request_bytes(int num_bytes);
    
  // Get the number of bytes available to write into the buffer
  // without resizing

  int get_available_write_bytes()
    {
    return buffer_size - write_position + read_position;
    }

  void get_max_seek_offsets(gint64 & backward, gint64 & forward)
    {
    backward = - read_position;
    forward  = write_position - read_position;
    }

  void seek(gint64 difference, GVMpegSeekMode mode);
  
  // Add data to the buffer, return 1 on success, 0 if the fifo is full
  
  int add_data(guchar * data, int size);
  
  void clear()   // Discard all data
    {
    read_position = 0;
    write_position = 0;
    flags = 0;
    }
  
  void set_eof() // Set the eof flag
    {
    flags |= GV_MPEG_FIFO_EOF;
    }

  void clear_eof() // Set the eof flag
    {
    flags &= ~GV_MPEG_FIFO_EOF;
    }

  void set_autoresize(int autoresize)
    {
    if(autoresize)
      flags |= GV_MPEG_FIFO_AUTORESIZE;
    else
      flags &= ~GV_MPEG_FIFO_AUTORESIZE;
    }
  
  // Return True if eof
  
  int eof()
    {
    //    cerr << "get_available_read_bytes: " << get_available_read_bytes() << "\n";
    return ((flags & GV_MPEG_FIFO_EOF) && (read_position == write_position));
    }
  
  GVMpegFifo(GVMpegSource * _source);
  virtual ~GVMpegFifo();
  
  void set_buffer_size(int size);

  int get_buffer_size() { return buffer_size; }
  
  // Obtain data from the fifo
  // the read_* functions increment the read position, the get_* functions don't
  
  guint32 read_32bits(int * ok = 0)
    {
    guint32 ret;
    guchar * ptr = read_data(4, ok);
    ret = ptr[3] | (ptr[2] << 8) | (ptr[1] << 16) | (ptr[0] << 24);
    return ret;
    }
  
  guint32 get_32bits(int * ok = 0)
    {
    guint32 ret;
    guchar * ptr = get_data(4, ok);
    ret = ptr[3] | (ptr[2] << 8) | (ptr[1] << 16) | (ptr[0] << 24);
    return ret;
    }

//   guint32 read_24bits()
//     {
//     guint32 ret;
//     ret = (buffer[read_position+2] << 8) | (buffer[read_position+1] << 16) |
//       (buffer[read_position] << 24);
//     read_position+= 3;
//     return ret;
//     }
  
//   guint32 get_24bits()
//     {
//     guint32 ret;
//     ret = (buffer[read_position+2] << 8) | (buffer[read_position+1] << 16) |
//       (buffer[read_position] << 24);
//     return ret;
//     }
  
  // Returns a pointer to the internal buffer
  
  guchar * read_data(int size, int * size_read = (int*)0);
  guchar * get_data(int size, int * size_read = (int*)0);

  friend class GVMpegInput;
    
  };
