/*****************************************************************

  id3read.cc

  Copyright (c) 2001 by Burkhard Plaum - plaum@ipf.uni-stuttgart.de

  http://gmerlin.sourceforge.net

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111, USA.

*****************************************************************/

#include <iostream.h>

#include <id3.h>
#include <id3/tag.h>
#include <id3/misc_support.h>
#include <string.h>

#include <glib.h>

#include <stdlib.h>

#include "source.h"
#include "fifo.h"
#include "input.h"
#include "in_file.h"

#include "../../trackinfo.h"

#include "id3.h"

static char *_genres[]={
  "Blues", "Classic Rock", "Country", "Dance", "Disco", "Funk",
  "Grunge", "Hip-Hop", "Jazz", "Metal", "New Age", "Oldies",
  "Other", "Pop", "R&B", "Rap", "Reggae", "Rock",
  "Techno", "Industrial", "Alternative", "Ska", "Death Metal", "Pranks",
  "Soundtrack", "Euro-Techno", "Ambient", "Trip-Hop", "Vocal", "Jazz+Funk",
  "Fusion", "Trance", "Classical", "Instrumental", "Acid", "House",
  "Game", "Sound Clip", "Gospel", "Noise", "AlternRock", "Bass",
  "Soul", "Punk", "Space", "Meditative", "Instrumental Pop",
  "Instrumental Rock", "Ethnic", "Gothic", "Darkwave", "Techno-Industrial",
  "Electronic", "Pop-Folk", "Eurodance", "Dream", "Southern Rock", "Comedy",
  "Cult", "Gangsta", "Top 40", "Christian Rap", "Pop/Funk", "Jungle",
  "Native American", "Cabaret", "New Wave", "Psychadelic", "Rave",
  "Showtunes", "Trailer", "Lo-Fi", "Tribal", "Acid Punk", "Acid Jazz",
  "Polka", "Retro", "Musical", "Rock & Roll", "Hard Rock", "Folk",
  "Folk/Rock", "National Folk", "Swing", "Fast-Fusion", "Bebob", "Latin",
  "Revival", "Celtic", "Bluegrass", "Avantgarde", "Gothic Rock",
  "Progressive Rock", "Psychedelic Rock", "Symphonic Rock", "Slow Rock",
  "Big Band", "Chorus", "Easy Listening", "Acoustic", "Humour", "Speech",
  "Chanson", "Opera", "Chamber Music", "Sonata", "Symphony", "Booty Bass",
  "Primus", "Porn Groove", "Satire", "Slow Jam", "Club", "Tango", "Samba",
  "Folklore", "Ballad", "Power Ballad", "Rhythmic Soul", "Freestyle", "Duet",
  "Punk Rock", "Drum Solo", "A capella", "Euro-House", "Dance Hall", "Goa",
  "Drum & Bass", "Club House", "Hardcore", "Terror", "Indie", "BritPop",
  "NegerPunk", "Polsk Punk", "Beat", "Christian Gangsta", "Heavy Metal",
  "Black Metal", "Crossover", "Contemporary C", "Christian Rock", "Merengue",
  "Salsa", "Thrash Metal", "Anime", "JPop", "SynthPop"
};

static void apply_id3_tag(ID3_Tag* id3_tag, GVTrackInfo * track_info)
  {
  char * ptr;

  ptr = ID3_GetTitle(id3_tag);

  if(ptr)
    {
    track_info->title = g_strdup(ptr);
    delete ptr;
    }

  ptr = ID3_GetArtist(id3_tag);

  if(ptr)
    {
    track_info->artist = g_strdup(ptr);
    delete ptr;
    }

  ptr = ID3_GetAlbum(id3_tag);

  if(ptr)
    {
    track_info->album = g_strdup(ptr);
    delete ptr;
    }

  ptr = ID3_GetComment(id3_tag);

  if(ptr)
    {
    track_info->comment = g_strdup(ptr);
    delete ptr;
    }

  ptr = ID3_GetYear(id3_tag);

  if(ptr)
    {
    track_info->year = atoi(ptr);
    delete ptr;
    }

  ptr = ID3_GetGenre(id3_tag);

  if(ptr)
    {
    if(*ptr == '(')
      {
      // We need the genre number

      unsigned int genre_number = atoi(&ptr[1]);

      if(genre_number >= sizeof(_genres) / sizeof(char*))
        track_info->genre = g_strdup("Unknown");
      else
        track_info->genre = g_strdup(_genres[genre_number]);
      }
    else
      track_info->genre = g_strdup(ptr);
    delete ptr;
    }
  track_info->track_number = ID3_GetTrackNum(id3_tag);
  
  }


void GVMpegInputFile::read_id3_tags()
  {
  //  cerr << "Blupp1\n";

  ID3_Tag * id3_tag;
  
  if(!filename)
    return;

  //  cerr << "Blupp2\n";

  id3_tag = new ID3_Tag(filename);
//   delete id3_tag;
//   id3_tag = new ID3_Tag(filename);

  
  //  cerr << "Blupp3\n";

  size_t appended_bytes = id3_tag->GetAppendedBytes();
  size_t prepended_bytes = id3_tag->GetPrependedBytes();

  if(prepended_bytes)
    {
    fifo.read_data(prepended_bytes);

    add_start_offset(prepended_bytes);
    //    seek(0, GVMpegSeekStart);
    }
  if(appended_bytes)
    add_end_offset(appended_bytes);

  if(!(appended_bytes) && !(prepended_bytes))
    {
    delete id3_tag;
    return;
    }

  // DEBUG
  
//   if((appended_bytes) || (prepended_bytes))
//     {
//     cerr << "ID3tags found, at start: " << prepended_bytes
//          << " at end: " <<  appended_bytes << "\n";
//     }

  // We found Id3V1 and ID3V2 tags, clear the tag and reread it with
  // only ID3V2 enabled

  track_info = new GVTrackInfo;

  apply_id3_tag(id3_tag, track_info);
    
  if(id3_tag->HasV1Tag() && id3_tag->HasV2Tag())
    {
//     cerr << "Found ID2V1 and ID3V2 tags, using ID3V2\n";
    //    delete id3_tag;
    id3_tag->Clear();
    id3_tag->Link(filename, ID3TT_ID3V2);
    apply_id3_tag(id3_tag, track_info);
    }
  
  delete id3_tag;
  }

// This is also used by oa_lame.cc. It's moved here
// because otherwise, we would have too long cpp options in src/plugins

ID3_Tag * gv_create_id3_tag(const GVTrackInfo * info)
  {
  ID3_Tag * ret = new ID3_Tag;
  
  if(info->artist)
    ID3_AddArtist(ret, info->artist, 1);
  if(info->album)
    ID3_AddAlbum(ret, info->album, 1);
  if(info->title)
    ID3_AddTitle(ret, info->title, 1);
  if(info->comment)
    ID3_AddComment(ret, info->comment, 1);
  if(info->genre)
    {
    size_t genre = 0xFF; // Unknown
    
    for(unsigned int i = 0; i < sizeof(_genres) / sizeof(char*); i++)
      {
      if(!strcmp(info->genre, _genres[i]))
        genre = i;
      }
    ID3_AddGenre(ret, genre, 1);
    }
  
  if(info->year >= 0)
    {
    char * buf = g_strdup_printf("%d", info->year);
    ID3_AddYear(ret, buf, 1);
    g_free(buf);
    }
  if(info->track_number >= 0)
    {
    ID3_AddTrack(ret, info->track_number, 0, 1);
    }
  return ret;
  }

void gv_delete_id3_tag(ID3_Tag * tag)
  {
  delete tag;
  }

void gv_write_id3v1_tag(ID3_Tag * tag, ostream & output)
  {
  guchar * buffer = new guchar[128];

  tag->Render(buffer, ID3TT_ID3V1);

  output.write((char*)buffer, 128);
  delete buffer;
  }

void gv_write_id3v2_tag(ID3_Tag * tag, ostream & output)
  {
  size_t render_size;
  size_t buffer_size = tag->Size();
  guchar * buffer = new guchar[buffer_size];
  
  render_size = tag->Render(buffer);
  
  output.write((char*)buffer, render_size);
  delete buffer;
  }

