/*****************************************************************

  in_file.cc

  Copyright (c) 2001 by Burkhard Plaum - plaum@ipf.uni-stuttgart.de

  http://gmerlin.sourceforge.net

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111, USA.

*****************************************************************/

#include <glib.h>
#include <iostream.h>
#include <fstream.h>

#include <string.h>
#include <ctype.h>

#include "source.h"
#include "fifo.h"
#include "input.h"

#include "in_file.h"

#include "../../trackinfo.h"

#define FILE_BLOCK_SIZE 4096

int GVMpegInputFile::open(const char * arg)
  {
  GVMpegInput::init();
  
  file.open(arg);

  if(!file)
    {
    cerr << "Couldn't open file " << arg << "\n";
    return 0;
    }

  if(filename)
    g_free(filename);
  filename = g_strdup(arg);
  
  file.seekg(0, ios::end);
  total_bytes = file.tellg();
  file.seekg(0, ios::beg);

  //  cerr << "Openend " << arg << " " << total_bytes << " Bytes\n";

  // Read ID3 tags and set offsets
  
  if(check_suffix("mp3") || check_suffix("mp2"))
    read_id3_tags();

  return 1;
  }

void GVMpegInputFile::close()
  {
  //  cerr << "Closing file\n";
  file.close();
  
  if(filename)
    {
    g_free(filename);
    filename = (char*)0;
    }
  if(track_info)
    {
    delete track_info;
    track_info = (GVTrackInfo*)0;
    }
  }

guint64 GVMpegInputFile::read_block()
  {
  guint64 bytes_read;
    
  if((total_bytes - input_position) < (guint64)block_size)
    bytes_read = total_bytes - input_position;
  else
    bytes_read = block_size;
  
  //  cerr << "Reading " << bytes_read << " Bytes\n";
  
  file.read((char*)read_buffer, bytes_read);

  return bytes_read;
  }

void GVMpegInputFile::real_seek(gint64 position)
  {
  //  cerr << "GVMpegInputFile::real_seek(position = " << position << ")\n";
  guint64 pos = block_size * position;
  file.seekg(pos, ios::beg);
  }

GVMpegInputFile::GVMpegInputFile() :
  GVMpegInput(FILE_BLOCK_SIZE)
  {
  flags = GV_MPEG_SOURCE_CAN_SEEK;
  filename = (char*)0;
  track_info = (GVTrackInfo*)0;
  }

GVMpegInputFile::~GVMpegInputFile()
  {
  close();
  if(filename)
    g_free(filename);
  if(track_info)
    delete track_info;
  }

// Check if filename matches a suffix, suffix MUST be lowercase

int GVMpegInputFile::check_suffix(const char * suffix) const
  {
  char * pos;
  char * lower_suffix;

  if(!filename)
    return 0;
  
  if(!(pos = strrchr(filename, '.')))
    return 0;

  pos++;

  if(*pos == '\0')
    return 0;
  
  lower_suffix = g_strdup(pos);

  pos = lower_suffix;
  pos++;
  while(*pos != '\0')
    {
    *pos = tolower(*pos);
    pos++;
    }

  int ret = strcmp(lower_suffix, suffix);

  g_free(lower_suffix);
  return !(ret);
  }

const GVTrackInfo * GVMpegInputFile::get_track_info() const
  {
  return track_info;
  }
