/*****************************************************************

  input.cc

  Copyright (c) 2001 by Burkhard Plaum - plaum@ipf.uni-stuttgart.de

  http://gmerlin.sourceforge.net

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111, USA.

*****************************************************************/

#ifdef __GNUG__
#pragma implementation "input.h"
#endif

#include <glib.h>

#include "source.h"
#include "fifo.h"
#include "input.h"

#include <iostream>
using std::cerr;
using std::ostream;

GVMpegInput::GVMpegInput(int _block_size) :
  fifo(this)
  {
  block_size = _block_size;

  read_buffer = new guchar[block_size];

  fifo.set_buffer_size(2 * block_size);
  }

void GVMpegInput::init()
  {
  total_bytes = 0; // Unknown
  start_offset = 0;
  end_offset = 0;

  input_position = 0;
  //  output_position = 0;
  }

GVMpegInput::~GVMpegInput()
  {
  delete read_buffer;
  }

void GVMpegInput::seek(gint64 position, GVMpegSeekMode mode)
  {
  gint64 real_position = 0;

  gint64 output_position = input_position -
    (fifo.write_position - fifo.read_position);
  
  switch(mode)
    {
    case GVMpegSeekStart:
      //      cerr << "GVMpegInput::seek(), position: " << position << "\n";
      real_position = position;
      break;
    case GVMpegSeekCurrent:
      real_position = position + output_position;
      break;
    case GVMpegSeekEnd:
      real_position = position + total_bytes;
      break;
    }

  // Check if we must seek at all

//   cerr << "Seeking to " << real_position << " Real position: "
//        << real_position + start_offset << "\n";
  
  gint64 difference = real_position - output_position;
  
  // No seeking necessary
  //  cerr << "difference: " << difference << "\n";
  if(!difference)
    return;

  gint64 backward;
  gint64 forward;
  
  fifo.get_max_seek_offsets(backward, forward);
  
  if(((difference < 0) && (difference >= backward)) ||
     ((difference > 0) && (difference <= forward)))
    {
    //    cerr << "Seek cached in Fifo\n";
    fifo.seek(difference, GVMpegSeekCurrent);
    output_position += difference;
    return;
    }

  // Calculate the real seek destination
  
  guint64 block_num = (start_offset + real_position) / block_size;
  guint64 block_offset = (start_offset + real_position) % block_size;

  fifo.clear();
  real_seek(block_num);

  input_position = block_num * block_size - start_offset;

  refill_fifo(&fifo);
  fifo.read_data(block_offset);
  
  //  output_position += difference;
  }

void GVMpegInput::real_seek(gint64 position)
  {
  //  cerr << "Input::real_seek();\n";
  }


void GVMpegInput::refill_fifo(GVMpegFifo * dummy_fifo)
  {
  //  cerr << "GVMpegInput::refill_fifo()\n";
  int bytes_to_write = fifo.get_available_write_bytes();

  int bytes_read;
  
  while(bytes_to_write >= block_size)
    {
    bytes_read = read_block();
    fifo.add_data(read_buffer, bytes_read);
    input_position += bytes_read;

    if((total_bytes) && ((input_position >= (gint64)(total_bytes))))
      {
      fifo.set_eof();
//       cerr << "EOF, input_position: " << input_position
//            << " total_bytes: " << (gint64)(total_bytes) << "\n";
      break;
      }
    else
      fifo.clear_eof();
    //    cerr << "Input Position: " << input_position << "\n";
    bytes_to_write = fifo.get_available_write_bytes();
    }
  }

void GVMpegInput::cleanup()
  {
  fifo.clear();
  }

const GVTrackInfo * GVMpegInput::get_track_info() const
  {
  return (GVTrackInfo*)0;
  }
