/*****************************************************************

  input.h

  Copyright (c) 2001 by Burkhard Plaum - plaum@ipf.uni-stuttgart.de

  http://gmerlin.sourceforge.net

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111, USA.

*****************************************************************/

#ifdef __GNUG__
#pragma interface
#endif

// Source of the data, can be file, http and vcd

#define GV_MPEG_SOURCE_CAN_SEEK  (1<<0)

class GVTrackInfo;

class GVMpegInput : public GVMpegSource
  {

protected:
  int block_size;

  guchar * read_buffer;   // Buffer for elemental read operations, fixed size

  gint64 input_position;
  //  gint64 output_position;

  // Size of non-mpeg (e.g. ID3) data at the start and the end of the file
  
  guint64 start_offset;
  guint64 end_offset;

  guint64 total_bytes;
  
  int flags;

  // Reads a block, return value is block_size or less
  // if the file was at the end

  virtual guint64 read_block() = 0;



  // Fifo
  
  GVMpegFifo fifo;

  // Do the real seeking here, position is in blocks
  
  virtual void real_seek(gint64 position);
  
public:

  // refill the buffer until it's full

  void refill_fifo(GVMpegFifo * fifo);
  
  virtual const GVTrackInfo * get_track_info() const;
  
  // Return true if eof
  
  int eof() { return fifo.eof(); }
  
  int get_buffer_size() { return fifo.get_buffer_size(); }
  
  void add_start_offset(guint64 _start_offset)
    {
    start_offset += _start_offset;
    total_bytes  -= _start_offset;
    //    output_position -= _start_offset;
    input_position -= _start_offset;
    }
  
  void add_end_offset(guint64 _end_offset)
    {
    end_offset += _end_offset;
    total_bytes -= _end_offset;
    }

  void init();
  
  guint64 get_total_bytes() { return total_bytes; }
  
  int can_seek() { return flags & GV_MPEG_SOURCE_CAN_SEEK; }

//   gint64 get_available_read_bytes()
//     {
//     if(!can_seek())
//       return -1;
//     return total_bytes - input_position + fifo.get_available_read_bytes();
//     }
  
  // Obtain data from the buffer
  // The read_* functions really read the data while the get_* functions
  // try not to discard any unread buffer
  
  GVMpegInput(int _block_size);
  virtual ~GVMpegInput();
  
  int get_flags();

  // Seek to a block
  
  void seek(gint64 position, GVMpegSeekMode mode);

  virtual int open(const char * arg) = 0;
  virtual void close() = 0;

//   guint32 read_32bits();
//   guint32 get_32bits();

//   guint32 read_24bits();
//   guint32 get_24bits();

  // Returns a pointer to the internal buffer
  
//   guchar * read_data(int size);
//   guchar * get_data(int size);

  // Reset the Fifo
  
  void cleanup();

  friend class GVMpegDecoder;
  
  };
