/*****************************************************************

  video.h

  Copyright (c) 2001 by Burkhard Plaum - plaum@ipf.uni-stuttgart.de

  http://gmerlin.sourceforge.net

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111, USA.

*****************************************************************/

extern "C" {
#include <inttypes.h>
#include "mpeg2dec/include/video_out.h"
#include "mpeg2dec/include/mpeg2.h"
}
  
struct GVMpegVideoInfo
  {
  int width;
  int height;
  double framerate;
  guint64 num_frames;
  };

struct GVMpegToc;

class GVMpegVideoDecoder
  {
public:

  struct VoFrame
    {
    vo_frame_t frame;
//     int type; // Frame type
//     int draw;
    };
  
  struct VoInstance
    {
    vo_instance_t vo;

    VoFrame frame[3];

    int prediction_index;
    
//     VoFrame * first_saved_frame;
//     VoFrame * last_saved_frame;

    int size; // For copying
    };

  
private:

  // Reads bytes from the source and send them to the decoder
  // until the next picture_start_code arrives
  
  guint32 process_bytes(int drop, int decode);

  guint32 start_offset;
  
  GVMpegFifo * fifo;
  GVMpegInput * input;

  GVMpegVideoInfo video_info;

  GVMpegToc * toc;
  
  void build_toc();

  // libmpeg2 specific stuff
  
  mpeg2dec_t * mpeg2dec;

  VoInstance * vo_instance;

  int eof_flag; // Set to 1 after the decoder has decoded the last frame
  int eof_seen; // Set to a after a sequence end code was seen or fifo is empty
  
  int last_picture_type;

  // This is set to 1 if a gop start code was found during the last
  // process_bytes call
  
  int gop_start_found; 

  int prediction_frame_seen;
  
public:

  int eof() { return eof_flag; }
  
  const GVMpegVideoInfo * get_video_info() const { return &video_info; }
  
  void cleanup();
  int init();
  
  GVMpegVideoDecoder(GVMpegFifo * _fifo);

  void decode_frame(guchar * y, guchar * u, guchar * v);

  // Works only for elemtal video streams
  
  void seek(float percentage);

  friend class GVMpegDecoder;
  
  };
