/*****************************************************************

  videodecoder.cc

  Copyright (c) 2001 by Burkhard Plaum - plaum@ipf.uni-stuttgart.de

  http://gmerlin.sourceforge.net

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111, USA.

*****************************************************************/

#include <iostream>
using std::ostream;
using std::cerr;

#include <string.h>
#include <glib.h>
#include "source.h"
#include "fifo.h"
#include "toc.h"
#include "input.h"
#include "video.h"

// Static functions needed by libmpeg2 and libvo

// 0x00000100

#define IS_START_CODE_PREFIX(ptr) ((ptr[0] == 0x00) && \
                                    (ptr[1] == 0x00) && \
                                    (ptr[2] == 0x01))

#define IS_PICTURE_START_CODE(ptr) (ptr[3] == 0x00)

#define IS_GOP_START_CODE(ptr) (ptr[3] == 0xb8)

#define IS_SEQUENCE_END_CODE(ptr) (ptr[3] == 0xb7)

static inline void __vo_copy_frame(vo_frame_t * src,
                                   vo_frame_t * dst,
                                   int size)
  {
  memcpy(dst->base[0], src->base[0], size);
  memcpy(dst->base[1], src->base[1], size/4);
  memcpy(dst->base[2], src->base[2], size/4);
  }

static void __vo_draw (vo_frame_t * frame)
  {
//   cerr << "__vo_draw\n";

//   GVMpegVideoDecoder::VoInstance * vo_instance =
//     (GVMpegVideoDecoder::VoInstance*)(frame->instance);

//   if(frame == (vo_frame_t*)(&(vo_instance->frame[2])))
//     return;
//   else
//     __vo_copy_frame(frame, &(vo_instance->frame[2].frame),
//                     vo_instance->size);
  
  //  ((GVMpegVideoDecoder::VoFrame*)(frame))->draw = 1;
  }

static void __vo_field (vo_frame_t * frame, int flags)
  {
  //  cerr << "__vo_field, flags : " << flags << "\n";
  }

static int __vo_setup(vo_instance_t * instance, int width, int height)
  {
  //  cerr << "__vo_setup: " << width << " " << height << "\n";
  
  GVMpegVideoDecoder::VoInstance * vo_instance =
    (GVMpegVideoDecoder::VoInstance*)instance;

  vo_instance->size = width * height;
  vo_instance->prediction_index = 0;
  
  for(int i = 0; i < 2; i++)
    {
    vo_instance->frame[i].frame.base[0] =
      new uint8_t[vo_instance->size];

    vo_instance->frame[i].frame.base[1] =
      new uint8_t[vo_instance->size/4];

    vo_instance->frame[i].frame.base[2] =
      new uint8_t[vo_instance->size/4];
    }

  for(int i = 0; i < 3; i++)
    {
    vo_instance->frame[i].frame.instance = instance;
    vo_instance->frame[i].frame.copy = NULL;
    vo_instance->frame[i].frame.field = __vo_field;
    vo_instance->frame[i].frame.draw = __vo_draw;
    }
  return 0;
  }

static void __vo_close(vo_instance_t * instance)
  {
  GVMpegVideoDecoder::VoInstance * vo_instance =
    (GVMpegVideoDecoder::VoInstance*)instance;

  for(int i = 0; i < 2; i++)
    {
    delete vo_instance->frame[i].frame.base[0];
    delete vo_instance->frame[i].frame.base[1];
    delete vo_instance->frame[i].frame.base[2];
    }
  }

static vo_frame_t * __vo_get_frame(vo_instance_t * instance, int flags)
  {
  //  cerr << "__vo_get_frame, flags: " << flags << "\n";
  GVMpegVideoDecoder::VoInstance * vo_instance =
    (GVMpegVideoDecoder::VoInstance*)instance;

  if(flags & VO_PREDICTION_FLAG)
    {
    vo_instance->prediction_index ^= 1;
    return (vo_frame_t*)(&(vo_instance->frame[vo_instance->prediction_index]));
    }
  else
    return (vo_frame_t*)(&(vo_instance->frame[2]));
  
  //  return (vo_frame_t*)0;
  }

float __frame_rate_table[16] =
{
  0.0,
  24000.0/1001.0,
  24.0,
  25.0,
  30000.0/1001.0,
  30.0,
  50.0,
  ((60.0*1000.0)/1001.0),
  60.0,
  1,
  5, 
  10,
  12,
  15,
  0,
  0,
};

GVMpegVideoDecoder::GVMpegVideoDecoder(GVMpegFifo * _fifo)
  {
  fifo = _fifo;
  input = (GVMpegInput*)0;
  }

void GVMpegVideoDecoder::cleanup()
  {
  //  cerr << "GVMpegVideoDecoder::cleanup()...";
  mpeg2_close(mpeg2dec);
  
  delete mpeg2dec;
  delete vo_instance;
  //  cerr << "done\n";
  }

int GVMpegVideoDecoder::init()
  {
  // Initialize the video decoder
  // We read what we need to initialize the decoder (64 bits) 
  // and pass the rest to the actual decoder
  eof_flag = 0;
  eof_seen = 0;
  
  last_picture_type = -1;

  prediction_frame_seen = 0;
  
  //  old_frame_present = 0;
  
  guchar * ptr = fifo->get_data(8);

  // Read width, height and framerate

  guint32 tmp = (ptr[4]<<24) | (ptr[5]<<16) | (ptr[6]<<8) | (ptr[7]);
    
  video_info.width =  (tmp & 0xFFF00000) >> 20;
  video_info.height = (tmp & 0x000FFF00) >> 8;

  int frame_rate_code = (tmp & 0x0000000F);

  video_info.framerate = __frame_rate_table[frame_rate_code];

//   cerr << "Found video stream, Width = " << video_info.width
//        << " Height: " << video_info.height << " Framerate: "
//        << video_info.framerate << "\n"; 

  // Set up the vo instance

  vo_instance = new VoInstance;
  vo_instance->vo.setup = __vo_setup;
  vo_instance->vo.close = __vo_close;
  vo_instance->vo.get_frame = __vo_get_frame;
  
  mpeg2dec = new mpeg2dec_t;
  mpeg2_init(mpeg2dec, 0, (vo_instance_t*)vo_instance);

  // Decode everything until the next picture start code
  // So, mpeg2dec will know about the Sequence stuff and
  // GOP header
  
  start_offset = process_bytes(0, 1);
  
  return 1;
  }

// How many bytes do we read at once and pass to the decoder

#define BYTES_TO_READ 1024

// This function processes bytes until the next picture start code
// all bytes (including the next startcode) are sent to the decoder
// and removes from the fifo.
// If there are no more bytes left or a SEQUENCE_END_CODE was seen,
// the flag eof_seen is set to 1

guint GVMpegVideoDecoder::process_bytes(int drop, int decode)
  {
  guchar * ptr;
  guchar * tmp_ptr;
  int end = 0;
  guint num_bytes = 0;

  gop_start_found = 0;

  if(decode)
    mpeg2_drop(mpeg2dec, drop);
  
  int bytes_read;
  
  while(1)
    {
    ptr = fifo->get_data(BYTES_TO_READ, &bytes_read);
    tmp_ptr = ptr;
    for(int i = 0; i < bytes_read - 3; i++)
      {
      if(IS_START_CODE_PREFIX(tmp_ptr))
        {
        if(IS_PICTURE_START_CODE(tmp_ptr))
          {
          //          cerr << "Found Picture start code...";
          if(decode)
            mpeg2_decode_data(mpeg2dec, ptr, tmp_ptr+4);
          fifo->read_data((int)(tmp_ptr - ptr + 4));
          //          cerr << "Done\n";
          end = 1;
          break;
          }
        else if(IS_SEQUENCE_END_CODE(tmp_ptr))
          {
          cerr << "Found Sequence end code\n";
          if(i)
            {
            if(decode)
              mpeg2_decode_data(mpeg2dec, ptr, tmp_ptr+4);
            fifo->read_data((int)(tmp_ptr - ptr));
            }
          eof_seen = 1;
          end = 1;
          break;
          }
//         else if(IS_GOP_START_CODE(tmp_ptr))
//           {
//           //          cerr << "Found GOP start code\n";
//           gop_start_found = 1;
//           tmp_ptr++;
//           num_bytes++;
//           }
        else
          {
          tmp_ptr++;
          num_bytes++;
          }
        }
      else
        {
        tmp_ptr++;
        num_bytes++;
        }
      if(end)
        break;
      }
    if(end)
      break;
    if(decode)
      mpeg2_decode_data(mpeg2dec, ptr, &(ptr[bytes_read-3]));
    fifo->read_data(bytes_read-3);

    if(!eof_seen)
      {
      if(fifo->eof())
        {
        eof_seen = 1;
        cerr << "EOF Seen\n";
        break;
        }
      fifo->get_data(4, &bytes_read);
      if(bytes_read < 4)
        {
        eof_seen = 1;
        cerr << "Fifo has less than 4 bytes left, it's EOF\n";
        break;
        }
      }
    }
  return num_bytes;
  }

#define PICTURE_TYPE_I 1
#define PICTURE_TYPE_P 2
#define PICTURE_TYPE_B 3

static inline void __copy_frame(guchar * y, guchar * u, guchar * v,
                                GVMpegVideoDecoder::VoFrame * frame,
                                int size)
  {
  memcpy(y, frame->frame.base[0], size);
  memcpy(u, frame->frame.base[1], size/4);
  memcpy(v, frame->frame.base[2], size/4);
  }

void GVMpegVideoDecoder::decode_frame(guchar * y, guchar * u, guchar * v)
  {
  // Decode one single video frame
  // This function assumes, that we have a valid picture start code
  // at the start of the data
 
  guchar * ptr = fifo->get_data(2);

  guint picture_type = (ptr[1] & 0x3f) >> 3;
       
  //  guint next_picture_type = 0;

  if(eof_seen && (last_picture_type == PICTURE_TYPE_B))
    {
//     cerr << "Found Eof, displaying last prediction frame\n";
    __copy_frame(y, u, v,
                 &(vo_instance->frame[(vo_instance->prediction_index ^ 1)]),
                 video_info.width*video_info.height);
    eof_flag = 1;
    return;
    }

  if(picture_type == PICTURE_TYPE_B)
    {
    // B-frames are right after decoding
    // and aren't saved
//     cerr << "Found B Frame\n";
    vo_instance->frame[2].frame.base[0] = y;
    vo_instance->frame[2].frame.base[1] = u;
    vo_instance->frame[2].frame.base[2] = v;

    process_bytes(0, 1);
    }
  
  // We saw no prediction frame yet, so we should
  // mempcy the last saved frame
  else if(!prediction_frame_seen)
    {
//     cerr << "Found " << ((picture_type == PICTURE_TYPE_P) ? "P" : "I")
//          << " Frame. ";
//     cerr << "No prediction frame yet, displaying this one "
//          << vo_instance->prediction_index << "\n";
    process_bytes(0, 1);
    __copy_frame(y, u, v,
                 &(vo_instance->frame[vo_instance->prediction_index]),
                 video_info.width*video_info.height);
    prediction_frame_seen = 1;
    }
  else
    {
    process_bytes(0, 1);

    if(last_picture_type != PICTURE_TYPE_B) 
      {
      ptr = fifo->get_data(2);
      guint next_picture_type = (ptr[1] & 0x3f) >> 3;

      // The last 2 frames were predictive frames, the next one is
      // a B Frame -> The B Frame is, that the user wants to see
      
      if(next_picture_type == PICTURE_TYPE_B)
        {
//         cerr << "Found " << ((picture_type == PICTURE_TYPE_P) ? "P" : "I")
//              << " Frame. ";
//         cerr << "Displaying next B frame\n";

        vo_instance->frame[2].frame.base[0] = y;
        vo_instance->frame[2].frame.base[1] = u;
        vo_instance->frame[2].frame.base[2] = v;
        process_bytes(0, 1);
        }
      else
        {
//         cerr << "Found " << ((picture_type == PICTURE_TYPE_P) ? "P" : "I")
//              << " Frame. ";
//         cerr << "Displaying last prediction frame\n";
        __copy_frame(y, u, v,
                     &(vo_instance->frame[(vo_instance->prediction_index ^ 1)]),
                     video_info.width*video_info.height);

        }
      }
    else
      {
//       cerr << "Found " << ((picture_type == PICTURE_TYPE_P) ? "P" : "I")
//            << " Frame. ";
//       cerr << "Displaying last prediction frame\n";
      __copy_frame(y, u, v,
                   &(vo_instance->frame[(vo_instance->prediction_index ^ 1)]),
                   video_info.width*video_info.height);
      
      
      }
    }
  
  //  cerr << "******** Decoded frame *******\n";

  // Set the eof flag if the stream is over

  if(eof_seen && (picture_type != PICTURE_TYPE_B))
    eof_flag = 1;
  
  last_picture_type = picture_type;
  }

#define FRAMES_TO_ALLOC 10000

void GVMpegVideoDecoder::build_toc()
  {
  //  int bytes_read;
  
  toc = new GVMpegToc;
 
  // First Run: Count the frames

  toc->num_frames = 0;

  //  cerr << "Building toc, first run\n";

  guint64 max_num_frames = FRAMES_TO_ALLOC;

  GVMpegTocEntry * file_positions = new GVMpegTocEntry[FRAMES_TO_ALLOC];
    
  GList * toc_list = (GList*)0;

  guint64 file_position = start_offset;

  guint64 num_i_frames = 0;
  
  int picture_type;
  guchar * ptr;
  
  while(1)
    {
    ptr = fifo->get_data(2);
    picture_type = (ptr[1] & 0x3f) >> 3;

    if(picture_type == PICTURE_TYPE_I)
      {
      if(toc->num_frames + 1 > max_num_frames)
        {
        // Put the array into the list
        
        toc_list = g_list_append(toc_list, (gpointer)file_positions);
        
        // Allocate a new one
        
        file_positions = new GVMpegTocEntry[FRAMES_TO_ALLOC];
        max_num_frames += FRAMES_TO_ALLOC;
        }

      // TODO: Skip some frames to find out, when to display this one
      
      file_positions[num_i_frames % FRAMES_TO_ALLOC].file_position
        = file_position;
      file_positions[num_i_frames % FRAMES_TO_ALLOC].frame_number =
        toc->num_frames;

      num_i_frames++;
            
      file_position += process_bytes(0, 0);
      toc->num_frames++;
      }
    else
      {
      file_position += process_bytes(0, 0);
      toc->num_frames++;
      }
    
    //    cerr << "toc->num_frames: " << toc->num_frames << "\n";

    if(eof_seen)
      break;
    }

  toc_list = g_list_append(toc_list, (gpointer)file_positions);
  
  // Second run: Fill the toc

//   guint64 next_frame;

//   guint64 * position_ptr;
  
//   for(int i = 0; i < 100; i++)
//     {
//     // Get the next frame for the TOC

//     next_frame = (guint64)(toc->num_frames*i/100.0);

//     position_ptr = (guint64*)(g_list_nth_data(toc_list,
//                                             next_frame/FRAMES_TO_ALLOC));
    
//     toc->toc_entry[i].frame_number = next_frame;
//     toc->toc_entry[i].file_position =
//       position_ptr[next_frame % FRAMES_TO_ALLOC]; 
//     }
  input->seek(start_offset, GVMpegSeekStart);   
  eof_seen = 0;

  video_info.num_frames = toc->num_frames;

  // Free the list

  int imax = (toc->num_frames + FRAMES_TO_ALLOC - 1) / FRAMES_TO_ALLOC;

  GVMpegTocEntry * position_ptr;
  
  for(int i = 0; i < imax; i++)
    {
    position_ptr = (GVMpegTocEntry*)(g_list_nth_data(toc_list, i));
    delete position_ptr;
    }
  g_list_free(toc_list);
  cerr << "Built toc: " << toc->num_frames << " Frames\n";
  }




void GVMpegVideoDecoder::seek(float percentage)
  {
  
  }
