/*****************************************************************

  oa_lame.cc

  Copyright (c) 2001 by Burkhard Plaum - plaum@ipf.uni-stuttgart.de

  http://gmerlin.sourceforge.net

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111, USA.

*****************************************************************/

// Lame mp3 encoder

#ifdef __GNUG__
#pragma implementation "oa_lame.h"
#endif

#include <iostream>

using std::ostream;
using std::cerr;

#include "../gv.h"
#include "../gvconfig.h"
#include "../gtkutil.h"

#include "../trackinfo.h"

extern "C" {
#include "avifile/plugins/libmp3lame_audioenc/lame3.70/lame.h"
#include "avifile/plugins/libmp3lame_audioenc/lame3.70/VbrTag.h"
}

#include "oa_lame.h"

#include <string.h>

#include <stdlib.h>
#include <unistd.h>
#include <stdio.h>

#include "libgmerlin_mpeg/id3.h"

#define LAME_CHANNELS_AUTO   -1
#define LAME_CHANNELS_STEREO  0
#define LAME_CHANNELS_JSTEREO 1
#define LAME_CHANNELS_DUAL    2
#define LAME_CHANNELS_MONO    3

#define BIT_PER_KBIT 1024

static char *lame_genres[]={
  "Blues", "Classic Rock", "Country", "Dance", "Disco", "Funk",
  "Grunge", "Hip-Hop", "Jazz", "Metal", "New Age", "Oldies",
  "Other", "Pop", "R&B", "Rap", "Reggae", "Rock",
  "Techno", "Industrial", "Alternative", "Ska", "Death Metal", "Pranks",
  "Soundtrack", "Euro-Techno", "Ambient", "Trip-Hop", "Vocal", "Jazz+Funk",
  "Fusion", "Trance", "Classical", "Instrumental", "Acid", "House",
  "Game", "Sound Clip", "Gospel", "Noise", "AlternRock", "Bass",
  "Soul", "Punk", "Space", "Meditative", "Instrumental Pop",
  "Instrumental Rock", "Ethnic", "Gothic", "Darkwave", "Techno-Industrial",
  "Electronic", "Pop-Folk", "Eurodance", "Dream", "Southern Rock", "Comedy",
  "Cult", "Gangsta", "Top 40", "Christian Rap", "Pop/Funk", "Jungle",
  "Native American", "Cabaret", "New Wave", "Psychadelic", "Rave",
  "Showtunes", "Trailer", "Lo-Fi", "Tribal", "Acid Punk", "Acid Jazz",
  "Polka", "Retro", "Musical", "Rock & Roll", "Hard Rock", "Folk",
  "Folk/Rock", "National Folk", "Swing", "Fast-Fusion", "Bebob", "Latin",
  "Revival", "Celtic", "Bluegrass", "Avantgarde", "Gothic Rock",
  "Progressive Rock", "Psychedelic Rock", "Symphonic Rock", "Slow Rock",
  "Big Band", "Chorus", "Easy Listening", "Acoustic", "Humour", "Speech",
  "Chanson", "Opera", "Chamber Music", "Sonata", "Symphony", "Booty Bass",
  "Primus", "Porn Groove", "Satire", "Slow Jam", "Club", "Tango", "Samba",
  "Folklore", "Ballad", "Power Ballad", "Rhythmic Soul", "Freestyle", "Duet",
  "Punk Rock", "Drum Solo", "A capella", "Euro-House", "Dance Hall", "Goa",
  "Drum & Bass", "Club House", "Hardcore", "Terror", "Indie", "BritPop",
  "NegerPunk", "Polsk Punk", "Beat", "Christian Gangsta", "Heavy Metal",
  "Black Metal", "Crossover", "Contemporary C", "Christian Rock", "Merengue",
  "Salsa", "Thrash Metal", "Anime", "JPop", "SynthPop"
};


static int lame_bitrates[] =
  {
    56,
    96,
    112,
    128,
    160,
    192,
    256,
    320
  };

static GVConfigMenuItemInt lame_stereo_modes[] =
  {
    { LAME_CHANNELS_AUTO,    "Auto" },         // Let lame decide
    { LAME_CHANNELS_STEREO,  "Stereo" },       // stereo
    { LAME_CHANNELS_JSTEREO, "Joint Stereo" }, // jstereo
    { LAME_CHANNELS_DUAL,    "Dual Channel" }, // dual channel
    { LAME_CHANNELS_MONO,    "Mono" }          // mono
  };

static GVPluginAttributes _attributes =
  {
    "oa_lame",
    "Lame mp3 encoder",
    "MPEG1/2 audio layer III (mp3) encoder\n\
Based on Lame. See\n\
http://lame.sourceforge.net\n\
for more information",
    "mp3",
    GV_PLUGIN_HAS_AUDIO|
    GV_PLUGIN_IS_FILE
  };

#define CONFIG_SECTION "oa_lame"

void GVOutputAudioLame::get_config()
  {
  int i_tmp;

  lame_init(&lame_flags);
  
  // Bitrate
  lame_flags.brate = 128;
  lame_flags.silent = 1;

  gv_config->get_int(CONFIG_SECTION, "Bitrate", lame_flags.brate);
  gv_config->get_int(CONFIG_SECTION, "Quality", lame_flags.quality);

  gv_config->get_bool(CONFIG_SECTION, "UseVBR", lame_flags.VBR);

  if(lame_flags.VBR)
    lame_flags.bWriteVbrTag = 1;
  else
    lame_flags.bWriteVbrTag = 0;
    
  
  gv_config->get_int(CONFIG_SECTION, "VbrMinBitrate",
                     lame_flags.VBR_min_bitrate_kbps);
  gv_config->get_int(CONFIG_SECTION, "VbrMaxBitrate",
                     lame_flags.VBR_max_bitrate_kbps);
  gv_config->get_int(CONFIG_SECTION, "VbrQuality",
                     lame_flags.VBR_q);

  build_id3_flags = GV_TRACK_INFO_FORCE_COMMENT;

  gv_config->get_int(CONFIG_SECTION, "BuildId3Flags",
                     build_id3_flags);
    
  i_tmp = LAME_CHANNELS_AUTO;
  gv_config->get_int(CONFIG_SECTION, "StereoMode", i_tmp);

  if(i_tmp == LAME_CHANNELS_AUTO)
    lame_flags.mode_fixed = 0;
  else
    {
    lame_flags.mode_fixed = 1;
    lame_flags.mode = i_tmp;
    }

  write_id3v1 = 1;
  gv_config->get_bool(CONFIG_SECTION, "WriteID3V1tags", write_id3v1);

  write_id3v2 = 0;
  gv_config->get_bool(CONFIG_SECTION, "WriteID3V2tags", write_id3v2);

  const char * tmp = "";
  gv_config->get_string(CONFIG_SECTION, "DefaultArtist", &tmp);
  default_tag_info.set_artist(tmp);

  tmp = "";
  gv_config->get_string(CONFIG_SECTION, "DefaultTitle", &tmp);
  default_tag_info.set_title(tmp);

  tmp = "";
  gv_config->get_string(CONFIG_SECTION, "DefaultAlbum", &tmp);
  default_tag_info.set_album(tmp);

  tmp = "Made with gmerlin";
  gv_config->get_string(CONFIG_SECTION, "DefaultComment", &tmp);
  default_tag_info.set_comment(tmp);

  tmp = "";
  gv_config->get_string(CONFIG_SECTION, "DefaultGenre", &tmp);
  default_tag_info.set_genre(tmp);

  default_tag_info.year = 2001;
  gv_config->get_int(CONFIG_SECTION, "DefaultYear", default_tag_info.year);

  default_tag_info.track_number = 0;
  gv_config->get_int(CONFIG_SECTION, "DefaultTrackNumber",
                     default_tag_info.year);
  
  }

void GVOutputAudioLame::put_config()
  {
  gv_config->put_int(CONFIG_SECTION, "Bitrate", lame_flags.brate);
  gv_config->put_int(CONFIG_SECTION, "Quality", lame_flags.quality);

  gv_config->put_bool(CONFIG_SECTION, "UseVBR", lame_flags.VBR);
  gv_config->put_int(CONFIG_SECTION, "VbrMinBitrate",
                     lame_flags.VBR_min_bitrate_kbps);
  gv_config->put_int(CONFIG_SECTION, "VbrMaxBitrate",
                     lame_flags.VBR_max_bitrate_kbps);
  gv_config->put_int(CONFIG_SECTION, "VbrQuality",
                     lame_flags.VBR_q);
  gv_config->put_int(CONFIG_SECTION, "BuildId3Flags",
                     build_id3_flags);

  
  if(lame_flags.mode_fixed)
    gv_config->put_int(CONFIG_SECTION, "StereoMode", lame_flags.mode);
  else
    gv_config->put_int(CONFIG_SECTION, "StereoMode", LAME_CHANNELS_AUTO);
  gv_config->put_bool(CONFIG_SECTION, "WriteID3V1tags", write_id3v1);
  gv_config->put_bool(CONFIG_SECTION, "WriteID3V2tags", write_id3v2);

  gv_config->put_string(CONFIG_SECTION, "DefaultArtist",
                        default_tag_info.artist);

  gv_config->put_string(CONFIG_SECTION, "DefaultTitle",
                        default_tag_info.title);
  gv_config->put_string(CONFIG_SECTION, "DefaultAlbum",
                        default_tag_info.album);

  gv_config->put_string(CONFIG_SECTION, "DefaultComment",
                        default_tag_info.comment);
  
  gv_config->put_string(CONFIG_SECTION, "DefaultGenre",
                        default_tag_info.genre);

  gv_config->put_int(CONFIG_SECTION, "DefaultYear",
                     default_tag_info.year);

  gv_config->put_int(CONFIG_SECTION, "DefaultTrackNumber",
                     default_tag_info.track_number);
  }


// Set defaults for the lame config

#define LAME_CONFIG_SECTION "oa_lame"

GVOutputAudioLame::GVOutputAudioLame() :
  GVOutputAudio(&_attributes)
  {
  //  int i_tmp;
  mp3_buffer = (char*)0;

  tmp_filename = (char*)0;
  
  lame_init(&lame_flags);

  get_config();

  config_window = (ConfigWindow*)0;
  }

GVOutputAudioLame::~GVOutputAudioLame()
  {
  // Write the configuration options
  
  //  int i_tmp;

  put_config();

  if(mp3_buffer)
    delete mp3_buffer;
  if(config_window)
    delete config_window;
  }

void GVOutputAudioLame::init()
  {
  const GVAudioFormat * input_format = input->get_audio_format();

  // Set sample rate

  lame_flags.in_samplerate = input_format->samplerate; 
  lame_flags.out_samplerate = input_format->samplerate;
  audio_format.samplerate = input_format->samplerate;
  cerr << "Samplerate: " << audio_format.samplerate << "\n";

  if(mp3_buffer)
    delete mp3_buffer;

  audio_format.sample_type = GVAudio_16_signed;
  audio_format.samples_per_frame = input_format->samples_per_frame;
  mp3_buffer_size = (int)(1.25*audio_format.samples_per_frame+7201);
  mp3_buffer = new char[mp3_buffer_size];
  
  // Set number of channels

  switch(input_format->num_channels)
    {
    case 1:
      lame_flags.num_channels = audio_format.num_channels = 1;
      break;
    default:
      lame_flags.num_channels = audio_format.num_channels = 2;
      break;
    }

  // Set total number of samples

  audio_format.num_samples = lame_flags.num_samples =
    input_format->num_samples;
  
  audio_format.interleave_mode = GVInterleaveNone; // No interleaved channels
  audio_format.init();
  
  lame_init_params(&lame_flags);
  
  // Open the output file For VBR files, we need a temporary file
  // if we want to add ID3 tags

  if(!create_outfilename("mp3"))
    return;

  // Create id3 tag from input
  
  if((write_id3v1) || (write_id3v2))
    build_id3_tag();
  
  if((lame_flags.VBR) && (write_id3v1 || write_id3v2))
    {
    tmp_filename = new char[strlen(filename)+7];
    strcpy(tmp_filename, filename);
    strcat(tmp_filename, "XXXXXX");
    int fd = mkstemp(tmp_filename);
    close(fd);
    output.open(tmp_filename);

    // Lame writes VBR tags only if outPath != NULL
    
    lame_flags.outPath = tmp_filename;
    }
  else
    {
    output.open(filename);
    if(write_id3v2)
      gv_write_id3v2_tag(id3_tag, output);
    }
      
  //  cerr << "Output File: " << filename << "\n";
  
  audio_samples_written = 0;
  }

void GVOutputAudioLame::put_frame(GVAudioFrame * frame)
  {
  int offset = audio_format.samples_per_frame;
  int num_bytes = lame_encode_buffer(&lame_flags,
                                     frame->samples.s_16,
                                     &(frame->samples.s_16[offset]),
                                     frame->valid_samples,
                                     mp3_buffer, mp3_buffer_size);

  output.write(mp3_buffer, num_bytes);
  }

void GVOutputAudioLame::cleanup()
  {
  int num_bytes
    = lame_encode_finish(&lame_flags, mp3_buffer, mp3_buffer_size);
  output.write(mp3_buffer, num_bytes);

  streampos oldpos, newpos;
  
  // We must copy the whole file now

  ifstream input;
  char * buffer;

  int bytes_read;
  
  if(tmp_filename)
    {
    output.close();
    // Write Id3 tag to the temporary file

    cerr << "Adding Xing VBR tag\n";
    
    // Calculate relative quality of VBR stream
    // 0=best, 100=worst
      
    int nQuality=lame_flags.VBR_q*100/9;
    // Write Xing header again
    PutVbrTag(tmp_filename, nQuality, 1 - lame_flags.version);

    // Now copy the whole file to it's final destination
    
    input.open(tmp_filename);

    oldpos = input.tellg();
    
    output.open(filename);
    
    buffer = new char[4096];
    
    // Write ID3V2 tags if requested
    
    if(write_id3v2)
      gv_write_id3v2_tag(id3_tag, output);

    oldpos = input.tellg();
    
    while(!input.eof())
      {
      input.read(buffer, 4096);
      newpos = input.tellg();
      bytes_read = newpos - oldpos;
      output.write(buffer, bytes_read);
      oldpos = newpos;
      }
    delete buffer;
    input.close();
    remove(tmp_filename);
    }
  else if(lame_flags.VBR)
    {
    output.close();
    // Write Id3 tag to the temporary file
    
    cerr << "Adding Xing VBR tag\n";
    
    // Calculate relative quality of VBR stream
    // 0=best, 100=worst
    
    int nQuality=lame_flags.VBR_q*100/9;
    // Write Xing header again
    PutVbrTag(filename, nQuality, 1 - lame_flags.version);
    }
    
  if(write_id3v1)
    gv_write_id3v1_tag(id3_tag, output);
  
  if(id3_tag)
    {
    gv_delete_id3_tag(id3_tag);
    id3_tag = (ID3_Tag*)0;
    }

  if(tmp_filename)
    {
    delete tmp_filename;
    tmp_filename = (char*)0;
    }
  
  output.close();
  }

void GVOutputAudioLame::configure(void)
  {
  if(!config_window)
    config_window = new ConfigWindow(this);
  config_window->run();
  }

void GVOutputAudioLame::build_id3_tag()
  {
  GVTrackInfo tag_info;

  const GVTrackInfo * input_info = input->get_track_info();

  // No input info

  const char * c_tmp;
  int i_tmp;
  
  if(!input_info)
    id3_tag = gv_create_id3_tag(&default_tag_info);
  else
    {
    c_tmp = default_tag_info.get_artist(input_info, build_id3_flags);
    if(c_tmp)
      tag_info.set_artist(c_tmp);

    c_tmp = default_tag_info.get_title(input_info, build_id3_flags);
    if(c_tmp)
      tag_info.set_title(c_tmp);

    c_tmp = default_tag_info.get_album(input_info, build_id3_flags);
    if(c_tmp)
      tag_info.set_album(c_tmp);

    c_tmp = default_tag_info.get_comment(input_info, build_id3_flags);
    if(c_tmp)
      tag_info.set_comment(c_tmp);

    c_tmp = default_tag_info.get_genre(input_info, build_id3_flags);
    if(c_tmp)
      tag_info.set_genre(c_tmp);

    i_tmp = default_tag_info.get_track_number(input_info, build_id3_flags);
    if(i_tmp > 0)
      tag_info.track_number = i_tmp;

    i_tmp = default_tag_info.get_year(input_info, build_id3_flags);
    if(i_tmp > 0)
      tag_info.year = i_tmp;
    
    id3_tag = gv_create_id3_tag(&tag_info);
    }
  }

static void __vbr_toggle_callback(GtkWidget * w, gpointer data)
  {
  ((GVOutputAudioLame::ConfigWindow*)data)->vbr_toggle_callback();
  }

// Configuration stuff

GVOutputAudioLame::ConfigWindow::ConfigWindow(GVOutputAudioLame * _plugin) :
  GVConfigWindow(_plugin->attributes->real_name),
  bitrate_menu("Bitrate (kb/s)"),
  quality("Quality", 0.0, 9.0, 0),
  vbr_min_bitrate_menu("Minimum Bitrate (kb/s)"),
  vbr_max_bitrate_menu("Maximum Bitrate (kb/s)"),
  stereo_mode_menu("Stereo Mode"),
  vbr_quality("VBR Quality", 0, 9, 0),
  default_title("Title"),
  default_artist("Artist"),
  default_album("Album"),
  default_comment("Comment"),
  default_year("Year", 0.0, 9999.0, 0),
  default_track("Track number", 0.0, 255.0, 0),
  default_genre("Genre", lame_genres, sizeof(lame_genres)/sizeof(char*))
  {
  GtkWidget * table;

  notebook = gtk_notebook_new();

  table = gvgtk_table_new(8, 2, 0);

  bitrate_menu.create_menu(lame_bitrates,
                           sizeof(lame_bitrates)/sizeof(int));
  vbr_min_bitrate_menu.create_menu(lame_bitrates,
                                   sizeof(lame_bitrates)/sizeof(int));
  
  vbr_max_bitrate_menu.create_menu(lame_bitrates,
                                   sizeof(lame_bitrates)/sizeof(int));
  
  stereo_mode_menu.create_menu(lame_stereo_modes,
                               sizeof(lame_stereo_modes)/
                               sizeof(GVConfigMenuItemInt));
  
  original = gtk_check_button_new_with_label("Mark as Original");
  copyright = gtk_check_button_new_with_label("Copyright");
  write_id3v1 = gtk_check_button_new_with_label("Write ID3V1 tags");
  write_id3v2 = gtk_check_button_new_with_label("Write ID3V2 tags");

  use_vbr = gtk_check_button_new_with_label("Variable Bitrate (VBR)");

  gtk_signal_connect(GTK_OBJECT(use_vbr), "toggled",
                     GTK_SIGNAL_FUNC(__vbr_toggle_callback), (gpointer)this);
  
  
  force_default_title = gtk_check_button_new_with_label("Use this always");
  force_default_artist = gtk_check_button_new_with_label("Use this always");
  force_default_album = gtk_check_button_new_with_label("Use this always");
  force_default_comment = gtk_check_button_new_with_label("Use this always");
  force_default_year = gtk_check_button_new_with_label("Use this always");
  force_default_track = gtk_check_button_new_with_label("Use this always");
  force_default_genre = gtk_check_button_new_with_label("Use this always");

  gtk_widget_show(force_default_title);
  gtk_widget_show(force_default_artist);
  gtk_widget_show(force_default_album);
  gtk_widget_show(force_default_comment);
  gtk_widget_show(force_default_year);
  gtk_widget_show(force_default_track);
  gtk_widget_show(force_default_genre);
    
  gtk_widget_show(original);
  gtk_widget_show(copyright);
  gtk_widget_show(write_id3v1);
  gtk_widget_show(write_id3v2);
  gtk_widget_show(use_vbr);

  stereo_mode_menu.attach(table, 0, 0);
  
  bitrate_menu.attach(table, 1, 0);
  quality.attach(table, 2, 0);

  gtk_table_attach_defaults(GTK_TABLE(table), use_vbr, 0, 1, 3, 4);

  vbr_min_bitrate_menu.attach(table, 4, 0);
  vbr_max_bitrate_menu.attach(table, 5, 0);
  vbr_quality.attach(table, 6, 0);
  gtk_table_attach_defaults(GTK_TABLE(table), original, 0, 1, 7, 8);
  gtk_table_attach_defaults(GTK_TABLE(table), copyright, 1, 2, 7, 8);

  gtk_widget_show(table);

  GtkWidget * tab_label;
  tab_label = gtk_label_new("Encoding options");
  gtk_widget_show(tab_label);
  
  gtk_notebook_append_page(GTK_NOTEBOOK(notebook), table, tab_label);

  table = gvgtk_table_new(8, 2, 0);
  
  gtk_table_attach_defaults(GTK_TABLE(table), write_id3v1,
                            0, 1, 0, 1);
  gtk_table_attach_defaults(GTK_TABLE(table), write_id3v2,
                            1, 2, 0, 1);

  default_title.attach(table,  1, 0);
  gtk_table_attach_defaults(GTK_TABLE(table), force_default_title,
                            2, 3, 1, 2);

  default_artist.attach(table, 2, 0);
  gtk_table_attach_defaults(GTK_TABLE(table), force_default_artist,
                            2, 3, 2, 3);

  default_album.attach(table,  3, 0);
  gtk_table_attach_defaults(GTK_TABLE(table), force_default_album,
                            2, 3, 3, 4);

  default_comment.attach(table, 4, 0);
  gtk_table_attach_defaults(GTK_TABLE(table), force_default_comment,
                            2, 3, 4, 5);

  default_year.attach(table,    5, 0);
  gtk_table_attach_defaults(GTK_TABLE(table), force_default_year,
                            2, 3, 5, 6);

  default_track.attach(table,   6, 0);
  gtk_table_attach_defaults(GTK_TABLE(table), force_default_track,
                            2, 3, 6, 7);
  
  default_genre.attach(table,   7, 0);
  gtk_table_attach_defaults(GTK_TABLE(table), force_default_genre,
                            2, 3, 7, 8);

  gtk_widget_show(table);

  tab_label = gtk_label_new("ID3 tags");
  gtk_widget_show(tab_label);

  gtk_notebook_append_page(GTK_NOTEBOOK(notebook), table, tab_label);

  gtk_widget_show(notebook);
  gtk_box_pack_start_defaults(GTK_BOX(mainbox), notebook);

  plugin = _plugin;
  }

void GVOutputAudioLame::ConfigWindow::get_values()
  {
  bitrate_menu.set(plugin->lame_flags.brate);
  vbr_min_bitrate_menu.set(plugin->lame_flags.VBR_min_bitrate_kbps);
  vbr_max_bitrate_menu.set(plugin->lame_flags.VBR_max_bitrate_kbps);

  vbr_quality.set_value(plugin->lame_flags.VBR_q);
  quality.set_value(plugin->lame_flags.quality);
  
  if(plugin->lame_flags.mode_fixed)
    stereo_mode_menu.set(plugin->lame_flags.mode);
  else
    stereo_mode_menu.set(LAME_CHANNELS_AUTO);

  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(use_vbr),
                               plugin->lame_flags.VBR);

  
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(original),
                               plugin->lame_flags.original);
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(copyright),
                               plugin->lame_flags.copyright);

  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(write_id3v1),
                               plugin->write_id3v1);
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(write_id3v2),
                               plugin->write_id3v2);

  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(force_default_title),
    !!(plugin->build_id3_flags & GV_TRACK_INFO_FORCE_TITLE));
  
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(force_default_artist),
    !!(plugin->build_id3_flags & GV_TRACK_INFO_FORCE_ARTIST));

  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(force_default_album),
    !!(plugin->build_id3_flags & GV_TRACK_INFO_FORCE_ALBUM));

  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(force_default_comment),
    !!(plugin->build_id3_flags & GV_TRACK_INFO_FORCE_COMMENT));

  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(force_default_year),
    !!(plugin->build_id3_flags & GV_TRACK_INFO_FORCE_YEAR));

  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(force_default_track),
    !!(plugin->build_id3_flags & GV_TRACK_INFO_FORCE_TRACK));

  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(force_default_genre),
    !!(plugin->build_id3_flags & GV_TRACK_INFO_FORCE_GENRE));

  default_artist.set_string(plugin->default_tag_info.artist);
  default_album.set_string(plugin->default_tag_info.album);
  default_title.set_string(plugin->default_tag_info.title);
  default_comment.set_string(plugin->default_tag_info.comment);

  default_year.set_value(plugin->default_tag_info.year);
  default_track.set_value(plugin->default_tag_info.track_number);
  default_genre.set_string(plugin->default_tag_info.genre);
  vbr_toggle_callback();
  }

void GVOutputAudioLame::ConfigWindow::apply_values()
  {
  plugin->lame_flags.brate = bitrate_menu.get_int();

  plugin->lame_flags.VBR_min_bitrate_kbps = vbr_min_bitrate_menu.get_int();
  plugin->lame_flags.VBR_max_bitrate_kbps = vbr_max_bitrate_menu.get_int();

  plugin->lame_flags.VBR_q = vbr_quality.get_value_int();
  plugin->lame_flags.quality = quality.get_value_int();

  plugin->lame_flags.VBR
    = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(use_vbr));

  // We always write VBR tags
  
  if(plugin->lame_flags.VBR)
    plugin->lame_flags.bWriteVbrTag = 1;
  else
    plugin->lame_flags.bWriteVbrTag = 0;
  
  int i_tmp = stereo_mode_menu.get_int();
  if(i_tmp == LAME_CHANNELS_AUTO)
    plugin->lame_flags.mode_fixed = 0;
  else
    {
    plugin->lame_flags.mode = i_tmp;
    plugin->lame_flags.mode_fixed = 1; 
    }

  plugin->lame_flags.original
    = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(original));
  plugin->lame_flags.copyright
    = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(copyright));
  plugin->write_id3v1
    = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(write_id3v1));
  plugin->write_id3v2
    = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(write_id3v2));

  plugin->build_id3_flags = 0;

  if(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(force_default_title)))
    plugin->build_id3_flags |= GV_TRACK_INFO_FORCE_TITLE;

  if(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(force_default_artist)))
    plugin->build_id3_flags |= GV_TRACK_INFO_FORCE_ARTIST;

  if(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(force_default_album)))
    plugin->build_id3_flags |= GV_TRACK_INFO_FORCE_ALBUM;

  if(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(force_default_comment)))
    plugin->build_id3_flags |= GV_TRACK_INFO_FORCE_COMMENT;

  if(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(force_default_year)))
    plugin->build_id3_flags |= GV_TRACK_INFO_FORCE_YEAR;

  if(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(force_default_track)))
    plugin->build_id3_flags |= GV_TRACK_INFO_FORCE_TRACK;

  if(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(force_default_genre)))
    plugin->build_id3_flags |= GV_TRACK_INFO_FORCE_GENRE;

  plugin->default_tag_info.set_artist(default_artist.get_string());
  plugin->default_tag_info.set_album(default_album.get_string());
  plugin->default_tag_info.set_title(default_title.get_string());
  plugin->default_tag_info.set_genre(default_genre.get_string());

  plugin->default_tag_info.year
    = (int)(default_year.get_value()+0.5);
  plugin->default_tag_info.track_number
    = (int)(default_track.get_value()+0.5);
  }

GVOutputAudioLame::ConfigWindow::~ConfigWindow()
  {
  //  gtk_widget_destroy(table);
  }

void GVOutputAudioLame::ConfigWindow::vbr_toggle_callback()
  {
  if(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(use_vbr)))
    {
    vbr_min_bitrate_menu.set_sensitive(1);
    vbr_max_bitrate_menu.set_sensitive(1);
    vbr_quality.set_sensitive(1);
    }
  else
    {
    vbr_min_bitrate_menu.set_sensitive(0);
    vbr_max_bitrate_menu.set_sensitive(0);
    vbr_quality.set_sensitive(0);
    }
  }


#define PLUGIN_NAME GVOutputAudioLame
#include "interface.cc"
