/*****************************************************************

  oa_lame.h

  Copyright (c) 2001 by Burkhard Plaum - plaum@ipf.uni-stuttgart.de

  http://gmerlin.sourceforge.net

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111, USA.

*****************************************************************/

// Lame Mp3 encoder

#ifdef __GNUG__
#pragma interface
#endif

// GV Interface for the lame mp3 encoder

#include <fstream.h>

class ID3_Tag;

class GVOutputAudioLame : public GVOutputAudio
  {
  int write_id3v1;
  int write_id3v2;

  int build_id3_flags;
  
  ofstream output;
  
  char * mp3_buffer;
  int mp3_buffer_size;
  lame_global_flags lame_flags;

  char * tmp_filename;
  
  void get_config();
  void put_config();
    
  void put_frame(GVAudioFrame * frame);

public:
  
  class ConfigWindow: public GVConfigWindow
    {
    GtkWidget * notebook;

    // Encoding options
    
    GtkWidget * original;
    GtkWidget * copyright;

    GVOutputAudioLame * plugin;
    
    GVConfigWidgetOptionmenu bitrate_menu;

    GVConfigWidgetSlider quality;
    
    GtkWidget * use_vbr; // Variable bitrate
    GVConfigWidgetOptionmenu vbr_min_bitrate_menu;
    GVConfigWidgetOptionmenu vbr_max_bitrate_menu;

    GVConfigWidgetOptionmenu stereo_mode_menu;

    GVConfigWidgetSlider vbr_quality;
    
    // Tagging options

    GtkWidget * write_id3v1;
    GtkWidget * write_id3v2;

    GVConfigWidgetString default_title;
    GVConfigWidgetString default_artist;
    GVConfigWidgetString default_album;
    GVConfigWidgetString default_comment;
    GVConfigWidgetSpinbutton default_year;
    GVConfigWidgetSpinbutton default_track;
    GVConfigWidgetStringmenu default_genre;
    
    GtkWidget * force_default_title;
    GtkWidget * force_default_artist;
    GtkWidget * force_default_album;
    GtkWidget * force_default_comment;
    GtkWidget * force_default_year;
    GtkWidget * force_default_track;
    GtkWidget * force_default_genre;
    
  public:

    void vbr_toggle_callback();
    
    ConfigWindow(GVOutputAudioLame * _plugin);
    virtual ~ConfigWindow();
    
    void get_values();
    void apply_values();
    
    };

private:
  
  friend class ConfigWindow;

  ConfigWindow * config_window;

  ID3_Tag * id3_tag;

  GVTrackInfo default_tag_info;
  
  void build_id3_tag();
 
public:

  void configure();

  void init();

  void cleanup();
  
  GVOutputAudioLame();
  virtual ~GVOutputAudioLame();
  
  };
