/*****************************************************************

  ov_x11.h

  Copyright (c) 2001 by Burkhard Plaum - plaum@ipf.uni-stuttgart.de

  http://gmerlin.sourceforge.net

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111, USA.

*****************************************************************/

// X11 Video driver

#include <gtk/gtk.h>
#include <X11/Xlib.h>
#include <X11/extensions/XShm.h>

#include "../xvwrapper.h"

// #include <X11/extensions/Xvlib.h>

#include <gdk/gdkx.h>
#include <gdk/gdkrgb.h>

#include "../gv.h"

#include "../gvgtk.h"


// Supported output modes

#define GV_X11_VIDEO_MODE_GDKRGB 1 // Slowest, but works with remote displays
#define GV_X11_VIDEO_MODE_XSHM   2 // Better, only for local displays
#define GV_X11_VIDEO_MODE_XV_SHM 3 // Best, not supported by every Xfree86 driver

class GVOutputVideoX11 : public GVOutputVideo
  {
  int render_depth;
  int video_mode; // One of the defines above

  int cleaned_up; // Plugin was already cleaned up

  int dpms;
  
  //  int stopped;
  
  GVVideoWindow * win;
  
  // Xv stuff
  
  XvImage * xv_image;
  int xv_format;
  
  //  XvAdaptorInfo *xv_adaptor_info;

  //  XShmSegmentInfo xv_shminfo;
  int xv_port;

  XvImage * create_xv_image(int num, int total);
  void destroy_xv_image(XvImage * image, int num, int total);

  Window last_window;
  
  // XShm stuff

  XImage *shm_image;
  XShmSegmentInfo * shminfo;
  
  XImage * create_xshm_image(int num, int total);
  void destroy_xshm_image(XImage * image, int num, int total);
    
  // Internal stuff
  
  int check_xv_support(GVColorspace requested_colorspace);
  int check_shm_support();

  void init_xv();
  void init_xshm();
  void init_gdkrgb();

  void alloc_frames_xshm(GVVideoFrame * frame, int num);
  void alloc_frames_xv(GVVideoFrame * frame, int num);

  void free_frames_xshm(GVVideoFrame * frame, int num);
  void free_frames_xv(GVVideoFrame * frame, int num);
  
  void put_frame_xv(GVVideoFrame * frame);
  void put_frame_xshm(GVVideoFrame * frame);
  void put_frame_gdkrgb(GVVideoFrame * frame);
  
  int window_width, window_height, draw_x, draw_y;
  int draw_width, draw_height;

  // Get coordinates for drawing
  
  void get_drawing_coords();

  class ConfigWindow : public GVConfigWindow
    {
    GVOutputVideoX11 * plugin;

    GtkWidget * force_aspect_ratio;
    GtkWidget * check_xv;
    GVConfigWidgetOptionmenu aspect_ratio_menu;

    GtkWidget * use_xv;
    GtkWidget * force_xv;
    GtkWidget * use_xshm;
    
    GtkWidget * table;
  public:

    ConfigWindow(GVOutputVideoX11 * _plugin);
    virtual ~ConfigWindow();

    void get_values();
    void apply_values();
    };

  friend class ConfigWindow;
  
  void get_config();
  void put_config();
  
  ConfigWindow * config_window;

  int force_aspect_ratio;
  double aspect_ratio;

  int check_xv; // Check for XVideo extension
  int force_xv; // Force XVideo extension for RGB Colormodels also
  
public:

  void alloc_frames(GVVideoFrame * frames, int num_frames);
  void free_frames(GVVideoFrame * frames, int num_frames);

  void print_info(std::ostream & output);
    
  int can_alloc_video();
  
  GVOutputVideoX11();
  ~GVOutputVideoX11();
  
  void init();

  void put_frame(GVVideoFrame * frame);

  void cleanup();
  void configure();

  };
