/*****************************************************************

  ov_x11_xv.cc

  Copyright (c) 2001 by Burkhard Plaum - plaum@ipf.uni-stuttgart.de

  http://gmerlin.sourceforge.net

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111, USA.

*****************************************************************/

// X11 video driver (XVideo subsystem)

#include <iostream>

using std::ostream;
using std::cerr;

#include <sys/ipc.h>
#include <sys/shm.h>
#include <string.h>

// #include <stdio.h>

#include "ov_x11.h"


// We need one of these formats

#define XV_ID_YUV12_PLANAR 0x32315659 
#define XV_ID_I420_PLANAR  0x30323449
#define XV_ID_YUY2_PACKED  0x32595559

// Xv related routines of the X11 video output plugin

static int x11_error = 0;

static int __dummy(Display* , XErrorEvent*)
  {
  x11_error = 1;
  //  cerr << "Scheisse!!!\n";
  return true;
  }

void GVOutputVideoX11::alloc_frames_xv(GVVideoFrame * frame, int num)
  {
  shminfo = new XShmSegmentInfo;
  memset(shminfo, 0, sizeof(XShmSegmentInfo));

  XvImage ** images = new XvImage*[num];

  unsigned char ** image_data = new unsigned char*[num]; 
  
  images[0] = XvShmCreateImage(win->x_display, xv_port, 
                               xv_format,
                               (char*)0, video_format.width,
                               video_format.height, shminfo);

//   images[0] = gv_XvShmCreateImage(win->x_display, xv_port, 
//                                   xv_format,
//                                   (char*)0, video_format.width,
//                                   video_format.height, shminfo);

  
  shminfo->shmid = shmget(IPC_PRIVATE, num*video_format.buffer_size, 
                          IPC_CREAT | 0777);

  if(shminfo->shmid < 0)
    {
    g_log(PACKAGE, G_LOG_LEVEL_CRITICAL, "shmget failed");
    return;
    }

  image_data[0] = (unsigned char *)shmat(shminfo->shmid, NULL, 0);

  images[0]->data = shminfo->shmaddr = (char*)image_data[0];

  shminfo->readOnly = 0;
  
  for(int i = 1; i < num; i++)
    {
    image_data[i] = image_data[0] + video_format.buffer_size * i;
//     images[i] = gv_XvShmCreateImage(win->x_display, 
//                                  xv_port, 
//                                  xv_format,
//                                  (char*)image_data[i], 
//                                  video_format.width,
//                                  video_format.height,
//                                  shminfo);
    images[i] = XvShmCreateImage(win->x_display, 
                                 xv_port, 
                                 xv_format,
                                 (char*)image_data[i], 
                                 video_format.width,
                                 video_format.height,
                                 shminfo);

    images[i]->data = (char*)image_data[i];
    }

  // Attach the shared memory

  if(!XShmAttach(win->x_display, shminfo))
    {
    g_log(PACKAGE, G_LOG_LEVEL_CRITICAL, "XShmAttach failed");
    return;
    }

  shmctl(shminfo->shmid, IPC_RMID, 0);
  
  // Set the pixels and pixels_rows
  
  for(int i = 0; i < num; i++)
    {
    frame[i].pixels = (guchar*)(images[i]->data);

    switch(xv_format)
      {
      case XV_ID_YUV12_PLANAR:
        //        frame[i].pixels_rows = new guchar*[3];

        frame[i].y = (guchar*)(images[i]->data +
                               images[i]->offsets[0]);

        frame[i].u = (guchar*)(images[i]->data +
                               images[i]->offsets[2]);

        frame[i].v = (guchar*)(images[i]->data +
                               images[i]->offsets[1]);
        
        frame[i].pixels_rows = new guchar*[3];
        frame[i].pixels_rows[0] = frame[i].y;
        frame[i].pixels_rows[1] = frame[i].u;
        frame[i].pixels_rows[2] = frame[i].v;
        
        frame[i].y_rows = new guchar*[video_format.height];
        frame[i].u_rows = new guchar*[video_format.height/2];
        frame[i].v_rows = new guchar*[video_format.height/2];
        
        for(int j = 0; j < video_format.height; j++)
          frame[i].y_rows[j] = &(frame[i].y[video_format.width*j]);
        
        for(int j = 0; j < (video_format.height)/2; j++)
          {
          frame[i].u_rows[j] = &(frame[i].u[video_format.width*j/2]);
          frame[i].v_rows[j] = &(frame[i].v[video_format.width*j/2]);
          }

        break;
      case XV_ID_I420_PLANAR:
        frame[i].y = (guchar*)(images[i]->data +
                               images[i]->offsets[0]);
        frame[i].u = (guchar*)(images[i]->data +
                               images[i]->offsets[1]);
        frame[i].v = (guchar*)(images[i]->data +
                               images[i]->offsets[2]);

        frame[i].pixels_rows = new guchar*[3];
        frame[i].pixels_rows[0] = frame[i].y;
        frame[i].pixels_rows[1] = frame[i].u;
        frame[i].pixels_rows[2] = frame[i].v;
        
        frame[i].y_rows = new guchar*[video_format.height];
        frame[i].u_rows = new guchar*[video_format.height/2];
        frame[i].v_rows = new guchar*[video_format.height/2];
        
        for(int j = 0; j < video_format.height; j++)
          frame[i].y_rows[j] = &(frame[i].y[video_format.width*j]);
        
        for(int j = 0; j < (video_format.height)/2; j++)
          {
          frame[i].u_rows[j] = &(frame[i].u[video_format.width*j/2]);
          frame[i].v_rows[j] = &(frame[i].v[video_format.width*j/2]);
          }
        break;
      case XV_ID_YUY2_PACKED:
        frame[i].pixels_rows = new guchar*[video_format.height];
        for(int j = 0; j < video_format.height; j++)
          frame[i].pixels_rows[j] =
            (guchar*)(&(images[i]->data[j*video_format.width*2]));
        break;
      }
    frame[i].data = (gpointer)(images[i]);
    }
  delete images;
  delete image_data;
  }


void GVOutputVideoX11::free_frames_xv(GVVideoFrame * frame, int num)
  {
  XvImage * xv_image;
  for(int i = 0; i < num; i++)
    {
    xv_image = (XvImage*)(frame[i].data);
    XFree(xv_image);
    }
  
  XShmDetach(win->x_display, shminfo);

  //  XvUngrabPort(win->display, xv_portid, CurrentTime);
  
  shmdt(shminfo->shmaddr);
  shmctl(shminfo->shmid, IPC_RMID, 0);

  for(int i = 0; i < num; i++)
    {
    delete frame[i].pixels_rows;
    }
  
  }

int GVOutputVideoX11::check_xv_support(GVColorspace requested_colorspace)
  {
  unsigned int          p_version=0;
  unsigned int          p_release=0;
  unsigned int          p_request_base=0;
  unsigned int          p_event_base=0;
  unsigned int          p_error_base=0;

  unsigned int          p_num_adaptors=0;

  xv_port = -1;
  
  XvAdaptorInfo         *ai;
  XvEncodingInfo        *ei;
  //  XvAttribute           *at;
  XvImageFormatValues   *fo;
  
  // Check if the Xv extension is present
  
//   if(gv_XvQueryExtension(win->x_display, &p_version, &p_release,
//                          &p_request_base, &p_event_base,
//                          &p_error_base) != Success)
  if(XvQueryExtension(win->x_display, &p_version, &p_release,
                      &p_request_base, &p_event_base,
                      &p_error_base) != Success)
    return 0;

  // Query the adaptors
  
//   if(gv_XvQueryAdaptors(win->x_display,
//                         DefaultRootWindow(win->x_display),
//                         &p_num_adaptors, &ai) != Success)
  if(XvQueryAdaptors(win->x_display,
                     DefaultRootWindow(win->x_display),
                     &p_num_adaptors, &ai) != Success)
    return 0;
  else if(!p_num_adaptors)
    return 0;     // No adaptors found
  
  //  cerr << p_num_adaptors << " Adaptors found\n";
  
  unsigned int i;

  int supported = 0;
  
  for (i = 0; i < p_num_adaptors; i++)
    {
    // Adaptor doesn't support XvImages     
    if(!(ai[i].type & XvImageMask))
      continue;
        
//       cerr << " name:        " << ai[i].name << "\n"
//            << " type:        " 
//            << ((ai[i].type & XvInputMask)        ? "input | "    : "")
//            << ((ai[i].type & XvOutputMask)       ? "output | "   : "")
//            << ((ai[i].type & XvVideoMask)        ? "video | "    : "")
//            << ((ai[i].type & XvStillMask)        ? "still | "    : "")
//            << ((ai[i].type & XvImageMask)        ? "image | "    : "") << "\n"
//            << " ports:       " << ai[i].num_ports << "\n"
//            << " first port:  " << ai[i].base_id << "\n";
    
      //    xv_port = ai[i].base_id;
    
    //    printf("adaptor %d ; format list:\n", i);

    //    for (j = 0; j < ai[i].num_formats; j++)
    //      {
    //      printf(" depth=%d, visual=%ld\n",
    //             ai[i].formats[j].depth,
    //             ai[i].formats[j].visual_id);
    //      }

    unsigned int p;
    unsigned int encodings;
    //    int attributes;
    int formats;
    
    for (p = ai[i].base_id; p < ai[i].base_id+ai[i].num_ports; p++)
      {
//       cerr << " encoding list for port " << p << "\n";
      //      if(gv_XvQueryEncodings(win->x_display, p, &encodings, &ei) != Success)
      if(XvQueryEncodings(win->x_display, p, &encodings, &ei) != Success)
        {
        //printf("XvQueryEncodings failed.\n");
        continue;
        }
//       for (j = 0; j < encodings; j++)
//         {
//         cerr << "  id=" << ei[j].encoding_id
//              << ", name=" << ei[j].name
//              << ", size=" << ei[j].width << "x" << ei[j].height
//              << ", numerator=" << ei[j].rate.numerator
//              << ", denominator=" << ei[j].rate.denominator << "\n";
//         }
      //      gv_XvFreeEncodingInfo(ei);
      XvFreeEncodingInfo(ei);

//       cerr << " attribute list for port " << p << "\n";
//       at = XvQueryPortAttributes(win->x_display, p, &attributes);
//       for (k = 0; k < attributes; k++)
//         {
//         cerr << "  name:       " << at[k].name << "\n"
//              << "  flags:      "
//              << ((at[k].flags & XvGettable) ? " get" : "")
//              << ((at[k].flags & XvSettable) ? " set" : "") << "\n"
//              << "  min_value:  " << at[k].min_value << "\n"
//              << "  max_value:  " << at[k].max_value << "\n";
//         }
//       if (at)
//         XFree(at);
      
      //      cerr << "Image format list for port " << p << "\n";
      //      fo = gv_XvListImageFormats(win->x_display, p, &formats);
      fo = XvListImageFormats(win->x_display, p, &formats);
      for (int k = 0; k < formats; k++)
        {
        char fourcc[5];
        ((int*)fourcc)[0] = fo[k].id;
        fourcc[4] = '\0';
        
        //        cerr << "Format: " << fourcc << "\n";

//         printf("  0x%x (%4.4s) %s\n",
//                fo[k].id,
//                (char *)&fo[k].id,
//                (fo[k].format == XvPacked) ? "packed" : "planar");
//         printf("y_sample_bits: %d\n", fo[k].y_sample_bits);
//         printf("u_sample_bits: %d\n", fo[k].u_sample_bits);
//         printf("v_sample_bits: %d\n", fo[k].v_sample_bits);
//         printf("horz_y_period: %d\n", fo[k].horz_y_period);
//         printf("horz_u_period: %d\n", fo[k].horz_u_period);
//         printf("horz_v_period: %d\n", fo[k].horz_v_period);
        
//         printf("vert_y_period: %d\n", fo[k].vert_y_period);
//         printf("vert_u_period: %d\n", fo[k].vert_u_period);
//         printf("vert_v_period: %d\n", fo[k].vert_v_period);
//         printf("component_order: %s\n", fo[k].component_order); /* eg. UYVY */
//         if(fo[k].scanline_order == XvTopToBottom)
//           printf("Top To Bottom\n");
//         else
//           printf("Bottom Tp Top\n");
        
        switch(requested_colorspace)
          {
          case GVColorspaceYUV_420_P:
            switch(fo[k].id)
              {
              case XV_ID_YUV12_PLANAR:
              case XV_ID_I420_PLANAR:
                supported = 1;
                break;
              default:
                  break;
              }
            break;
          case GVColorspaceYUY2:
            switch(fo[k].id)
              {
              case XV_ID_YUY2_PACKED:
                supported = 1;
                break;
              default:
                break;
              }
            break;
          }

        if(supported)
          {
          xv_port = p;
          xv_format = fo[k].id;
          XFree(fo);
          //          gv_XvFreeAdaptorInfo(ai);
          XvFreeAdaptorInfo(ai);
          return 1;
          }
        
        }
      
      if(fo)
        XFree(fo);
      }
    //    printf("\n");
    }

  if (p_num_adaptors > 0)
    //    gv_XvFreeAdaptorInfo(ai);
    XvFreeAdaptorInfo(ai);
  
  return 0;
  }

void GVOutputVideoX11::init_xv()
  {
  //  int id = XV_ID_YUV12_PLANAR; // ??????
  last_window = 0;
  //  if(Success == gv_XvGrabPort(win->x_display, xv_port, CurrentTime))
  if(Success == XvGrabPort(win->x_display, xv_port, CurrentTime))
    ;
  }

XvImage * GVOutputVideoX11::create_xv_image(int num, int total)
  {
  //  win->lock();
  XvImage * ret;

  if(!num)
    {
    shminfo = new XShmSegmentInfo;
    memset(shminfo, 0, sizeof(XShmSegmentInfo));
    }
  if(!num)
    {
//     ret = gv_XvShmCreateImage(win->x_display, xv_port, 
//                            XV_ID_YUV12_PLANAR, (char*)0, video_format.width,
//                            video_format.height, shminfo);
    ret = XvShmCreateImage(win->x_display, xv_port, 
                           XV_ID_YUV12_PLANAR, (char*)0, video_format.width,
                           video_format.height, shminfo);
    }
  else
    {
    char * data = (char*)(shminfo->shmaddr) + num*video_format.buffer_size;
    //    cerr << "Data: " << (void*)data << "\n"
//     ret = gv_XvShmCreateImage(win->x_display, xv_port,
//                            XV_ID_YUV12_PLANAR, data, video_format.width,
//                            video_format.height, shminfo);
    ret = XvShmCreateImage(win->x_display, xv_port,
                           XV_ID_YUV12_PLANAR, data, video_format.width,
                           video_format.height, shminfo);
    //    ret->data = data;
    }

  if(!ret)
    {
    //    cerr << "gv_XvShmCreateImage failed\n";
    return (XvImage*)0;
    }
  if(!num)
    {
    // Create shared memory

    shminfo->shmid=shmget(IPC_PRIVATE,
                          total*ret->data_size,
                          IPC_CREAT|0777);
    
    if(shminfo->shmid<0)
      {
      //      cerr << "shmget failed\n";
      return (XvImage *)0;
      }
    shminfo->shmaddr = (char*)shmat(shminfo->shmid, (const void*)0, 0 );
//     if(shminfo->shmaddr == (char *) -1)
//       cerr << "shmat failed\n";
    ret->data = shminfo->shmaddr;
    shminfo->readOnly=False;
    }

  //  cerr << "Allocated shared Memory "
  //       << total*(video_format.height*ret->bytes_per_line+4) << " Bytes"
  //       << " Start Adress: " << (void*)(shminfo->shmaddr) << "\n";
  
  
  //  ret->data += num * (video_format.height * ret->bytes_per_line + 4); 
  
  //  cerr << "ret->data: " << (void*)ret->data << "\n";
  
  if(num == total - 1)
    {
    //    cerr << "Last allocated Frame\n";
    XSetErrorHandler(__dummy);
    if(!XShmAttach(win->x_display,shminfo))
      cerr << "XShmAttach failed\n";
    shmctl(shminfo->shmid, IPC_RMID, 0 ); /* nobody else needs it */
    XSync(win->x_display, False);
    XSetErrorHandler(NULL);
    //    XFlush(win->x_display);
    }
  //  win->unlock();
  return ret;
  }

void GVOutputVideoX11::destroy_xv_image(XvImage * image, int num, int total)

  {
  //  cerr << "Destroying Image " << num+1 << "/" << total << "\n";

  XFree(image);

  if(num == total -1)
    {
    XShmDetach(win->x_display, shminfo);
    //    gv_XvUngrabPort(win->x_display, xv_port, CurrentTime);
    XvUngrabPort(win->x_display, xv_port, CurrentTime);
    shmdt( shminfo->shmaddr );
    shmctl(shminfo->shmid, IPC_RMID, 0);
    
    delete shminfo;
    shminfo = (XShmSegmentInfo*)0;
    }
  }

void GVOutputVideoX11::put_frame_xv(GVVideoFrame * frame)
  {
  //  cerr << "Put Video Frame (Xv)\n";

  //  int x_ret;
  //  int y_ret;

  //   cerr << "width_ret: " << width_ret << "  height_ret: "
  //        << height_ret << "\n";

  if(!last_window)
    last_window = win->x_win;
  
  else if(last_window != win->x_win)
    {
    //    gv_XvStopVideo(win->x_display, xv_port, last_window);
    XvStopVideo(win->x_display, xv_port, last_window);
    last_window = win->x_win;
    }

  // The video window changed
  
//   gv_XvShmPutImage(win->x_display, xv_port, win->x_win,
//                    win->x_gc, (XvImage*)(frame->data),
//                    0, 0,
//                    ((XvImage*)(frame->data))->width,
//                    ((XvImage*)(frame->data))->height,
//                    draw_x, draw_y, draw_width, draw_height+1, True);
  XvShmPutImage(win->x_display, xv_port, win->x_win,
                   win->x_gc, (XvImage*)(frame->data),
                   0, 0,
                   ((XvImage*)(frame->data))->width,
                   ((XvImage*)(frame->data))->height,
                   draw_x, draw_y, draw_width, draw_height+1, True);
  //  XFlush(win->x_display);

  // Don't know exactly why we need the following line, but it makes a
  // smooth playback

  XSync(win->x_display, FALSE);
  }

