/*****************************************************************

  quicktime_common.cc

  Copyright (c) 2001 by Burkhard Plaum - plaum@ipf.uni-stuttgart.de

  http://gmerlin.sourceforge.net

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111, USA.

*****************************************************************/

// Convert GVColorspace to quicktime colorspace
// Used by input and output

#include "../gv.h"
#include "quicktime_common.h"

#include "quicktime4linux/colormodels.h"

int get_quicktime_colorspace(GVColorspace colorspace)
  {
  switch(colorspace)
    {
    case GVColorspaceRGB16:
      return BC_RGB565;     // 565
      break;
    case GVColorspaceBGR16:
      return BC_BGR565;     // 565
      break;
    case GVColorspaceRGB24:
      return BC_RGB888;     // 888
      break;
    case GVColorspaceBGR24:
      return BC_BGR888;
      break;
    case GVColorspaceBGR32:
      return BC_BGR8888;
      break;
    case GVColorspaceRGBA32:
      return BC_RGBA8888;
      break;
    case GVColorspaceYUV_420_P:
      return BC_YUV420P;
      break;

    case GVColorspaceYUV_422_P:
      return BC_YUV422P;
      break;

    // 4:2:2 packed YUV, Ordering |Y0|U0|Y1|V0| ....
    case GVColorspaceYUY2:
      return BC_YUV422;
      break;
    default:
      return -1; // Undefined
    }
  }
