/*****************************************************************

  remote.h

  Copyright (c) 2001 by Burkhard Plaum - plaum@ipf.uni-stuttgart.de

  http://gmerlin.sourceforge.net

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111, USA.

*****************************************************************/

// C-Routines for remote controlling

// WARNING: The remote interface isn't frozen yet. It will
// change in the future.

// It's not documented yet, but can in principal be used by gnome-applets
// for remote controlling gmerlin

// It has a pure C interface

// The following things have to be added:

// - Retrieval of the time from a running gmerlin process
// - Update for passing input plugins together with devices as arguments

#ifndef __gv_remote_h_
#define __gv_remote_h_

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

#define GV_REMOTE_COMMAND_ATOM "GMERLIN_COMNMAND"
#define GV_REMOTE_ARG_ATOM     "GMERLIN_ARG"

#define GV_REMOTE_STOP              "Stop"
#define GV_REMOTE_PLAY              "Play"
#define GV_REMOTE_NEXT              "Next"
#define GV_REMOTE_PREVIOUS          "Previous"
#define GV_REMOTE_PLAYFILE          "Playfile"
#define GV_REMOTE_ADDFILE           "Addfile"
#define GV_REMOTE_SET_INPUT         "SetInput"
#define GV_REMOTE_SET_AUDIO_OUTPUT  "SetAudioOutput"
#define GV_REMOTE_SET_VIDEO_OUTPUT  "SetVideoOutput"

#define GV_REMOTE_TEST_REQUEST "TEST"
#define GV_REMOTE_ANSWER       "OK"
#define GV_REMOTE_DONE         "DONE"

#define GV_REMOTE_DUMMY_ARG    "DUMMY"

  
typedef struct
  {
  GdkWindow * window;

  char * command;
  char * arg;

  GdkAtom command_atom;
  GdkAtom arg_atom;
  int receiver;   /* 1 if the remote is a receiver */ 
  } GVRemote;

/* Find an already running program and return a remote */
  
GVRemote * gv_remote_find();

  
void gv_remote_destroy(GVRemote * remote);

GVRemote * gv_remote_create(GdkWindow * window);
  
void gv_remote_play(GVRemote * remote);         // Start playing
void gv_remote_stop(GVRemote * remote);         // Stop playing

void gv_remote_next(GVRemote * remote);         // Next track
void gv_remote_previous(GVRemote * remote);     // Previous track
  
// Add a file to the playlist
void gv_remote_play_file(GVRemote * remote, const char * filename); 

void gv_remote_add_file(GVRemote * remote, const char * filename); 
  
// Load an input plugin
void gv_remote_set_input(GVRemote * remote, const char * plugin_name);        

// Load an video output plugin
void gv_remote_set_video_output(GVRemote * remote, const char * plugin_name);

// Load an audio output plugin
void gv_remote_set_audio_output(GVRemote * remote, const char * plugin_name);

int gv_remote_get_command(GVRemote * remote);    // Get the Command

void gv_remote_confirm(GVRemote * remote);       // Send confirmation string

// Send the done confirmation
  
void gv_remote_send_done(GVRemote * remote);

// Wait for the done confirmation

int gv_remote_get_done(GVRemote * remote);

  
  
#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif
