/*****************************************************************

  skin.cc

  Copyright (c) 2001 by Burkhard Plaum - plaum@ipf.uni-stuttgart.de

  http://gmerlin.sourceforge.net

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111, USA.

*****************************************************************/

// Skin

#include "config.h"
#include <gtk/gtk.h>

#include "button.h"

#include "gvconfig.h"

#include "skin.h"
#include <sstream>
using std::stringstream;

#include <string.h>

#include "gvtime.h"
#include "videoformat.h"
#include "videoconvert.h"

#include "gvwindow.h"
#include "videowindow.h"
#include "configwindow.h"
#include "fileselect.h"
#include "playlistwidget.h"


#include "engine.h"

#include "display.h"
#include "remote.h"

#include "slider.h"

#include "mainwin.h"


#include "xmanager.h"

GVButtonSkin::GVButtonSkin()
  {
  mask = (GdkBitmap*)0;
  normal_pixmap = (GdkPixmap*)0;
  highlight_pixmap = (GdkPixmap*)0;
  pressed_pixmap = (GdkPixmap*)0;
  x = 0;
  y = 0;
  }


GVWindowSkin::GVWindowSkin()
  {
  top_pixmap    = (GdkPixmap*)0;
  bottom_pixmap = (GdkPixmap*)0;
  left_pixmap   = (GdkPixmap*)0;
  right_pixmap  = (GdkPixmap*)0;
  ll_pixmap     = (GdkPixmap*)0;
  ul_pixmap     = (GdkPixmap*)0;
  lr_pixmap     = (GdkPixmap*)0;
  ur_pixmap     = (GdkPixmap*)0;
  title_pixmap  = (GdkPixmap*)0;

  ll_mask = (GdkBitmap*)0;
  ul_mask = (GdkBitmap*)0;
  lr_mask = (GdkBitmap*)0;
  ur_mask = (GdkBitmap*)0;
  }

void GVSkin::load(const char * filename)
  {
  if(skin_data)
    delete skin_data;

  skin_data = new GVConfig;
  skin_data->read(filename);

  if(directory)
    g_free(directory);
  
  directory = g_strdup(filename);

  char * pos = strrchr(directory, G_DIR_SEPARATOR);
  if(pos)
    *pos = '\0';
  }

void GVSkin::load_info()
  {
  author    = (const char*)0;
  comment1  = (const char*)0;
  comment2  = (const char*)0;
  comment3  = (const char*)0;
  comment4  = (const char*)0;
  
  skin_data->get_string("Info", "Author", &author);
  skin_data->get_string("Info", "Comment1", &comment1);
  skin_data->get_string("Info", "Comment2", &comment2);
  skin_data->get_string("Info", "Comment3", &comment3);
  skin_data->get_string("Info", "Comment4", &comment4);
  
  preview_pixmap = load_pixmap("Info", "Preview", (GdkBitmap**)0);
  }

void GVSkin::destroy_info()
  {
  if(preview_pixmap)
    {
    gdk_pixmap_unref(preview_pixmap);
    preview_pixmap = (GdkPixmap*)0;
    }
  }

void GVSkin::load_skin()
  {
  load_button("PlayButton", play_button);
  load_button("NextButton", next_button);
  load_button("PreviousButton", previous_button);
  load_button("StopButton", stop_button);
  load_button("PauseButton", pause_button);
  load_button("MainWindowMenuButton", main_window_menu_button);
  load_button("MainWindowCloseButton", main_window_close_button);
//   load_button("VideoWindowMenuButton", video_window_menu_button);
//   load_button("VideoWindowCloseButton", video_window_close_button);
//   load_button("PlaylistWindowMenuButton", playlist_window_menu_button);
//   load_button("PlaylistWindowCloseButton", playlist_window_close_button);

  seek_offset_x = 0;
  seek_offset_y = 0;
  display_offset_x = 0;
  display_offset_y = 0;
  
  skin_data->get_int("Slider", "OffsetX", seek_offset_x);
  skin_data->get_int("Slider", "OffsetY", seek_offset_y);

  skin_data->get_int("Display", "OffsetX", display_offset_x);
  skin_data->get_int("Display", "OffsetY", display_offset_y);

  seek_background = load_pixmap("Slider", "Background",
                                (GdkBitmap**)0);

  seek_slider = load_pixmap("Slider", "Slider", &seek_slider_mask);

  seek_slider_highlight = load_pixmap("Slider", "SliderHighlight",
                                      (GdkBitmap**)0);
  
  main_window = load_pixmap("MainWindow", "Background",
                            &main_window_mask);

  load_window("VideoWindow", video_window);
  load_window("PlaylistWindow", playlist_window);

  // Load the colors

  load_color("Display", "Background", display_color_background);
  load_color("Display", "Foreground", display_color_foreground);
  load_color("Display", "Error", display_color_error);
  
  load_color("PlaylistWindow", "CurrentForeground",
             playlist_color_current_foreground);
  load_color("PlaylistWindow", "NormalForeground",
             playlist_color_normal_foreground);
  load_color("PlaylistWindow", "Background",
             playlist_color_background);
  load_color("PlaylistWindow", "SelectedBackground", 
             playlist_color_selected_background);
    
  }

void GVSkin::destroy_skin()
  {
  destroy_button(play_button);
  destroy_button(next_button);
  destroy_button(previous_button);
  destroy_button(stop_button);
  destroy_button(pause_button);
  destroy_window(video_window);
  destroy_window(playlist_window);
  }

GVSkin::GVSkin()
  {
  skin_data = (GVConfig*)0;
  directory = (char*)0;
  }

void GVSkin::load_button(const char * name, GVButtonSkin & button_skin)
  {
  int i_tmp;

  //  cerr << "Loading Button " << name << "\n";
  
  i_tmp = 0;
  skin_data->get_int(name, "OffsetX", i_tmp);
  button_skin.x = i_tmp;

  i_tmp = 0;
  skin_data->get_int(name, "OffsetY", i_tmp);
  button_skin.y = i_tmp;

//   cerr << "Button position: "
//        << button_skin.x << " " <<  button_skin.y << "\n";
  
  button_skin.normal_pixmap = load_pixmap(name, "Normal", &(button_skin.mask));
    
  button_skin.highlight_pixmap = load_pixmap(name, "Highlight",
                                             (GdkBitmap**)0);
  
  button_skin.pressed_pixmap = load_pixmap(name, "Pressed", (GdkBitmap**)0);
  }

void GVSkin::destroy_button(GVButtonSkin & button_skin)
  {
  if(button_skin.normal_pixmap)
    {
    gdk_pixmap_unref(button_skin.normal_pixmap);
    button_skin.normal_pixmap = (GdkPixmap*)0;
    }
  if(button_skin.highlight_pixmap)
    {
    gdk_pixmap_unref(button_skin.highlight_pixmap);
    button_skin.highlight_pixmap = (GdkPixmap*)0;
    }
  if(button_skin.pressed_pixmap)
    {
    gdk_pixmap_unref(button_skin.pressed_pixmap);
    button_skin.pressed_pixmap = (GdkPixmap*)0;
    }
  if(button_skin.mask)
    {
    gdk_bitmap_unref(button_skin.mask);
    button_skin.mask = (GdkBitmap*)0;
    }
  }

void GVSkin::load_window(const char * name, GVWindowSkin & window_skin)
  {
  char * c_tmp;
  //  cerr << "Loading window skin for " << name << "\n";
  window_skin.left_pixmap = load_pixmap(name, "LeftPixmap", (GdkBitmap**)0);
  window_skin.right_pixmap = load_pixmap(name, "RightPixmap", (GdkBitmap**)0);
  window_skin.top_pixmap = load_pixmap(name, "TopPixmap", (GdkBitmap**)0);
  window_skin.bottom_pixmap = load_pixmap(name, "BottomPixmap",
                                          (GdkBitmap**)0);

  window_skin.ll_pixmap = load_pixmap(name, "LowerLeftPixmap",
                                      &(window_skin.ll_mask));
  window_skin.lr_pixmap = load_pixmap(name, "LowerRightPixmap",
                                      &(window_skin.lr_mask));
  window_skin.ul_pixmap = load_pixmap(name, "UpperLeftPixmap",
                                      &(window_skin.ul_mask));
  window_skin.ur_pixmap = load_pixmap(name, "UpperRightPixmap",
                                      &(window_skin.ur_mask));

  window_skin.title_pixmap = load_pixmap(name, "TitlePixmap", (GdkBitmap**)0);

  c_tmp = g_strdup_printf("%s%s", name, "CloseButton");
  load_button(c_tmp, window_skin.close_button);
  g_free(c_tmp);

  c_tmp = g_strdup_printf("%s%s", name, "MenuButton");
  load_button(c_tmp, window_skin.menu_button);
  g_free(c_tmp);
  }

void GVSkin::destroy_window(GVWindowSkin & window_skin)
  {
  if(window_skin.top_pixmap)
    gdk_pixmap_unref(window_skin.top_pixmap);

  if(window_skin.bottom_pixmap)
    gdk_pixmap_unref(window_skin.bottom_pixmap);

  if(window_skin.left_pixmap)
    gdk_pixmap_unref(window_skin.left_pixmap);

  if(window_skin.right_pixmap)
    gdk_pixmap_unref(window_skin.right_pixmap);

  if(window_skin.ll_pixmap)
    gdk_pixmap_unref(window_skin.ll_pixmap);

  if(window_skin.lr_pixmap)
    gdk_pixmap_unref(window_skin.lr_pixmap);

  if(window_skin.ul_pixmap)
    gdk_pixmap_unref(window_skin.ul_pixmap);

  if(window_skin.ur_pixmap)
    gdk_pixmap_unref(window_skin.ur_pixmap);

  if(window_skin.title_pixmap)
    gdk_pixmap_unref(window_skin.title_pixmap);

  if(window_skin.ll_mask)
    gdk_bitmap_unref(window_skin.ll_mask);
  if(window_skin.lr_mask)
    gdk_bitmap_unref(window_skin.lr_mask);
  if(window_skin.ul_mask)
    gdk_bitmap_unref(window_skin.ul_mask);
  if(window_skin.ur_mask)
    gdk_bitmap_unref(window_skin.ur_mask);

  destroy_button(window_skin.close_button);
  destroy_button(window_skin.menu_button);
  }

GdkPixmap * GVSkin::load_pixmap(const char * section,
                                const char * name, GdkBitmap ** mask)
  {
  GdkPixmap * ret;

  const char * c_tmp = (const char*)0;
  skin_data->get_string(section, name, &c_tmp);

  if(!c_tmp)
    return (GdkPixmap*)0;
  
  char * filename = g_strdup_printf("%s/%s", directory, c_tmp);
  //  cerr << "Loading image " << filename << "\n";
  ret = gv_x_manager->load_image(filename, mask);
  g_free(filename);
  return ret;
  }

void GVSkin::apply()
  {
  gv_main_win->apply_skin(*this);
  gv_video_window->apply_skin(&video_window);
  }

GVSkin::~GVSkin()
  {
  destroy_info();
  destroy_skin();
  }

void GVSkin::get_color(guchar * rgb, GdkColor * color)
  {
  color->red =   rgb[0] * (65535/255);
  color->green = rgb[1] * (65535/255);
  color->blue =  rgb[2] * (65535/255); 
  color->pixel = (gulong)(rgb[0]*65536 + rgb[1]*256 + rgb[2]);
  }

void GVSkin::load_color(const char * section, const char * name,
                        guchar * rgb)
  {
  skin_data->get_color(section, name, rgb[0], rgb[1], rgb[2]);  
  }
