/*****************************************************************

  skin.h

  Copyright (c) 2001 by Burkhard Plaum - plaum@ipf.uni-stuttgart.de

  http://gmerlin.sourceforge.net

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111, USA.

*****************************************************************/

// Class for a skin

class GVConfig;

// This one contains all information, a skinned button needs from the
// skin description

struct GVButtonSkin
  {
  GdkBitmap * mask;
  GdkPixmap * normal_pixmap;
  GdkPixmap * highlight_pixmap;
  GdkPixmap * pressed_pixmap;
  int x;
  int y;

  GVButtonSkin();
  };

struct GVWindowSkin
  {
  GVButtonSkin close_button;
  GVButtonSkin menu_button;
  
  GdkPixmap * top_pixmap;
  GdkPixmap * bottom_pixmap;
  GdkPixmap * left_pixmap;
  GdkPixmap * right_pixmap;
  GdkPixmap * ll_pixmap;
  GdkPixmap * ul_pixmap;
  GdkPixmap * lr_pixmap;
  GdkPixmap * ur_pixmap;

  GdkBitmap * ll_mask;
  GdkBitmap * ul_mask;
  GdkBitmap * lr_mask;
  GdkBitmap * ur_mask;

  GdkPixmap * title_pixmap;

  GVWindowSkin();
  };


class GVSkin
  {
public:

  // Stuff for the main window
  
  GVButtonSkin play_button;
  GVButtonSkin pause_button;
  GVButtonSkin next_button;
  GVButtonSkin previous_button;
  GVButtonSkin stop_button;

  GVButtonSkin main_window_menu_button;
  GVButtonSkin main_window_close_button;
    
  GdkPixmap * main_window;
  GdkBitmap * main_window_mask;
  
  GdkPixmap * seek_background;

  GdkPixmap * seek_slider;
  GdkPixmap * seek_slider_highlight;
  GdkBitmap * seek_slider_mask;

  int seek_offset_x;
  int seek_offset_y;
  int display_offset_x;
  int display_offset_y;

  // Video- and Playlist window
  
  GVWindowSkin video_window;
  GVWindowSkin playlist_window;
  
  // Info data (read by the skin browser)
  
  const char * author;
  const char * comment1;
  const char * comment2;
  const char * comment3;
  const char * comment4;
  GdkPixmap * preview_pixmap;

  // GVConfig class to read the file
  
  GVConfig * skin_data;

  guchar display_color_background[3];
  guchar display_color_foreground[3];
  guchar display_color_error[3];

  guchar playlist_color_background[3];
  guchar playlist_color_normal_foreground[3];
  guchar playlist_color_current_foreground[3];
  guchar playlist_color_selected_background[3];
 
  GVSkin();
  ~GVSkin();

  void load(const char * file);

  void load_info();
  void destroy_info();
  void load_skin();
  void destroy_skin();

  void apply();

  // This enables getting a color using one of the above defined
  // guchar arrays
  
  void get_color(guchar * rgb, GdkColor * color);
  
private:

  void load_color(const char * section, const char * name,
                  guchar * rgb);

  
  char * directory;
  
  // Some utility functions
  
  void load_button(const char * name, GVButtonSkin & button_skin);
  void destroy_button(GVButtonSkin & button_skin);

  void load_window(const char * name, GVWindowSkin & window_skin);
  void destroy_window(GVWindowSkin & window_skin);
  
  GdkPixmap * load_pixmap(const char *section,
                          const char * name,
                          GdkBitmap ** mask);
   
  };
