/*****************************************************************

  skinbrowser.cc

  Copyright (c) 2001 by Burkhard Plaum - plaum@ipf.uni-stuttgart.de

  http://gmerlin.sourceforge.net

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111, USA.

*****************************************************************/

// Skin browser

#ifdef __GNUG__
#pragma implementation "skinbrowser.h"
#endif

#include "config.h"
#include <gtk/gtk.h>

#include <iostream>
#include <sstream>
using std::stringstream;

#include <string.h>


#include "button.h"

#include "gvconfig.h"

#include "configwindow.h"

#include "skin.h"
#include "skinbrowser.h"

#include "gvtime.h"
#include "videoformat.h"
#include "videoconvert.h"
#include "xmanager.h"
#include "gtkutil.h"

#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <dirent.h>
#include <stdio.h>

using std::cerr;
using std::ends;


GVSkinBrowser * gv_skin_browser = (GVSkinBrowser*)0;

#define MIN_PREVIEW_WIDTH  400
#define MIN_PREVIEW_HEIGHT 200

static void __select_row_callback(GtkWidget * w, gint row, gint column,
                                  GdkEvent * event, gpointer data)
  {
  ((GVSkinBrowser*)data)->select_row_callback(row);
  }

static gboolean __expose_callback(GtkWidget * w, GdkEventExpose * event,
                                  gpointer data)
  {
  ((GVSkinBrowser*)data)->expose_callback();
  return TRUE;
  }



void GVSkinBrowser::select_row_callback(int row)
  {
  if((display_skin != current_skin) && (display_skin))
    delete display_skin;
  
  display_skin = new GVSkin;

  display_skin_file = (char*)gtk_clist_get_row_data(GTK_CLIST(skinlist), row);
  
  display_skin->load(display_skin_file);
  display_skin->load_info();
  
  //  gtk_label_set_text(GTK_LABEL(author), display_skin->author);
  
  stringstream line;
  //  line.freeze(0);
  line.seekp(0);

  line << "Author: " << display_skin->author << "\n\n";
  
  if(display_skin->comment1)
    {
    line << display_skin->comment1;
    if(display_skin->comment2)
      line << "\n" << display_skin->comment2;
    if(display_skin->comment3)
      line << "\n" << display_skin->comment3;
    if(display_skin->comment4)
      line << "\n" << display_skin->comment4;
    line << ends;
    }

  description->set_text(line.str().c_str());
  
  gdk_window_get_size(display_skin->preview_pixmap, &preview_width,
                      &preview_height);

  drawingarea_width = MIN_PREVIEW_WIDTH;
  drawingarea_height = MIN_PREVIEW_HEIGHT;
  
  if(preview_width > MIN_PREVIEW_WIDTH)
    drawingarea_width = preview_width;

  if(preview_height > MIN_PREVIEW_HEIGHT)
    drawingarea_height = preview_height;

  gtk_drawing_area_size(GTK_DRAWING_AREA(preview_drawingarea),
                        drawingarea_width, drawingarea_height);
  
    
  
  gdk_window_clear(preview_drawingarea->window);
  expose_callback();
  }

void GVSkinBrowser::expose_callback()
  {
  if(!display_skin || !(display_skin->preview_pixmap))
    return;

  gdk_draw_pixmap(preview_drawingarea->window,
                  gv_x_manager->get_gc(),
                  display_skin->preview_pixmap,
                  0,
                  0,
                  (drawingarea_width - preview_width)/2,
                  (drawingarea_height - preview_height)/2,
                  preview_width,
                  preview_height);
  }

GVSkinBrowser::GVSkinBrowser() :
  GVConfigWindow("Gmerlin Skin Browser")
  {
  current_skin = (GVSkin*)0;
  display_skin = (GVSkin*)0;

  // Create the objects
  
  description = new GVLabel();
  
  preview_drawingarea = gtk_drawing_area_new();

  skinlist = gtk_clist_new(1);
  gtk_clist_set_column_title(GTK_CLIST(skinlist),
                             0,
                             "Installed skins");

  // Set attributes

  drawingarea_width = MIN_PREVIEW_WIDTH;
  drawingarea_height = MIN_PREVIEW_HEIGHT;
  gtk_drawing_area_size(GTK_DRAWING_AREA(preview_drawingarea),
                        MIN_PREVIEW_WIDTH, MIN_PREVIEW_HEIGHT);

  gtk_widget_set_events(preview_drawingarea, GDK_EXPOSURE_MASK);
  
  // Set callbacks

  gtk_signal_connect(GTK_OBJECT(skinlist), "select_row",
                     GTK_SIGNAL_FUNC(__select_row_callback),
                     (gpointer)this); 

  gtk_signal_connect(GTK_OBJECT(preview_drawingarea), "expose_event",
                     GTK_SIGNAL_FUNC(__expose_callback),
                     (gpointer)this); 


  // Set the minimum height

  gtk_widget_set_usize(skinlist, skinlist->requisition.width, 100);
  
  // show the objects
  
  gtk_widget_show(preview_drawingarea);
  gtk_widget_show(skinlist);
    
  // Pack the objects

  GtkWidget * hbox = gtk_hbox_new(1, 5);

  GtkWidget * scrolledwindow = gtk_scrolled_window_new((GtkAdjustment*)0,
                                                       (GtkAdjustment*)0);
  gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrolledwindow),
                                 GTK_POLICY_NEVER,
                                 GTK_POLICY_ALWAYS);
  gtk_container_add(GTK_CONTAINER(scrolledwindow), skinlist);
  gtk_widget_show(scrolledwindow);
  
  gtk_box_pack_start_defaults(GTK_BOX(hbox), scrolledwindow);
  gtk_box_pack_start_defaults(GTK_BOX(hbox), description->widget);
  gtk_widget_show(hbox);
  gtk_box_pack_start_defaults(GTK_BOX(mainbox), hbox);
  gtk_box_pack_start_defaults(GTK_BOX(mainbox), preview_drawingarea);
  
  char * directory = g_strdup_printf("%s/%s", DATA_DIR, "skins");

  read_skins(directory);
  g_free(directory);

  const char * c_tmp;
  gv_config->get_string("_", "HomeDir", &c_tmp);
  
  directory = g_strdup_printf("%s%s", c_tmp, "skins");
  read_skins(directory);
  g_free(directory);

  c_tmp = DATA_DIR"/skins/Default/skin.def";

  gv_config->get_string("General", "Skin", &c_tmp);
  current_skin = new GVSkin;

  current_skin_file = g_strdup(c_tmp);
  current_skin->load(current_skin_file);
  
  current_skin->load_skin();
  current_skin->apply();
  }

GVSkinBrowser::~GVSkinBrowser()
  {
  gv_config->put_string("General", "Skin", current_skin_file);
  }

void GVSkinBrowser::apply_values()
  {
  if(display_skin == current_skin)
    return;
  GVSkin * tmp_skin = current_skin;
  current_skin = display_skin;
  g_free(current_skin_file);
  current_skin_file = g_strdup(display_skin_file);
  current_skin->load_skin();
  current_skin->apply();
  delete tmp_skin;
  }

// get_values isn't needed, we load everything in the constructor

void GVSkinBrowser::get_values()
  {
  
  }

void GVSkinBrowser::read_skins(const char * directory_name)
  {
  int skinsfound = 0;

  cerr << "Searching in " << directory_name << " for skins...";
  
  DIR * dir = opendir(directory_name);

  if(!dir)
    {
    cerr << "Directory doesn't exist\n";
    return;
    }
  struct dirent * entry;

  FILE * testfile;

  char * filename;
    
  while(1)
    {
    if(!(entry = readdir(dir)))
      break;

    if(!strcmp(entry->d_name, "..") || !strcmp(entry->d_name, "."))
      continue;

    filename = g_strdup_printf("%s/%s/skin.def", directory_name,
                               entry->d_name);

    cerr << "Trying " << filename << "\n";
    if(!(testfile = fopen(filename, "r")))
      {
      g_free(filename);
      continue;
      }
    fclose(testfile);
    char * c_tmp = g_strdup(entry->d_name);
    int row = gtk_clist_append(GTK_CLIST(skinlist), &c_tmp);
    g_free(c_tmp);
    gtk_clist_set_row_data(GTK_CLIST(skinlist), row, (gpointer)filename);
    skinsfound++;
    }
  cerr << "done, found " << skinsfound << " Skins\n";
  }
