/*****************************************************************

  slider.cc

  Copyright (c) 2001 by Burkhard Plaum - plaum@ipf.uni-stuttgart.de

  http://gmerlin.sourceforge.net

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111, USA.

*****************************************************************/

// Gmerlins own seek slider widget

#include "config.h"
#include <gtk/gtk.h>
#include <iostream>

#include "gvtime.h"
#include "videoformat.h"
#include "videoconvert.h"

// #include "baseplugin.h"
// #include "imagereader.h"

#include "xmanager.h"


#include "slider.h"

static int __button_callback(GtkWidget * w, GdkEventButton *event,
                              gpointer data)
   {
   ((GVSlider*)data)->button_callback(w, event);
   return TRUE;
   }

static int __notify_callback(GtkWidget * w, GdkEventAny *event,
                             gpointer data)
   {
   ((GVSlider*)data)->notify_callback(event);
   return TRUE;
   }

static int __motion_callback(GtkWidget * w, GdkEventMotion *event,
                              gpointer data)
   {
   ((GVSlider*)data)->motion_callback(event);
   return TRUE;
   }

static int __expose_callback(GtkWidget * w, GdkEventExpose *event,
                             gpointer data)
   {
   ((GVSlider*)data)->expose_callback(w);
   return TRUE;
   }

static void __set_style_callback(GtkWidget * w, GtkStyle * old_style,
                                 gpointer data)
  {
  ((GVSlider*)data)->set_style_callback();
  //  return true;
  }

void GVSlider::notify_callback(GdkEventAny * event)
  {
  switch(event->type)
    {
    case GDK_ENTER_NOTIFY:
      if(sensitive)
        {
        highlight = 1;
        expose_callback(slider);
        }
      break;
    case GDK_LEAVE_NOTIFY:
      if(!seek_active)
        {
        highlight = 0;
        expose_callback(slider);
        }
      break;
    }
  }


void GVSlider::expose_callback(GtkWidget * w)
  {
  //  cerr << "GVSlider::expose_callback\n";
  if(w == slider)
    {
    if(highlight)
      {
      gdk_draw_pixmap(slider->window, gv_x_manager->get_gc(),
                      slider_pixmap_highlight,
                      0, // xsrc
                      0, // ysrc
                      0, // xdest
                      0, // ydest
                      slider_width,
                      slider_height);
      }
    else
      gdk_draw_pixmap(slider->window, gv_x_manager->get_gc(), slider_pixmap,
                      0, // xsrc
                      0, // ysrc
                      0, // xdest
                      0, // ydest
                      slider_width,
                      slider_height);
    }
  else
    gdk_draw_pixmap(background->window, gv_x_manager->get_gc(),
                    background_pixmap,
                    0, // xsrc
                    0, // ysrc
                    0, // xdest
                    0, // ydest
                    background_width,
                    background_height);
  
  }

GVSlider::GVSlider()
  {
  seek_active = 0;
  seek_x = 0;
  x_offset = 0;
  highlight = 0;
  
  change_callback = (void (*)(float, gpointer))0;
  seek_callback = (void (*)(float, gpointer))0;

  change_callback_data = (gpointer)0;
  seek_callback_data = (gpointer)0;
  
  slider = gtk_drawing_area_new();
  background = gtk_drawing_area_new();

  widget = gtk_layout_new((GtkAdjustment*)0, (GtkAdjustment*)0);

  gtk_layout_set_size(GTK_LAYOUT(widget), 240, 15);
  gtk_drawing_area_size(GTK_DRAWING_AREA(background), 240, 15);
  gtk_drawing_area_size(GTK_DRAWING_AREA(slider), 20, 15);

  gtk_widget_set_events(background, GDK_EXPOSURE_MASK|
                        GDK_BUTTON_PRESS_MASK);

  gtk_widget_set_events(slider, GDK_EXPOSURE_MASK|GDK_BUTTON_PRESS_MASK|
                        GDK_BUTTON1_MOTION_MASK|
                        GDK_BUTTON_RELEASE_MASK|
                        GDK_ENTER_NOTIFY_MASK|
                        GDK_LEAVE_NOTIFY_MASK);
  
  gtk_signal_connect(GTK_OBJECT(background), "button_press_event",
                     GTK_SIGNAL_FUNC(__button_callback), (gpointer)this);
  
  gtk_signal_connect(GTK_OBJECT(slider), "button_press_event",
                     GTK_SIGNAL_FUNC(__button_callback), (gpointer)this);
  gtk_signal_connect(GTK_OBJECT(slider), "button_release_event",
                     GTK_SIGNAL_FUNC(__button_callback), (gpointer)this);
  gtk_signal_connect(GTK_OBJECT(slider), "motion_notify_event",
                     GTK_SIGNAL_FUNC(__motion_callback), (gpointer)this);

  gtk_signal_connect(GTK_OBJECT(slider), "expose_event",
                     GTK_SIGNAL_FUNC(__expose_callback), (gpointer)this);
  gtk_signal_connect(GTK_OBJECT(background), "expose_event",
                     GTK_SIGNAL_FUNC(__expose_callback), (gpointer)this);

  gtk_signal_connect(GTK_OBJECT(background), "style_set",
                     GTK_SIGNAL_FUNC(__set_style_callback), (gpointer)this);
  gtk_signal_connect(GTK_OBJECT(slider), "style_set",
                     GTK_SIGNAL_FUNC(__set_style_callback), (gpointer)this);

  gtk_signal_connect(GTK_OBJECT(slider), "enter_notify_event",
                     GTK_SIGNAL_FUNC(__notify_callback), (gpointer)this);
  gtk_signal_connect(GTK_OBJECT(slider), "leave_notify_event",
                     GTK_SIGNAL_FUNC(__notify_callback), (gpointer)this);
  
  background_pixmap = (GdkPixmap*)0;
  slider_pixmap = (GdkPixmap*)0;
  slider_mask = (GdkBitmap*)0;

  //  cerr << "Showing objects..";
  gtk_widget_show(slider);
  gtk_widget_show(background);

  gtk_layout_put(GTK_LAYOUT(widget), background, 0, 0);
  gtk_layout_put(GTK_LAYOUT(widget), slider, 0, 0);
  gtk_widget_show(widget);
  //  cerr << "Done\n";
  }

void GVSlider::realize()
  {
  gtk_widget_realize(widget);
  gtk_widget_realize(background);
  gtk_widget_realize(slider);

  }

void GVSlider::set_pixmaps(GdkPixmap * _background_pixmap,
                           GdkPixmap * _slider_pixmap,
                           GdkPixmap * _slider_pixmap_highlight,
                           GdkBitmap * _slider_mask)
  {
  gtk_layout_freeze(GTK_LAYOUT(widget));

  background_pixmap = _background_pixmap;
  slider_pixmap = _slider_pixmap;
  slider_pixmap_highlight = _slider_pixmap_highlight;
  slider_mask = _slider_mask;

  int tmp_x, tmp_y, tmp_width, tmp_height, tmp_depth;

  gdk_window_get_geometry(background_pixmap,
                          &tmp_x, &tmp_y, &background_width,
                          &background_height, &tmp_depth);

  gdk_window_get_geometry(slider_pixmap,
                          &tmp_x, &tmp_y, &slider_width,
                          &slider_height, &tmp_depth);

  gtk_layout_set_size(GTK_LAYOUT(widget), background_width, background_height);
  gtk_widget_set_usize(widget, background_width, background_height);
  
  gtk_drawing_area_size(GTK_DRAWING_AREA(background), background_width,
                        background_height);

  gdk_window_get_geometry(slider_pixmap,
                          &tmp_x, &tmp_y, &tmp_width, &tmp_height, &tmp_depth);

  max_x_offset = background_width - tmp_width;
  
  gtk_drawing_area_size(GTK_DRAWING_AREA(slider), tmp_width,
                        tmp_height);

  gdk_window_shape_combine_mask(slider->window, slider_mask, 0, 0);
  
  gtk_layout_thaw(GTK_LAYOUT(widget));

  expose_callback(widget);
    
  set_percentage(percentage);
  }

void GVSlider::set_style_callback()
  {
  if(!(slider->window) || !(background->window))
    return;

  gdk_window_shape_combine_mask(slider->window,
                                slider_mask, 0, 0);

  expose_callback(slider);
  expose_callback(background);
  
  }

void GVSlider::set_percentage(float _percentage)
  {
  if(seek_active)
    return;
  
  percentage = _percentage;
  x_offset = (int)(max_x_offset * percentage + 0.5);
  if(x_offset < 0)
    x_offset = 0;
  if(x_offset > max_x_offset)
    x_offset = max_x_offset;
    
  gtk_layout_move(GTK_LAYOUT(widget), slider, x_offset, 0);
  }

float GVSlider::get_percentage()
  {
  return percentage;
  }

void GVSlider::button_callback(GtkWidget * w, GdkEventButton * event)
  {
  if(!sensitive)
    return;

  // Support seeking with the mousewheel
  
  if((event->type == GDK_BUTTON_PRESS) && (event->button == 5))
    {
    float new_percentage = percentage + 0.005;

    if(new_percentage > 1.0)
      new_percentage = 1.0;
    
    set_percentage(new_percentage);
    if(change_callback)
      change_callback(percentage, change_callback_data);
    if(seek_callback)
      seek_callback(percentage, seek_callback_data);
    return;
    }

  else if((event->type == GDK_BUTTON_PRESS) && (event->button == 4))
    {
    float new_percentage = percentage - 0.005;
    
    if(new_percentage < 0.0)
      new_percentage = 0.0;
    
    set_percentage(new_percentage);
    if(change_callback)
      change_callback(percentage, change_callback_data);
    if(seek_callback)
      seek_callback(percentage, seek_callback_data);
    return;
    }
  
  else if(w == slider)
    {
    if(event->button == 1)
      {
      if(event->type == GDK_BUTTON_PRESS)
        {
        //      cerr << "Button clicked on slider\n";
        seek_active = 1;
        seek_x = (int)(event->x_root+0.5);
        gdk_pointer_grab(slider->window, 0,
                         (GdkEventMask)(GDK_BUTTON1_MOTION_MASK|
                                        GDK_BUTTON_RELEASE_MASK),
                         (GdkWindow*)0, (GdkCursor*)0,
                         event->time);
        
        }
      else if(event->type == GDK_BUTTON_RELEASE)
        {
        seek_active = 0;
        highlight = 0;
        expose_callback(slider);
        if(seek_callback)
          seek_callback(percentage, seek_callback_data);
        gdk_pointer_ungrab(event->time);
        }
      }
    }
  }

void GVSlider::motion_callback(GdkEventMotion * event)
  {
  if(!sensitive)
    return;
  
  if(!seek_active)
    return;

  int new_seek_x = (int)(event->x_root+0.5);

  x_offset += new_seek_x - seek_x;

  //  cerr << "seek_x: " << seek_x << " new_seek_x : " << new_seek_x << "\n";
  
  seek_x = new_seek_x;
  
  if(x_offset > max_x_offset)
    x_offset = max_x_offset;
  else if(x_offset < 0)
    x_offset = 0;

  percentage = (float)x_offset / (float)max_x_offset;

  if(change_callback)
    change_callback(percentage, change_callback_data);
  
  gtk_layout_move(GTK_LAYOUT(widget), slider, x_offset, 0);
  }

void GVSlider::set_visible(int visible)
  {
  if(visible)
    gtk_widget_show(slider);
  else
    gtk_widget_hide(slider);
  }
