/*****************************************************************

  socket.cc

  Copyright (c) 2001 by Burkhard Plaum - plaum@ipf.uni-stuttgart.de

  http://gmerlin.sourceforge.net

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111, USA.

*****************************************************************/

// Functions for the gmerlin socket class, which handles all networking
// stuff

#include "config.h"

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/stat.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>
#include <string.h>
#include <pwd.h>
#include <errno.h>
#include <unistd.h>

#include <string.h>

#include <gtk/gtk.h>

#include <iostream>
#include <sstream>
using std::stringstream;

using std::cerr;
using std::ends;
using std::ostream;

#include "socket.h"
#include "configwindow.h"
#include "globalconfig.h"


void GVSocket::open(char * hostname, int port)
  {
  if(servername)
    g_free(servername);
  servername = g_strdup(hostname);

  real_port = port;
  
  struct sockaddr_in sin;
  struct hostent *host;
  char *sname = (char*)0;
  
  sin.sin_family = AF_INET;

  if(gv_global_config->http_use_proxy)
    sin.sin_port=htons(gv_global_config->http_proxy_port);
  else
    sin.sin_port=htons(port);
  
  if(gv_global_config->http_use_proxy)
    sname = gv_global_config->http_proxy_server;
  else
    sname = hostname;
  
  if((sin.sin_addr.s_addr = inet_addr(sname))==INADDR_NONE)
    {
    cerr << "Looking up host " << sname << "...";
    if((host = gethostbyname(sname))==NULL)
      {
      sock = -1;
      cerr << "Failed\n";
      return;
      }
    cerr << "Ok\n";
    bcopy(host->h_addr,&sin.sin_addr,host->h_length);
    }

  cerr << "Connecting...";
  
  if((sock=socket(AF_INET,SOCK_STREAM,0))<0)
    {
    sock = -1;
    return;
    }
  
  if(connect(sock,(struct sockaddr *)&sin,sizeof(sin))<0)
    {
    sock = -1;
    return;
    }
  cerr << "Connected\n";
  }

void GVSocket::close()
  {
  shutdown(sock,2);
  ::close(sock);
  sock = -1;

  if(servername)
    {
    g_free(servername);
    servername = (char*)0;
    }
  free_header();
  }

void GVSocket::send_http_request(const char * url, const char * accept_type)
  {
  stringstream line;
  //  line.freeze(0);
  line.seekp(0);
  line << "GET ";
  if(gv_global_config->http_use_proxy)
    line << "http://" << servername << ":" << real_port;

  if(*url != '/')
    line << '/';
  
  line << url << " HTTP/1.1\r\nHost: " << servername
       << "\r\nUser-Agent: " << PACKAGE << "/" << VERSION
       << "\r\nAccept: " << accept_type << "\n\n" << ends;

  const char * request = line.str().c_str();

  cerr << "Sending request:\n" << request << "\n";
  
  write(sock, request, strlen(request));
  }

void GVSocket::read_line(ostream & output)
  {
  char inchar;

  while(1)
    {
    read(sock, &inchar, 1);
    if(inchar == '\n')
      break;
    if(inchar != '\r')
      output << inchar;
    }
  }

ssize_t GVSocket::read_raw(char * buffer, size_t size)
  {
  return read(sock, buffer, size);
  }

GVSocket::GVSocket()
  {
  servername = (char*)0;
  header = (GList*)0;
  keep_header = 0;
  }

// Skip the http header, return 0, if 200 was found, return 1 on error

int GVSocket::skip_http()
  {
//   char inchar;
//   int len;
  cerr << "Skip http:\n";

  // Check, if we got a 200

  stringstream line;
  //  line.freeze(0);
  line.seekp(0);
  read_line(line);
  line << ends;
  //  cerr << line.str() << "\n";

  // Todo: handle other codes

  const char * pos;
  const char * new_header_line;
  
  pos = line.str().c_str();
  
  if(!strstr(pos, " 200 "))
    return 0;

  if(keep_header)
    {
    new_header_line = g_strdup(pos);
    header = g_list_append(header, (gpointer)(new_header_line));
    }
  do{
  //    line.freeze(0);
    line.seekp(0);
    line.seekg(0);
    read_line(line);
    line << ends;
    pos = line.str().c_str();
    //    cerr << pos << "\n";
    
    if(keep_header)
      {
      new_header_line = g_strdup(pos);
      header = g_list_append(header, (gpointer)(new_header_line));
      }
  }while(strlen(pos) > 2);
  cerr << "Done skipping\n";
  return 1;
  }

// This one parses an URL of the form:
// http://www.somehost.com:9000/stream1
// On return, url points to the protocol string (e.g. "http"),
// directory points to "stream1" and port is set, if it's found
// in the URL string

// The return value is 0 is the string isn't a valid URL, 1 else

static int parse_url(char * url, char ** host, char ** directory, int & port)
  {
  char * pos = strchr(url, ':');

  if(!pos)
    return 0;
  
  *pos = '\0';

  pos++;

  if((*pos != '/') || (pos[1] != '/'))
    return 0;

  pos+=2;

  *host = pos;

  // Look for a port
  
  pos = strchr(*host, ':');
  if(pos)
    {
    *pos = '\0';
    pos++;

    if(!isdigit(*pos))
      return 0;
    
    port = atoi(pos);
    while(isdigit(*pos))
      pos++;
    if(*pos == '/')
      {
      pos++;
      *directory = pos;
      }
    else if(*pos == '\0')
      *directory = pos;
    else
      return 0;
    }
  else
    {
    pos = strchr(*host, '/');
    if(!pos)
      return 0;
    *pos = '\0';
    pos++;
    *directory = pos;
    }

  cerr << "Parsed URL: " << url
       << " Host: " << *host
       << " Directory: " << *directory
       << " Port: " << port << "\n";
  
  return 1;
  }

int GVSocket::open_url(const char * _url, const char * accept_type)
  {
  char * host;
  char * directory;
  int port = -1;
  
  char * url = g_strdup(_url);
  int ret = 1;
  
  if(!parse_url(url, &host, &directory, port))
    {
    g_free(url);
    return 0;
    }
  
  if(!strcmp(url, "http"))
    {
    if(port == -1)
      port = 80;
    open(host, port);

    if(sock == -1)
      {
      g_free(url);
      return 0;
      }
    send_http_request(directory, accept_type);
    if(!skip_http())
      {
      g_free(url);
      return 0;
      }
    }
  else
    cerr << "Protocol " << url << " not supported\n";

  g_free(url);
  return ret;
  }

void GVSocket::free_header()
  {
  if(!header)
    return;
  int num = g_list_length(header);
  char * line;

  for(int i = 0; i < num; i++)
    {
    line = (char*)g_list_nth_data(header, i);
    if(line)
      g_free(line);
    }
  g_list_free(header);
  header = (GList*)0;
  }

const char * GVSocket::get_header_key(const char * key)
  {
  int num = g_list_length(header);
  
  int keylen = strlen(key);
  char * line;
  
  for(int i = 0; i < num; i++)
    {
    line = (char*)g_list_nth_data(header, i);
    cerr << "Checking line: " << line << ", key: " << key
         << " len: " << keylen << "\n";
    if(!strncmp(line, key, keylen))
      return &(line[keylen]);
    }
  return (const char*)0;
  
  }
