/*****************************************************************

  trackinfo.cc

  Copyright (c) 2001 by Burkhard Plaum - plaum@ipf.uni-stuttgart.de

  http://gmerlin.sourceforge.net

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111, USA.

*****************************************************************/

// Infos for playlist tracks

#include "config.h"
#include <iostream>
#include <glib.h>
#include <ctype.h>

using std::ostream;
using std::ends;

#include "trackinfo.h"
#include "gv.h"
#include "gvconfig.h"

#include "globalconfig.h"

GVTrackInfo::GVTrackInfo()
  {
  title = (char*)0;
  artist = (char*)0;
  album = (char*)0;
  comment = (char*)0;
  genre = (char*)0;
  year = 0;
  track_number = 0;
  }

GVTrackInfo::~GVTrackInfo()
  {
  if(title)
    g_free(title);
  if(artist)
    g_free(artist);
  if(album)
    g_free(album);
  if(comment)
    g_free(comment);
  if(genre)
    g_free(genre);
  }

ostream& operator<<(ostream & output, const GVTrackInfo& track_info)
  {
  if(track_info.title)
    output << "Title: " << track_info.title << "\n";
  if(track_info.artist)
    output << "Artist: " << track_info.artist << "\n";
  if(track_info.album)
    output << "Album: " << track_info.album << "\n";
  if(track_info.comment)
    output << "Comment: " << track_info.comment << "\n";
  if(track_info.genre)
    output << "Genre: " << track_info.genre << "\n";
  if(track_info.year > 0)
    output << "Year: " << track_info.year << "\n";
  if(track_info.track_number > 0)
    output << "Track Number: " << track_info.track_number << "\n";
  return output;
  }

void GVTrackInfo::set_title(const char * _title)
  {
  if(title)
    g_free(title);

  if(_title && (*_title != '\0'))
    title = g_strdup(_title);
  else
    title = (char*)0;
  }

void GVTrackInfo::set_artist(const char * _artist)
  {
  if(artist)
    g_free(artist);

  if(_artist && (*_artist != '\0'))
    artist = g_strdup(_artist);
  else
    artist = (char*)0;
  }

void GVTrackInfo::set_album(const char * _album)
  {
  if(album)
    g_free(album);

  if(_album && (*_album != '\0'))
    album = g_strdup(_album);
  else
    album = (char*)0;
  }

void GVTrackInfo::set_comment(const char * _comment)
  {
  if(comment)
    g_free(comment);

  if(_comment && (*_comment != '\0'))
    comment = g_strdup(_comment);
  else
    comment = (char*)0;
  }

void GVTrackInfo::set_genre(const char * _genre)
  {
  if(genre)
    g_free(genre);
  if(_genre && (*_genre != '\0'))
    genre = g_strdup(_genre);
  else
    genre = (char*)0; 
  }

void GVTrackInfo::print_string(ostream & output) const
  {
  char * pos = gv_global_config->trackname_mask;

  while(*pos != '\0')
    {
    if(*pos == '%')
      {
      pos++;
      switch(*pos)
        {
        case 'y': // year
          if(year > 0)
            output << year;
          break;
        case 'a': // artist
          if(artist)
            output << artist;
          break;
        case 'A':
          if(album)
            output << album;
          break;
        case 'n': // track number
          if(track_number > 0)
            output << track_number;
          break;
        case 't':
          if(title)
            output << title;
          break;
        case 'c':
          if(comment)
            output << comment;
          break;
        default:
          if(isdigit(*pos))
            {
            char num_digits = *pos;
            pos++;
            if((*pos == 'n') && (num_digits > '0'))
              {
              char mask[5];
              mask[0] = '%';
              mask[1] = '0';
              mask[2] = num_digits;
              mask[3] = 'd';
              mask[4] = '\0';
              //              cerr << "Mask: " << mask << "\n";
              char * number_string = g_strdup_printf(mask, track_number);
              // cerr << "number_string: " << number_string << "\n";
              output << number_string;
              g_free(number_string);
              }
            else
              output << '%' << num_digits;
            }
          else
            output << '%' << *pos;
        }
      }
    else
      output << *pos;
    pos++;
    }
  output << ends;
  }

// Some routines for creating ID3TAGS or similar
// call default_track_info.get_*(input->get_track_info())

const char * GVTrackInfo::get_title(const GVTrackInfo * track_info,
                                    int build_comment_flags) const
  {
  if((build_comment_flags & GV_TRACK_INFO_FORCE_TITLE) && (title))
    return title;
  else if(track_info && (track_info->title))
    return track_info->title;
  return (const char *)0;
  }

const char * GVTrackInfo::get_artist(const GVTrackInfo * track_info,
                                     int build_comment_flags) const
  {
  if((build_comment_flags & GV_TRACK_INFO_FORCE_ARTIST) && (artist))
    return artist;
  else if(track_info && (track_info->artist))
    return track_info->artist;
  return (const char *)0;
  }

const char * GVTrackInfo::get_album(const GVTrackInfo * track_info,
                                    int build_comment_flags) const
  {
  if((build_comment_flags & GV_TRACK_INFO_FORCE_ALBUM) && (album))
    return album;
  else if(track_info && (track_info->album))
    return track_info->album;
  return (const char *)0;
  }
  
const char * GVTrackInfo::get_comment(const GVTrackInfo * track_info,
                                      int build_comment_flags) const
  {
  if((build_comment_flags & GV_TRACK_INFO_FORCE_COMMENT) && (comment))
    return comment;
  else if(track_info && (track_info->comment))
    return track_info->comment;
  return (const char *)0;
  }

const char * GVTrackInfo::get_genre(const GVTrackInfo * track_info,
                                    int build_comment_flags) const
  {
  if((build_comment_flags & GV_TRACK_INFO_FORCE_GENRE) && (genre))
    return genre;
  else if(track_info && (track_info->genre))
    return track_info->genre;
  return (const char *)0;

  }

int GVTrackInfo::get_year(const GVTrackInfo * track_info,
                          int build_comment_flags) const
  {
  if((build_comment_flags & GV_TRACK_INFO_FORCE_YEAR) && (year > 0))
    return year;
  else if(track_info && (track_info->year > 0))
    return track_info->year;
  return 0;
  }

int GVTrackInfo::get_track_number(const GVTrackInfo * track_info,
                                  int build_comment_flags) const
  {
  if((build_comment_flags & GV_TRACK_INFO_FORCE_TRACK) && (track_number > 0))
    return track_number;
  else if(track_info && (track_info->track_number) > 0)
    return track_info->track_number;
  return 0;
  }
