/*****************************************************************

  trackinfo.h

  Copyright (c) 2001 by Burkhard Plaum - plaum@ipf.uni-stuttgart.de

  http://gmerlin.sourceforge.net

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111, USA.

*****************************************************************/

// Infos for playlist tracks

class ostream;

// flags for building ID3 tags

#define GV_TRACK_INFO_FORCE_TITLE    (1<<0)
#define GV_TRACK_INFO_FORCE_ARTIST   (1<<1)
#define GV_TRACK_INFO_FORCE_ALBUM    (1<<2)
#define GV_TRACK_INFO_FORCE_COMMENT  (1<<3)
#define GV_TRACK_INFO_FORCE_YEAR     (1<<4)
#define GV_TRACK_INFO_FORCE_TRACK    (1<<5)
#define GV_TRACK_INFO_FORCE_GENRE    (1<<6)

struct GVTrackInfo
  {
  char * title;
  char * artist;
  char * album;

  char * comment;
  char * genre;

  int year;
  int track_number;
  
  GVTrackInfo();
  ~GVTrackInfo();

  void set_title(const char * _title);
  void set_artist(const char * _artist);
  void set_album(const char * _album);
  void set_comment(const char * _comment);
  void set_genre(const char * _genre);

  // Some routines for creating ID3TAGS or similar
  // call default_track_info.get_*(input->get_track_info())
  
  const char * get_title(const GVTrackInfo * track_info,
                         int build_comment_flags) const;

  const char * get_artist(const GVTrackInfo * track_info,
                          int build_comment_flags) const;

  const char * get_album(const GVTrackInfo * track_info,
                         int build_comment_flags) const;
  
  const char * get_comment(const GVTrackInfo * track_info,
                           int build_comment_flags) const;
  
  const char * get_genre(const GVTrackInfo * track_info,
                         int build_comment_flags) const;

  int get_year(const GVTrackInfo * track_info,
               int build_comment_flags) const;

  int get_track_number(const GVTrackInfo * track_info,
                       int build_comment_flags) const;

  // Print a string using the fields and the program wide
  // translation string
  
  void print_string(ostream & output) const;
  
  };

ostream& operator<<(ostream & output, const GVTrackInfo& track_info);
