/*****************************************************************

  urlwindow.cc

  Copyright (c) 2001 by Burkhard Plaum - plaum@ipf.uni-stuttgart.de

  http://gmerlin.sourceforge.net

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111, USA.

*****************************************************************/

// URL window

#include <gtk/gtk.h>
#include <pthread.h>

#include <iostream>

#include "config.h"
#include "gvtime.h"
#include "gvconfig.h"
#include "videoformat.h"
#include "audioformat.h"
#include "buffer.h"
#include "audiobuf.h"
#include "videobuf.h"

#include "playlist.h"

#include "baseplugin.h"
#include "plugin.h"
#include "input.h"

#include "output.h"
#include "audioout.h"
#include "videoout.h"

#include "configwindow.h"
#include "gvloader.h"
#include "engine.h"

#include "urlwindow.h"
#include "gtkutil.h"

static void __button_callback(GtkWidget * w, gpointer data)
  {
  ((GVURLWindow*)data)->button_callback(w);
  }

static void __menu_callback(GtkWidget * w, gpointer data)
  {
  ((GVURLWindow*)data)->menu_callback(w);
  }

static gboolean __delete_callback(GtkWidget * w, GdkEventAny * event,
                                  gpointer data)
  {
  ((GVURLWindow*)data)->button_callback(w);
  return TRUE;
  }

void GVURLWindow::menu_callback(GtkWidget * w)
  {
  plugin_info
    = (const GVPluginInfo*)(gtk_object_get_user_data(GTK_OBJECT(w)));
  }

void GVURLWindow::button_callback(GtkWidget * w)
  {
  if(w == add_button)
    gv_engine->add_file_with_plugin(gtk_entry_get_text(GTK_ENTRY(entry)),
                                    plugin_info->name);
  
  else if((w == close_button) || (w == window))
    gtk_widget_hide(window);
  }

GVURLWindow::GVURLWindow()
  {
  window = gtk_window_new(GTK_WINDOW_TOPLEVEL);

  GtkWidget * menu = gtk_menu_new();

  // Set the plugin items

  int num_plugins = gv_input_file_loader->get_num_plugins();
    
  GtkWidget * item;
  const GVPluginInfo * info;  

  int first_item = 1;

  //  const char * c_tmp = "i_gmerlin_mpeg";
  
  for(int i = 0; i < num_plugins; i++)
    {
    info = gv_input_file_loader->get_plugin_info(i);
    if(info->flags & GV_PLUGIN_HANDLES_URLS)
      {
      item = gtk_menu_item_new_with_label(info->real_name);
      if(first_item)
        {
        plugin_info = info;
        first_item = 0;
        }
      gtk_signal_connect(GTK_OBJECT(item), "activate",
                         GTK_SIGNAL_FUNC(__menu_callback),
                         (gpointer)this);
      
      gtk_object_set_user_data(GTK_OBJECT(item), (gpointer)info);
      gtk_widget_show(item);
      gtk_menu_append(GTK_MENU(menu), item);
      }
    }
  gtk_widget_show(menu);
    
  entry = gtk_entry_new();
  add_button = gtk_button_new_with_label("Add");
  close_button = gtk_button_new_with_label("Close");
  
  option_menu = gtk_option_menu_new();
  gtk_option_menu_set_menu(GTK_OPTION_MENU(option_menu), menu);

  // Set callbacks

  gtk_signal_connect(GTK_OBJECT(add_button), "clicked",
                     GTK_SIGNAL_FUNC(__button_callback), 
                     (gpointer)this);
  gtk_signal_connect(GTK_OBJECT(close_button), "clicked",
                     GTK_SIGNAL_FUNC(__button_callback), 
                     (gpointer)this);

  gtk_signal_connect(GTK_OBJECT(window), "delete_event",
                     GTK_SIGNAL_FUNC(__delete_callback), 
                     (gpointer)this);
                     
  // Show the objects
  
  gtk_widget_show(entry);
  gtk_widget_show(add_button);
  gtk_widget_show(close_button);
  gtk_widget_show(option_menu);
  
  // Pack the objects
  
  GtkWidget * table;
  
  table = gvgtk_table_new(3, 2, 0);
  GtkWidget * label;
  label = gtk_label_new("URL");
  gtk_widget_show(label);
  gvgtk_table_attach_noresize(table, label, 0, 1, 0, 1);
  gtk_table_attach_defaults(GTK_TABLE(table), entry, 1, 2, 0, 1);
  
  label = gtk_label_new("Plugin");
  gtk_widget_show(label);
  
  gvgtk_table_attach_noresize(table, label, 0, 1, 1, 2);
  
  gtk_table_attach_defaults(GTK_TABLE(table), option_menu, 1, 2, 1, 2);
  
  GtkWidget * buttonbox = gtk_hbutton_box_new();
  
  //  gtk_container_set_border_width(GTK_CONTAINER(buttonbox), 10);
  
  gtk_container_add(GTK_CONTAINER(buttonbox), add_button);
  gtk_container_add(GTK_CONTAINER(buttonbox), close_button);
  gtk_widget_show(buttonbox);
  
  gvgtk_table_attach_noresize(table, buttonbox, 0, 2, 2, 3);
  
  gtk_widget_show(table);
  gtk_container_add(GTK_CONTAINER(window), table);
  
  gtk_option_menu_set_history(GTK_OPTION_MENU(option_menu), 0);
  }

GVURLWindow::~GVURLWindow()
  {

  }

void GVURLWindow::show()
  {
  gtk_entry_set_text(GTK_ENTRY(entry), "");
  gtk_widget_show(window);
  }
