/*****************************************************************

  videobuf.cc

  Copyright (c) 2001 by Burkhard Plaum - plaum@ipf.uni-stuttgart.de

  http://gmerlin.sourceforge.net

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111, USA.

*****************************************************************/

// Video buffer routines

// #ifdef __GNUG__
// #pragma implementation "videobuf.h"
// #endif


#include "config.h"
#include <gtk/gtk.h>
#include <pthread.h>

#include <iostream>
#include <string.h>

#include "gvtime.h"
#include "gvconfig.h"

#include "buffer.h"
#include "videoformat.h"
#include "audioformat.h"
#include "videoconvert.h"

#include "videobuf.h"
#include "audiobuf.h"

#include "playlist.h"
#include "baseplugin.h"
#include "plugin.h"
#include "input.h"
#include "output.h"
#include "audioout.h"
#include "videoout.h"

#include "configwindow.h"
#include "engine.h"

void GVBufferVideo::free_frame(GVVideoFrame * frame,
                               const GVVideoFormat * format)
  {
  frame->free();
  }

void GVBufferVideo::alloc_frame(GVVideoFrame * frame,
                                const GVVideoFormat * format)
  {
  frame->alloc(format);

//   cerr << "Allocating Frame:\n"
//        << format->width << " x " << format->height << " x " << format->bytes_per_pixel << "\n";

  }

void GVBufferVideo::clear_frame(GVVideoFrame * frame,
                                const GVVideoFormat * format)
  {
  switch(format->colorspace)
    {
    case GVColorspaceYUV_420_P:
      memset(frame->pixels_rows[0], 0x00, format->width * format->height);
      memset(frame->pixels_rows[1], 0x80, format->width * format->height/4);
      memset(frame->pixels_rows[2], 0x80, format->width * format->height/4);
      break;      
    case GVColorspaceYUY2:
      for(int i = 0; i < format->height; i++)
        {
        for(int j = 0; j < format->width; j++)
          {
          frame->pixels_rows[i][2*j]   = 0x00; // Y
          frame->pixels_rows[i][2*j+1] = 0x80; // U/V
          }
        }
      break;
    case GVColorspaceRGBA32:
      {
      guchar * pixels = frame->pixels;
      for(int i = 0; i < format->width*format->height; i++)
        {
        pixels[0] = 0x00;
        pixels[1] = 0x00;
        pixels[2] = 0x00;
        pixels[3] = 0xFF;
        pixels += 4;
        }
      break;
      }
    default:
      memset(frame->pixels, 0x00, format->buffer_size);
      break;
    }
  }


void GVBufferVideo::convert(GVVideoFrame * src, GVVideoFrame * dst)
  {
  //  cerr << "Convert\n";

  converter->convert(src, dst);
  }

GVBufferVideo::GVBufferVideo()
  {
  converter = new GVVideoConverter;
  gv_config->get_int("VideoBuffer", "NumFrames", max_num_frames);
  }

GVBufferVideo::~GVBufferVideo()
  {
  gv_config->put_int("VideoBuffer", "NumFrames", max_num_frames);
  delete converter;
  }

int GVBufferVideo::output_can_alloc()
  {
  return output->can_alloc_video();
  }

int GVBufferVideo::input_can_alloc()
  {
  return input->can_alloc_video();
  }

void GVBufferVideo::set_formats()
  {
  convert_mode = 0;
  input_format = *(input->get_video_format());
  output_format = *(output->get_video_format());
    
//   in_x = 0;
//   in_y = 0;

//   out_x = 0;
//   out_y = 0;
  
//   in_w = input_format.width;
//   in_h = input_format.height;
//   out_w = output_format.width;
//   out_h = output_format.height;

  //  bg_color = 0;

//   total_in_w = input_format.width*input_format.bytes_per_pixel;
//   total_out_w = output_format.width*output_format.bytes_per_pixel;;

  // total_in_w and total_out_w aren't used by colormodels.c, but we set them here
  // anyway.
  
  //  total_in_w = input_format.width;
  //  total_out_w = output_format.width;
    
  if((input_format.width != output_format.width) ||
     (input_format.height != output_format.height) ||
     (input_format.colorspace != output_format.colorspace))
    {
    convert_mode = GV_BUFFER_VIDEO_CONVERT;
    converter->init(&input_format, &output_format);

    //    cerr << "===== Doing colorspace conversion ========\n";
    }
  }
