/*****************************************************************

  videoconvert.cc

  Copyright (c) 2001 by Burkhard Plaum - plaum@ipf.uni-stuttgart.de

  http://gmerlin.sourceforge.net

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111, USA.

*****************************************************************/

// Gmerlin video format converter
// Converts between colorspaces
// In the future, maybe scaling and cropping will also be supported

#include "config.h"

#include <gtk/gtk.h>
#include <iostream>
#include <iomanip.h>
#include "gvmmx.h"
#include "gvtime.h"
#include "videoformat.h"
#include "videoconvert.h"
#include "configwindow.h"
#include "globalconfig.h"

void GVVideoConverter::convert(GVVideoFrame * input_frame,
                               GVVideoFrame * output_frame)
  {
  int num_pixels = input_num_pixels;
  int width = input_format.width;
  int height = input_format.height;

  switch(convert_mode)
    {
    case Convert_PACKED_PACKED:
      for(int i = 0; i < height; i++)
        convert_packed_packed(input_frame->pixels_rows[i],
                              output_frame->pixels_rows[i], width);
      break;
    case Convert_PACKED_ALPHA_PACKED:
      for(int i = 0; i < height; i++)
        convert_packed_alpha_packed(input_frame->pixels_rows[i],
                                    output_frame->pixels_rows[i], width,
                                    background);
      break;

    case Convert_PACKED_PLANAR:
      convert_packed_planar(input_frame->pixels,
                            output_frame->y,
                            output_frame->u,
                            output_frame->v, width, height);
      break;
    case Convert_PACKED_ALPHA_PLANAR:
      convert_packed_alpha_planar(input_frame->pixels,
                                  output_frame->y,
                                  output_frame->u,
                                  output_frame->v, width, height,
                                  background);
      break;
    case Convert_PLANAR_PACKED:
      convert_planar_packed(input_frame->y,
                            input_frame->u,
                            input_frame->v,
                            output_frame->pixels, width, height);
      break;
    case Convert_PLANAR_PLANAR:
      convert_planar_planar(input_frame->y_rows,
                            input_frame->u_rows,
                            input_frame->v_rows,
                            output_frame->y_rows,
                            output_frame->u_rows,
                            output_frame->v_rows,
                            width, height);
      break;
    default:
      break;

    }
  output_frame->time = input_frame->time;
  }

void GVVideoConverter::reset()
  {
  RGB16_to_BGR16 = (GVConvert_PACKED_PACKED)0;
  RGB16_to_RGB24 = (GVConvert_PACKED_PACKED)0;
  RGB16_to_BGR24 = (GVConvert_PACKED_PACKED)0;
  RGB16_to_RGB32 = (GVConvert_PACKED_PACKED)0;
  RGB16_to_BGR32 = (GVConvert_PACKED_PACKED)0;
  RGB16_to_YUV420_P = (GVConvert_PACKED_PLANAR)0;
  RGB16_to_YUV422_P = (GVConvert_PACKED_PLANAR)0;
  RGB16_to_YUY2 = (GVConvert_PACKED_PACKED)0;

  BGR16_to_RGB16 = (GVConvert_PACKED_PACKED)0;
  BGR16_to_RGB24 = (GVConvert_PACKED_PACKED)0;
  BGR16_to_BGR24 = (GVConvert_PACKED_PACKED)0;
  BGR16_to_RGB32 = (GVConvert_PACKED_PACKED)0;
  BGR16_to_BGR32 = (GVConvert_PACKED_PACKED)0;
  BGR16_to_YUV420_P = (GVConvert_PACKED_PLANAR)0;
  BGR16_to_YUV422_P = (GVConvert_PACKED_PLANAR)0;
  BGR16_to_YUY2 = (GVConvert_PACKED_PACKED)0;

  RGB24_to_RGB16 = (GVConvert_PACKED_PACKED)0;
  RGB24_to_BGR16 = (GVConvert_PACKED_PACKED)0;
  RGB24_to_BGR24 = (GVConvert_PACKED_PACKED)0;
  RGB24_to_RGB32 = (GVConvert_PACKED_PACKED)0;
  RGB24_to_BGR32 = (GVConvert_PACKED_PACKED)0;
  RGB24_to_YUV420_P = (GVConvert_PACKED_PLANAR)0;
  RGB24_to_YUV422_P = (GVConvert_PACKED_PLANAR)0;
  RGB24_to_YUY2 = (GVConvert_PACKED_PACKED)0;

  BGR24_to_RGB16 = (GVConvert_PACKED_PACKED)0;
  BGR24_to_BGR16 = (GVConvert_PACKED_PACKED)0;
  BGR24_to_RGB24 = (GVConvert_PACKED_PACKED)0;
  BGR24_to_RGB32 = (GVConvert_PACKED_PACKED)0;
  BGR24_to_BGR32 = (GVConvert_PACKED_PACKED)0;
  BGR24_to_YUV420_P = (GVConvert_PACKED_PLANAR)0;
  BGR24_to_YUV422_P = (GVConvert_PACKED_PLANAR)0;
  BGR24_to_YUY2 = (GVConvert_PACKED_PACKED)0;

  RGB32_to_RGB16 = (GVConvert_PACKED_PACKED)0;
  RGB32_to_BGR16 = (GVConvert_PACKED_PACKED)0;
  RGB32_to_RGB24 = (GVConvert_PACKED_PACKED)0;
  RGB32_to_BGR24 = (GVConvert_PACKED_PACKED)0;
  RGB32_to_BGR32 = (GVConvert_PACKED_PACKED)0;
  RGB32_to_RGBA32 = (GVConvert_PACKED_PACKED)0;
  RGB32_to_YUV420_P = (GVConvert_PACKED_PLANAR)0;
  RGB32_to_YUV422_P = (GVConvert_PACKED_PLANAR)0;
  RGB32_to_YUY2 = (GVConvert_PACKED_PACKED)0;
  
  BGR32_to_RGB16 = (GVConvert_PACKED_PACKED)0;
  BGR32_to_BGR16 = (GVConvert_PACKED_PACKED)0;
  BGR32_to_RGB24 = (GVConvert_PACKED_PACKED)0;
  BGR32_to_BGR24 = (GVConvert_PACKED_PACKED)0;
  BGR32_to_RGB32 = (GVConvert_PACKED_PACKED)0;
  BGR32_to_YUV420_P = (GVConvert_PACKED_PLANAR)0;
  BGR32_to_YUV422_P = (GVConvert_PACKED_PLANAR)0;
  BGR32_to_YUY2 = (GVConvert_PACKED_PACKED)0;

  RGBA32_to_RGB16 = (GVConvert_PACKED_ALPHA_PACKED)0;
  RGBA32_to_BGR16 = (GVConvert_PACKED_ALPHA_PACKED)0;
  RGBA32_to_RGB24 = (GVConvert_PACKED_ALPHA_PACKED)0;
  RGBA32_to_BGR24 = (GVConvert_PACKED_ALPHA_PACKED)0;
  RGBA32_to_RGB32 = (GVConvert_PACKED_ALPHA_PACKED)0;
  RGBA32_to_BGR32 = (GVConvert_PACKED_ALPHA_PACKED)0;
  RGBA32_to_YUY2  = (GVConvert_PACKED_ALPHA_PACKED)0;
  RGBA32_to_YUV420_P = (GVConvert_PACKED_ALPHA_PLANAR)0;
  RGBA32_to_YUV422_P = (GVConvert_PACKED_ALPHA_PLANAR)0;
  
  YUV420_P_to_RGB16 = (GVConvert_PLANAR_PACKED)0;
  YUV420_P_to_BGR16 = (GVConvert_PLANAR_PACKED)0;
  YUV420_P_to_RGB24 = (GVConvert_PLANAR_PACKED)0;
  YUV420_P_to_BGR24 = (GVConvert_PLANAR_PACKED)0;
  YUV420_P_to_RGB32 = (GVConvert_PLANAR_PACKED)0;
  YUV420_P_to_BGR32 = (GVConvert_PLANAR_PACKED)0;
  YUV420_P_to_YUY2 =  (GVConvert_PLANAR_PACKED)0;
  YUV420_P_to_YUV422_P = (GVConvert_PLANAR_PLANAR)0;

  YUV422_P_to_RGB16  = (GVConvert_PLANAR_PACKED)0;
  YUV422_P_to_BGR16  = (GVConvert_PLANAR_PACKED)0;
  YUV422_P_to_RGB24  = (GVConvert_PLANAR_PACKED)0;
  YUV422_P_to_BGR24  = (GVConvert_PLANAR_PACKED)0;
  YUV422_P_to_RGB32  = (GVConvert_PLANAR_PACKED)0;
  YUV422_P_to_BGR32  = (GVConvert_PLANAR_PACKED)0;
  YUV422_P_to_YUY2   =  (GVConvert_PLANAR_PACKED)0;
  YUV422_P_to_YUV420_P = (GVConvert_PLANAR_PLANAR)0;
  
  YUY2_to_RGB16 = (GVConvert_PACKED_PACKED)0;
  YUY2_to_BGR16 = (GVConvert_PACKED_PACKED)0;
  YUY2_to_RGB24 = (GVConvert_PACKED_PACKED)0;
  YUY2_to_BGR24 = (GVConvert_PACKED_PACKED)0;
  YUY2_to_RGB32 = (GVConvert_PACKED_PACKED)0;
  YUY2_to_BGR32 = (GVConvert_PACKED_PACKED)0;
  YUY2_to_YUV420_P = (GVConvert_PACKED_PLANAR)0;
  YUY2_to_YUV422_P = (GVConvert_PACKED_PLANAR)0;
  }

void GVVideoConverter::init(const GVVideoFormat * _input_format,
                            const GVVideoFormat * _output_format)
  {
  do_scale = 0;

  reset();
  
  init_c_funcs(); // Initalize with the C functions

  if(gv_mmx->do_mmx())
    init_mmx_funcs();

  // Initialize the background color for alpha blending

  background[0] = gv_global_config->alpha_background[0];
  background[1] = gv_global_config->alpha_background[1];
  background[2] = gv_global_config->alpha_background[2];
  
  input_format = *_input_format;
  output_format = *_output_format;

  input_num_pixels = input_format.width * input_format.height;
  output_num_pixels = output_format.width * output_format.height;

  if((input_format.width != output_format.width) ||
     (input_format.height != output_format.height))
    {
    do_scale = 1;
    g_log(PACKAGE, G_LOG_LEVEL_CRITICAL, "Video scaling not supported yet!");
    return;
    }

  // Now set the conversion function

  convert_mode = Convert_NONE;

  convert_packed_packed = (GVConvert_PACKED_PACKED)0;
  convert_packed_planar = (GVConvert_PACKED_PLANAR)0;
  convert_planar_packed = (GVConvert_PLANAR_PACKED)0;
  convert_planar_planar = (GVConvert_PLANAR_PLANAR)0;
  
  switch(input_format.colorspace)
    {
    case GVColorspaceRGB16:
      switch(output_format.colorspace)
        {
        case GVColorspaceBGR16:
          convert_mode = Convert_PACKED_PACKED;
          convert_packed_packed = RGB16_to_BGR16;
          break;
        case GVColorspaceRGB24:
          convert_mode = Convert_PACKED_PACKED;
          convert_packed_packed = RGB16_to_RGB24;
          break;
        case GVColorspaceBGR24:
          convert_mode = Convert_PACKED_PACKED;
          convert_packed_packed = RGB16_to_BGR24;
          break;
        case GVColorspaceRGB32:
        case GVColorspaceRGBA32:
          convert_mode = Convert_PACKED_PACKED;
          convert_packed_packed = RGB16_to_RGB32;
          break;
        case GVColorspaceBGR32:
          convert_mode = Convert_PACKED_PACKED;
          convert_packed_packed = RGB16_to_BGR32;
          break;
        case GVColorspaceYUV_420_P:
          convert_mode = Convert_PACKED_PLANAR;
          convert_packed_planar = RGB16_to_YUV420_P;
          break;
        case GVColorspaceYUV_422_P:
          convert_mode = Convert_PACKED_PLANAR;
          convert_packed_planar = RGB16_to_YUV422_P;
          break;
        case GVColorspaceYUY2:
          convert_mode = Convert_PACKED_PACKED;
          convert_packed_packed = RGB16_to_YUY2;
          break;
        default:
          break;
        }  
      break;
    case GVColorspaceBGR16:
      switch(output_format.colorspace)
        {
        case GVColorspaceRGB16:
          convert_mode = Convert_PACKED_PACKED;
          convert_packed_packed = BGR16_to_RGB16;
          break;
        case GVColorspaceRGB24:
          convert_mode = Convert_PACKED_PACKED;
          convert_packed_packed = BGR16_to_RGB24;
          break;
        case GVColorspaceBGR24:
          convert_mode = Convert_PACKED_PACKED;
          convert_packed_packed = BGR16_to_BGR24;
          break;
        case GVColorspaceRGB32:
        case GVColorspaceRGBA32:
          convert_mode = Convert_PACKED_PACKED;
          convert_packed_packed = BGR16_to_RGB32;
          break;
        case GVColorspaceBGR32:
          convert_mode = Convert_PACKED_PACKED;
          convert_packed_packed = BGR16_to_BGR32;
          break;
        case GVColorspaceYUV_420_P:
          convert_mode = Convert_PACKED_PLANAR;
          convert_packed_planar = BGR16_to_YUV420_P;
          break;
        case GVColorspaceYUV_422_P:
          convert_mode = Convert_PACKED_PLANAR;
          convert_packed_planar = BGR16_to_YUV422_P;
          break;
        case GVColorspaceYUY2:
          convert_mode = Convert_PACKED_PACKED;
          convert_packed_packed = BGR16_to_YUY2;
          break;
        }  
      break;
    case GVColorspaceRGB24:
      switch(output_format.colorspace)
        {
        case GVColorspaceRGB16:
          convert_mode = Convert_PACKED_PACKED;
          convert_packed_packed = RGB24_to_RGB16;
          break;
        case GVColorspaceBGR16:
          convert_mode = Convert_PACKED_PACKED;
          convert_packed_packed = RGB24_to_BGR16;
          break;
        case GVColorspaceBGR24:
          convert_mode = Convert_PACKED_PACKED;
          convert_packed_packed = RGB24_to_BGR24;
          break;
        case GVColorspaceRGB32:
        case GVColorspaceRGBA32:
          convert_mode = Convert_PACKED_PACKED;
          convert_packed_packed = RGB24_to_RGB32;
          break;
        case GVColorspaceBGR32:
          convert_mode = Convert_PACKED_PACKED;
          convert_packed_packed = RGB24_to_BGR32;
          break;
        case GVColorspaceYUV_420_P:
          convert_mode = Convert_PACKED_PLANAR;
          convert_packed_planar = RGB24_to_YUV420_P;
          break;
        case GVColorspaceYUV_422_P:
          convert_mode = Convert_PACKED_PLANAR;
          convert_packed_planar = RGB24_to_YUV422_P;
          break;
        case GVColorspaceYUY2:
          convert_mode = Convert_PACKED_PACKED;
          convert_packed_packed = RGB24_to_YUY2;
          break;
        }  
      break;
    case GVColorspaceBGR24:
      switch(output_format.colorspace)
        {
        case GVColorspaceRGB16:
          convert_mode = Convert_PACKED_PACKED;
          convert_packed_packed = BGR24_to_RGB16;
          break;
        case GVColorspaceBGR16:
          convert_mode = Convert_PACKED_PACKED;
          convert_packed_packed = BGR24_to_BGR16;
          break;
        case GVColorspaceRGB24:
          convert_mode = Convert_PACKED_PACKED;
          convert_packed_packed = BGR24_to_RGB24;
          break;
        case GVColorspaceRGB32:
        case GVColorspaceRGBA32:
          convert_mode = Convert_PACKED_PACKED;
          convert_packed_packed = BGR24_to_RGB32;
          break;
        case GVColorspaceBGR32:
          convert_mode = Convert_PACKED_PACKED;
          convert_packed_packed = BGR24_to_BGR32;
          break;
        case GVColorspaceYUV_420_P:
          convert_mode = Convert_PACKED_PLANAR;
          convert_packed_planar = BGR24_to_YUV420_P;
          break;
        case GVColorspaceYUV_422_P:
          convert_mode = Convert_PACKED_PLANAR;
          convert_packed_planar = BGR24_to_YUV422_P;
          break;
        case GVColorspaceYUY2:
          convert_mode = Convert_PACKED_PACKED;
          convert_packed_packed = BGR24_to_YUY2;
          break;
        }  
      break;
    case GVColorspaceRGB32:
      switch(output_format.colorspace)
        {
        case GVColorspaceRGB16:
          convert_mode = Convert_PACKED_PACKED;
          convert_packed_packed = RGB32_to_RGB16;
          break;
        case GVColorspaceBGR16:
          convert_mode = Convert_PACKED_PACKED;
          convert_packed_packed = RGB32_to_BGR16;
          break;
        case GVColorspaceRGB24:
          convert_mode = Convert_PACKED_PACKED;
          convert_packed_packed = RGB32_to_RGB24;
          break;
        case GVColorspaceBGR32:
          convert_mode = Convert_PACKED_PACKED;
          convert_packed_packed = RGB32_to_BGR32;
          break;
        case GVColorspaceBGR24:
          convert_mode = Convert_PACKED_PACKED;
          convert_packed_packed = RGB32_to_BGR24;
          break;
        case GVColorspaceRGBA32:
          convert_mode = Convert_PACKED_PACKED;
          convert_packed_packed = RGB32_to_RGBA32;
          break;
        case GVColorspaceYUV_420_P:
          convert_mode = Convert_PACKED_PLANAR;
          convert_packed_planar = RGB32_to_YUV420_P;
          break;
        case GVColorspaceYUV_422_P:
          convert_mode = Convert_PACKED_PLANAR;
          convert_packed_planar = RGB32_to_YUV422_P;
          break;
        case GVColorspaceYUY2:
          convert_mode = Convert_PACKED_PACKED;
          convert_packed_packed = RGB32_to_YUY2;
          break;
        }
      break;
    case GVColorspaceBGR32:
      switch(output_format.colorspace)
        {
        case GVColorspaceRGB16:
          convert_mode = Convert_PACKED_PACKED;
          convert_packed_packed = BGR32_to_RGB16;
          break;
        case GVColorspaceBGR16:
          convert_mode = Convert_PACKED_PACKED;
          convert_packed_packed = BGR32_to_BGR16;
          break;
        case GVColorspaceRGB24:
          convert_mode = Convert_PACKED_PACKED;
          convert_packed_packed = BGR32_to_RGB24;
          break;
        case GVColorspaceRGB32:
        case GVColorspaceRGBA32:
          convert_mode = Convert_PACKED_PACKED;
          convert_packed_packed = BGR32_to_RGB32;
          break;
        case GVColorspaceBGR24:
          convert_mode = Convert_PACKED_PACKED;
          convert_packed_packed = BGR32_to_BGR24;
          break;
        case GVColorspaceYUV_420_P:
          convert_mode = Convert_PACKED_PLANAR;
          convert_packed_planar = BGR32_to_YUV420_P;
          break;
        case GVColorspaceYUV_422_P:
          convert_mode = Convert_PACKED_PLANAR;
          convert_packed_planar = BGR32_to_YUV422_P;
          break;
        case GVColorspaceYUY2:
          convert_mode = Convert_PACKED_PACKED;
          convert_packed_packed = BGR32_to_YUY2;
          break;
        }
      break;
    case GVColorspaceRGBA32:
      switch(output_format.colorspace)
        {
        case GVColorspaceRGB16:
          convert_mode = Convert_PACKED_ALPHA_PACKED;
          convert_packed_alpha_packed = RGBA32_to_RGB16;
          break;
        case GVColorspaceBGR16:
          convert_mode = Convert_PACKED_ALPHA_PACKED;
          convert_packed_alpha_packed = RGBA32_to_BGR16;
          break;
        case GVColorspaceRGB24:
          convert_mode = Convert_PACKED_ALPHA_PACKED;
          convert_packed_alpha_packed = RGBA32_to_RGB24;
          break;
        case GVColorspaceBGR24:
          convert_mode = Convert_PACKED_ALPHA_PACKED;
          convert_packed_alpha_packed = RGBA32_to_BGR24;
          break;
        case GVColorspaceRGB32:
          convert_mode = Convert_PACKED_ALPHA_PACKED;
          convert_packed_alpha_packed = RGBA32_to_RGB32;
          break;
        case GVColorspaceBGR32:
          convert_mode = Convert_PACKED_ALPHA_PACKED;
          convert_packed_alpha_packed = RGBA32_to_BGR32;
          break;
        case GVColorspaceYUV_420_P:
          convert_mode = Convert_PACKED_ALPHA_PLANAR;
          convert_packed_alpha_planar = RGBA32_to_YUV420_P;
          break;
        case GVColorspaceYUV_422_P:
          convert_mode = Convert_PACKED_ALPHA_PLANAR;
          convert_packed_alpha_planar = RGBA32_to_YUV422_P;
          break;
        case GVColorspaceYUY2:
          convert_mode = Convert_PACKED_ALPHA_PACKED;
          convert_packed_alpha_packed = RGBA32_to_YUY2;
          break;
        default:
          break;
        }  
      break;
    case GVColorspaceYUV_420_P:
      switch(output_format.colorspace)
        {
        case GVColorspaceRGB16:
          convert_mode = Convert_PLANAR_PACKED;
          convert_planar_packed = YUV420_P_to_RGB16;
          break;
        case GVColorspaceBGR16:
          convert_mode = Convert_PLANAR_PACKED;
          convert_planar_packed = YUV420_P_to_BGR16;
          break;
        case GVColorspaceRGB24:
          convert_mode = Convert_PLANAR_PACKED;
          convert_planar_packed = YUV420_P_to_RGB24;
          break;
        case GVColorspaceBGR24:
          convert_mode = Convert_PLANAR_PACKED;
          convert_planar_packed = YUV420_P_to_BGR24;
          break;
        case GVColorspaceRGB32:
        case GVColorspaceRGBA32:
          convert_mode = Convert_PLANAR_PACKED;
          convert_planar_packed = YUV420_P_to_RGB32;
          break;
        case GVColorspaceBGR32:
          convert_mode = Convert_PLANAR_PACKED;
          convert_planar_packed = YUV420_P_to_BGR32;
          break;
        case GVColorspaceYUY2:
          convert_mode = Convert_PLANAR_PACKED;
          convert_planar_packed = YUV420_P_to_YUY2;
          break;
        case GVColorspaceYUV_422_P:
          convert_mode = Convert_PLANAR_PLANAR;
          convert_planar_planar = YUV420_P_to_YUV422_P;
          break;
        }  
      break;
    case GVColorspaceYUV_422_P:
      switch(output_format.colorspace)
        {
        case GVColorspaceRGB16:
          convert_mode = Convert_PLANAR_PACKED;
          convert_planar_packed = YUV422_P_to_RGB16;
          break;
        case GVColorspaceBGR16:
          convert_mode = Convert_PLANAR_PACKED;
          convert_planar_packed = YUV422_P_to_BGR16;
          break;
        case GVColorspaceRGB24:
          convert_mode = Convert_PLANAR_PACKED;
          convert_planar_packed = YUV422_P_to_RGB24;
          break;
        case GVColorspaceBGR24:
          convert_mode = Convert_PLANAR_PACKED;
          convert_planar_packed = YUV422_P_to_BGR24;
          break;
        case GVColorspaceRGB32:
        case GVColorspaceRGBA32:
          convert_mode = Convert_PLANAR_PACKED;
          convert_planar_packed = YUV422_P_to_RGB32;
          break;
        case GVColorspaceBGR32:
          convert_mode = Convert_PLANAR_PACKED;
          convert_planar_packed = YUV422_P_to_BGR32;
          break;
        case GVColorspaceYUY2:
          convert_mode = Convert_PLANAR_PACKED;
          convert_planar_packed = YUV422_P_to_YUY2;
          break;
        case GVColorspaceYUV_420_P:
          convert_mode = Convert_PLANAR_PLANAR;
          convert_planar_planar = YUV422_P_to_YUV420_P;
          break;
        }  
      break;
    case GVColorspaceYUY2:
      switch(output_format.colorspace)
        {
        case GVColorspaceRGB16:
          convert_mode = Convert_PACKED_PACKED;
          convert_packed_packed = YUY2_to_RGB16;
          break;
        case GVColorspaceBGR16:
          convert_mode = Convert_PACKED_PACKED;
          convert_packed_packed = YUY2_to_BGR16;
          break;
        case GVColorspaceRGB24:
          convert_mode = Convert_PACKED_PACKED;
          convert_packed_packed = YUY2_to_RGB24;
          break;
        case GVColorspaceBGR24:
          convert_mode = Convert_PACKED_PACKED;
          convert_packed_packed = YUY2_to_BGR24;
          break;
        case GVColorspaceRGB32:
        case GVColorspaceRGBA32:
          convert_mode = Convert_PACKED_PACKED;
          convert_packed_packed = YUY2_to_RGB32;
          break;
        case GVColorspaceBGR32:
          convert_mode = Convert_PACKED_PACKED;
          convert_packed_packed = YUY2_to_BGR32;
          break;
        case GVColorspaceYUV_420_P:
          convert_mode = Convert_PACKED_PLANAR;
          convert_packed_planar = YUY2_to_YUV420_P;
          break;
        case GVColorspaceYUV_422_P:
          convert_mode = Convert_PACKED_PLANAR;
          convert_packed_planar = YUY2_to_YUV422_P;
          break;
        }
      break;
    }
  if((convert_mode == Convert_NONE) ||
     (!(convert_packed_packed) &&
      !(convert_packed_alpha_packed) &&
      !(convert_planar_packed) &&
      !(convert_packed_planar) &&
      !(convert_packed_alpha_planar) &&
      !(convert_planar_planar)))
    {
    char * in_colorspace_name
      = gv_colorspace_to_string(input_format.colorspace);
    char * out_colorspace_name
      = gv_colorspace_to_string(output_format.colorspace);
    
    g_log(PACKAGE, G_LOG_LEVEL_CRITICAL,
          "Cannot convert colorspace \"%s\" to colorspace \"%s\"",
          in_colorspace_name, out_colorspace_name);
    g_free(in_colorspace_name);
    g_free(out_colorspace_name);
    }
  
  }
