/*****************************************************************

  videoframe.cc

  Copyright (c) 2001 by Burkhard Plaum - plaum@ipf.uni-stuttgart.de

  http://gmerlin.sourceforge.net

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111, USA.

*****************************************************************/

// Routines for the video frame

#include "config.h"

#include <gtk/gtk.h>
#include <pthread.h>

#include <iostream>
#include <string.h>

#include "gvtime.h"
#include "gvconfig.h"

#include "buffer.h"
#include "videoformat.h"
#include "audioformat.h"

void GVVideoFrame::alloc(const GVVideoFormat * format)
  {
  pixels
    = new guchar[format->buffer_size];
  
  switch(format->colorspace)
    {
    case GVColorspaceYUV_420_P:

      y = pixels;
      u = &(pixels[format->width*format->height]);
      v = &(pixels[format->width*format->height +
                               (format->width*format->height)/4]);

      pixels_rows = new guchar*[3];
      pixels_rows[0] = y;
      pixels_rows[1] = u;
      pixels_rows[2] = v;
      
      y_rows = new guchar*[format->height];
      u_rows = new guchar*[format->height/2];
      v_rows = new guchar*[format->height/2];

      for(int i = 0; i < format->height; i++)
        y_rows[i] = &(y[format->width*i]);

      for(int i = 0; i < (format->height)/2; i++)
        {
        u_rows[i] = &(u[format->width*i/2]);
        v_rows[i] = &(v[format->width*i/2]);
        }
      
      break;
    case GVColorspaceYUV_422_P:
      pixels_rows = (guchar**)0;
      y = pixels;
      u = &(pixels[format->width*format->height]);
      v = &(pixels[format->width*format->height +
                  (format->width*format->height)/2]);

      pixels_rows = new guchar*[3];
      pixels_rows[0] = y;
      pixels_rows[1] = u;
      pixels_rows[2] = v;
      
      y_rows = new guchar*[format->height];
      u_rows = new guchar*[format->height];
      v_rows = new guchar*[format->height];

      for(int i = 0; i < (format->height); i++)
        {
        y_rows[i] = &(y[format->width*i]);
        u_rows[i] = &(u[format->width*i/2]);
        v_rows[i] = &(v[format->width*i/2]);
        }
      break;
    default:
      y = (guchar*)0;
      u = (guchar*)0;
      v = (guchar*)0;

      y_rows = (guchar**)0;
      u_rows = (guchar**)0;
      v_rows = (guchar**)0;

      pixels_rows = new guchar*[format->height];
      for(int i = 0; i < format->height; i++)
        pixels_rows[i]
          = &(pixels[i*format->width*format->bytes_per_pixel]);
      break;
    }
  }

void GVVideoFrame::free()
  {
  if(pixels)
    {
    delete pixels;
    pixels = (guchar*)0;
    }
  if(pixels_rows)
    {
    delete pixels_rows;
    pixels_rows = (guchar**)0;
    }
  if(y_rows)
    {
    delete y_rows;
    y_rows = (guchar**)0;
    }
  if(u_rows)
    {
    delete u_rows;
    u_rows = (guchar**)0;
    }
  if(v_rows)
    {
    delete v_rows;
    v_rows = (guchar**)0;
    }
  }

GVVideoFrame::GVVideoFrame()
  {
  pixels = (guchar*)0;
  pixels_rows = (guchar**)0;
  y_rows = (guchar**)0;
  u_rows = (guchar**)0;
  v_rows = (guchar**)0;
  }
  
