/*****************************************************************

  videoout.cc

  Copyright (c) 2001 by Burkhard Plaum - plaum@ipf.uni-stuttgart.de

  http://gmerlin.sourceforge.net

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111, USA.

*****************************************************************/

// Write a short explanation here!!

#ifdef __GNUG__
#pragma implementation "videoout.h"
#endif


#include <iostream>

using std::cerr;
using std::ostream;

#include <gtk/gtk.h>
#include <pthread.h>

#include "config.h"
#include "gvtime.h"
#include "buffer.h"
#include "videoformat.h"
#include "audioformat.h"

#include "videobuf.h"
#include "audiobuf.h"

#include "playlist.h"

#include "baseplugin.h"
#include "plugin.h"
#include "input.h"
#include "output.h"
#include "audioout.h"
#include "videoout.h"

#include "configwindow.h"
#include "engine.h"


GVOutputVideo::GVOutputVideo(const GVPluginAttributes * _attributes) :
  GVOutput(_attributes)
  {
  flags = GV_PLUGIN_TRACK_HAS_VIDEO;
  do_audio = 0;
  do_video = 1;
  }

#define NSEC2MSEC 1000000

void GVOutputVideo::playback_func() // Playback Function
  {

  //  cout << attributes->name << ": playback_func() started\n";
  GVVideoFrame * video_frame;

  GVTime long_time;
  GVTime short_time;
  
  long_time = video_format.frame_time;
  short_time = video_format.frame_time;

  long_time += long_time;
  long_time += long_time;
  short_time /= 5;  
  
  GVTime current_time;
  GVTime diff_time;

  GVTime last_time;
  last_time.reset();
  
  //  GVTime last_time;
  //  last_time.seconds = 0;
  //  last_time.nanoseconds = 0;

  //  int paused = 0;
  //  GVEngineState state;
  //  GVTime time_diff;

  gv_engine->plugin_attach();
  gv_engine->wait_for_start();

  double time_difference;

  int skip_active = 0;
  
  while(1)
    {
    // Check if we should stop
    //    cerr << attributes->name << ": Checking for stop...";
    if(check_engine_state())
      break;
    //   cerr << "done\n";
   
   if (video_buffer->empty()) 
     {
     if(gv_engine->get_state() == GVEngineCompleting)
       {
       //       cerr << "VideoOut: Playback completed\n";
       break;
       }
     short_time.sleep();
     continue;
     }
   else
     {
     video_frame = video_buffer->lock_read();

     if(realtime)
       {
       gv_engine->get_current_track_time(current_time, 1);
       time_difference = video_frame->time.get_double() -
         current_time.get_double();
       
//      cerr << "video_frame->time: " << video_frame->time.get_double()
//           << " current_time: " << current_time.get_double()
//           << " Time difference: " << time_difference << "\n";

       // We do the following:
       // if(time_difference > 0): Wait until the frame can be displayed
       // if(time_difference < video_format.frame_time * 0.5)
       
       // Send a skip request to the input
       if(!(skip_active) &&
          (time_difference < - 2.0 * video_format.frame_time.get_double()))
         {
         int frames_to_skip =
           (int)(-time_difference / video_format.frame_time.get_double()+0.5);
         cerr << attributes->name << ": trying to skip " << frames_to_skip
              << " Video frames (Frame was: "
              << -time_difference*1000.0 << " msec. too late)\n";
         input->skip_video_frames(frames_to_skip);
         skip_active = 1;
         }
       // Wait until it's time to display the frame
     
       else if(current_time < video_frame->time)
         {
//          cerr << "Sleeping, video_frame->time:"
//               << video_frame->time.get_double()
//               << " current_time:"
//               << current_time.get_double() << "\n";
         diff_time = time_difference;
         diff_time.sleep();
         }
       
      // Check if the first frame after skipping is here
     
       if((skip_active) && (video_frame->skip))
         {
         cerr << attributes->name << ": Got first frame after skip\n";
         skip_active = 0;
         }
       }
     put_frame(video_frame);
     last_time = video_frame->time;
     
     video_buffer->unlock_read();
     if(realtime)
       short_time.sleep();
     }
    }
  cleanup();
  gv_engine->plugin_detach();
  //  cout << attributes->name << ": playback_func() finished\n";
  }


GVOutputVideo::~GVOutputVideo()
  {
  
  }
