/*****************************************************************

  videowindow.cc

  Copyright (c) 2001 by Burkhard Plaum - plaum@ipf.uni-stuttgart.de

  http://gmerlin.sourceforge.net

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111, USA.

*****************************************************************/

// Methods for the video window
// The video window is currently only used by the X11 video driver.

#include "config.h"
#include <pthread.h>
#include <iostream>

using std::cerr;
using std::ostream;

#include <gtk/gtk.h>
#include "button.h"
#include "gvwindow.h"

#include "videowindow.h"
#include <gdk/gdkx.h>

#include "gvconfig.h"


#include "gvtime.h"
#include "videoformat.h"


#include "xmanager.h"

static unsigned char cursor_data[] = { 0x00 };
static unsigned char cursor_mask[] = { 0x00 };

// static gboolean __delete_event(GtkWidget * w, GdkEventAny * event,
//                                gpointer data)
//   {
//   ((GVVideoWindow *)data)->hide();
//   return TRUE;
//   }

static gboolean __key_press_event(GtkWidget * w, GdkEventKey * event,
                                  gpointer data)
  {
  ((GVVideoWindow*)data)->key_press_event(event);
  return TRUE;
  }

static gboolean __expose_event(GtkWidget * w, GdkEventExpose * event,
                               gpointer data)
  {
  ((GVVideoWindow*)data)->expose_callback();
  return TRUE;
  }

static gboolean __configure_notify_event(GtkWidget * w,
                                         GdkEventConfigure * event, 
                                         gpointer data)
  {
  ((GVVideoWindow*)data)->configure_notify_event(event);
  return TRUE;
  }

static void __menu_callback(GtkWidget * w, gpointer data)
  {
  ((GVVideoWindow*)data)->menu_callback(w);
  }

void GVVideoWindow::configure_notify_event(GdkEventConfigure * event)
  {
  //  cerr << "Configure Notify!!\n";
  normal_width = event->width;
  normal_height = event->height;

  if(!(flags & GV_VIDEO_WINDOW_FULLSCREEN))
    {
    pthread_mutex_lock(&size_mutex);
    width = normal_width;
    height = normal_height;
    pthread_mutex_unlock(&size_mutex);
    }
  }

void GVVideoWindow::key_press_event(GdkEventKey * event)
  {
  cerr << "Key press event\n";
  switch(*(event->string))
    {
    case 'F':
      toggle_fullscreen(event->time);
      break;
    default:
      break;
    }
  }

void GVVideoWindow::menu_callback(GtkWidget * w)
  {
  if(w == fullscreen_item)
    set_fullscreen(gdk_time_get(), 1);
  else if(w == original_item)
    set_size(original_width, original_height);
  else if(w == auto_resize_item)
    {
    if(GTK_CHECK_MENU_ITEM(w)->active)
      flags |= GV_VIDEO_WINDOW_AUTO_RESIZE;
    else
      flags &= ~GV_VIDEO_WINDOW_AUTO_RESIZE;
    }
  else if(w == remember_size_item)
    {
    if(GTK_CHECK_MENU_ITEM(w)->active)
      flags |= GV_VIDEO_WINDOW_REMEMBER_SIZE;
    else
      flags &= ~GV_VIDEO_WINDOW_REMEMBER_SIZE;
    }
  else if(w == remember_position_item)
    {
    if(GTK_CHECK_MENU_ITEM(w)->active)
      flags |= GV_VIDEO_WINDOW_REMEMBER_POSITION;
    else
      flags &= ~GV_VIDEO_WINDOW_REMEMBER_POSITION;
    }
  }

GVVideoWindow::GVVideoWindow()
  {
  active = 0;

  pthread_mutex_init(&size_mutex,(pthread_mutexattr_t *)0);
  pthread_mutex_init(&active_mutex,(pthread_mutexattr_t *)0);

  normal_window.set_name("Video");
    
  //  gtk_normal_window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
  gtk_fullscreen_window = gtk_window_new(GTK_WINDOW_TOPLEVEL);

//   gtk_window_set_title(GTK_WINDOW(gtk_normal_window),
//                        "Gmerlin Video");
  gtk_window_set_title(GTK_WINDOW(gtk_fullscreen_window),
                       "Gmerlin Fullscreen Video");
  
  normal_drawingarea = gtk_drawing_area_new();
  normal_window.drawingarea = normal_drawingarea;

  fullscreen_drawingarea = gtk_drawing_area_new();

  //  gtk_container_set_border_width(GTK_CONTAINER(gtk_normal_window), 0);
  gtk_container_set_border_width(GTK_CONTAINER(gtk_fullscreen_window), 0);
  
  gtk_widget_set_events(gtk_fullscreen_window, GDK_KEY_PRESS_MASK);

  gtk_widget_set_events(normal_drawingarea, GDK_EXPOSURE_MASK);
  gtk_widget_set_events(fullscreen_drawingarea, GDK_EXPOSURE_MASK);
  
  gtk_signal_connect(GTK_OBJECT(gtk_fullscreen_window), "key_press_event",
                     GTK_SIGNAL_FUNC(__key_press_event), (gpointer)this);

  gtk_signal_connect(GTK_OBJECT(normal_drawingarea), "expose_event",
                     GTK_SIGNAL_FUNC(__expose_event), (gpointer)this);

//   gtk_signal_connect(GTK_OBJECT(gtk_normal_window), "delete_event",
//                      GTK_SIGNAL_FUNC(__delete_event), (gpointer)this);

  gtk_signal_connect(GTK_OBJECT(normal_drawingarea), "configure_event",
                     GTK_SIGNAL_FUNC(__configure_notify_event),
                     (gpointer)this);
  
  gtk_widget_show(normal_drawingarea);
  gtk_widget_show(fullscreen_drawingarea);
  
//   gtk_container_set_resize_mode(GTK_CONTAINER(gtk_window),
//                                 GTK_RESIZE_IMMEDIATE);

  // By default, we never have fullscreen
  
  //  gtk_container_add(GTK_CONTAINER(gtk_normal_window), normal_drawingarea);

  normal_window.attach(normal_drawingarea);
  gtk_drawing_area_size(GTK_DRAWING_AREA(normal_drawingarea), 320, 240);
  gtk_container_add(GTK_CONTAINER(gtk_fullscreen_window),
                    fullscreen_drawingarea);

  flags = 0;
  
  normal_window.set_resizeable(1);
  
  normal_window.realize();
  
  gtk_widget_realize(gtk_fullscreen_window);
  
  gdk_window_set_decorations(gtk_fullscreen_window->window,
                             (GdkWMDecoration)0);
  gdk_window_set_functions(gtk_fullscreen_window->window,
                           (GdkWMFunction)0);
  //  gdk_window_set_override_redirect(gtk_fullscreen_window->window, TRUE);
  
  gdk_window_move(gtk_fullscreen_window->window, 0, 0);
  
  gtk_widget_realize(normal_drawingarea);
  gtk_widget_realize(fullscreen_drawingarea);

  gdk_normal_gc = gdk_gc_new(normal_drawingarea->window);
  gdk_fullscreen_gc = gdk_gc_new(fullscreen_drawingarea->window);

  gdk_gc = gdk_normal_gc;
  
  set_public_vars();

  // set the background color of the drawingarea

  gdk_color_parse("#000000", &black);

  gdk_color_alloc(gdk_window_get_colormap(gdk_window), &black);
  
  gdk_window_set_background(normal_drawingarea->window, &black);
  gdk_window_set_background(fullscreen_drawingarea->window, &black);

  // Create the fullscreen cursor

  create_fullscreen_cursor();
  gdk_window_set_cursor(gtk_fullscreen_window->window, fullscreen_cursor);

  //  normal_window.set_size(320, 240);

  menu = gtk_menu_new();
  
  fullscreen_item =
    gtk_menu_item_new_with_label("Fullscreen (exit with Shift+F)");
  gtk_signal_connect(GTK_OBJECT(fullscreen_item), "activate",
                     GTK_SIGNAL_FUNC(__menu_callback), (gpointer)this);
  
  original_item =
    gtk_menu_item_new_with_label("Original Size");
  gtk_signal_connect(GTK_OBJECT(original_item), "activate",
                     GTK_SIGNAL_FUNC(__menu_callback), (gpointer)this);

  auto_resize_item = gtk_check_menu_item_new_with_label("Auto Resize");
  gtk_signal_connect(GTK_OBJECT(auto_resize_item), "toggled",
                     GTK_SIGNAL_FUNC(__menu_callback),
                     (gpointer)this);
  gtk_check_menu_item_set_show_toggle(GTK_CHECK_MENU_ITEM(auto_resize_item),
                                      TRUE);

  remember_size_item = gtk_check_menu_item_new_with_label("Remember Size");
  gtk_signal_connect(GTK_OBJECT(remember_size_item), "toggled",
                     GTK_SIGNAL_FUNC(__menu_callback),
                     (gpointer)this);
  gtk_check_menu_item_set_show_toggle(GTK_CHECK_MENU_ITEM(remember_size_item),
                                      TRUE);

  remember_position_item =
    gtk_check_menu_item_new_with_label("Remember Position");
  gtk_signal_connect(GTK_OBJECT(remember_position_item), "toggled",
                     GTK_SIGNAL_FUNC(__menu_callback),
                     (gpointer)this);
  gtk_check_menu_item_set_show_toggle(GTK_CHECK_MENU_ITEM(remember_position_item),
                                      TRUE);

  gtk_widget_show(fullscreen_item);
  gtk_widget_show(original_item);
  gtk_widget_show(auto_resize_item);
  
  gtk_widget_show(remember_size_item);
  //   gtk_widget_show(remember_position_item);

  gtk_menu_append(GTK_MENU(menu), fullscreen_item);
  gtk_menu_append(GTK_MENU(menu), original_item);
  gtk_menu_append(GTK_MENU(menu), auto_resize_item);
  gtk_menu_append(GTK_MENU(menu), remember_size_item);
  gtk_menu_append(GTK_MENU(menu), remember_position_item);
  
  gtk_widget_show(menu);
  normal_window.set_ul_menu(menu);

  //  cerr << "Loading Image: " << DATA_DIR"/gmerlin.png" << "\n";
  default_background_pixmap = gv_x_manager->load_image(DATA_DIR"/gmerlin.png",
                                                       (GdkBitmap**)0);

  gdk_window_get_size(default_background_pixmap,
                      &background_width, &background_height);

  set_size(background_width, background_height);
  set_original_size(background_width, background_height);
  current_background_pixmap = default_background_pixmap;
  
  get_config();
  
  }

void GVVideoWindow::expose_callback()
  {
  if(get_active())
    return;
  int tmp_width, tmp_height;
  get_size(tmp_width, tmp_height);
  int x_coord = (tmp_width - background_width)/2;
  int y_coord = (tmp_height - background_height)/2;

  gdk_draw_pixmap(normal_drawingarea->window, gv_x_manager->get_gc(),
                  current_background_pixmap, 0, 0, x_coord, y_coord,
                  background_width,
                  background_height);
  
  }


void GVVideoWindow::create_fullscreen_cursor()
  {
  GdkPixmap *src;
  GdkPixmap *msk;

  GdkColor fg = { 0, 255 << 8, 255 << 8, 255 << 8 };
  GdkColor bg = { 0, 0, 0, 0 };
  
  src = gdk_bitmap_create_from_data(gtk_fullscreen_window->window,
                                    (const char *)cursor_data, 1, 1);
  
  msk = gdk_bitmap_create_from_data(gtk_fullscreen_window->window,
                                    (const char *)cursor_mask, 1, 1);
  
  fullscreen_cursor = gdk_cursor_new_from_pixmap( src, msk, &fg, &bg, 0, 0);

  gdk_pixmap_unref( src );
  gdk_pixmap_unref( msk );
  }

void GVVideoWindow::set_public_vars()
  {
  if(flags & GV_VIDEO_WINDOW_FULLSCREEN)
    {
    gdk_gc = gdk_fullscreen_gc;
    gdk_window = fullscreen_drawingarea->window;
    }
  else
    {
    gdk_gc = gdk_normal_gc;
    gdk_window = normal_drawingarea->window;
    }
  
  gdk_gc = gdk_gc_new(gdk_window);
  gdk_visual = gdk_window_get_visual(gdk_window);
  
  // Now set the Xlib variables
  
  x_win = GDK_WINDOW_XWINDOW(gdk_window);
  x_gc = GDK_GC_XGC(gdk_gc);
  x_display = GDK_WINDOW_XDISPLAY(gdk_window);   
  x_visual = GDK_VISUAL_XVISUAL(gdk_visual);
  }

void GVVideoWindow::lock()
  {
  gdk_threads_enter();
  }

void GVVideoWindow::unlock()
  {
  gdk_threads_leave();
  }

void GVVideoWindow::toggle_fullscreen(guint32 time)
  {
  if(flags & GV_VIDEO_WINDOW_FULLSCREEN)
    set_fullscreen(0, time);
  else
    set_fullscreen(1, time);
  }

void GVVideoWindow::set_fullscreen(int fullscreen, guint32 time)
  {
  if(!!fullscreen == !!(flags & GV_VIDEO_WINDOW_FULLSCREEN))
    return;

  //  int depth;
  
  if(fullscreen)
    {
    // Make a fullscreen window

    fullscreen_width = gdk_screen_width();
    fullscreen_height = gdk_screen_height();

    //    normal_window.hide();
        
    gtk_widget_set_usize(gtk_fullscreen_window,
                         fullscreen_width,
                         fullscreen_height);
    
    gtk_widget_show(gtk_fullscreen_window);
    gdk_window_move(gtk_fullscreen_window->window, 0, 0);
    
    flags |= GV_VIDEO_WINDOW_FULLSCREEN;
    set_public_vars();

    gdk_window_raise(gtk_fullscreen_window->window);
    //    gtk_window_activate_focus(GTK_WINDOW(gtk_fullscreen_window));
    
//     gdk_pointer_grab(gtk_fullscreen_window->window,
//                      FALSE,
//                      (GdkEventMask)0,
//                      (GdkWindow*)0,
//                      fullscreen_cursor,
//                      time);
        
    pthread_mutex_lock(&size_mutex);    
    width = fullscreen_width;
    height = fullscreen_height;
    pthread_mutex_unlock(&size_mutex);    
    
    gdk_flush();
    }
  else
    {
    // Make a non fullscreen window

    flags &= ~GV_VIDEO_WINDOW_FULLSCREEN;

    set_public_vars();

    //    gdk_pointer_ungrab(time);
    
    gtk_widget_hide(gtk_fullscreen_window);

    //    gtk_widget_set_usize(gtk_fullscreen_window, display_width, display_height);
    
    normal_window.show();
    
    pthread_mutex_lock(&size_mutex);    
    width = normal_width;
    height = normal_height;
    pthread_mutex_unlock(&size_mutex);    
    
    //    set_size(width_save, height_save);
    
    //    gdk_window_move(gtk_window->window, x_save, y_save);
    
    }
  }

void GVVideoWindow::get_size(int & _width, int & _height)
  {
  pthread_mutex_lock(&size_mutex);
  _width = width;
  _height = height;
  pthread_mutex_unlock(&size_mutex);
  }

void GVVideoWindow::get_position(int & x, int & y)
  {
  //  int width, height, depth;
  gdk_window_get_origin (gdk_window, &x, &y);
  }

int GVVideoWindow::get_depth()
  {
  int x, y, width, height, depth;
  gdk_window_get_geometry(gdk_window, &x, &y,
                          &width, &height, &depth);
  return depth;
  }

void GVVideoWindow::clear(int expose)     // Clear the window
  {
  if(!expose)
    gdk_window_clear(gdk_window);
  else
    {
    pthread_mutex_lock(&size_mutex);
    gdk_window_clear_area_e(gdk_window, 0, 0, width, height);
    pthread_mutex_unlock(&size_mutex);
    }
  }

void GVVideoWindow::set_original_size(int _width, int _height)
  {
  original_width = _width;
  original_height = _height;
  if(flags & GV_VIDEO_WINDOW_AUTO_RESIZE)
    set_size(original_width, original_height);
  }

void GVVideoWindow::set_size(int _width, int _height)
  {
  normal_width = _width;
  normal_height = _height;

  gtk_drawing_area_size(GTK_DRAWING_AREA(normal_drawingarea),
                        _width, _height);

  normal_window.set_size(_width, _height);

  if(flags & GV_VIDEO_WINDOW_FULLSCREEN)
    return;
  
  pthread_mutex_lock(&size_mutex);
  width = _width;
  height = _height;
  pthread_mutex_unlock(&size_mutex);
  }

GVVideoWindow::~GVVideoWindow()
  {
  put_config();
  hide();
  gtk_widget_destroy(gtk_fullscreen_window);
  }

#define SECTION "VideoWindow"

void GVVideoWindow::get_config()
  {
  int i_tmp = 1;
  gv_config->get_bool(SECTION, "AutoResize", i_tmp);
  if(i_tmp)
    {
    flags |=GV_VIDEO_WINDOW_AUTO_RESIZE;
    gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(auto_resize_item), 1);
    }

  i_tmp = 0;
  gv_config->get_bool(SECTION, "RememberSize", i_tmp);
  if(i_tmp)
    {
    flags |=GV_VIDEO_WINDOW_REMEMBER_SIZE;
    gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(remember_size_item), 1);

    int tmp_width;
    int tmp_height;

    get_size(tmp_width, tmp_height);

    gv_config->get_int(SECTION, "Width", tmp_width);
    gv_config->get_int(SECTION, "Height", tmp_height);

    set_size(tmp_width, tmp_height);
    }

  i_tmp = 0;
  gv_config->get_bool(SECTION, "RememberPosition", i_tmp);
  if(i_tmp)
    {
    flags |=GV_VIDEO_WINDOW_REMEMBER_POSITION;
    gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(remember_position_item), 1);
    }
  }

void GVVideoWindow::put_config()
  {
  gv_config->put_bool(SECTION, "ShowAtStart", !!is_visible());
  gv_config->put_bool(SECTION, "AutoResize",
                      !!(flags & GV_VIDEO_WINDOW_AUTO_RESIZE));
  gv_config->put_bool(SECTION, "RememberSize",
                      !!(flags & GV_VIDEO_WINDOW_REMEMBER_SIZE));
  gv_config->put_bool(SECTION, "RememberPosition",
                      !!(flags & GV_VIDEO_WINDOW_REMEMBER_POSITION));

  int tmp_width;
  int tmp_height;
  
  get_size(tmp_width, tmp_height);
  
  gv_config->put_int(SECTION, "Width", tmp_width);
  gv_config->put_int(SECTION, "Height", tmp_height);
  
  }

void GVVideoWindow::NormalWindow::before_resize_hook(int old_width,
                                                     int old_height,
                                                     int & new_width,
                                                     int & new_height)
  {
  gtk_drawing_area_size(GTK_DRAWING_AREA(drawingarea),
                        new_width, new_height);
  }

void GVVideoWindow::show()
  {
  if(!(flags & GV_VIDEO_WINDOW_FULLSCREEN))
    normal_window.show();
  }

void GVVideoWindow::hide()
  {
  set_fullscreen(0, gdk_time_get());
  normal_window.hide();
  }

int GVVideoWindow::get_active()
  {
  int ret;
  pthread_mutex_lock(&active_mutex);
  ret = active;
  pthread_mutex_unlock(&active_mutex);
  return ret;
  }

void GVVideoWindow::set_active(int _active)
  {
  //  cerr << "GVVideoWindow::set_active: " << _active << "\n";
  pthread_mutex_lock(&active_mutex);
  active = _active;
  pthread_mutex_unlock(&active_mutex);
  if(!_active)
    set_original_size(background_width, background_height);
  clear(1);
  }


GVVideoWindow * gv_video_window = (GVVideoWindow*)0;
