/*****************************************************************

  videowindow.h

  Copyright (c) 2001 by Burkhard Plaum - plaum@ipf.uni-stuttgart.de

  http://gmerlin.sourceforge.net

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111, USA.

*****************************************************************/

// Video window

#include <X11/Xlib.h>

#define GV_VIDEO_WINDOW_VISIBLE           (1<<0)
#define GV_VIDEO_WINDOW_FULLSCREEN        (1<<1)
#define GV_VIDEO_WINDOW_AUTO_RESIZE       (1<<2)
#define GV_VIDEO_WINDOW_ACTIVE            (1<<3)
#define GV_VIDEO_WINDOW_REMEMBER_SIZE     (1<<4)
#define GV_VIDEO_WINDOW_REMEMBER_POSITION (1<<5)

class GVVideoWindow
  {
  
  class NormalWindow : public GVWindow
    {
  public:
    void before_resize_hook(int old_width, int old_height,
                            int & new_width, int & new_height);

    GtkWidget * drawingarea;
    };

  // Gdk/Gtk Variables
  
  GdkGC     * gdk_normal_gc;
  GdkGC     * gdk_fullscreen_gc;
  GdkGC     * gdk_gc;
  
  GdkVisual * gdk_visual;
  //  GtkWidget * gtk_normal_window;
  GtkWidget * gtk_fullscreen_window;
  GdkWindow * gdk_window;
  GdkCursor * cursor;

  GtkWidget * normal_drawingarea;
  GtkWidget * fullscreen_drawingarea;
  
  GdkColor black;
  
  int fullscreen_width, fullscreen_height;
  int normal_width, normal_height;
  int original_width, original_height;
  
  int width, height;

  void set_public_vars();
  
  pthread_mutex_t size_mutex;

  GdkCursor * fullscreen_cursor;

  void create_fullscreen_cursor();

  pthread_mutex_t active_mutex;
  int active;
  
  int flags;

  NormalWindow normal_window;
  GtkWidget * fullscreen_item;
  GtkWidget * original_item;
  GtkWidget * auto_resize_item;
  GtkWidget * remember_size_item;
  GtkWidget * remember_position_item;
  
  GtkWidget * menu;

  void get_config();
  void put_config();

  int background_width;
  int background_height;
  
  GdkPixmap * default_background_pixmap; // This is loaded at startup
  GdkPixmap * current_background_pixmap;
    
public:

  void expose_callback();
    
  int get_active();
  void set_active(int _active);
    
  void apply_skin(GVWindowSkin * skin) { normal_window.apply_skin(skin); }
  
  // Xlib variables
  
  Window x_win;         // Window to draw to
  GC x_gc;              // Graphics context
  Display * x_display;  // Display variable for the Xlib functions
  Visual * x_visual;    // X Visual
  
  GVVideoWindow();

  void show();
  void hide();

  int is_visible() { return normal_window.is_visible(); }
  
  void lock();
  void unlock();
  void toggle_fullscreen(guint32 time);
  void set_fullscreen(int fullscreen, guint32 time);

  void get_size(int & width, int & height);
  void set_size(int width, int height);
  void set_original_size(int width, int height);

  void get_position(int & x, int & y);
  int get_depth();

  void clear(int expose = 0); // Clear the window

  void key_press_event(GdkEventKey * event);
  void configure_notify_event(GdkEventConfigure * event);
  void menu_callback(GtkWidget * w);

  ~GVVideoWindow();
  };

extern GVVideoWindow * gv_video_window;
