/*****************************************************************

  xmanager.cc

  Copyright (c) 2001 by Burkhard Plaum - plaum@ipf.uni-stuttgart.de

  http://gmerlin.sourceforge.net

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111, USA.

*****************************************************************/

// Manages various X things globally.
// It's main purpose is to convert GVVideoFrames into GdkPixmaps
// (With mask taken from alpha channel) for making skins and stuff,
// But other stuff also could go here

#include "config.h"
#include <iostream>

using std::cerr;
using std::ostream;

#include <gtk/gtk.h>
#include <string.h>

#include "gvtime.h"
#include "videoformat.h"
#include "videoconvert.h"

#include "baseplugin.h"
#include "imagereader.h"

#include "gvloader.h"

#include "xmanager.h"

#include <stdlib.h>

GVXManager * gv_x_manager = (GVXManager*)0;

GVXManager::GVXManager()
  {
  GdkWindowAttr attr;
  attr.title = (gchar*)0;
  attr.event_mask = GDK_EXPOSURE_MASK;
  attr.x = 0;
  attr.y = 0;
  attr.width = 10;
  attr.height = 10;
  attr.wclass =  GDK_INPUT_OUTPUT;
  attr.visual = (GdkVisual*)0;
  attr.colormap = (GdkColormap*)0;
  attr.window_type = GDK_WINDOW_TOPLEVEL;  
  attr.cursor = (GdkCursor*)0;
  attr.wmclass_name = (gchar*)0;
  attr.wmclass_class = (gchar*)0;
  attr.override_redirect = FALSE;

  window = gdk_window_new((GdkWindow*)0,
                          &attr,
                          0);

  int x, y, width, height;

  gdk_window_get_geometry(window, &x, &y, &width, &height, &window_depth);
  
  GdkVisual* gdk_visual = gdk_visual_get_system();

  gc = gdk_gc_new(window);
  
  // TODO: Handle the byte order

  int colorspace_not_supported = 0;

//   cerr << "gdk_visual->bits_per_rgb: "
//        << gdk_visual->bits_per_rgb << "\n";

//   cerr << "gdk_visual->depth: "
//        << gdk_visual->depth << "\n";
  
  GdkImage * gdk_image = gdk_image_new(GDK_IMAGE_FASTEST,
                                       gdk_visual,
                                       10, 10);

//   cerr << "image->bpp: "
//        << gdk_image->bpp << "\n";
  
  switch(gdk_image->bpp*8)
    {
    case 24:
      // Check for colorspace BGR32
      if((gdk_visual->red_mask == 0xff0000) && 
         (gdk_visual->green_mask == 0xff00) &&
         (gdk_visual->blue_mask == 0xff))
        colorspace = GVColorspaceBGR24;
      // Check for colorspace RGB32
      else if((gdk_visual->blue_mask == 0xff0000) && 
              (gdk_visual->green_mask == 0xff00) &&
              (gdk_visual->red_mask == 0xff))
        colorspace = GVColorspaceRGB24;
      else
        colorspace_not_supported = 1;
      break;
    case 32:
      // Check for colorspace RGB32
      if((gdk_visual->red_mask == 0xff) && 
         (gdk_visual->green_mask == 0xff00) &&
         (gdk_visual->blue_mask == 0xff0000))
        colorspace = GVColorspaceRGB32;
      // Check for colorspace BGR32
      if((gdk_visual->red_mask == 0xff0000) && 
         (gdk_visual->green_mask == 0xff00) &&
         (gdk_visual->blue_mask == 0xff))
        colorspace = GVColorspaceBGR32;
      else
        colorspace_not_supported = 1;
      break;
    case 16:
      // Check for colorspace BGR16
      if((gdk_visual->red_mask == 63488) &&
         (gdk_visual->green_mask == 2016) &&
         (gdk_visual->blue_mask == 31))
        colorspace = GVColorspaceBGR16;
      // Check for colorspace RGB16
      else if((gdk_visual->blue_mask == 63488) &&
              (gdk_visual->green_mask == 2016) &&
              (gdk_visual->red_mask == 31))
        colorspace = GVColorspaceRGB16;
      else
        colorspace_not_supported = 1;
      break;
    default:
      colorspace_not_supported = 1;
    }

  if(colorspace_not_supported)
    {
    cerr << "Sorry, but starting with version 0.1.4, gmerlin will run\n"
         << "only on displays with 16, 24 or 32 bits/pixel.\n"
         << "If you think, you have a Graphics card and an X11 setup\n"
         << "which should be supported by gmerlin, copy the following\n"
         << "lines and post them into the gmerlin developers forum at\n"
         << "http://sourcegorge.net/projects/gmerlin\n";

    cerr << "=====================================================";
    cerr << "GdkVisual:\ntype: ";
    
    switch(gdk_visual->type)
      {
      case GDK_VISUAL_STATIC_GRAY:
        cerr << "GDK_VISUAL_STATIC_GRAY\n";
        break;
      case GDK_VISUAL_GRAYSCALE:
        cerr << "GDK_VISUAL_GRAYSCALE\n";
        break;
      case GDK_VISUAL_STATIC_COLOR:
        cerr << "GDK_VISUAL_STATIC_COLOR\n";
        break;
      case GDK_VISUAL_PSEUDO_COLOR:
        cerr << "GDK_VISUAL_PSEUDO_COLOR\n";
        break;
      case GDK_VISUAL_TRUE_COLOR:
        cerr << "GDK_VISUAL_TRUE_COLOR\n";
        break;
      case GDK_VISUAL_DIRECT_COLOR:
        cerr << "GDK_VISUAL_DIRECT_COLOR\n";
        break;
      }
    cerr << "depth: " << gdk_visual->depth << "\n";
    cerr << "bits_per_rgb: " << gdk_visual->bits_per_rgb << "\n";

    cerr << "red_mask: "   << gdk_visual->red_mask   << "\n";
    cerr << "green_mask: " << gdk_visual->green_mask << "\n";
    cerr << "blue_mask: "  << gdk_visual->blue_mask  << "\n";

    cerr << "Bits per pixel (for images): "
         << gdk_image->bpp*8 << "\n";
    cerr << "=====================================================";
    exit(-1);
    }

  gdk_image_destroy(gdk_image);

  video_converter = new GVVideoConverter;

  // Create some cursors

  move_cursor = gdk_cursor_new(GDK_FLEUR);

  gmerlin_icon =      (GdkPixmap*)0;
  gmerlin_icon_mask = (GdkBitmap*)0;
  
  gmerlin_icon = load_image(DATA_DIR"/gmerlin_icon.png", &gmerlin_icon_mask);
  
  cerr << "X Manager initialized, Display colorspace: "
       << colorspace << "\n";
  }

GdkPixmap * GVXManager::load_image(const char * filename, GdkBitmap ** mask)
  {
  GdkPixmap * ret;
  GVVideoFrame output_frame;
  GVVideoFrame input_frame;
   
  const GVVideoFormat * input_format;

  GVVideoFormat output_format;
  
  GVImageReader * image_reader;
  GVPluginInfo * info = gv_image_reader_loader->find_plugin_by_suffix(filename);

  if(!info)
    return (GdkPixmap*)0;

//   cerr << "Loading Image reader...";
  image_reader = gv_image_reader_loader->load_image_reader(info);
//   cerr << "Done, attributes: "
//        << (const void*)image_reader->attributes << "\n";
  
  image_reader->init(filename, 1);

  input_format = image_reader->get_video_format();
 
  output_format = *input_format;

  output_format.colorspace = colorspace;
  output_format.init();

  alloc_video_frame(&output_frame, &output_format);
  
  if(input_format->colorspace == colorspace)
    {
    // We are lucky, we can display the image just as it comes
    // from the file
    image_reader->load_image(&output_frame);
    if(mask)
      *mask = get_bitmap(&output_frame, &output_format);
    }
  else
    {
    video_converter->init(input_format,
                          &output_format);

    input_frame.alloc(input_format);

    image_reader->load_image(&input_frame);
    if(mask)
      *mask = get_bitmap(&input_frame, input_format);
    video_converter->convert(&input_frame, &output_frame);
    input_frame.free();    
    }
  
  output_format.colorspace = colorspace;
  
  gv_image_reader_loader->destroy_plugin(image_reader);
  
  ret = gdk_pixmap_new(window, output_format.width, output_format.height,
                       window_depth);

  // Now, put the image onto the pixmap

  gdk_draw_image(ret,
                 gc,
                 (GdkImage*)(output_frame.data),
                 0, 0, 0, 0,
                 output_format.width,
                 output_format.height);
  free_video_frame(&output_frame);
  return ret;
  }

void GVXManager::alloc_video_frame(GVVideoFrame * frame,
                                   const GVVideoFormat * format)
  {
  GdkImage * image = gdk_image_new(GDK_IMAGE_FASTEST,
                                   gdk_visual_get_system(),
                                   format->width, format->height);

  frame->pixels = (guchar*)(image->mem);
  frame->pixels_rows = new guchar*[format->height];

  guchar * ptr = (guchar*)(image->mem);

  for(int i = 0; i < format->height; i++)
    {
    frame->pixels_rows[i] = ptr;
    ptr += image->bpl;
    }
  
  frame->data = (gpointer)(image);

  frame->y = (guchar*)0;
  frame->u = (guchar*)0;
  frame->v = (guchar*)0;

  frame->y_rows = (guchar**)0;
  frame->u_rows = (guchar**)0;
  frame->v_rows = (guchar**)0;
  }

void GVXManager::free_video_frame(GVVideoFrame * frame)
  {
  GdkImage * image = (GdkImage*)(frame->data);
  delete frame->pixels_rows;
  gdk_image_destroy(image);
  }

GdkBitmap * GVXManager::get_bitmap(const GVVideoFrame * video_frame,
                                   const GVVideoFormat * video_format)
  {
  GdkBitmap * ret;
  char * bitmap_pixels;
  char ** bitmap_pixels_rows;

  // Return NULL if we don't have an alpha channel
  // The following should be made extensible for future support
  // of other alpha capable colorspaces (but I hope, there won't come
  // more)

  if(!((int)(video_format->colorspace) & GV_COLORSPACE_HAS_ALPHA))
    return (GdkBitmap*)0;
  
  int bytes_per_line = (video_format->width + 7)/8;
  bitmap_pixels = new char[video_format->height * bytes_per_line];
  bitmap_pixels_rows = new char*[video_format->height];

  for(int i = 0; i < video_format->height; i++)
    bitmap_pixels_rows[i] = (&bitmap_pixels[i*bytes_per_line]);

  // Now, set the bits

  int current_bit;
  int current_byte;
  int val;

  memset(bitmap_pixels, 0x00, video_format->height * bytes_per_line);

  switch(video_format->colorspace)
    {
    case GVColorspaceRGBA32:
      for(int i = 0; i < video_format->height; i++)
        {
        current_bit = 0;
        current_byte = 0;
        for(int j = 0; j < video_format->width; j++)
          {
          // We accept pixels with alpha values > 128 for being visible
          val = !!(video_frame->pixels_rows[i][4*j+3] & 0x80);
          bitmap_pixels_rows[i][current_byte] |= (char)(val << current_bit);

          current_bit++;

          if(current_bit == 8)
            {
            current_bit = 0;
            current_byte++;
            }
          }
        }
      break;
    default:
      // This never happens, but keeps g++ quiet
      return (GdkBitmap*)0;
    }

  ret = gdk_bitmap_create_from_data(window,
                                    bitmap_pixels,
                                    video_format->width,
                                    video_format->height);
  delete bitmap_pixels;
  delete bitmap_pixels_rows;
  return ret;
  }
